@@ -198,3 +198,91 @@ assert_equal %q{[10, main]}, %q{
     end
   }, '[ruby-dev:31372]'
 end
+
+assert_normal_exit %q{
+  STDERR.reopen(STDOUT)
+  class Foo
+     def self.add_method
+       class_eval("def some-bad-name; puts 'hello' unless @some_variable.some_function(''); end")
+     end
+  end
+  Foo.add_method
+}, '[ruby-core:14556] reported by Frederick Cheung'
+
+assert_equal 'ok', %q{
+  class Module
+    def my_module_eval(&block)
+      module_eval(&block)
+    end
+  end
+  class String
+    Integer.my_module_eval do
+      def hoge; end
+    end
+  end
+  if Integer.instance_methods(false).map{|m|m.to_sym}.include?(:hoge) &&
+     !String.instance_methods(false).map{|m|m.to_sym}.include?(:hoge)
+    :ok
+  else
+    :ng
+  end
+}, "[ruby-dev:34236]"
+
+assert_equal 'ok', %q{
+  begin
+    eval("class nil::Foo; end")
+    :ng
+  rescue Exception
+    :ok
+  end
+}
+
+assert_equal 'ok', %q{
+  begin
+    0.instance_eval { def m() :m end }
+    1.m
+    :ng
+  rescue Exception
+    :ok
+  end
+}, '[ruby-dev:34579]'
+
+assert_equal 'ok', %q{
+  begin
+    12.instance_eval { @@a }
+  rescue NameError
+    :ok
+  end
+}, '[ruby-core:16794]'
+
+assert_equal 'ok', %q{
+  begin
+    12.instance_exec { @@a }
+  rescue NameError
+    :ok
+  end
+}, '[ruby-core:16794]'
+
+assert_equal 'ok', %q{
+  begin
+    nil.instance_eval {
+      def a() :a end
+    }
+  rescue TypeError
+    :ok
+  end
+}, '[ruby-core:16796]'
+
+assert_equal 'ok', %q{
+  begin
+    nil.instance_exec {
+      def a() :a end
+    }
+  rescue TypeError
+    :ok
+  end
+}, '[ruby-core:16796]'
+
+assert_normal_exit %q{
+  eval("", method(:proc).call {}.binding)
+}
