@@ -2,8 +2,7 @@
 
   compar.c -
 
-  $Author: akr $
-  $Date: 2007-09-01 21:02:36 +0900 (Sat, 01 Sep 2007) $
+  $Author: nobu $
   created at: Thu Aug 26 14:39:48 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -16,22 +15,6 @@ VALUE rb_mComparable;
 
 static ID cmp;
 
-int
-rb_cmpint(VALUE val, VALUE a, VALUE b)
-{
-    if (NIL_P(val)) {
-	rb_cmperr(a, b);
-    }
-    if (FIXNUM_P(val)) return FIX2INT(val);
-    if (TYPE(val) == T_BIGNUM) {
-	if (RBIGNUM_SIGN(val)) return 1;
-	return -1;
-    }
-    if (RTEST(rb_funcall(val, '>', 1, INT2FIX(0)))) return 1;
-    if (RTEST(rb_funcall(val, '<', 1, INT2FIX(0)))) return -1;
-    return 0;
-}
-
 void
 rb_cmperr(VALUE x, VALUE y)
 {
@@ -215,6 +198,8 @@ cmp_between(VALUE x, VALUE min, VALUE max)
 void
 Init_Comparable(void)
 {
+#undef rb_intern
+
     rb_mComparable = rb_define_module("Comparable");
     rb_define_method(rb_mComparable, "==", cmp_equal, 1);
     rb_define_method(rb_mComparable, ">", cmp_gt, 1);
