@@ -3,7 +3,6 @@
   cont.c - 
 
   $Author: ko1 $
-  $Date: 2007-08-26 12:31:20 +0900 (日, 26  8月 2007) $
   created at: Thu May 23 09:03:43 2007
 
   Copyright (C) 2007 Koichi Sasada
@@ -15,6 +14,12 @@
 #include "gc.h"
 #include "eval_intern.h"
 
+enum context_type {
+    CONTINUATION_CONTEXT = 0,
+    FIBER_CONTEXT = 1,
+    ROOT_FIBER_CONTEXT = 2,
+};
+
 typedef struct rb_context_struct {
     VALUE self;
     VALUE value;
@@ -29,9 +34,9 @@ typedef struct rb_context_struct {
     rb_thread_t saved_thread;
     rb_jmpbuf_t jmpbuf;
     int machine_stack_size;
-    /* for cont */
     VALUE prev;
     int alive;
+    enum context_type type;
 } rb_context_t;
 
 static VALUE rb_cContinuation;
@@ -86,6 +91,11 @@ cont_free(void *ptr)
 	RUBY_FREE_UNLESS_NULL(cont->machine_register_stack);
 #endif
 	RUBY_FREE_UNLESS_NULL(cont->vm_stack);
+
+	if (cont->type == FIBER_CONTEXT) {
+	    st_free_table(cont->saved_thread.local_storage);
+	}
+
 	ruby_xfree(ptr);
     }
     RUBY_FREE_LEAVE("cont");
@@ -118,6 +128,7 @@ cont_save_machine_stack(rb_thread_t *th, rb_context_t *cont)
 	cont->machine_stack = ALLOC_N(VALUE, size);
     }
 
+    FLUSH_REGISTER_WINDOWS;
     MEMCPY(cont->machine_stack, cont->machine_stack_src, VALUE, size);
 
 #ifdef __ia64
@@ -147,8 +158,7 @@ cont_new(VALUE klass)
     volatile VALUE contval;
     rb_thread_t *th = GET_THREAD();
 
-    contval = Data_Make_Struct(klass, rb_context_t,
-			       cont_mark, cont_free, cont);
+    contval = Data_Make_Struct(klass, rb_context_t, cont_mark, cont_free, cont);
 
     cont->self = contval;
     cont->alive = Qtrue;
@@ -201,13 +211,7 @@ cont_restore_1(rb_context_t *cont)
     rb_thread_t *th = GET_THREAD(), *sth = &cont->saved_thread;
 
     /* restore thread context */
-    if (sth->stack) {
-	/* fiber */
-	th->stack = sth->stack;
-	th->stack_size = sth->stack_size;
-	th->fiber = cont->self;
-    }
-    else {
+    if (cont->type == CONTINUATION_CONTEXT) {
 	/* continuation */
 	VALUE fib;
 
@@ -222,6 +226,13 @@ cont_restore_1(rb_context_t *cont)
 	}
 	MEMCPY(th->stack, cont->vm_stack, VALUE, sth->stack_size);
     }
+    else {
+	/* fiber */
+	th->stack = sth->stack;
+	th->stack_size = sth->stack_size;
+	th->local_storage = sth->local_storage;
+	th->fiber = cont->self;
+    }
 
     th->cfp = sth->cfp;
     th->safe_level = sth->safe_level;
@@ -244,6 +255,7 @@ cont_restore_1(rb_context_t *cont)
     }
 #endif
     if (cont->machine_stack_src) {
+	FLUSH_REGISTER_WINDOWS;
 	MEMCPY(cont->machine_stack_src, cont->machine_stack,
 	       VALUE, cont->machine_stack_size);
     }
@@ -310,7 +322,7 @@ cont_restore_0(rb_context_t *cont, VALUE *addr_in_prev_frame)
 #else
 	if (addr_in_prev_frame > &space[0]) {
 	    /* Stack grows downward */
-	    if (addr_in_prev_frame > cont->saved_thread.machine_stack_src) {
+	    if (addr_in_prev_frame > cont->machine_stack_src) {
 		cont_restore_0(cont, &space[0]);
 	    }
 	}
@@ -471,15 +483,25 @@ rb_cont_call(int argc, VALUE *argv, VALUE contval)
 
 #define FIBER_VM_STACK_SIZE (4 * 1024)
 
-static VALUE
-fiber_alloc(VALUE klass, VALUE proc)
+static rb_context_t *
+fiber_alloc(VALUE klass)
 {
     rb_context_t *cont = cont_new(klass);
+
+    cont->type = FIBER_CONTEXT;
+    cont->prev = Qnil;
+
+    return cont;
+}
+
+static VALUE
+fiber_new(VALUE klass, VALUE proc)
+{
+    rb_context_t *cont = fiber_alloc(klass);
     VALUE contval = cont->self;
     rb_thread_t *th = &cont->saved_thread;
 
     /* initialize */
-    cont->prev = Qnil;
     cont->vm_stack = 0;
 
     th->stack = 0;
@@ -500,6 +522,7 @@ fiber_alloc(VALUE klass, VALUE proc)
     th->cfp->proc = 0;
     th->cfp->block_iseq = 0;
     th->tag = 0;
+    th->local_storage = st_init_numtable();
 
     th->first_proc = proc;
 
@@ -511,13 +534,13 @@ fiber_alloc(VALUE klass, VALUE proc)
 VALUE
 rb_fiber_new(VALUE (*func)(ANYARGS), VALUE obj)
 {
-    return fiber_alloc(rb_cFiber, rb_proc_new(func, obj));
+    return fiber_new(rb_cFiber, rb_proc_new(func, obj));
 }
 
 static VALUE
 rb_fiber_s_new(VALUE self)
 {
-    return fiber_alloc(self, rb_block_proc());
+    return fiber_new(self, rb_block_proc());
 }
 
 static VALUE
@@ -573,7 +596,7 @@ rb_fiber_start(void)
 	th->local_lfp = proc->block.lfp;
 	th->local_svar = Qnil;
 
-	cont->value = vm_invoke_proc(th, proc, proc->block.self, 1, &args);
+	cont->value = vm_invoke_proc(th, proc, proc->block.self, 1, &args, 0);
     }
     TH_POP_TAG();
 
@@ -585,7 +608,7 @@ rb_fiber_start(void)
 	    th->thrown_errinfo =
 	      vm_make_jump_tag_but_local_jump(state, th->errinfo);
 	}
-	th->interrupt_flag = 1;
+	RUBY_VM_SET_INTERRUPT(th);
     }
 
     rb_fiber_terminate(cont);
@@ -598,15 +621,15 @@ rb_fiber_current()
     rb_thread_t *th = GET_THREAD();
     if (th->fiber == 0) {
 	/* save root */
-	rb_context_t *cont = cont_new(rb_cFiber);
-	cont->prev = Qnil;
+	rb_context_t *cont = fiber_alloc(rb_cFiber);
+	cont->type = ROOT_FIBER_CONTEXT;
 	th->root_fiber = th->fiber = cont->self;
     }
     return th->fiber;
 }
 
 static VALUE
-cont_store(rb_context_t *next_cont)
+fiber_store(rb_context_t *next_cont)
 {
     rb_thread_t *th = GET_THREAD();
     rb_context_t *cont;
@@ -617,8 +640,8 @@ cont_store(rb_context_t *next_cont)
     }
     else {
 	/* create current fiber */
-	cont = cont_new(rb_cFiber); /* no need to allocate vm stack */
-	cont->prev = Qnil;
+	cont = fiber_alloc(rb_cFiber); /* no need to allocate vm stack */
+	cont->type = ROOT_FIBER_CONTEXT;
 	th->root_fiber = th->fiber = cont->self;
     }
 
@@ -659,7 +682,7 @@ fiber_switch(VALUE fib, int argc, VALUE *argv, int is_resume)
 
     cont->value = make_passing_arg(argc, argv);
 
-    if ((value = cont_store(cont)) == Qundef) {
+    if ((value = fiber_store(cont)) == Qundef) {
 	cont_restore_0(cont, &value);
 	rb_bug("rb_fiber_resume: unreachable");
     }
@@ -732,6 +755,9 @@ Init_Cont(void)
     rb_cFiber = rb_define_class("Fiber", rb_cObject);
     rb_undef_alloc_func(rb_cFiber);
     rb_eFiberError = rb_define_class("FiberError", rb_eStandardError);
+    rb_define_singleton_method(rb_cFiber, "new", rb_fiber_s_new, 0);
+    rb_define_singleton_method(rb_cFiber, "yield", rb_fiber_s_yield, -1);
+    rb_define_method(rb_cFiber, "resume", rb_fiber_m_resume, -1);
 }
 
 void
@@ -746,12 +772,9 @@ Init_Continuation_body(void)
 }
 
 void
-Init_Fiber_body(void)
+Init_Fiber_as_Coroutine(void)
 {
-    rb_define_singleton_method(rb_cFiber, "new", rb_fiber_s_new, 0);
-    rb_define_method(rb_cFiber, "resume", rb_fiber_m_resume, -1);
     rb_define_method(rb_cFiber, "transfer", rb_fiber_m_transfer, -1);
     rb_define_method(rb_cFiber, "alive?", rb_fiber_alive_p, 0);
-    rb_define_singleton_method(rb_cFiber, "yield", rb_fiber_s_yield, -1);
     rb_define_singleton_method(rb_cFiber, "current", rb_fiber_s_current, 0);
 }
