@@ -2,8 +2,7 @@
 
   dir.c -
 
-  $Author: matz $
-  $Date: 2007-12-25 13:17:06 +0900 (Tue, 25 Dec 2007) $
+  $Author: usa $
   created at: Wed Jan  5 09:51:01 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -13,6 +12,7 @@
 **********************************************************************/
 
 #include "ruby/ruby.h"
+#include "ruby/encoding.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -67,14 +67,6 @@ char *strchr(char*,char);
 #define lstat stat
 #endif
 
-#ifndef CASEFOLD_FILESYSTEM
-# if defined DOSISH || defined __VMS
-#   define CASEFOLD_FILESYSTEM 1
-# else
-#   define CASEFOLD_FILESYSTEM 0
-# endif
-#endif
-
 #define FNM_NOESCAPE	0x01
 #define FNM_PATHNAME	0x02
 #define FNM_DOTMATCH	0x04
@@ -88,7 +80,7 @@ char *strchr(char*,char);
 #define FNM_NOMATCH	1
 #define FNM_ERROR	2
 
-#define downcase(c) (nocase && ISUPPER(c) ? tolower(c) : (c))
+#define downcase(c) (nocase && ISUPPER(c) ? TOLOWER(c) : (c))
 #define compare(c1, c2) (((unsigned char)(c1)) - ((unsigned char)(c2)))
 
 /* caution: in case *p == '\0'
@@ -351,6 +343,8 @@ VALUE rb_cDir;
 struct dir_data {
     DIR *dir;
     char *path;
+    rb_encoding *intenc;
+    rb_encoding *extenc;
 };
 
 static void
@@ -358,9 +352,9 @@ free_dir(struct dir_data *dir)
 {
     if (dir) {
 	if (dir->dir) closedir(dir->dir);
-	if (dir->path) free(dir->path);
+	if (dir->path) xfree(dir->path);
     }
-    free(dir);
+    xfree(dir);
 }
 
 static VALUE dir_close(VALUE);
@@ -373,6 +367,8 @@ dir_s_alloc(VALUE klass)
 
     dirp->dir = NULL;
     dirp->path = NULL;
+    dirp->intenc = NULL;
+    dirp->extenc = NULL;
 
     return obj;
 }
@@ -384,16 +380,71 @@ dir_s_alloc(VALUE klass)
  *  Returns a new directory object for the named directory.
  */
 static VALUE
-dir_initialize(VALUE dir, VALUE dirname)
+dir_initialize(int argc, VALUE *argv, VALUE dir)
 {
     struct dir_data *dp;
+    static rb_encoding *fs_encoding;
+    rb_encoding  *intencoding, *extencoding;
+    VALUE dirname, opt;
+    static VALUE sym_intenc, sym_extenc;
 
+    if (!sym_intenc) {
+	sym_intenc = ID2SYM(rb_intern("internal_encoding"));
+	sym_extenc = ID2SYM(rb_intern("external_encoding"));
+	fs_encoding = rb_filesystem_encoding();
+    }
+
+    intencoding = NULL;
+    extencoding = fs_encoding;
+    rb_scan_args(argc, argv, "11", &dirname, &opt);
+
+    if (!NIL_P(opt)) {
+        VALUE v, extenc=Qnil, intenc=Qnil;
+        opt = rb_check_convert_type(opt, T_HASH, "Hash", "to_hash");
+
+        v = rb_hash_aref(opt, sym_intenc);
+        if (!NIL_P(v)) intenc = v;
+        v = rb_hash_aref(opt, sym_extenc);
+        if (!NIL_P(v)) extenc = v;
+
+	if (!NIL_P(extenc)) {
+	    extencoding = rb_to_encoding(extenc);
+	    if (!NIL_P(intenc)) {
+		intencoding = rb_to_encoding(intenc);
+		if (extencoding == intencoding) {
+		    rb_warn("Ignoring internal encoding '%s': it is identical to external encoding '%s'",
+			    RSTRING_PTR(rb_inspect(intenc)),
+			    RSTRING_PTR(rb_inspect(extenc)));
+		    intencoding = NULL;
+		}
+	    }
+	}
+	else if (!NIL_P(intenc)) {
+	    rb_raise(rb_eArgError, "External encoding must be specified when internal encoding is given");
+	}
+    }
+
+    {
+	rb_encoding  *dirname_encoding = rb_enc_get(dirname);
+	if (rb_usascii_encoding() != dirname_encoding
+	    && rb_ascii8bit_encoding() != dirname_encoding
+#if defined __APPLE__
+	    && rb_utf8_encoding() != dirname_encoding
+#endif
+	    && extencoding != dirname_encoding) {
+	    if (!intencoding) intencoding = dirname_encoding;
+	    dirname = rb_str_transcode(dirname, rb_enc_from_encoding(extencoding));
+	}
+    }
     FilePathValue(dirname);
+
     Data_Get_Struct(dir, struct dir_data, dp);
     if (dp->dir) closedir(dp->dir);
-    if (dp->path) free(dp->path);
+    if (dp->path) xfree(dp->path);
     dp->dir = NULL;
     dp->path = NULL;
+    dp->intenc = intencoding;
+    dp->extenc = extencoding;
     dp->dir = opendir(RSTRING_PTR(dirname));
     if (dp->dir == NULL) {
 	if (errno == EMFILE || errno == ENFILE) {
@@ -421,12 +472,12 @@ dir_initialize(VALUE dir, VALUE dirname)
  *  block.
  */
 static VALUE
-dir_s_open(VALUE klass, VALUE dirname)
+dir_s_open(int argc, VALUE *argv, VALUE klass)
 {
     struct dir_data *dp;
     VALUE dir = Data_Make_Struct(klass, struct dir_data, 0, free_dir, dp);
 
-    dir_initialize(dir, dirname);
+    dir_initialize(argc, argv, dir);
     if (rb_block_given_p()) {
 	return rb_ensure(rb_yield, dir, dir_close, dir);
     }
@@ -454,6 +505,16 @@ dir_check(VALUE dir)
     if (dirp->dir == NULL) dir_closed();\
 } while (0)
 
+static VALUE
+dir_enc_str(VALUE str, struct dir_data *dirp)
+{
+    rb_enc_associate(str, dirp->extenc);
+    if (dirp->intenc) {
+        str = rb_str_transcode(str, rb_enc_from_encoding(dirp->intenc));
+    }
+    return str;
+}
+
 /*
  *  call-seq:
  *     dir.inspect => string
@@ -467,7 +528,7 @@ dir_inspect(VALUE dir)
 
     Data_Get_Struct(dir, struct dir_data, dirp);
     if (dirp->path) {
-	char *c = rb_obj_classname(dir);
+	const char *c = rb_obj_classname(dir);
 	int len = strlen(c) + strlen(dirp->path) + 4;
 	VALUE s = rb_str_new(0, len);
 	snprintf(RSTRING_PTR(s), len+1, "#<%s:%s>", c, dirp->path);
@@ -492,7 +553,7 @@ dir_path(VALUE dir)
 
     Data_Get_Struct(dir, struct dir_data, dirp);
     if (!dirp->path) return Qnil;
-    return rb_str_new2(dirp->path);
+    return dir_enc_str(rb_str_new2(dirp->path), dirp);
 }
 
 /*
@@ -517,7 +578,7 @@ dir_read(VALUE dir)
     errno = 0;
     dp = readdir(dirp->dir);
     if (dp) {
-	return rb_tainted_str_new(dp->d_name, NAMLEN(dp));
+	return dir_enc_str(rb_tainted_str_new(dp->d_name, NAMLEN(dp)), dirp);
     }
     else if (errno == 0) {	/* end of stream */
 	return Qnil;
@@ -555,7 +616,7 @@ dir_each(VALUE dir)
     GetDIR(dir, dirp);
     rewinddir(dirp->dir);
     for (dp = readdir(dirp->dir); dp != NULL; dp = readdir(dirp->dir)) {
-	rb_yield(rb_tainted_str_new(dp->d_name, NAMLEN(dp)));
+	rb_yield(dir_enc_str(rb_tainted_str_new(dp->d_name, NAMLEN(dp)), dirp));
 	if (dirp->dir == NULL) dir_closed();
     }
     return dir;
@@ -581,7 +642,7 @@ dir_tell(VALUE dir)
     struct dir_data *dirp;
     long pos;
 
-    Data_Get_Struct(dir, struct dir_data, dirp);
+    GetDIR(dir, dirp);
     pos = telldir(dirp->dir);
     return rb_int2inum(pos);
 #else
@@ -789,7 +850,7 @@ dir_s_chdir(int argc, VALUE *argv, VALUE obj)
 	struct chdir_data args;
 	char *cwd = my_getcwd();
 
-	args.old_path = rb_tainted_str_new2(cwd); free(cwd);
+	args.old_path = rb_tainted_str_new2(cwd); xfree(cwd);
 	args.new_path = path;
 	args.done = Qfalse;
 	return rb_ensure(chdir_yield, (VALUE)&args, chdir_restore, (VALUE)&args);
@@ -820,7 +881,7 @@ dir_s_getwd(VALUE dir)
     path = my_getcwd();
     cwd = rb_tainted_str_new2(path);
 
-    free(path);
+    xfree(path);
     return cwd;
 }
 
@@ -926,6 +987,7 @@ sys_warning_1(const char* mesg)
 
 #define GLOB_ALLOC(type) (type *)malloc(sizeof(type))
 #define GLOB_ALLOC_N(type, n) (type *)malloc(sizeof(type) * (n))
+#define GLOB_FREE(ptr) free(ptr)
 #define GLOB_JUMP_TAG(status) ((status == -1) ? rb_memerror() : rb_jump_tag(status))
 
 /*
@@ -1035,7 +1097,7 @@ find_dirsep(const char *s, int flags)
     return (char *)p-1;
 }
 
-/* Remove escaping baskclashes */
+/* Remove escaping backslashes */
 static void
 remove_backslashes(char *p)
 {
@@ -1090,7 +1152,7 @@ glob_make_pattern(const char *p, int flags)
 	    const char *m = find_dirsep(p, flags);
 	    char *buf = GLOB_ALLOC_N(char, m-p+1);
 	    if (!buf) {
-		free(tmp);
+		GLOB_FREE(tmp);
 		goto error;
 	    }
 	    memcpy(buf, p, m-p);
@@ -1132,8 +1194,8 @@ glob_free_pattern(struct glob_pattern *list)
 	struct glob_pattern *tmp = list;
 	list = list->next;
 	if (tmp->str)
-	    free(tmp->str);
-	free(tmp);
+	    GLOB_FREE(tmp->str);
+	GLOB_FREE(tmp);
     }
 }
 
@@ -1255,7 +1317,7 @@ glob_helper(
 	    char *tmp = join_path(path, dirsep, "");
 	    if (!tmp) return -1;
 	    status = glob_call_func(func, tmp, arg);
-	    free(tmp);
+	    GLOB_FREE(tmp);
 	    if (status) return status;
 	}
     }
@@ -1307,8 +1369,8 @@ glob_helper(
 	    }
 
 	    status = glob_helper(buf, 1, YES, new_isdir, new_beg, new_end, flags, func, arg);
-	    free(buf);
-	    free(new_beg);
+	    GLOB_FREE(buf);
+	    GLOB_FREE(new_beg);
 	    if (status) break;
 	}
 
@@ -1336,7 +1398,7 @@ glob_helper(
 
 		new_beg = new_end = GLOB_ALLOC_N(struct glob_pattern *, end - beg);
 		if (!new_beg) {
-		    free(name);
+		    GLOB_FREE(name);
 		    status = -1;
 		    break;
 		}
@@ -1349,20 +1411,20 @@ glob_helper(
 		}
 
 		buf = join_path(path, dirsep, name);
-		free(name);
+		GLOB_FREE(name);
 		if (!buf) {
-		    free(new_beg);
+		    GLOB_FREE(new_beg);
 		    status = -1;
 		    break;
 		}
 		status = glob_helper(buf, 1, UNKNOWN, UNKNOWN, new_beg, new_end, flags, func, arg);
-		free(buf);
-		free(new_beg);
+		GLOB_FREE(buf);
+		GLOB_FREE(new_beg);
 		if (status) break;
 	    }
 	}
 
-	free(copy_beg);
+	GLOB_FREE(copy_beg);
     }
 
     return status;
@@ -1393,12 +1455,12 @@ ruby_glob0(const char *path, int flags, ruby_glob_func *func, VALUE arg)
 
     list = glob_make_pattern(root, flags);
     if (!list) {
-	free(buf);
+	GLOB_FREE(buf);
 	return -1;
     }
     status = glob_helper(buf, 0, UNKNOWN, UNKNOWN, &list, &list + 1, flags, func, arg);
     glob_free_pattern(list);
-    free(buf);
+    GLOB_FREE(buf);
 
     return status;
 }
@@ -1495,7 +1557,7 @@ ruby_brace_expand(const char *str, int flags, ruby_glob_func *func, VALUE arg)
 	    status = ruby_brace_expand(buf, flags, func, arg);
 	    if (status) break;
 	}
-	free(buf);
+	GLOB_FREE(buf);
     }
     else if (!lbrace && !rbrace) {
 	status = (*func)(s, arg);
@@ -1654,14 +1716,14 @@ dir_s_aref(int argc, VALUE *argv, VALUE obj)
  *
  *     rbfiles = File.join("**", "*.rb")
  *     Dir.glob(rbfiles)                   #=> ["main.rb",
- *                                              "lib/song.rb",
- *                                              "lib/song/karaoke.rb"]
+ *                                         #    "lib/song.rb",
+ *                                         #    "lib/song/karaoke.rb"]
  *     libdirs = File.join("**", "lib")
  *     Dir.glob(libdirs)                   #=> ["lib"]
  *
  *     librbfiles = File.join("**", "lib", "**", "*.rb")
  *     Dir.glob(librbfiles)                #=> ["lib/song.rb",
- *                                              "lib/song/karaoke.rb"]
+ *                                         #    "lib/song/karaoke.rb"]
  *
  *     librbfiles = File.join("**", "lib", "*.rb")
  *     Dir.glob(librbfiles)                #=> ["lib/song.rb"]
@@ -1694,9 +1756,9 @@ dir_s_glob(int argc, VALUE *argv, VALUE obj)
 }
 
 static VALUE
-dir_open_dir(VALUE path)
+dir_open_dir(int argc, VALUE *argv)
 {
-    VALUE dir = rb_funcall(rb_cDir, rb_intern("open"), 1, path);
+    VALUE dir = rb_funcall2(rb_cDir, rb_intern("open"), argc, argv);
 
     if (TYPE(dir) != T_DATA ||
 	RDATA(dir)->dfree != (RUBY_DATA_FUNC)free_dir) {
@@ -1725,12 +1787,12 @@ dir_open_dir(VALUE path)
  *
  */
 static VALUE
-dir_foreach(VALUE io, VALUE dirname)
+dir_foreach(int argc, VALUE *argv, VALUE io)
 {
     VALUE dir;
 
-    RETURN_ENUMERATOR(io, 1, &dirname);
-    dir = dir_open_dir(dirname);
+    RETURN_ENUMERATOR(io, argc, argv);
+    dir = dir_open_dir(argc, argv);
     rb_ensure(dir_each, dir, dir_close, dir);
     return Qnil;
 }
@@ -1747,11 +1809,11 @@ dir_foreach(VALUE io, VALUE dirname)
  *
  */
 static VALUE
-dir_entries(VALUE io, VALUE dirname)
+dir_entries(int argc, VALUE *argv, VALUE io)
 {
     VALUE dir;
 
-    dir = dir_open_dir(dirname);
+    dir = dir_open_dir(argc, argv);
     return rb_ensure(rb_Array, dir, dir_close, dir);
 }
 
@@ -1789,31 +1851,31 @@ dir_entries(VALUE io, VALUE dirname)
  *  parameters. The same glob pattern and flags are used by
  *  <code>Dir::glob</code>.
  *
- *     File.fnmatch('cat',       'cat')        #=> true  : match entire string
- *     File.fnmatch('cat',       'category')   #=> false : only match partial string
- *     File.fnmatch('c{at,ub}s', 'cats')       #=> false : { } isn't supported
+ *     File.fnmatch('cat',       'cat')        #=> true  # match entire string
+ *     File.fnmatch('cat',       'category')   #=> false # only match partial string
+ *     File.fnmatch('c{at,ub}s', 'cats')       #=> false # { } isn't supported
  *
- *     File.fnmatch('c?t',     'cat')          #=> true  : '?' match only 1 character
- *     File.fnmatch('c??t',    'cat')          #=> false : ditto
- *     File.fnmatch('c*',      'cats')         #=> true  : '*' match 0 or more characters
- *     File.fnmatch('c*t',     'c/a/b/t')      #=> true  : ditto
- *     File.fnmatch('ca[a-z]', 'cat')          #=> true  : inclusive bracket expression
- *     File.fnmatch('ca[^t]',  'cat')          #=> false : exclusive bracket expression ('^' or '!')
+ *     File.fnmatch('c?t',     'cat')          #=> true  # '?' match only 1 character
+ *     File.fnmatch('c??t',    'cat')          #=> false # ditto
+ *     File.fnmatch('c*',      'cats')         #=> true  # '*' match 0 or more characters
+ *     File.fnmatch('c*t',     'c/a/b/t')      #=> true  # ditto
+ *     File.fnmatch('ca[a-z]', 'cat')          #=> true  # inclusive bracket expression
+ *     File.fnmatch('ca[^t]',  'cat')          #=> false # exclusive bracket expression ('^' or '!')
  *
- *     File.fnmatch('cat', 'CAT')                     #=> false : case sensitive
- *     File.fnmatch('cat', 'CAT', File::FNM_CASEFOLD) #=> true  : case insensitive
+ *     File.fnmatch('cat', 'CAT')                     #=> false # case sensitive
+ *     File.fnmatch('cat', 'CAT', File::FNM_CASEFOLD) #=> true  # case insensitive
  *
- *     File.fnmatch('?',   '/', File::FNM_PATHNAME)  #=> false : wildcard doesn't match '/' on FNM_PATHNAME
- *     File.fnmatch('*',   '/', File::FNM_PATHNAME)  #=> false : ditto
- *     File.fnmatch('[/]', '/', File::FNM_PATHNAME)  #=> false : ditto
+ *     File.fnmatch('?',   '/', File::FNM_PATHNAME)  #=> false # wildcard doesn't match '/' on FNM_PATHNAME
+ *     File.fnmatch('*',   '/', File::FNM_PATHNAME)  #=> false # ditto
+ *     File.fnmatch('[/]', '/', File::FNM_PATHNAME)  #=> false # ditto
  *
- *     File.fnmatch('\?',   '?')                       #=> true  : escaped wildcard becomes ordinary
- *     File.fnmatch('\a',   'a')                       #=> true  : escaped ordinary remains ordinary
- *     File.fnmatch('\a',   '\a', File::FNM_NOESCAPE)  #=> true  : FNM_NOESACPE makes '\' ordinary
- *     File.fnmatch('[\?]', '?')                       #=> true  : can escape inside bracket expression
+ *     File.fnmatch('\?',   '?')                       #=> true  # escaped wildcard becomes ordinary
+ *     File.fnmatch('\a',   'a')                       #=> true  # escaped ordinary remains ordinary
+ *     File.fnmatch('\a',   '\a', File::FNM_NOESCAPE)  #=> true  # FNM_NOESACPE makes '\' ordinary
+ *     File.fnmatch('[\?]', '?')                       #=> true  # can escape inside bracket expression
  *
- *     File.fnmatch('*',   '.profile')                      #=> false : wildcard doesn't match leading
- *     File.fnmatch('*',   '.profile', File::FNM_DOTMATCH)  #=> true    period by default.
+ *     File.fnmatch('*',   '.profile')                      #=> false # wildcard doesn't match leading
+ *     File.fnmatch('*',   '.profile', File::FNM_DOTMATCH)  #=> true  # period by default.
  *     File.fnmatch('.*',  '.profile')                      #=> true
  *
  *     rbfiles = '**' '/' '*.rb' # you don't have to do like this. just write in single string.
@@ -1876,11 +1938,11 @@ Init_Dir(void)
     rb_include_module(rb_cDir, rb_mEnumerable);
 
     rb_define_alloc_func(rb_cDir, dir_s_alloc);
-    rb_define_singleton_method(rb_cDir, "open", dir_s_open, 1);
-    rb_define_singleton_method(rb_cDir, "foreach", dir_foreach, 1);
-    rb_define_singleton_method(rb_cDir, "entries", dir_entries, 1);
+    rb_define_singleton_method(rb_cDir, "open", dir_s_open, -1);
+    rb_define_singleton_method(rb_cDir, "foreach", dir_foreach, -1);
+    rb_define_singleton_method(rb_cDir, "entries", dir_entries, -1);
 
-    rb_define_method(rb_cDir,"initialize", dir_initialize, 1);
+    rb_define_method(rb_cDir,"initialize", dir_initialize, -1);
     rb_define_method(rb_cDir,"path", dir_path, 0);
     rb_define_method(rb_cDir,"inspect", dir_inspect, 0);
     rb_define_method(rb_cDir,"read", dir_read, 0);
