@@ -2,8 +2,7 @@
 
   dln.h -
 
-  $Author: matz $
-  $Date: 2007-08-25 12:29:39 +0900 (Sat, 25 Aug 2007) $
+  $Author: nobu $
   created at: Wed Jan 19 16:53:09 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -31,6 +30,8 @@
 
 char *dln_find_exe(const char*,const char*);
 char *dln_find_file(const char*,const char*);
+char *dln_find_exe_r(const char*,const char*,char*,int);
+char *dln_find_file_r(const char*,const char*,char*,int);
 
 #ifdef USE_DLN_A_OUT
 extern char *dln_argv0;
