@@ -2,8 +2,7 @@
 
   encoding.c -
 
-  $Author: nobu $
-  $Date: 2007-12-25 19:01:06 +0900 (Tue, 25 Dec 2007) $
+  $Author: naruse $
   created at: Thu May 24 17:23:27 JST 2007
 
   Copyright (C) 2007 Yukihiro Matsumoto
@@ -18,25 +17,29 @@
 #include <langinfo.h>
 #endif
 
-static ID id_encoding, id_based_encoding;
-static VALUE rb_cEncoding;
+static ID id_encoding, id_base_encoding;
+VALUE rb_cEncoding;
 
 struct rb_encoding_entry {
     const char *name;
     rb_encoding *enc;
 };
 
-static struct rb_encoding_entry *enc_table;
-static int enc_table_count;
-static int enc_table_size;
-static st_table *enc_table_alias;
+static struct {
+    struct rb_encoding_entry *list;
+    int count;
+    int size;
+    st_table *names;
+} enc_table;
 
-#define ENC_UNINITIALIZED (&rb_cEncoding)
-#define enc_initialized_p(enc) ((enc)->auxiliary_data != &rb_cEncoding)
-#define ENC_FROM_ENCODING(enc) ((VALUE)(enc)->auxiliary_data)
+void rb_enc_init(void);
 
-#define ENC_DUMMY FL_USER2
-#define ENC_DUMMY_P(enc) (RBASIC(enc)->flags & ENC_DUMMY)
+#define ENCODING_COUNT ENCINDEX_BUILTIN_MAX
+
+#define enc_autoload_p(enc) (!rb_enc_mbmaxlen(enc))
+
+static int load_encoding(const char *name);
+static VALUE enc_base_encoding(VALUE self);
 
 static void
 enc_mark(void *ptr)
@@ -46,7 +49,7 @@ enc_mark(void *ptr)
 static VALUE
 enc_new(rb_encoding *encoding)
 {
-    VALUE enc = Data_Wrap_Struct(rb_cEncoding, enc_mark, -1, encoding);
+    VALUE enc = Data_Wrap_Struct(rb_cEncoding, enc_mark, 0, encoding);
     encoding->auxiliary_data = (void *)enc;
     return enc;
 }
@@ -64,13 +67,19 @@ static int
 enc_check_encoding(VALUE obj)
 {
     int index;
+    rb_encoding *enc;
+
     if (SPECIAL_CONST_P(obj) || BUILTIN_TYPE(obj) != T_DATA ||
 	RDATA(obj)->dmark != enc_mark) {
 	return -1;
     }
-    index = rb_enc_to_index(RDATA(obj)->data);
-    if (rb_enc_from_index(index) != RDATA(obj)->data)
+    enc = (rb_encoding*)RDATA(obj)->data;
+    index = rb_enc_to_index(enc);
+    if (rb_enc_from_index(index) != enc)
 	return -1;
+    if (enc_autoload_p(enc)) {
+	index = rb_enc_find_index(enc->name);
+    }
     return index;
 }
 
@@ -79,7 +88,6 @@ rb_to_encoding_index(VALUE enc)
 {
     int idx;
 
-    if (NIL_P(enc)) return 0;
     idx = enc_check_encoding(enc);
     if (idx >= 0) {
 	return idx;
@@ -97,12 +105,8 @@ rb_to_encoding(VALUE enc)
 {
     int idx;
 
-    if (NIL_P(enc)) return 0;
     idx = enc_check_encoding(enc);
     if (idx >= 0) return RDATA(enc)->data;
-    if (NIL_P(enc = rb_check_string_type(enc))) {
-	return 0;
-    }
     if ((idx = rb_enc_find_index(StringValueCStr(enc))) < 0) {
 	rb_raise(rb_eArgError, "unknown encoding name - %s", RSTRING_PTR(enc));
     }
@@ -113,8 +117,8 @@ void
 rb_gc_mark_encodings(void)
 {
     int i;
-    for (i = 0; i < enc_table_size; ++i) {
-	rb_encoding *enc = enc_table[i].enc;
+    for (i = 0; i < enc_table.count; ++i) {
+	rb_encoding *enc = enc_table.list[i].enc;
 	if (enc && enc_initialized_p(enc)) {
 	    rb_gc_mark(ENC_FROM_ENCODING(enc));
 	}
@@ -125,27 +129,50 @@ static int
 enc_table_expand(int newsize)
 {
     struct rb_encoding_entry *ent;
+    int count = newsize;
 
-    if (enc_table_size >= newsize) return newsize;
-    ent = realloc(enc_table, sizeof(*enc_table) * newsize);
+    if (enc_table.size >= newsize) return newsize;
+    newsize = (newsize + 7) / 8 * 8;
+    ent = realloc(enc_table.list, sizeof(*enc_table.list) * newsize);
     if (!ent) return -1;
-    memset(ent + enc_table_size, 0, sizeof(*ent)*(newsize - enc_table_size));
-    enc_table = ent;
-    enc_table_size = newsize;
-    return newsize;
+    memset(ent + enc_table.size, 0, sizeof(*ent)*(newsize - enc_table.size));
+    enc_table.list = ent;
+    enc_table.size = newsize;
+    return count;
 }
 
 static int
 enc_register_at(int index, const char *name, rb_encoding *encoding)
 {
-    struct rb_encoding_entry *ent = &enc_table[index];
+    struct rb_encoding_entry *ent = &enc_table.list[index];
+    void *obj = ENC_UNINITIALIZED;
 
-    ent->name = name;
-    if (!ent->enc) ent->enc = malloc(sizeof(rb_encoding));
-    *ent->enc = *encoding;
+    if (!ent->name) {
+	ent->name = name = strdup(name);
+    }
+    else if (STRCASECMP(name, ent->name)) {
+	return -1;
+    }
+    if (!ent->enc) {
+	ent->enc = xmalloc(sizeof(rb_encoding));
+    }
+    else {
+	obj = ent->enc->auxiliary_data;
+    }
+    if (encoding) {
+	*ent->enc = *encoding;
+    }
+    else {
+	memset(ent->enc, 0, sizeof(*ent->enc));
+    }
     encoding = ent->enc;
     encoding->name = name;
-    if (rb_cEncoding) {
+    encoding->ruby_encoding_index = index;
+    st_insert(enc_table.names, (st_data_t)name, (st_data_t)index);
+    if (obj != ENC_UNINITIALIZED) {
+	encoding->auxiliary_data = obj;
+    }
+    else if (rb_cEncoding) {
 	/* initialize encoding data */
 	enc_new(encoding);
     }
@@ -158,11 +185,10 @@ enc_register_at(int index, const char *name, rb_encoding *encoding)
 static int
 enc_register(const char *name, rb_encoding *encoding)
 {
-    int index = enc_table_count;
+    int index = enc_table.count;
 
-    if (index >= ENCODING_INLINE_MAX) index = enc_table_size;
     if ((index = enc_table_expand(index + 1)) < 0) return -1;
-    enc_table_count = index;
+    enc_table.count = index;
     return enc_register_at(index - 1, name, encoding);
 }
 
@@ -176,13 +202,12 @@ rb_enc_register(const char *name, rb_encoding *encoding)
 
     if (index >= 0) {
 	rb_encoding *oldenc = rb_enc_from_index(index);
-	if (strcasecmp(name, rb_enc_name(oldenc))) {
-	    st_data_t key = (st_data_t)name, alias;
-	    st_delete(enc_table_alias, &key, &alias);
+	if (STRCASECMP(name, rb_enc_name(oldenc))) {
 	    index = enc_register(name, encoding);
 	}
-	else if (enc_initialized_p(oldenc) &&
-		 !ENC_DUMMY_P(ENC_FROM_ENCODING(oldenc))) {
+	else if (!enc_autoload_p(oldenc) ||
+		 (enc_initialized_p(oldenc) &&
+		  !ENC_DUMMY_P(ENC_FROM_ENCODING(oldenc)))) {
 	    enc_register_at(index, name, encoding);
 	}
 	else {
@@ -191,11 +216,21 @@ rb_enc_register(const char *name, rb_encoding *encoding)
     }
     else {
 	index = enc_register(name, encoding);
+	set_encoding_const(name, rb_enc_from_index(index));
     }
-    set_encoding_const(name, rb_enc_from_index(index));
     return index;
 }
 
+void
+rb_encdb_declare(const char *name)
+{
+    int idx = rb_enc_registered(name);
+    if (idx < 0) {
+	idx = enc_register(name, 0);
+    }
+    set_encoding_const(name, rb_enc_from_index(idx));
+}
+
 static void
 enc_check_duplication(const char *name)
 {
@@ -205,58 +240,84 @@ enc_check_duplication(const char *name)
 }
 
 static VALUE
-set_based_encoding(int index, rb_encoding *based)
+set_base_encoding(int index, rb_encoding *base)
 {
-    VALUE enc = rb_enc_from_encoding(enc_table[index].enc);
+    VALUE enc = rb_enc_from_encoding(enc_table.list[index].enc);
 
-    rb_ivar_set(enc, id_based_encoding, rb_enc_from_encoding(based));
+    rb_ivar_set(enc, id_base_encoding, rb_enc_from_encoding(base));
+    if (rb_enc_dummy_p(base)) ENC_SET_DUMMY(enc);
     return enc;
 }
 
 int
 rb_enc_replicate(const char *name, rb_encoding *encoding)
 {
-    int index = enc_table_size;
+    int idx;
 
     enc_check_duplication(name);
-    if (enc_table_expand(index + 1) < 0) return -1;
-    enc_register_at(index, name, encoding);
-    set_based_encoding(index, encoding);
-    return index;
+    idx = enc_register(name, encoding);
+    set_base_encoding(idx, encoding);
+    set_encoding_const(name, rb_enc_from_index(idx));
+    return idx;
+}
+
+static int
+enc_replicate(int idx, const char *name, rb_encoding *origenc)
+{
+    if (idx < 0) {
+	idx = enc_register(name, origenc);
+    }
+    else {
+	idx = enc_register_at(idx, name, origenc);
+    }
+    if (idx >= 0) {
+	set_base_encoding(idx, origenc);
+	set_encoding_const(name, rb_enc_from_index(idx));
+    }
+    return idx;
+}
+
+int
+rb_encdb_replicate(const char *name, const char *orig)
+{
+    int origidx = rb_enc_registered(orig);
+    int idx = rb_enc_registered(name);
+
+    if (origidx < 0) {
+	origidx = enc_register(orig, 0);
+    }
+    return enc_replicate(idx, name, rb_enc_from_index(origidx));
 }
 
 int
 rb_define_dummy_encoding(const char *name)
 {
-    int index = enc_table_size;
-    rb_encoding *encoding;
-    VALUE enc;
+    int index = rb_enc_replicate(name, rb_ascii8bit_encoding());
+    VALUE enc = rb_enc_from_encoding(enc_table.list[index].enc);
 
-    enc_check_duplication(name);
-    if (index < ENCODING_INLINE_MAX) index = ENCODING_INLINE_MAX;
-    if (enc_table_expand(index + 1) < 0) return -1;
-    encoding = rb_ascii8bit_encoding();
-    enc_register_at(index, name, encoding);
-    enc = set_based_encoding(index, encoding);
-    FL_SET(enc, ENC_DUMMY);
+    ENC_SET_DUMMY(enc);
     return index;
 }
 
 int
-rb_enc_dummy_p(rb_encoding *enc)
+rb_encdb_dummy(const char *name)
 {
-    VALUE encoding = rb_enc_from_encoding(enc);
-    return ENC_DUMMY_P(encoding);
+    int index = enc_replicate(rb_enc_registered(name), name,
+			      rb_ascii8bit_encoding());
+    VALUE enc = rb_enc_from_encoding(enc_table.list[index].enc);
+
+    ENC_SET_DUMMY(enc);
+    return index;
 }
 
 /*
  * call-seq:
  *   enc.dummy? => true or false
  *
- * Returns true for dummy encoding.
- * A dummy encoding is a encoding which character handling is not properly
+ * Returns true for dummy encodings.
+ * A dummy encoding is an encoding for which character handling is not properly
  * implemented.
- * It is used for stateful encoding.
+ * It is used for stateful encodings.
  *
  *   Encoding::ISO_2022_JP.dummy?       #=> true
  *   Encoding::UTF_8.dummy?             #=> false
@@ -265,93 +326,90 @@ rb_enc_dummy_p(rb_encoding *enc)
 static VALUE
 enc_dummy_p(VALUE enc)
 {
-    return rb_enc_dummy_p(rb_to_encoding(enc)) ? Qtrue : Qfalse;
+    return ENC_DUMMY_P(enc) ? Qtrue : Qfalse;
 }
 
 static int
-enc_alias(const char *alias, const char *orig)
+enc_alias(const char *alias, int idx)
 {
-    st_data_t data;
-    int idx;
-
-    if (!enc_table_alias) {
-	enc_table_alias = st_init_strcasetable();
-    }
-    if ((idx = rb_enc_find_index(orig)) < 0) {
-	if (!st_lookup(enc_table_alias, (st_data_t)orig, &data))
-	    return -1;
-	idx = (int)data;
-    }
-    st_insert(enc_table_alias, (st_data_t)alias, (st_data_t)idx);
+    alias = strdup(alias);
+    st_insert(enc_table.names, (st_data_t)alias, (st_data_t)idx);
+    set_encoding_const(alias, rb_enc_from_index(idx));
     return idx;
 }
 
 int
 rb_enc_alias(const char *alias, const char *orig)
 {
+    int idx;
+
     enc_check_duplication(alias);
-    return enc_alias(alias, orig);
+    if (!enc_table.list) {
+	rb_enc_init();
+    }
+    if ((idx = rb_enc_find_index(orig)) < 0) {
+	return -1;
+    }
+    return enc_alias(alias, idx);
+}
+
+int
+rb_encdb_alias(const char *alias, const char *orig)
+{
+    int idx = rb_enc_registered(orig);
+
+    if (idx < 0) {
+	idx = enc_register(orig, 0);
+    }
+    return enc_alias(alias, idx);
 }
 
 enum {
     ENCINDEX_ASCII,
-    ENCINDEX_EUC_JP,
-    ENCINDEX_SJIS,
-    ENCINDEX_UTF8,
+    ENCINDEX_UTF_8,
+    ENCINDEX_US_ASCII,
     ENCINDEX_BUILTIN_MAX
 };
 
+extern rb_encoding OnigEncodingUTF_8;
+extern rb_encoding OnigEncodingUS_ASCII;
+
 void
 rb_enc_init(void)
 {
-    enc_table_count = enc_table_expand(ENCINDEX_BUILTIN_MAX);
-#define ENC_REGISTER(enc) enc_register_at(ENCINDEX_##enc, rb_enc_name(ONIG_ENCODING_##enc), ONIG_ENCODING_##enc)
+    enc_table_expand(ENCODING_COUNT + 1);
+    if (!enc_table.names) {
+	enc_table.names = st_init_strcasetable();
+    }
+#define ENC_REGISTER(enc) enc_register_at(ENCINDEX_##enc, rb_enc_name(&OnigEncoding##enc), &OnigEncoding##enc)
     ENC_REGISTER(ASCII);
-    ENC_REGISTER(EUC_JP);
-    ENC_REGISTER(SJIS);
-    ENC_REGISTER(UTF8);
+    ENC_REGISTER(UTF_8);
+    ENC_REGISTER(US_ASCII);
 #undef ENC_REGISTER
-    enc_alias("ASCII", rb_enc_name(ONIG_ENCODING_ASCII));
-    enc_alias("BINARY", rb_enc_name(ONIG_ENCODING_ASCII));
-    enc_alias("eucJP", rb_enc_name(ONIG_ENCODING_EUC_JP)); /* UI-OSF Application Platform Profile for Japanese Environment Version 1.1 */
-    enc_alias("SJIS", rb_enc_name(ONIG_ENCODING_SJIS));
+    enc_table.count = ENCINDEX_BUILTIN_MAX;
 }
 
 rb_encoding *
 rb_enc_from_index(int index)
 {
-    if (!enc_table) {
+    if (!enc_table.list) {
 	rb_enc_init();
     }
-    if (index < 0 || enc_table_size <= index) {
+    if (index < 0 || enc_table.count <= index) {
 	return 0;
     }
-    return enc_table[index].enc;
+    return enc_table.list[index].enc;
 }
 
 int
 rb_enc_registered(const char *name)
 {
-    int i;
-    st_data_t alias = 0;
+    st_data_t idx = 0;
 
     if (!name) return -1;
-    if (!enc_table) {
-	rb_enc_init();
-    }
-    for (i=0; i<enc_table_size; i++) {
-	if (!enc_table[i].name) {
-	    if (i < ENCODING_INLINE_MAX - 1) i = ENCODING_INLINE_MAX - 1;
-	    continue;
-	}
-	if (strcasecmp(name, enc_table[i].name) == 0) {
-	    return i;
-	}
-    }
-    if (!alias && enc_table_alias) {
-	if (st_lookup(enc_table_alias, (st_data_t)name, &alias)) {
-	    return (int)alias;
-	}
+    if (!enc_table.list) return -1;
+    if (st_lookup(enc_table.names, (st_data_t)name, &idx)) {
+	return (int)idx;
     }
     return -1;
 }
@@ -362,22 +420,61 @@ require_enc(VALUE enclib)
     return rb_require_safe(enclib, rb_safe_level());
 }
 
+static int
+load_encoding(const char *name)
+{
+    VALUE enclib = rb_sprintf("enc/%s", name);
+    VALUE verbose = ruby_verbose;
+    VALUE debug = ruby_debug;
+    VALUE loaded;
+    char *s = RSTRING_PTR(enclib) + 4, *e = RSTRING_END(enclib);
+    int idx;
+
+    while (s < e) {
+	if (!ISALNUM(*s)) *s = '_';
+	else if (ISUPPER(*s)) *s = TOLOWER(*s);
+	++s;
+    }
+    OBJ_FREEZE(enclib);
+    ruby_verbose = Qfalse;
+    ruby_debug = Qfalse;
+    loaded = rb_protect(require_enc, enclib, 0);
+    ruby_verbose = verbose;
+    ruby_debug = debug;
+    rb_set_errinfo(Qnil);
+    if (NIL_P(loaded)) return -1;
+    if ((idx = rb_enc_registered(name)) < 0) return -1;
+    if (enc_autoload_p(enc_table.list[idx].enc)) return -1;
+    return idx;
+}
+
 int
 rb_enc_find_index(const char *name)
 {
-    int i = rb_enc_registered(name);
+    int i = rb_enc_registered(name), b;
+    rb_encoding *enc;
+    VALUE base;
+
     if (i < 0) {
-	VALUE enclib = rb_sprintf("enc/%s", name);
-	char *s = RSTRING_PTR(enclib) + 4, *e = RSTRING_END(enclib);
-	while (s < e) {
-	    if (!ISALNUM(*s)) *s = '_';
-	    else if (ISUPPER(*s)) *s = tolower(*s);
-	    ++s;
+	i = load_encoding(name);
+    }
+    else if (enc_autoload_p(enc = rb_enc_from_index(i))) {
+	if (enc_initialized_p(enc) &&
+	    (base = enc_base_encoding(ENC_FROM_ENCODING(enc)), !NIL_P(base))) {
+	    if ((b = enc_check_encoding(base)) < 0) {
+		goto failed;
+	    }
+	    enc_register_at(i, rb_enc_name(enc), rb_enc_from_index(b));
+	}
+	else {
+	    i = load_encoding(rb_enc_name(enc));
+	    if (i < 0) {
+	      failed:
+		rb_warn("failed to load encoding (%s); use ASCII-8BIT instead",
+			name);
+		return 0;
+	    }
 	}
-	OBJ_FREEZE(enclib);
-	if (RTEST(rb_protect(require_enc, enclib, 0)))
-	    i = rb_enc_registered(name);
-	rb_set_errinfo(Qnil);
     }
     return i;
 }
@@ -393,7 +490,7 @@ rb_enc_find(const char *name)
 static inline int
 enc_capable(VALUE obj)
 {
-    if (IMMEDIATE_P(obj)) return Qfalse;
+    if (SPECIAL_CONST_P(obj)) return Qfalse;
     switch (BUILTIN_TYPE(obj)) {
       case T_STRING:
       case T_REGEXP:
@@ -406,6 +503,7 @@ enc_capable(VALUE obj)
     }
 }
 
+#if 0
 static void
 enc_check_capable(VALUE x)
 {
@@ -430,60 +528,21 @@ enc_check_capable(VALUE x)
 	rb_raise(rb_eTypeError, "wrong argument type %s (not encode capable)", etype);
     }
 }
+#endif
 
 ID
 rb_id_encoding(void)
 {
-    if (!id_encoding) {
-	id_encoding = rb_intern("encoding");
-    }
+    CONST_ID(id_encoding, "encoding");
     return id_encoding;
 }
 
-void
-rb_enc_associate_index(VALUE obj, int idx)
-{
-    enc_check_capable(obj);
-    if (!ENC_CODERANGE_ASCIIONLY(obj) ||
-	!rb_enc_asciicompat(rb_enc_from_index(idx))) {
-	ENC_CODERANGE_CLEAR(obj);
-    }
-    if (idx < ENCODING_INLINE_MAX) {
-	ENCODING_SET(obj, idx);
-	return;
-    }
-    ENCODING_SET(obj, ENCODING_INLINE_MAX);
-    rb_ivar_set(obj, rb_id_encoding(), INT2NUM(idx));
-    return;
-}
-
-int
-rb_enc_to_index(rb_encoding *enc)
-{
-    int i;
-
-    if (!enc) return 0;
-    for (i=0; i<enc_table_size; i++) {
-	if (enc_table[i].enc == enc) {
-	    return i;
-	}
-    }
-    return 0;
-}
-
-void
-rb_enc_associate(VALUE obj, rb_encoding *enc)
-{
-    rb_enc_associate_index(obj, rb_enc_to_index(enc));
-}
-
 int
 rb_enc_get_index(VALUE obj)
 {
     int i;
 
-    if (!enc_capable(obj)) return -1;
-    i = ENCODING_GET(obj);
+    i = ENCODING_GET_INLINED(obj);
     if (i == ENCODING_INLINE_MAX) {
 	VALUE iv;
 
@@ -493,6 +552,38 @@ rb_enc_get_index(VALUE obj)
     return i;
 }
 
+void
+rb_enc_set_index(VALUE obj, int idx)
+{
+    if (idx < ENCODING_INLINE_MAX) {
+	ENCODING_SET_INLINED(obj, idx);
+	return;
+    }
+    ENCODING_SET_INLINED(obj, ENCODING_INLINE_MAX);
+    rb_ivar_set(obj, rb_id_encoding(), INT2NUM(idx));
+    return;
+}
+
+VALUE
+rb_enc_associate_index(VALUE obj, int idx)
+{
+/*    enc_check_capable(obj);*/
+    if (rb_enc_get_index(obj) == idx)
+    	return obj;
+    if (!ENC_CODERANGE_ASCIIONLY(obj) ||
+	!rb_enc_asciicompat(rb_enc_from_index(idx))) {
+	ENC_CODERANGE_CLEAR(obj);
+    }
+    rb_enc_set_index(obj, idx);
+    return obj;
+}
+
+VALUE
+rb_enc_associate(VALUE obj, rb_encoding *enc)
+{
+    return rb_enc_associate_index(obj, rb_enc_to_index(enc));
+}
+
 rb_encoding*
 rb_enc_get(VALUE obj)
 {
@@ -504,7 +595,9 @@ rb_enc_check(VALUE str1, VALUE str2)
 {
     rb_encoding *enc = rb_enc_compatible(str1, str2);
     if (!enc)
-	rb_raise(rb_eArgError, "character encodings differ");
+	rb_raise(rb_eArgError, "character encodings differ: %s and %s",
+		 rb_enc_name(rb_enc_get(str1)),
+		 rb_enc_name(rb_enc_get(str2)));
     return enc;
 }
 
@@ -512,14 +605,28 @@ rb_encoding*
 rb_enc_compatible(VALUE str1, VALUE str2)
 {
     int idx1, idx2;
-    rb_encoding *enc;
+    rb_encoding *enc1, *enc2;
 
     idx1 = rb_enc_get_index(str1);
     idx2 = rb_enc_get_index(str2);
 
+    if (idx1 < 0 || idx2 < 0)
+        return 0;
+
     if (idx1 == idx2) {
 	return rb_enc_from_index(idx1);
     }
+    enc1 = rb_enc_from_index(idx1);
+    enc2 = rb_enc_from_index(idx2);
+
+    if (TYPE(str2) == T_STRING && RSTRING_LEN(str2) == 0)
+	return enc1;
+    if (TYPE(str1) == T_STRING && RSTRING_LEN(str1) == 0)
+	return enc2;
+    if (!rb_enc_asciicompat(enc1) || !rb_enc_asciicompat(enc2)) {
+	return 0;
+    }
+
     if (BUILTIN_TYPE(str1) != T_STRING) {
 	VALUE tmp = str1;
 	int idx0 = idx1;
@@ -536,17 +643,16 @@ rb_enc_compatible(VALUE str1, VALUE str2)
 	    cr2 = rb_enc_str_coderange(str2);
 	    if (cr1 != cr2) {
 		/* may need to handle ENC_CODERANGE_BROKEN */
-		if (cr1 == ENC_CODERANGE_7BIT) return rb_enc_from_index(idx2);
-		if (cr2 == ENC_CODERANGE_7BIT) return rb_enc_from_index(idx1);
+		if (cr1 == ENC_CODERANGE_7BIT) return enc2;
+		if (cr2 == ENC_CODERANGE_7BIT) return enc1;
 	    }
 	    if (cr2 == ENC_CODERANGE_7BIT) {
-		if (idx1 == 0) return rb_enc_from_index(idx2);
-		return rb_enc_from_index(idx1);
+		if (idx1 == 0) return enc2;
+		return enc1;
 	    }
 	}
-	if (cr1 == ENC_CODERANGE_7BIT &&
-	    rb_enc_asciicompat(enc = rb_enc_from_index(idx2)))
-	    return enc;
+	if (cr1 == ENC_CODERANGE_7BIT)
+	    return enc2;
     }
     return 0;
 }
@@ -575,54 +681,16 @@ rb_obj_encoding(VALUE obj)
     return rb_enc_from_encoding(enc);
 }
 
-
-char*
-rb_enc_nth(const char *p, const char *e, int nth, rb_encoding *enc)
-{
-    int c;
-
-    if (rb_enc_mbmaxlen(enc) == 1) {
-	p += nth;
-    }
-    else if (rb_enc_mbmaxlen(enc) == rb_enc_mbminlen(enc)) {
-	p += nth * rb_enc_mbmaxlen(enc);
-    }
-    else {
-	for (c=0; p<e && nth--; c++) {
-	    int n = rb_enc_mbclen(p, e, enc);
-
-	    p += n;
-	}
-    }
-    return (char*)p;
-}
-
-long
-rb_enc_strlen(const char *p, const char *e, rb_encoding *enc)
-{
-    long c;
-
-    if (rb_enc_mbmaxlen(enc) == rb_enc_mbminlen(enc)) {
-	return (e - p) / rb_enc_mbminlen(enc);
-    }
-
-    for (c=0; p<e; c++) {
-	int n = rb_enc_mbclen(p, e, enc);
-
-	p += n;
-    }
-    return c;
-}
-
 int
 rb_enc_mbclen(const char *p, const char *e, rb_encoding *enc)
 {
     int n = ONIGENC_PRECISE_MBC_ENC_LEN(enc, (UChar*)p, (UChar*)e);
-    n = MBCLEN_CHARFOUND(n);
-    if (0 < n && n <= e-p)
-        return n;
-    else
-        return 1;
+    if (MBCLEN_CHARFOUND_P(n) && MBCLEN_CHARFOUND_LEN(n) <= e-p)
+        return MBCLEN_CHARFOUND_LEN(n);
+    else {
+        int min = rb_enc_mbminlen(enc);
+        return min <= e-p ? min : e-p;
+    }
 }
 
 int
@@ -651,9 +719,9 @@ rb_enc_ascget(const char *p, const char *e, int *len, rb_encoding *enc)
         return c;
     }
     l = rb_enc_precise_mbclen(p, e, enc);
-    if (!MBCLEN_CHARFOUND(l))
+    if (!MBCLEN_CHARFOUND_P(l))
         return -1;
-    c = rb_enc_codepoint(p, e, enc);
+    c = rb_enc_mbc_to_codepoint(p, e, enc);
     if (!rb_enc_isascii(c, enc))
         return -1;
     if (len) *len = l;
@@ -667,7 +735,7 @@ rb_enc_codepoint(const char *p, const char *e, rb_encoding *enc)
     if (e <= p)
         rb_raise(rb_eArgError, "empty string");
     r = rb_enc_precise_mbclen(p, e, enc);
-    if (MBCLEN_CHARFOUND(r))
+    if (MBCLEN_CHARFOUND_P(r))
         return rb_enc_mbc_to_codepoint(p, e, enc);
     else
 	rb_raise(rb_eArgError, "invalid mbstring sequence");
@@ -678,7 +746,7 @@ rb_enc_codelen(int c, rb_encoding *enc)
 {
     int n = ONIGENC_CODE_TO_MBCLEN(enc,c);
     if (n == 0) {
-	rb_raise(rb_eArgError, "invalid mbstring sequence");
+	rb_raise(rb_eArgError, "invalid codepoint 0x%x", c);
     }
     return n;
 }
@@ -707,9 +775,11 @@ rb_enc_tolower(int c, rb_encoding *enc)
 static VALUE
 enc_inspect(VALUE self)
 {
-    return rb_sprintf("#<%s:%s%s>", rb_obj_classname(self),
+    VALUE str = rb_sprintf("#<%s:%s%s>", rb_obj_classname(self),
 		      rb_enc_name((rb_encoding*)DATA_PTR(self)),
 		      (ENC_DUMMY_P(self) ? " (dummy)" : ""));
+    ENCODING_CODERANGE_SET(str, rb_usascii_encindex(), ENC_CODERANGE_7BIT);
+    return str;
 }
 
 /*
@@ -723,13 +793,13 @@ enc_inspect(VALUE self)
 static VALUE
 enc_name(VALUE self)
 {
-    return rb_str_new2(rb_enc_name((rb_encoding*)DATA_PTR(self)));
+    return rb_usascii_str_new2(rb_enc_name((rb_encoding*)DATA_PTR(self)));
 }
 
 static VALUE
-enc_based_encoding(VALUE self)
+enc_base_encoding(VALUE self)
 {
-    return rb_attr_get(self, id_based_encoding);
+    return rb_attr_get(self, id_base_encoding);
 }
 
 /*
@@ -739,26 +809,24 @@ enc_based_encoding(VALUE self)
  * Returns the list of loaded encodings.
  *
  *   Encoding.list
- *   => [#<Encoding:ASCII-8BIT>, #<Encoding:EUC-JP>,
- *       #<Encoding:Shift_JIS>, #<Encoding:UTF-8>,
+ *   => [#<Encoding:ASCII-8BIT>, #<Encoding:UTF-8>,
  *       #<Encoding:ISO-2022-JP (dummy)>]
  *
  *   Encoding.find("US-ASCII")
  *   => #<Encoding:US-ASCII>
  *
  *   Encoding.list
- *   => [#<Encoding:ASCII-8BIT>, #<Encoding:EUC-JP>,
- *       #<Encoding:Shift_JIS>, #<Encoding:UTF-8>,
+ *   => [#<Encoding:ASCII-8BIT>, #<Encoding:UTF-8>,
  *       #<Encoding:US-ASCII>, #<Encoding:ISO-2022-JP (dummy)>]
  *
  */
 static VALUE
 enc_list(VALUE klass)
 {
-    VALUE ary = rb_ary_new2(enc_table_size);
+    VALUE ary = rb_ary_new2(enc_table.count);
     int i;
-    for (i = 0; i < enc_table_size; ++i) {
-	rb_encoding *enc = enc_table[i].enc;
+    for (i = 0; i < enc_table.count; ++i) {
+	rb_encoding *enc = enc_table.list[i].enc;
 	if (enc) {
 	    rb_ary_push(ary, rb_enc_from_encoding(enc));
 	}
@@ -782,7 +850,11 @@ static VALUE
 enc_find(VALUE klass, VALUE enc)
 {
     int idx;
-    if (SYMBOL_P(enc)) enc = rb_id2str(SYM2ID(enc));
+
+    StringValue(enc);
+    if (!rb_enc_asciicompat(rb_enc_get(enc))) {
+	rb_raise(rb_eArgError, "invalid name encoding (non ASCII)");
+    }
     idx = rb_enc_find_index(StringValueCStr(enc));
     if (idx < 0) {
 	rb_raise(rb_eArgError, "unknown encoding name - %s", RSTRING_PTR(enc));
@@ -795,7 +867,7 @@ enc_find(VALUE klass, VALUE enc)
  *   Encoding.compatible?(str1, str2) => enc or nil
  *
  * Checks the compatibility of two strings.
- * If they are compabible, means concatinatable, 
+ * If they are compatible, means concatenatable, 
  * returns an encoding which the concatinated string will be.
  * If they are not compatible, nil is returned.
  *
@@ -811,11 +883,13 @@ enc_find(VALUE klass, VALUE enc)
 static VALUE
 enc_compatible_p(VALUE klass, VALUE str1, VALUE str2)
 {
-    rb_encoding *enc = rb_enc_compatible(str1, str2);
-    VALUE encoding = Qnil;
-    if (!enc || !(encoding = rb_enc_from_encoding(enc)))
-	encoding = Qnil;
-    return encoding;
+    rb_encoding *enc;
+
+    if (!enc_capable(str1)) return Qnil;
+    if (!enc_capable(str2)) return Qnil;
+    enc = rb_enc_compatible(str1, str2);
+    if (!enc) return Qnil;
+    return rb_enc_from_encoding(enc);
 }
 
 /* :nodoc: */
@@ -836,19 +910,72 @@ enc_load(VALUE klass, VALUE str)
 rb_encoding *
 rb_ascii8bit_encoding(void)
 {
-    if (!enc_table) {
+    if (!enc_table.list) {
 	rb_enc_init();
     }
-    return enc_table[0].enc;
+    return enc_table.list[ENCINDEX_ASCII].enc;
+}
+
+int
+rb_ascii8bit_encindex(void)
+{
+    return ENCINDEX_ASCII;
 }
 
 rb_encoding *
 rb_utf8_encoding(void)
 {
-    if (!enc_table) {
+    if (!enc_table.list) {
 	rb_enc_init();
     }
-    return enc_table[ENCINDEX_UTF8].enc;
+    return enc_table.list[ENCINDEX_UTF_8].enc;
+}
+
+rb_encoding *
+rb_usascii_encoding(void)
+{
+    if (!enc_table.list) {
+	rb_enc_init();
+    }
+    return enc_table.list[ENCINDEX_US_ASCII].enc;
+}
+
+int
+rb_usascii_encindex(void)
+{
+    return ENCINDEX_US_ASCII;
+}
+
+rb_encoding *
+rb_locale_encoding(void)
+{
+    VALUE charmap = rb_locale_charmap(rb_cEncoding);
+    int idx;
+
+    if (NIL_P(charmap))
+        return rb_usascii_encoding();
+    else
+        idx = rb_enc_find_index(StringValueCStr(charmap));
+    if (idx < 0)
+        return rb_ascii8bit_encoding();
+
+    return rb_enc_from_index(idx);
+}
+
+rb_encoding *
+rb_filesystem_encoding(void)
+{
+    static rb_encoding *enc;
+    if (!enc) {
+#if defined _WIN32
+	enc = rb_locale_encoding();
+#elif defined __APPLE__
+	enc = rb_enc_find("UTF8-MAC");
+#else
+	enc = rb_ascii8bit_encoding();
+#endif
+    }
+    return enc;
 }
 
 static int default_external_index;
@@ -907,10 +1034,12 @@ rb_enc_set_default_external(VALUE encoding)
 VALUE
 rb_locale_charmap(VALUE klass)
 {
-#ifdef HAVE_LANGINFO_H
+#if defined NO_LOCALE_CHARMAP
+    return rb_usascii_str_new2("ASCII-8BIT");
+#elif defined HAVE_LANGINFO_H
     char *codeset;
     codeset = nl_langinfo(CODESET);
-    return rb_str_new2(codeset);
+    return rb_usascii_str_new2(codeset);
 #elif defined _WIN32
     return rb_sprintf("CP%d", GetACP());
 #else
@@ -923,9 +1052,11 @@ set_encoding_const(const char *name, rb_encoding *enc)
 {
     VALUE encoding = rb_enc_from_encoding(enc);
     char *s = (char *)name;
-    int haslower = 0, valid = 0;
+    int haslower = 0, hasupper = 0, valid = 0;
 
+    if (ISDIGIT(*s)) return;
     if (ISUPPER(*s)) {
+	hasupper = 1;
 	while (*++s && (ISALNUM(*s) || *s == '_')) {
 	    if (ISLOWER(*s)) haslower = 1;
 	}
@@ -936,9 +1067,11 @@ set_encoding_const(const char *name, rb_encoding *enc)
     }
     if (!valid || haslower) {
 	int len = strlen(name) + 1;
-	if (!haslower) {
-	    while (!ISLOWER(*s) && *++s);
-	    if (*s) haslower = 1;
+	if (!haslower || !hasupper) {
+	    do {
+		if (ISLOWER(*s)) haslower = 1;
+		if (ISUPPER(*s)) hasupper = 1;
+	    } while (*++s && (!haslower || !hasupper));
 	}
 	MEMCPY(s = ALLOCA_N(char, len), name, char, len);
 	name = s;
@@ -947,7 +1080,9 @@ set_encoding_const(const char *name, rb_encoding *enc)
 	    for (; *s; ++s) {
 		if (!ISALNUM(*s)) *s = '_';
 	    }
-	    rb_define_const(rb_cEncoding, name, encoding);
+	    if (hasupper) {
+		rb_define_const(rb_cEncoding, name, encoding);
+	    }
 	}
 	if (haslower) {
 	    for (s = (char *)name; *s; ++s) {
@@ -959,28 +1094,102 @@ set_encoding_const(const char *name, rb_encoding *enc)
 }
 
 static int
-set_encoding_alias(st_data_t name, st_data_t orig, st_data_t arg)
+rb_enc_name_list_i(st_data_t name, st_data_t idx, st_data_t arg)
 {
-    rb_encoding *enc = rb_enc_from_index((int)orig);
-    set_encoding_const((const char *)name, enc);
+    VALUE ary = (VALUE)arg;
+    VALUE str = rb_usascii_str_new2((char *)name);
+    OBJ_FREEZE(str);
+    rb_ary_push(ary, str);
     return ST_CONTINUE;
 }
 
+/*
+ * call-seq:
+ *   Encoding.name_list => ["enc1", "enc2", ...]
+ *
+ * Returns the list of available encoding names.
+ *
+ *   Encoding.name_list
+ *   => ["US-ASCII", "ASCII-8BIT", "UTF-8",
+ *       "ISO-8859-1", "Shift_JIS", "EUC-JP",
+ *       "Windows-31J",
+ *       "BINARY", "CP932", "eucJP"]
+ *
+ * This list doesn't include dummy encodings.
+ *
+ */
+
+static VALUE
+rb_enc_name_list(VALUE klass)
+{
+    VALUE ary = rb_ary_new2(enc_table.names->num_entries);
+    st_foreach(enc_table.names, rb_enc_name_list_i, (st_data_t)ary);
+    return ary;
+}
+
+static int
+rb_enc_aliases_enc_i(st_data_t name, st_data_t orig, st_data_t arg)
+{
+    VALUE *p = (VALUE *)arg;
+    VALUE aliases = p[0], ary = p[1];
+    int idx = (int)orig;
+    VALUE key, str = rb_ary_entry(ary, idx);
+
+    if (NIL_P(str)) {
+	rb_encoding *enc = rb_enc_from_index(idx);
+
+	if (STRCASECMP((char*)name, rb_enc_name(enc)) == 0) {
+	    return ST_CONTINUE;
+	}
+	str = rb_usascii_str_new2(rb_enc_name(enc));
+	OBJ_FREEZE(str);
+	rb_ary_store(ary, idx, str);
+    }
+    key = rb_usascii_str_new2((char *)name);
+    OBJ_FREEZE(key);
+    rb_hash_aset(aliases, key, str);
+    return ST_CONTINUE;
+}
+
+/*
+ * call-seq:
+ *   Encoding.aliases => {"alias1" => "orig1", "alias2" => "orig2", ...}
+ *
+ * Returns the hash of available encoding alias and original encoding name.
+ *
+ *   Encoding.aliases
+ *   => {"BINARY"=>"ASCII-8BIT", "ASCII"=>"US-ASCII", "ANSI_X3.4-1986"=>"US-ASCII",
+ *       "SJIS"=>"Shift_JIS", "eucJP"=>"EUC-JP", "CP932"=>"Windows-31J"}
+ *
+ */
+
+static VALUE
+rb_enc_aliases(VALUE klass)
+{
+    VALUE aliases[2];
+    aliases[0] = rb_hash_new();
+    aliases[1] = rb_ary_new();
+    st_foreach(enc_table.names, rb_enc_aliases_enc_i, (st_data_t)aliases);
+    return aliases[0];
+}
+
 void
 Init_Encoding(void)
 {
-    int i;
+#undef rb_intern
 
-    id_based_encoding = rb_intern("#based_encoding");
+    id_base_encoding = rb_intern("#base_encoding");
 
     rb_cEncoding = rb_define_class("Encoding", rb_cObject);
     rb_undef_alloc_func(rb_cEncoding);
     rb_define_method(rb_cEncoding, "to_s", enc_name, 0);
     rb_define_method(rb_cEncoding, "inspect", enc_inspect, 0);
     rb_define_method(rb_cEncoding, "name", enc_name, 0);
-    rb_define_method(rb_cEncoding, "based_encoding", enc_based_encoding, 0);
+    rb_define_method(rb_cEncoding, "base_encoding", enc_base_encoding, 0);
     rb_define_method(rb_cEncoding, "dummy?", enc_dummy_p, 0);
     rb_define_singleton_method(rb_cEncoding, "list", enc_list, 0);
+    rb_define_singleton_method(rb_cEncoding, "name_list", rb_enc_name_list, 0);
+    rb_define_singleton_method(rb_cEncoding, "aliases", rb_enc_aliases, 0);
     rb_define_singleton_method(rb_cEncoding, "find", enc_find, 1);
     rb_define_singleton_method(rb_cEncoding, "compatible?", enc_compatible_p, 2);
 
@@ -989,13 +1198,35 @@ Init_Encoding(void)
 
     rb_define_singleton_method(rb_cEncoding, "default_external", get_default_external, 0);
     rb_define_singleton_method(rb_cEncoding, "locale_charmap", rb_locale_charmap, 0);
-
-    /* dummy for unsupported, statefull encoding */
-    rb_define_dummy_encoding("ISO-2022-JP");
-
-    for (i = 0; i < enc_table_size; ++i) {
-	rb_encoding *enc = enc_table[i].enc;
-	if (enc) set_encoding_const(rb_enc_name(enc), enc);
-    }
-    st_foreach(enc_table_alias, set_encoding_alias, 0);
 }
+
+/* locale insensitive functions */
+
+#define ctype_test(c, ctype) \
+    (rb_isascii(c) && ONIGENC_IS_ASCII_CODE_CTYPE((c), ctype))
+
+int rb_isalnum(int c) { return ctype_test(c, ONIGENC_CTYPE_ALNUM); }
+int rb_isalpha(int c) { return ctype_test(c, ONIGENC_CTYPE_ALPHA); }
+int rb_isblank(int c) { return ctype_test(c, ONIGENC_CTYPE_BLANK); }
+int rb_iscntrl(int c) { return ctype_test(c, ONIGENC_CTYPE_CNTRL); }
+int rb_isdigit(int c) { return ctype_test(c, ONIGENC_CTYPE_DIGIT); }
+int rb_isgraph(int c) { return ctype_test(c, ONIGENC_CTYPE_GRAPH); }
+int rb_islower(int c) { return ctype_test(c, ONIGENC_CTYPE_LOWER); }
+int rb_isprint(int c) { return ctype_test(c, ONIGENC_CTYPE_PRINT); }
+int rb_ispunct(int c) { return ctype_test(c, ONIGENC_CTYPE_PUNCT); }
+int rb_isspace(int c) { return ctype_test(c, ONIGENC_CTYPE_SPACE); }
+int rb_isupper(int c) { return ctype_test(c, ONIGENC_CTYPE_UPPER); }
+int rb_isxdigit(int c) { return ctype_test(c, ONIGENC_CTYPE_XDIGIT); }
+
+int
+rb_tolower(int c)
+{
+    return rb_isascii(c) ? ONIGENC_ASCII_CODE_TO_LOWER_CASE(c) : c;
+}
+
+int
+rb_toupper(int c)
+{
+    return rb_isascii(c) ? ONIGENC_ASCII_CODE_TO_UPPER_CASE(c) : c;
+}
+
