@@ -1,6 +1,6 @@
-/* $Id: rmd160ossl.c 28341 2010-06-16 09:38:14Z knu $ */
+/* $Id: rmd160ossl.c 11902 2007-02-27 10:08:39Z knu $ */
 
-#include "../defs.h"
+#include "defs.h"
 #include "rmd160ossl.h"
 
 void RMD160_Finish(RMD160_CTX *ctx, char *buf) {
