@@ -1,4 +1,4 @@
-/* $Id: rmd160ossl.h,v 1.2 2005/08/30 10:43:15 gotoyuzo Exp $ */
+/* $Id: rmd160ossl.h 11708 2007-02-12 23:01:19Z shyouhei $ */
 
 #ifndef RMD160OSSL_H_INCLUDED
 #define RMD160OSSL_H_INCLUDED
@@ -10,12 +10,10 @@
 
 #define RMD160_Init	RIPEMD160_Init
 #define RMD160_Update	RIPEMD160_Update
-#define RMD160_Final	RIPEMD160_Final
 
 #define RMD160_BLOCK_LENGTH		RIPEMD160_CBLOCK
 #define RMD160_DIGEST_LENGTH		RIPEMD160_DIGEST_LENGTH
 
-char *RMD160_End(RMD160_CTX *ctx, char *buf);
-int RMD160_Equal(RMD160_CTX *pctx1, RMD160_CTX *pctx2);
+void RMD160_Finish(RMD160_CTX *ctx, char *buf);
 
 #endif
