@@ -1,9 +1,10 @@
 # $RoughId: extconf.rb,v 1.3 2001/08/14 19:54:51 knu Exp $
-# $Id: extconf.rb,v 1.6 2004/01/07 22:06:38 nobu Exp $
+# $Id: extconf.rb 11708 2007-02-12 23:01:19Z shyouhei $
 
 require "mkmf"
 
-$CFLAGS << " -DHAVE_CONFIG_H -I#{File.dirname(__FILE__)}/.."
+$defs << "-DHAVE_CONFIG_H"
+$INCFLAGS << " -I$(srcdir)/.."
 
 $objs = [ "sha1init.#{$OBJEXT}" ]
 
@@ -13,7 +14,7 @@ if !with_config("bundled-sha1") &&
     have_library("crypto") && have_header("openssl/sha.h")
   $objs << "sha1ossl.#{$OBJEXT}"
 else
-  $objs << "sha1.#{$OBJEXT}" << "sha1hl.#{$OBJEXT}"
+  $objs << "sha1.#{$OBJEXT}"
 end
 
 have_header("sys/cdefs.h")
