@@ -4,9 +4,7 @@ have_library("sun", "getpwnam")	# NIS (== YP) interface for IRIX 4
 a = have_func("getlogin")
 b = have_func("getpwent")
 c = have_func("getgrent")
-sysconfdir = RbConfig.expand(RbConfig::MAKEFILE_CONFIG["sysconfdir"].dup, "prefix"=>"")
-$defs.push("-DSYSCONFDIR=#{Shellwords.escape(sysconfdir.dump)}")
-if  a or b or c or sysconfdir
+if  a or b or c
   have_struct_member('struct passwd', 'pw_gecos', 'pwd.h')
   have_struct_member('struct passwd', 'pw_change', 'pwd.h')
   have_struct_member('struct passwd', 'pw_quota', 'pwd.h')
