@@ -1,13 +1,19 @@
 #! /usr/local/bin/ruby
 # -*- ruby -*-
 
+$extension = nil
+$extstatic = nil
 $force_static = nil
 $install = nil
 $destdir = nil
+$dryrun = false
 $clean = nil
 $nodynamic = nil
 $extinit = nil
 $extobjs = nil
+$extflags = ""
+$extlibs = nil
+$extpath = nil
 $ignore = nil
 $message = nil
 
@@ -18,17 +24,18 @@ alias $0 $progname
 $extlist = []
 $compiled = {}
 
-$:.replace([Dir.pwd])
-require 'rbconfig'
-
 srcdir = File.dirname(File.dirname(__FILE__))
-
-$:.unshift(srcdir, File.expand_path("lib", srcdir))
+unless defined?(CROSS_COMPILING) and CROSS_COMPILING
+  $:.replace([File.expand_path("lib", srcdir), Dir.pwd])
+end
+$:.unshift(srcdir)
+require 'rbconfig'
 
 $topdir = "."
 $top_srcdir = srcdir
 
-require 'mkmf'
+$" << "mkmf.rb"
+load File.expand_path("lib/mkmf.rb", srcdir)
 require 'optparse/shellwords'
 
 def sysquote(x)
@@ -36,15 +43,6 @@ def sysquote(x)
   @quote ? x.quote : x
 end
 
-def relative_from(path, base)
-  dir = File.join(path, "")
-  if File.expand_path(dir) == File.expand_path(dir, base)
-    path
-  else
-    File.join(base, path)
-  end
-end
-
 def extract_makefile(makefile, keep = true)
   m = File.read(makefile)
   if !(target = m[/^TARGET[ \t]*=[ \t]*(\S*)/, 1])
@@ -63,7 +61,7 @@ def extract_makefile(makefile, keep = true)
       unless installrb.empty?
         config = CONFIG.dup
         install_dirs(target_prefix).each {|var, val| config[var] = val}
-        FileUtils.rm_f(installrb.values.collect {|f| RbConfig.expand(f, config)}, verbose: true)
+        FileUtils.rm_f(installrb.values.collect {|f| RbConfig.expand(f, config)}, :verbose => true)
       end
     end
     return false
@@ -73,12 +71,14 @@ def extract_makefile(makefile, keep = true)
   $static ||= m[/^EXTSTATIC[ \t]*=[ \t]*(\S+)/, 1] || false
   /^STATIC_LIB[ \t]*=[ \t]*\S+/ =~ m or $static = nil
   $preload = Shellwords.shellwords(m[/^preload[ \t]*=[ \t]*(.*)/, 1] || "")
-  $DLDFLAGS += " " + (m[/^DLDFLAGS[ \t]*=[ \t]*(.*)/, 1] || "")
+  $DLDFLAGS += " " + (m[/^dldflags[ \t]*=[ \t]*(.*)/, 1] || "")
   if s = m[/^LIBS[ \t]*=[ \t]*(.*)/, 1]
     s.sub!(/^#{Regexp.quote($LIBRUBYARG)} */, "")
     s.sub!(/ *#{Regexp.quote($LIBS)}$/, "")
     $libs = s
   end
+  $objs = (m[/^OBJS[ \t]*=[ \t](.*)/, 1] || "").split
+  $srcs = (m[/^SRCS[ \t]*=[ \t](.*)/, 1] || "").split
   $LOCAL_LIBS = m[/^LOCAL_LIBS[ \t]*=[ \t]*(.*)/, 1] || ""
   $LIBPATH = Shellwords.shellwords(m[/^libpath[ \t]*=[ \t]*(.*)/, 1] || "") - %w[$(libdir) $(topdir)]
   true
@@ -87,15 +87,6 @@ end
 def extmake(target)
   print "#{$message} #{target}\n"
   $stdout.flush
-  if $force_static or $static_ext[target]
-    $static = target
-  else
-    $static = false
-  end
-
-  unless $ignore
-    return true if $nodynamic and not $static
-  end
 
   FileUtils.mkpath target unless File.directory?(target)
   begin
@@ -104,31 +95,51 @@ def extmake(target)
     Dir.chdir target
     top_srcdir = $top_srcdir
     topdir = $topdir
-    mk_srcdir = CONFIG["srcdir"]
-    mk_topdir = CONFIG["topdir"]
+    hdrdir = $hdrdir
     prefix = "../" * (target.count("/")+1)
-    $hdrdir = $top_srcdir = relative_from(top_srcdir, prefix)
+    $top_srcdir = relative_from(top_srcdir, prefix)
+    $hdrdir = relative_from(hdrdir, prefix)
     $topdir = prefix + $topdir
     $target = target
     $mdir = target
     $srcdir = File.join($top_srcdir, "ext", $mdir)
     $preload = nil
+    $objs = ""
+    $srcs = ""
     $compiled[target] = false
     makefile = "./Makefile"
     ok = File.exist?(makefile)
     unless $ignore
-      RbConfig::CONFIG["hdrdir"] = $hdrdir
-      RbConfig::CONFIG["srcdir"] = $srcdir
-      RbConfig::CONFIG["topdir"] = $topdir
-      CONFIG["hdrdir"] = ($hdrdir == top_srcdir) ? top_srcdir : "$(topdir)"+top_srcdir[2..-1]
-      CONFIG["srcdir"] = "$(hdrdir)/ext/#{$mdir}"
-      CONFIG["topdir"] = $topdir
+      rbconfig0 = RbConfig::CONFIG
+      mkconfig0 = CONFIG
+      rbconfig = {
+	"hdrdir" => $hdrdir,
+	"srcdir" => $srcdir,
+	"topdir" => $topdir,
+      }
+      mkconfig = {
+	"hdrdir" => ($hdrdir == top_srcdir) ? top_srcdir : "$(top_srcdir)/include",
+	"srcdir" => "$(top_srcdir)/ext/#{$mdir}",
+	"topdir" => $topdir,
+      }
+      rbconfig0.each_pair {|key, val| rbconfig[key] ||= val.dup}
+      mkconfig0.each_pair {|key, val| mkconfig[key] ||= val.dup}
+      RbConfig.module_eval {
+	remove_const(:CONFIG)
+	const_set(:CONFIG, rbconfig)
+	remove_const(:MAKEFILE_CONFIG)
+	const_set(:MAKEFILE_CONFIG, mkconfig)
+      }
+      Object.class_eval {
+	remove_const(:CONFIG)
+	const_set(:CONFIG, mkconfig)
+      }
       begin
 	$extconf_h = nil
 	ok &&= extract_makefile(makefile)
 	if (($extconf_h && !File.exist?($extconf_h)) ||
 	    !(t = modified?(makefile, MTIMES)) ||
-            %W"#{$srcdir}/makefile.rb #{$srcdir}/extconf.rb #{$srcdir}/depend".any? {|f| modified?(f, [t])})
+	    ["#{$srcdir}/makefile.rb", "#{$srcdir}/extconf.rb", "#{$srcdir}/depend"].any? {|f| modified?(f, [t])})
         then
 	  ok = false
           init_mkmf
@@ -155,7 +166,7 @@ def extmake(target)
     ok = yield(ok) if block_given?
     unless ok
       open(makefile, "w") do |f|
-	f.print dummy_makefile(CONFIG["srcdir"])
+	f.print(*dummy_makefile(CONFIG["srcdir"]))
       end
       return true
     end
@@ -171,8 +182,11 @@ def extmake(target)
       $ignore or $continue or return false
     end
     $compiled[target] = true
-    if $clean and $clean != true
-      File.unlink(makefile) rescue nil
+    if $clean
+      FileUtils.rm_f("mkmf.log")
+      if $clean != true
+	FileUtils.rm_f([makefile, $extconf_h || "extconf.h"])
+      end
     end
     if $static
       $extflags ||= ""
@@ -185,13 +199,21 @@ def extmake(target)
       $extpath |= $LIBPATH
     end
   ensure
-    RbConfig::CONFIG["srcdir"] = $top_srcdir
-    RbConfig::CONFIG["topdir"] = topdir
-    CONFIG["srcdir"] = mk_srcdir
-    CONFIG["topdir"] = mk_topdir
-    CONFIG.delete("hdrdir")
-    $hdrdir = $top_srcdir = top_srcdir
+    unless $ignore
+      RbConfig.module_eval {
+	remove_const(:CONFIG)
+	const_set(:CONFIG, rbconfig0)
+	remove_const(:MAKEFILE_CONFIG)
+	const_set(:MAKEFILE_CONFIG, mkconfig0)
+      }
+      Object.class_eval {
+	remove_const(:CONFIG)
+	const_set(:CONFIG, mkconfig0)
+      }
+    end
+    $top_srcdir = top_srcdir
     $topdir = topdir
+    $hdrdir = hdrdir
     Dir.chdir dir
   end
   begin
@@ -210,7 +232,6 @@ end
 def parse_args()
   $mflags = []
 
-  opts = nil
   $optparser ||= OptionParser.new do |opts|
     opts.on('-n') {$dryrun = true}
     opts.on('--[no-]extension [EXTS]', Array) do |v|
@@ -249,7 +270,7 @@ def parse_args()
   rescue OptionParser::InvalidOption => e
     retry if /^--/ =~ e.args[0]
     $optparser.warn(e)
-    abort opts.to_s
+    abort $optparser.to_s
   end
 
   $destdir ||= ''
@@ -258,7 +279,7 @@ def parse_args()
   $mflags.unshift(*rest) unless rest.empty?
 
   def $mflags.set?(flag)
-    grep(/\A-(?!-).*#{'%c' % flag}/i) { return true }
+    grep(/\A-(?!-).*#{flag.chr}/i) { return true }
     false
   end
   def $mflags.defined?(var)
@@ -275,6 +296,7 @@ def parse_args()
   $continue = $mflags.set?(?k)
   if $extout
     $extout = '$(topdir)/'+$extout
+    RbConfig::CONFIG["extout"] = CONFIG["extout"] = $extout
     $extout_prefix = $extout ? "$(extout)$(target_prefix)/" : ""
     $mflags << "extout=#$extout" << "extout_prefix=#$extout_prefix"
   end
@@ -284,9 +306,9 @@ parse_args()
 
 if target = ARGV.shift and /^[a-z-]+$/ =~ target
   $mflags.push(target)
-  target = target.sub(/^(dist|real)(?=(?:clean)?$)/, '')
   case target
-  when /clean/
+  when /^(dist|real)?(clean)$/
+    target = $2
     $ignore ||= true
     $clean = $1 ? $1[0] : true
   when /^install\b/
@@ -307,21 +329,30 @@ end
 
 EXEEXT = CONFIG['EXEEXT']
 if CROSS_COMPILING
-  $ruby = CONFIG['MINIRUBY']
+  $ruby = $mflags.defined?("MINIRUBY") || CONFIG['MINIRUBY']
 elsif sep = config_string('BUILD_FILE_SEPARATOR')
   $ruby = "$(topdir:/=#{sep})#{sep}miniruby" + EXEEXT
 else
   $ruby = '$(topdir)/miniruby' + EXEEXT
 end
-$ruby << " -I'$(topdir)' -I'$(hdrdir)/lib'"
-$config_h = '$(topdir)/config.h'
+$ruby << " -I'$(topdir)'"
+unless CROSS_COMPILING
+  $ruby << " -I'$(top_srcdir)/lib'"
+  $ruby << " -I'$(extout)/$(arch)' -I'$(extout)/common'" if $extout
+  $ruby << " -I./- -I'$(top_srcdir)/ext' -rpurelib.rb"
+  ENV["RUBYLIB"] = "-"
+  ENV["RUBYOPT"] = "-r#{File.expand_path('ext/purelib.rb', $top_srcdir)}"
+end
+$config_h = '$(arch_hdrdir)/ruby/config.h'
+$mflags << "ruby=#$ruby"
 
 MTIMES = [__FILE__, 'rbconfig.rb', srcdir+'/lib/mkmf.rb'].collect {|f| File.mtime(f)}
 
 # get static-link modules
 $static_ext = {}
 if $extstatic
-  $extstatic.each do |target|
+  $extstatic.each do |t|
+    target = t
     target = target.downcase if /mswin32|bccwin32/ =~ RUBY_PLATFORM
     $static_ext[target] = $static_ext.size
   end
@@ -358,17 +389,23 @@ if $extension
   exts |= $extension.select {|d| File.directory?("#{ext_prefix}/#{d}")}
 else
   withes, withouts = %w[--with --without].collect {|w|
-    if not (w = %w[-extensions -ext].collect {|opt|arg_config(w+opt)}).any?
-      proc {false}
+    if not (w = %w[-extensions -ext].collect {|o|arg_config(w+o)}).any?
+      nil
     elsif (w = w.grep(String)).empty?
       proc {true}
     else
-      w.collect {|opt| opt.split(/,/)}.flatten.method(:any?)
+      proc {|c1| w.collect {|o| o.split(/,/)}.flatten.any?(&c1)}
     end
   }
-  cond = proc {|ext|
-    cond1 = proc {|n| File.fnmatch(n, ext, File::FNM_PATHNAME)}
-    withes.call(&cond1) or !withouts.call(&cond1)
+  if withes
+    withouts ||= proc {true}
+  else
+    withes = proc {false}
+    withouts ||= withes
+  end
+  cond = proc {|ext, *|
+    cond1 = proc {|n| File.fnmatch(n, ext)}
+    withes.call(cond1) or !withouts.call(cond1)
   }
   exts |= Dir.glob("#{ext_prefix}/*/**/extconf.rb").collect {|d|
     d = File.dirname(d)
@@ -380,12 +417,7 @@ else
 end
 
 if $extout
-  RbConfig.expand(extout = "#$extout", RbConfig::CONFIG.merge("topdir"=>$topdir))
-  if $install
-    RbConfig.expand(dest = "#{$destdir}#{$rubylibdir}")
-    FileUtils.cp_r(extout+"/.", dest, :remove_destination => true, :verbose => true, :noop => $dryrun)
-    exit
-  end
+  extout = RbConfig.expand("#{$extout}", RbConfig::CONFIG.merge("topdir"=>$topdir))
   unless $ignore
     FileUtils.mkpath(extout)
   end
@@ -395,12 +427,18 @@ dir = Dir.pwd
 FileUtils::makedirs('ext')
 Dir::chdir('ext')
 
-$hdrdir = $top_srcdir = relative_from(srcdir, $topdir = "..")
+hdrdir = $hdrdir
+$hdrdir = ($top_srcdir = relative_from(srcdir, $topdir = "..")) + "/include"
 exts.each do |d|
-  extmake(d) or abort
+  $static = $force_static ? $static_ext[target] : false
+
+  if $ignore or !$nodynamic or $static
+    extmake(d) or abort
+  end
 end
-$hdrdir = $top_srcdir = srcdir
+$top_srcdir = srcdir
 $topdir = "."
+$hdrdir = hdrdir
 
 extinit = Struct.new(:c, :o) {
   def initialize(src)
@@ -412,14 +450,27 @@ if $ignore
   Dir.chdir ".."
   if $clean
     Dir.rmdir('ext') rescue nil
-    FileUtils.rm_rf(extout) if $extout
+    if $extout
+      FileUtils.rm_rf([extout+"/common", extout+"/include/ruby", extout+"/rdoc"])
+      FileUtils.rm_rf(extout+"/"+CONFIG["arch"])
+      if $clean != true
+	FileUtils.rm_rf(extout+"/include/"+CONFIG["arch"])
+	FileUtils.rm_f($mflags.defined?("INSTALLED_LIST")||ENV["INSTALLED_LIST"]||".installed.list")
+	Dir.rmdir(extout+"/include") rescue nil
+	Dir.rmdir(extout) rescue nil
+      end
+    end
   end
   exit
 end
 
-if $extlist.size > 0
-  $extinit ||= ""
-  $extobjs ||= ""
+$extinit ||= ""
+$extobjs ||= ""
+$extpath ||= []
+$extflags ||= ""
+$extlibs ||= []
+unless $extlist.empty?
+  $extinit << "\n" unless $extinit.empty?
   list = $extlist.dup
   built = []
   while e = list.shift
@@ -433,45 +484,53 @@ if $extlist.size > 0
     end
     f = format("%s/%s.%s", s, i, $LIBEXT)
     if File.exist?(f)
-      $extinit += "\tinit(Init_#{i}, \"#{t}.so\");\n"
-      $extobjs += "ext/#{f} "
+      $extinit << "    init(Init_#{i}, \"#{t}.so\");\n"
+      $extobjs << "ext/#{f} "
       built << t
     end
   end
 
   src = %{\
-extern char *ruby_sourcefile, *rb_source_filename();
-#define init(func, name) (ruby_sourcefile = src = rb_source_filename(name), func(), rb_provide(src))
-void Init_ext() {\n\tchar* src;\n#$extinit}
+#include "ruby.h"
+
+#define init(func, name) {	\\
+    extern void func _((void));	\\
+    ruby_init_ext(name, func);	\\
+}
+
+void ruby_init_ext _((const char *name, void (*init)(void)));
+
+void Init_ext _((void))\n{\n#$extinit}
 }
   if !modified?(extinit.c, MTIMES) || IO.read(extinit.c) != src
-    open(extinit.c, "w") {|f| f.print src}
+    open(extinit.c, "w") {|fe| fe.print src}
   end
 
-  $extobjs = "ext/#{extinit.o} " + $extobjs
+  $extobjs = "ext/#{extinit.o} #{$extobjs}"
   if RUBY_PLATFORM =~ /m68k-human|beos/
     $extflags.delete("-L/usr/local/lib")
   end
   $extpath.delete("$(topdir)")
   $extflags = libpathflag($extpath) << " " << $extflags.strip
   conf = [
+    ['LIBRUBY_SO_UPDATE', '$(LIBRUBY_EXTS)'],
     ['SETUP', $setup],
     [enable_config("shared", $enable_shared) ? 'DLDOBJS' : 'EXTOBJS', $extobjs],
     ['EXTLIBS', $extlibs.join(' ')], ['EXTLDFLAGS', $extflags]
   ].map {|n, v|
     "#{n}=#{v}" if v and !(v = v.strip).empty?
   }.compact
-  puts conf
+  puts(*conf)
   $stdout.flush
   $mflags.concat(conf)
 else
   FileUtils.rm_f(extinit.to_a)
 end
 rubies = []
-%w[RUBY RUBYW STATIC_RUBY].each {|r|
-  n = r
+%w[RUBY RUBYW STATIC_RUBY].each {|n|
+  r = n
   if r = arg_config("--"+r.downcase) || config_string(r+"_INSTALL_NAME")
-    rubies << r+EXEEXT
+    rubies << Config.expand(r+=EXEEXT)
     $mflags << "#{n}=#{r}"
   end
 }
@@ -485,8 +544,19 @@ $stdout.flush
 $mflags.concat(rubies)
 
 if $nmake == ?b
-  $mflags.collect {|flag| flag.sub!(/\A(?=\w+=)/, "-D")}
+  unless (vars = $mflags.grep(/\A\w+=/n)).empty?
+    open(mkf = "libruby.mk", "wb") do |tmf|
+      tmf.puts("!include Makefile")
+      tmf.puts
+      tmf.puts(*vars.map {|v| v.sub(/=/, " = ")})
+      tmf.puts("PRE_LIBRUBY_UPDATE = del #{mkf}")
+    end
+    $mflags.unshift("-f#{mkf}")
+    vars.each {|flag| flag.sub!(/\A/, "-D")}
+  end
 end
+$mflags.unshift("topdir=#$topdir")
+ENV.delete("RUBYOPT")
 system($make, *sysquote($mflags)) or exit($?.exitstatus)
 
 #Local variables:
