@@ -2,8 +2,7 @@
 
   gdbm.c -
 
-  $Author: shyouhei $
-  $Date: 2007-02-13 08:01:19 +0900 (Tue, 13 Feb 2007) $
+  $Author: ko1 $
   modified at: Mon Jan 24 15:59:52 JST 1994
 
   Documentation by Peter Adolphs < futzilogik at users dot sourceforge dot net >
@@ -111,7 +110,7 @@ free_dbm(struct dbmdata *dbmp)
 {
     if (dbmp) {
         if (dbmp->di_dbm) gdbm_close(dbmp->di_dbm);
-        free(dbmp);
+        xfree(dbmp);
     }
 }
 
@@ -802,6 +801,8 @@ fgdbm_each_value(VALUE obj)
     GDBM_FILE dbm;
     VALUE keystr;
 
+    RETURN_ENUMERATOR(obj, 0, 0);
+
     GetDBM2(obj, dbmp, dbm);
     for (keystr = rb_gdbm_firstkey(dbm); RTEST(keystr);
          keystr = rb_gdbm_nextkey(dbm, keystr)) {
@@ -826,6 +827,8 @@ fgdbm_each_key(VALUE obj)
     GDBM_FILE dbm;
     VALUE keystr;
 
+    RETURN_ENUMERATOR(obj, 0, 0);
+
     GetDBM2(obj, dbmp, dbm);
     for (keystr = rb_gdbm_firstkey(dbm); RTEST(keystr);
          keystr = rb_gdbm_nextkey(dbm, keystr)) {
@@ -850,6 +853,8 @@ fgdbm_each_pair(VALUE obj)
     struct dbmdata *dbmp;
     VALUE keystr;
 
+    RETURN_ENUMERATOR(obj, 0, 0);
+
     GetDBM2(obj, dbmp, dbm);
     for (keystr = rb_gdbm_firstkey(dbm); RTEST(keystr);
          keystr = rb_gdbm_nextkey(dbm, keystr)) {
