@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_ns_spki.c,v 1.5 2004/12/31 19:23:31 gotoyuzo Exp $
+ * $Id: ossl_ns_spki.c 11708 2007-02-12 23:01:19Z shyouhei $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -56,19 +56,45 @@ ossl_spki_initialize(int argc, VALUE *argv, VALUE self)
 {
     NETSCAPE_SPKI *spki;
     VALUE buffer;
+    unsigned char *p;
 	
     if (rb_scan_args(argc, argv, "01", &buffer) == 0) {
 	return self;
     }
-    if (!(spki = NETSCAPE_SPKI_b64_decode(StringValuePtr(buffer), -1))) {
-	ossl_raise(eSPKIError, NULL);
+    StringValue(buffer);
+    if (!(spki = NETSCAPE_SPKI_b64_decode(RSTRING_PTR(buffer), -1))) {
+	p = RSTRING_PTR(buffer);
+	if (!(spki = d2i_NETSCAPE_SPKI(NULL, &p, RSTRING_LEN(buffer)))) {
+	    ossl_raise(eSPKIError, NULL);
+	}
     }
     NETSCAPE_SPKI_free(DATA_PTR(self));
     DATA_PTR(self) = spki;
+    ERR_clear_error();
 
     return self;
 }
 
+static VALUE
+ossl_spki_to_der(VALUE self)
+{
+    NETSCAPE_SPKI *spki;
+    VALUE str;
+    long len;
+    unsigned char *p;
+
+    GetSPKI(self, spki);
+    if ((len = i2d_NETSCAPE_SPKI(spki, NULL)) <= 0)
+        ossl_raise(eX509CertError, NULL);
+    str = rb_str_new(0, len);
+    p = RSTRING_PTR(str);
+    if (i2d_NETSCAPE_SPKI(spki, &p) <= 0)
+        ossl_raise(eX509CertError, NULL);
+    ossl_str_adjust(str, p);
+    
+    return str;
+}
+
 static VALUE
 ossl_spki_to_pem(VALUE self)
 {
@@ -143,7 +169,7 @@ ossl_spki_get_challenge(VALUE self)
     GetSPKI(self, spki);
     if (spki->spkac->challenge->length <= 0) {
 	OSSL_Debug("Challenge.length <= 0?");
-	return rb_str_new2(NULL);
+	return rb_str_new(0, 0);
     }
 
     return rb_str_new(spki->spkac->challenge->data,
@@ -157,8 +183,8 @@ ossl_spki_set_challenge(VALUE self, VALUE str)
 
     StringValue(str);
     GetSPKI(self, spki);
-    if (!ASN1_STRING_set(spki->spkac->challenge, RSTRING(str)->ptr,
-			 RSTRING(str)->len)) {
+    if (!ASN1_STRING_set(spki->spkac->challenge, RSTRING_PTR(str),
+			 RSTRING_LEN(str))) {
 	ossl_raise(eSPKIError, NULL);
     }
     
@@ -217,6 +243,7 @@ Init_ossl_ns_spki()
     rb_define_alloc_func(cSPKI, ossl_spki_alloc);
     rb_define_method(cSPKI, "initialize", ossl_spki_initialize, -1);
 	
+    rb_define_method(cSPKI, "to_der", ossl_spki_to_der, 0);
     rb_define_method(cSPKI, "to_pem", ossl_spki_to_pem, 0);
     rb_define_alias(cSPKI, "to_s", "to_pem");
     rb_define_method(cSPKI, "to_text", ossl_spki_print, 0);
