@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_x509crl.c 27440 2010-04-22 08:21:01Z nobu $
+ * $Id: ossl_x509crl.c 11708 2007-02-12 23:01:19Z shyouhei $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002 Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -66,14 +66,14 @@ ossl_x509crl_new(X509_CRL *crl)
     tmp = crl ? X509_CRL_dup(crl) : X509_CRL_new();
     if(!tmp) ossl_raise(eX509CRLError, NULL);
     WrapX509CRL(cX509CRL, obj, tmp);
-
+        
     return obj;
 }
 
 /*
  * PRIVATE
  */
-static VALUE
+static VALUE 
 ossl_x509crl_alloc(VALUE klass)
 {
     X509_CRL *crl;
@@ -87,11 +87,11 @@ ossl_x509crl_alloc(VALUE klass)
     return obj;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_initialize(int argc, VALUE *argv, VALUE self)
 {
     BIO *in;
-    X509_CRL *crl, *x = DATA_PTR(self);
+    X509_CRL *crl;
     VALUE arg;
 
     if (rb_scan_args(argc, argv, "01", &arg) == 0) {
@@ -99,12 +99,10 @@ ossl_x509crl_initialize(int argc, VALUE *argv, VALUE self)
     }
     arg = ossl_to_der_if_possible(arg);
     in = ossl_obj2bio(arg);
-    crl = PEM_read_bio_X509_CRL(in, &x, NULL, NULL);
-    DATA_PTR(self) = x;
+    crl = PEM_read_bio_X509_CRL(in, (X509_CRL **)&DATA_PTR(self), NULL, NULL);
     if (!crl) {
-	(void)BIO_reset(in);
-	crl = d2i_X509_CRL_bio(in, &x);
-	DATA_PTR(self) = x;
+	BIO_reset(in);
+	crl = d2i_X509_CRL_bio(in, (X509_CRL **)&DATA_PTR(self));
     }
     BIO_free(in);
     if (!crl) ossl_raise(eX509CRLError, NULL);
@@ -116,7 +114,7 @@ static VALUE
 ossl_x509crl_copy(VALUE self, VALUE other)
 {
     X509_CRL *a, *b, *crl;
-
+	
     rb_check_frozen(self);
     if (self == other) return self;
     GetX509CRL(self, a);
@@ -130,7 +128,7 @@ ossl_x509crl_copy(VALUE self, VALUE other)
     return self;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_get_version(VALUE self)
 {
     X509_CRL *crl;
@@ -142,7 +140,7 @@ ossl_x509crl_get_version(VALUE self)
     return LONG2NUM(ver);
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_set_version(VALUE self, VALUE version)
 {
     X509_CRL *crl;
@@ -159,7 +157,7 @@ ossl_x509crl_set_version(VALUE self, VALUE version)
     return version;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_get_signature_algorithm(VALUE self)
 {
     X509_CRL *crl;
@@ -181,7 +179,7 @@ ossl_x509crl_get_signature_algorithm(VALUE self)
     return str;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_get_issuer(VALUE self)
 {
     X509_CRL *crl;
@@ -191,7 +189,7 @@ ossl_x509crl_get_issuer(VALUE self)
     return ossl_x509name_new(X509_CRL_get_issuer(crl)); /* NO DUP - don't free */
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_set_issuer(VALUE self, VALUE issuer)
 {
     X509_CRL *crl;
@@ -204,7 +202,7 @@ ossl_x509crl_set_issuer(VALUE self, VALUE issuer)
     return issuer;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_get_last_update(VALUE self)
 {
     X509_CRL *crl;
@@ -214,7 +212,7 @@ ossl_x509crl_get_last_update(VALUE self)
     return asn1time_to_time(X509_CRL_get_lastUpdate(crl));
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_set_last_update(VALUE self, VALUE time)
 {
     X509_CRL *crl;
@@ -229,7 +227,7 @@ ossl_x509crl_set_last_update(VALUE self, VALUE time)
     return time;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_get_next_update(VALUE self)
 {
     X509_CRL *crl;
@@ -239,7 +237,7 @@ ossl_x509crl_get_next_update(VALUE self)
     return asn1time_to_time(X509_CRL_get_nextUpdate(crl));
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_set_next_update(VALUE self, VALUE time)
 {
     X509_CRL *crl;
@@ -264,7 +262,7 @@ ossl_x509crl_get_revoked(VALUE self)
     VALUE ary, revoked;
 
     GetX509CRL(self, crl);
-    num = sk_X509_REVOKED_num(X509_CRL_get_REVOKED(crl));
+    num = sk_X509_CRL_num(X509_CRL_get_REVOKED(crl));
     if (num < 0) {
 	OSSL_Debug("num < 0???");
 	return rb_ary_new();
@@ -272,7 +270,7 @@ ossl_x509crl_get_revoked(VALUE self)
     ary = rb_ary_new2(num);
     for(i=0; i<num; i++) {
 	/* NO DUP - don't free! */
-	rev = sk_X509_REVOKED_value(X509_CRL_get_REVOKED(crl), i);
+	rev = (X509_REVOKED *)sk_X509_CRL_value(X509_CRL_get_REVOKED(crl), i);
 	revoked = ossl_x509revoked_new(rev);
 	rb_ary_push(ary, revoked);
     }
@@ -280,7 +278,7 @@ ossl_x509crl_get_revoked(VALUE self)
     return ary;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_set_revoked(VALUE self, VALUE ary)
 {
     X509_CRL *crl;
@@ -306,7 +304,7 @@ ossl_x509crl_set_revoked(VALUE self, VALUE ary)
     return ary;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_add_revoked(VALUE self, VALUE revoked)
 {
     X509_CRL *crl;
@@ -322,7 +320,7 @@ ossl_x509crl_add_revoked(VALUE self, VALUE revoked)
     return revoked;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_sign(VALUE self, VALUE key, VALUE digest)
 {
     X509_CRL *crl;
@@ -339,7 +337,7 @@ ossl_x509crl_sign(VALUE self, VALUE key, VALUE digest)
     return self;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_verify(VALUE self, VALUE key)
 {
     X509_CRL *crl;
@@ -356,7 +354,7 @@ ossl_x509crl_verify(VALUE self, VALUE key)
     return Qfalse;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_to_der(VALUE self)
 {
     X509_CRL *crl;
@@ -379,7 +377,7 @@ ossl_x509crl_to_der(VALUE self)
     return str;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_to_pem(VALUE self)
 {
     X509_CRL *crl;
@@ -402,7 +400,7 @@ ossl_x509crl_to_pem(VALUE self)
     return str;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_to_text(VALUE self)
 {
     X509_CRL *crl;
@@ -421,14 +419,14 @@ ossl_x509crl_to_text(VALUE self)
     BIO_get_mem_ptr(out, &buf);
     str = rb_str_new(buf->data, buf->length);
     BIO_free(out);
-
+	
     return str;
 }
 
 /*
  * Gets X509v3 extensions as array of X509Ext objects
  */
-static VALUE
+static VALUE 
 ossl_x509crl_get_extensions(VALUE self)
 {
     X509_CRL *crl;
@@ -454,13 +452,13 @@ ossl_x509crl_get_extensions(VALUE self)
 /*
  * Sets X509_EXTENSIONs
  */
-static VALUE
+static VALUE 
 ossl_x509crl_set_extensions(VALUE self, VALUE ary)
 {
     X509_CRL *crl;
     X509_EXTENSION *ext;
     int i;
-
+	
     Check_Type(ary, T_ARRAY);
     /* All ary members should be X509 Extensions */
     for (i=0; i<RARRAY_LEN(ary); i++) {
@@ -481,7 +479,7 @@ ossl_x509crl_set_extensions(VALUE self, VALUE ary)
     return ary;
 }
 
-static VALUE
+static VALUE 
 ossl_x509crl_add_extension(VALUE self, VALUE extension)
 {
     X509_CRL *crl;
@@ -501,17 +499,17 @@ ossl_x509crl_add_extension(VALUE self, VALUE extension)
 /*
  * INIT
  */
-void
+void 
 Init_ossl_x509crl()
 {
     eX509CRLError = rb_define_class_under(mX509, "CRLError", eOSSLError);
 
     cX509CRL = rb_define_class_under(mX509, "CRL", rb_cObject);
-
+	
     rb_define_alloc_func(cX509CRL, ossl_x509crl_alloc);
     rb_define_method(cX509CRL, "initialize", ossl_x509crl_initialize, -1);
     rb_define_copy_func(cX509CRL, ossl_x509crl_copy);
-
+    
     rb_define_method(cX509CRL, "version", ossl_x509crl_get_version, 0);
     rb_define_method(cX509CRL, "version=", ossl_x509crl_set_version, 1);
     rb_define_method(cX509CRL, "signature_algorithm", ossl_x509crl_get_signature_algorithm, 0);
