@@ -1,4 +1,7 @@
-#include	"config.h"
+#include	"ruby/config.h"
+#ifdef RUBY_EXTCONF_H
+#include RUBY_EXTCONF_H
+#endif
 #include	<stdio.h>
 #include	<sys/types.h>
 #include	<sys/stat.h>
@@ -22,9 +25,9 @@
 #endif
 #include <ctype.h>
 
-#include "ruby.h"
-#include "rubyio.h"
-#include "util.h"
+#include "ruby/ruby.h"
+#include "ruby/io.h"
+#include "ruby/util.h"
 
 #include <signal.h>
 #ifdef HAVE_SYS_STROPTS_H
@@ -39,10 +42,10 @@
 
 #if !defined(HAVE_OPENPTY)
 #if defined(__hpux)
-static
-char	*MasterDevice = "/dev/ptym/pty%s",
-	*SlaveDevice =  "/dev/pty/tty%s",
-	*deviceNo[] = {
+static const
+char	MasterDevice[] = "/dev/ptym/pty%s",
+	SlaveDevice[] =  "/dev/pty/tty%s",
+	*const deviceNo[] = {
 		"p0","p1","p2","p3","p4","p5","p6","p7",
 		"p8","p9","pa","pb","pc","pd","pe","pf",
 		"q0","q1","q2","q3","q4","q5","q6","q7",
@@ -62,10 +65,10 @@ char	*MasterDevice = "/dev/ptym/pty%s",
 		0,
 	};
 #elif defined(_IBMESA)  /* AIX/ESA */
-static 
-char	*MasterDevice = "/dev/ptyp%s",
-  	*SlaveDevice = "/dev/ttyp%s",
-	*deviceNo[] = {
+static const
+char	MasterDevice[] = "/dev/ptyp%s",
+  	SlaveDevice[] = "/dev/ttyp%s",
+	*const deviceNo[] = {
 "00","01","02","03","04","05","06","07","08","09","0a","0b","0c","0d","0e","0f",
 "10","11","12","13","14","15","16","17","18","19","1a","1b","1c","1d","1e","1f",
 "20","21","22","23","24","25","26","27","28","29","2a","2b","2c","2d","2e","2f",
@@ -84,10 +87,10 @@ char	*MasterDevice = "/dev/ptyp%s",
 "f0","f1","f2","f3","f4","f5","f6","f7","f8","f9","fa","fb","fc","fd","fe","ff",
 		};
 #elif !defined(HAVE_PTSNAME)
-static 
-char	*MasterDevice = "/dev/pty%s",
-	*SlaveDevice = "/dev/tty%s",
-	*deviceNo[] = {
+static const
+char	MasterDevice[] = "/dev/pty%s",
+	SlaveDevice[] = "/dev/tty%s",
+	*const deviceNo[] = {
 		"p0","p1","p2","p3","p4","p5","p6","p7",
 		"p8","p9","pa","pb","pc","pd","pe","pf",
 		"q0","q1","q2","q3","q4","q5","q6","q7",
@@ -101,10 +104,6 @@ char	*MasterDevice = "/dev/pty%s",
 #endif
 #endif /* !defined(HAVE_OPENPTY) */
 
-static char SlaveName[DEVICELEN];
-
-extern int errno;
-
 #ifndef HAVE_SETEUID
 # ifdef HAVE_SETREUID
 #  define seteuid(e)	setreuid(-1, (e))
@@ -120,8 +119,7 @@ extern int errno;
 static VALUE eChildExited;
 
 static VALUE
-echild_status(self)
-    VALUE self;
+echild_status(VALUE self)
 {
     return rb_ivar_get(self, rb_intern("status"));
 }
@@ -133,41 +131,36 @@ struct pty_info {
 };
 
 static void
-raise_from_wait(state, info)
-    struct pty_info *info;
-    char *state;
+raise_from_wait(char *state, struct pty_info *info)
 {
-    extern VALUE rb_last_status;
     char buf[1024];
     VALUE exc;
 
-    snprintf(buf, sizeof(buf), "pty - %s: %d", state, info->child_pid);
+    snprintf(buf, sizeof(buf), "pty - %s: %ld", state, (long)info->child_pid);
     exc = rb_exc_new2(eChildExited, buf);
-    rb_iv_set(exc, "status", rb_last_status);
+    rb_iv_set(exc, "status", rb_last_status_get());
     rb_funcall(info->thread, rb_intern("raise"), 1, exc);
 }
 
 static VALUE
-pty_syswait(info)
-    struct pty_info *info;
+pty_syswait(struct pty_info *info)
 {
-    int cpid, status;
+    rb_pid_t cpid;
+    int status;
 
     for (;;) {
 	cpid = rb_waitpid(info->child_pid, &status, WUNTRACED);
 	if (cpid == -1) return Qnil;
 
-#if defined(IF_STOPPED)
-	if (IF_STOPPED(status)) { /* suspend */
-	    raise_from_wait("stopped", info);
-	}
-#elif defined(WIFSTOPPED)
-	if (WIFSTOPPED(status)) { /* suspend */
-	    raise_from_wait("stopped", info);
-	}
+#if defined(WIFSTOPPED)
+#elif defined(IF_STOPPED)
+#define WIFSTOPPED(status) IF_STOPPED(status)
 #else
 ---->> Either IF_STOPPED or WIFSTOPPED is needed <<----
 #endif /* WIFSTOPPED | IF_STOPPED */
+	if (WIFSTOPPED(status)) { /* suspend */
+	    raise_from_wait("stopped", info);
+	}
 	else if (kill(info->child_pid, 0) == 0) {
 	    raise_from_wait("changed", info);
 	}
@@ -178,7 +171,7 @@ pty_syswait(info)
     }
 }
 
-static void getDevice _((int*, int*));
+static void getDevice(int*, int*, char [DEVICELEN]);
 
 struct exec_info {
     int argc;
@@ -186,27 +179,26 @@ struct exec_info {
 };
 
 static VALUE
-pty_exec(arg)
-    struct exec_info *arg;
+pty_exec(VALUE v)
 {
+    struct exec_info *arg = (struct exec_info *)v;
     return rb_f_exec(arg->argc, arg->argv);
 }
 
 static void
-establishShell(argc, argv, info)
-    int argc;
-    VALUE *argv;
-    struct pty_info *info;
-{	
-    static int		i,master,slave,currentPid;
-    char		*p,*getenv();
+establishShell(int argc, VALUE *argv, struct pty_info *info,
+	       char SlaveName[DEVICELEN])
+{
+    int 		master,slave;
+    rb_pid_t		pid;
+    char		*p, tmp, *getenv();
     struct passwd	*pwent;
     VALUE		v;
     struct exec_info	arg;
     int			status;
 
     if (argc == 0) {
-	char *shellname;
+	const char *shellname;
 
 	if ((p = getenv("SHELL")) != NULL) {
 	    shellname = p;
@@ -222,19 +214,16 @@ establishShell(argc, argv, info)
 	argc = 1;
 	argv = &v;
     }
-    getDevice(&master,&slave);
+    getDevice(&master, &slave, SlaveName);
 
     info->thread = rb_thread_current();
-    currentPid = getpid();
-    if((i = fork()) < 0) {
+    if ((pid = fork()) < 0) {
 	close(master);
 	close(slave);
 	rb_sys_fail("fork failed");
     }
 
-    if(i == 0) {	/* child */
-	currentPid = getpid();	
-
+    if (pid == 0) {	/* child */
 	/*
 	 * Set free from process group and controlling terminal
 	 */
@@ -246,11 +235,11 @@ establishShell(argc, argv, info)
 	if (setpgrp() == -1)
 	    perror("setpgrp()");
 #  else /* SETGRP_VOID */
-	if (setpgrp(0, currentPid) == -1)
+	if (setpgrp(0, getpid()) == -1)
 	    rb_sys_fail("setpgrp()");
-	if ((i = open("/dev/tty", O_RDONLY)) < 0)
-	    rb_sys_fail("/dev/tty");
-	else {
+	{
+	    int i = open("/dev/tty", O_RDONLY);
+	    if (i < 0) rb_sys_fail("/dev/tty");
 	    if (ioctl(i, TIOCNOTTY, (char *)0))
 		perror("ioctl(TIOCNOTTY)");
 	    close(i);
@@ -275,6 +264,7 @@ establishShell(argc, argv, info)
 	}
 	close(master);
 #endif
+	write(slave, "", 1);
 	dup2(slave,0);
 	dup2(slave,1);
 	dup2(slave,2);
@@ -290,15 +280,15 @@ establishShell(argc, argv, info)
 	_exit(1);
     }
 
+    read(master, &tmp, 1);
     close(slave);
 
-    info->child_pid = i;
+    info->child_pid = pid;
     info->fd = master;
 }
 
 static VALUE
-pty_finalize_syswait(info)
-    struct pty_info *info;
+pty_finalize_syswait(struct pty_info *info)
 {
     rb_thread_kill(info->thread);
     rb_funcall(info->thread, rb_intern("value"), 0);
@@ -306,40 +296,34 @@ pty_finalize_syswait(info)
     return Qnil;
 }
 
-#ifdef HAVE_OPENPTY
+static int
+get_device_once(int *master, int *slave, char SlaveName[DEVICELEN], int fail)
+{
+#if defined HAVE_OPENPTY
 /*
  * Use openpty(3) of 4.3BSD Reno and later,
  * or the same interface function.
  */
-static void
-getDevice(master,slave)
-    int	*master,*slave;
-{
     if (openpty(master, slave, SlaveName,
 		(struct termios *)0, (struct winsize *)0) == -1) {
+	if (!fail) return -1;
 	rb_raise(rb_eRuntimeError, "openpty() failed");
     }
-}
-#else /* HAVE_OPENPTY */
-#ifdef HAVE__GETPTY
-static void
-getDevice(master,slave)
-    int	*master,*slave;
-{
+
+    return 0;
+#elif defined HAVE__GETPTY
     char *name;
 
     if (!(name = _getpty(master, O_RDWR, 0622, 0))) {
+	if (!fail) return -1;
 	rb_raise(rb_eRuntimeError, "_getpty() failed");
     }
 
     *slave = open(name, O_RDWR);
-    strcpy(SlaveName, name);
-}
+    strlcpy(SlaveName, name, sizeof SlaveName);
+
+    return 0;
 #else /* HAVE__GETPTY */
-static void
-getDevice(master,slave)
-    int	*master,*slave;
-{
     int	 i,j;
 
 #ifdef HAVE_PTSNAME
@@ -360,11 +344,12 @@ getDevice(master,slave)
 #if defined I_PUSH && !defined linux
 			if(ioctl(j, I_PUSH, "ptem") != -1) {
 			    if(ioctl(j, I_PUSH, "ldterm") != -1) {
+				ioctl(j, I_PUSH, "ttcompat");
 #endif
 				*master = i;
 				*slave = j;
-				strcpy(SlaveName, pn);
-				return;
+				strlcpy(SlaveName, pn, sizeof SlaveName);
+				return 0;
 #if defined I_PUSH && !defined linux
 			    }
 			}
@@ -375,56 +360,57 @@ getDevice(master,slave)
 	}
 	close(i);
     }
-    rb_raise(rb_eRuntimeError, "can't get Master/Slave device");
+    if (!fail) rb_raise(rb_eRuntimeError, "can't get Master/Slave device");
+    return -1;
 #else
     char **p;
     char MasterName[DEVICELEN];
 
     for (p = deviceNo; *p != NULL; p++) {
-	sprintf(MasterName,MasterDevice,*p);
+	snprintf(MasterName, sizeof MasterName, MasterDevice, *p);
 	if ((i = open(MasterName,O_RDWR,0)) >= 0) {
 	    *master = i;
-	    sprintf(SlaveName,SlaveDevice,*p);
+	    snprintf(SlaveName, sizeof SlaveName, SlaveDevice, *p);
 	    if ((j = open(SlaveName,O_RDWR,0)) >= 0) {
 		*slave = j;
 		chown(SlaveName, getuid(), getgid());
 		chmod(SlaveName, 0622);
-		return;
+		return 0;
 	    }
 	    close(i);
 	}
     }
-    rb_raise(rb_eRuntimeError, "can't get %s", SlaveName);
+    if (fail) rb_raise(rb_eRuntimeError, "can't get %s", SlaveName);
+    return -1;
+#endif
 #endif
 }
-#endif /* HAVE__GETPTY */
-#endif /* HAVE_OPENPTY */
 
 static void
-freeDevice()
+getDevice(int *master, int *slave, char SlaveName[DEVICELEN])
 {
-    chmod(SlaveName, 0666);
-    chown(SlaveName, 0, 0);
+    if (get_device_once(master, slave, SlaveName, 0)) {
+	rb_gc();
+	get_device_once(master, slave, SlaveName, 1);
+    }
 }
 
 /* ruby function: getpty */
 static VALUE
-pty_getpty(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
+pty_getpty(int argc, VALUE *argv, VALUE self)
 {
     VALUE res;
     struct pty_info info;
     struct pty_info thinfo;
-    OpenFile *wfptr,*rfptr;
+    rb_io_t *wfptr,*rfptr;
     VALUE rport = rb_obj_alloc(rb_cFile);
     VALUE wport = rb_obj_alloc(rb_cFile);
-  
+    char SlaveName[DEVICELEN];
+
     MakeOpenFile(rport, rfptr);
     MakeOpenFile(wport, wfptr);
 
-    establishShell(argc, argv, &info);
+    establishShell(argc, argv, &info, SlaveName);
 
     rfptr->mode = rb_io_mode_flags("r");
     rfptr->fd = info.fd;
@@ -437,7 +423,7 @@ pty_getpty(argc, argv, self)
     res = rb_ary_new2(3);
     rb_ary_store(res,0,(VALUE)rport);
     rb_ary_store(res,1,(VALUE)wport);
-    rb_ary_store(res,2,INT2FIX(info.child_pid));
+    rb_ary_store(res,2,PIDT2NUM(info.child_pid));
 
     thinfo.thread = rb_thread_create(pty_syswait, (void*)&info);
     thinfo.child_pid = info.child_pid;
@@ -452,8 +438,7 @@ pty_getpty(argc, argv, self)
 
 /* ruby function: protect_signal - obsolete */
 static VALUE
-pty_protect(self)
-    VALUE self;
+pty_protect(VALUE self)
 {
     rb_warn("PTY::protect_signal is no longer needed");
     rb_yield(Qnil);
@@ -462,8 +447,7 @@ pty_protect(self)
 
 /* ruby function: reset_signal - obsolete */
 static VALUE
-pty_reset_signal(self)
-    VALUE self;
+pty_reset_signal(VALUE self)
 {
     rb_warn("PTY::reset_signal is no longer needed");
     return self;
