@@ -15,14 +15,6 @@ def have_readline_var(var)
   return have_var(var, $readline_headers)
 end
 
-def have_readline_func(func)
-  return have_func(func, $readline_headers)
-end
-
-def have_readline_macro(macro)
-  return have_macro(macro, $readline_headers)
-end
-
 dir_config('curses')
 dir_config('ncurses')
 dir_config('termcap')
@@ -50,11 +42,9 @@ else
   end
 end
 
-have_readline_func("rl_getc_function")
-have_readline_func("rl_filename_completion_function")
-have_readline_func("rl_username_completion_function")
-have_readline_func("rl_completion_matches")
-have_readline_func("rl_refresh_line")
+have_func("rl_filename_completion_function")
+have_func("rl_username_completion_function")
+have_func("rl_completion_matches")
 have_readline_var("rl_deprep_term_function")
 have_readline_var("rl_completion_append_character")
 have_readline_var("rl_basic_word_break_characters")
@@ -64,22 +54,12 @@ have_readline_var("rl_completer_quote_characters")
 have_readline_var("rl_filename_quote_characters")
 have_readline_var("rl_attempted_completion_over")
 have_readline_var("rl_library_version")
-have_readline_var("rl_editing_mode")
-have_readline_var("rl_line_buffer")
-have_readline_var("rl_point")
 # workaround for native windows.
 /mswin|bccwin|mingw/ !~ RUBY_PLATFORM && have_readline_var("rl_event_hook")
-/mswin|bccwin|mingw/ !~ RUBY_PLATFORM && have_readline_var("rl_catch_sigwinch")
-/mswin|bccwin|mingw/ !~ RUBY_PLATFORM && have_readline_var("rl_catch_signals")
-have_readline_func("rl_cleanup_after_signal")
-have_readline_func("rl_clear_signals")
-have_readline_func("rl_set_screen_size")
-have_readline_func("rl_get_screen_size")
-have_readline_func("rl_vi_editing_mode")
-have_readline_func("rl_emacs_editing_mode")
-have_readline_func("replace_history_entry")
-have_readline_func("remove_history")
-have_readline_func("clear_history")
-have_readline_macro("RL_PROMPT_START_IGNORE")
-have_readline_macro("RL_PROMPT_END_IGNORE")
+have_func("rl_cleanup_after_signal")
+have_func("rl_clear_signals")
+have_func("rl_vi_editing_mode")
+have_func("rl_emacs_editing_mode")
+have_func("replace_history_entry")
+have_func("remove_history")
 create_makefile("readline")
