@@ -1,9 +1,13 @@
 /* readline.c -- GNU Readline module
    Copyright (C) 1997-2001  Shugo Maeda */
 
+#ifdef RUBY_EXTCONF_H
+#include RUBY_EXTCONF_H
+#endif
+
+#include "ruby/config.h"
 #include <errno.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <string.h>
 #ifdef HAVE_READLINE_READLINE_H
 #include <readline/readline.h>
@@ -15,9 +19,9 @@
 #include <editline/readline.h>
 #endif
 
-#include "ruby.h"
-#include "rubyio.h"
-#include "rubysig.h"
+#include "ruby/ruby.h"
+#include "ruby/io.h"
+#include "ruby/signal.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -25,35 +29,45 @@
 
 static VALUE mReadline;
 
-#define TOLOWER(c) (isupper(c) ? tolower(c) : c)
-
 #define COMPLETION_PROC "completion_proc"
 #define COMPLETION_CASE_FOLD "completion_case_fold"
 static ID completion_proc, completion_case_fold;
 
 #ifndef HAVE_RL_FILENAME_COMPLETION_FUNCTION
 # define rl_filename_completion_function filename_completion_function
+#endif
+#ifndef HAVE_RL_USERNAME_COMPLETION_FUNCTION
 # define rl_username_completion_function username_completion_function
+#endif
+#ifndef HAVE_RL_COMPLETION_MATCHES
 # define rl_completion_matches completion_matches
 #endif
 
-static int readline_event(void);
 static char **readline_attempted_completion_function(const char *text,
                                                      int start, int end);
 
+#ifdef HAVE_RL_EVENT_HOOK
+#define BUSY_WAIT 0
+
+static int readline_event(void);
 static int
-readline_event()
+readline_event(void)
 {
-    CHECK_INTS;
+#if BUSY_WAIT
     rb_thread_schedule();
+#else
+    fd_set rset;
+
+    FD_ZERO(&rset);
+    FD_SET(fileno(rl_instream), &rset);
+    rb_thread_select(fileno(rl_instream) + 1, &rset, NULL, NULL, NULL);
     return 0;
+#endif
 }
+#endif
 
 static VALUE
-readline_readline(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
+readline_readline(int argc, VALUE *argv, VALUE self)
 {
     VALUE tmp, add_hist, result;
     char *prompt = NULL;
@@ -63,7 +77,7 @@ readline_readline(argc, argv, self)
     rb_secure(4);
     if (rb_scan_args(argc, argv, "02", &tmp, &add_hist) > 0) {
 	SafeStringValue(tmp);
-	prompt = RSTRING(tmp)->ptr;
+	prompt = RSTRING_PTR(tmp);
     }
 
     if (!isatty(0) && errno == EBADF) rb_raise(rb_eIOError, "stdin closed");
@@ -76,7 +90,9 @@ readline_readline(argc, argv, self)
         rl_cleanup_after_signal();
 #elif defined HAVE_RL_DEPREP_TERM_FUNCTION
         /* restore terminal mode */
-        (*rl_deprep_term_function)();
+	if (rl_deprep_term_function != NULL) /* NULL in libedit. [ruby-dev:29116] */
+	    (*rl_deprep_term_function)();
+	else
 #else
         rl_deprep_terminal();
 #endif
@@ -86,8 +102,10 @@ readline_readline(argc, argv, self)
     if (RTEST(add_hist) && buff) {
 	add_history(buff);
     }
-    if (buff)
+    if (buff) {
 	result = rb_tainted_str_new2(buff);
+	rb_enc_associate(result, rb_locale_encoding());
+    }
     else
 	result = Qnil;
     if (buff) free(buff);
@@ -95,10 +113,9 @@ readline_readline(argc, argv, self)
 }
 
 static VALUE
-readline_s_set_input(self, input)
-    VALUE self, input;
+readline_s_set_input(VALUE self, VALUE input)
 {
-    OpenFile *ifp;
+    rb_io_t *ifp;
 
     rb_secure(4);
     Check_Type(input, T_FILE);
@@ -108,10 +125,9 @@ readline_s_set_input(self, input)
 }
 
 static VALUE
-readline_s_set_output(self, output)
-    VALUE self, output;
+readline_s_set_output(VALUE self, VALUE output)
 {
-    OpenFile *ofp;
+    rb_io_t *ofp;
 
     rb_secure(4);
     Check_Type(output, T_FILE);
@@ -121,9 +137,7 @@ readline_s_set_output(self, output)
 }
 
 static VALUE
-readline_s_set_completion_proc(self, proc)
-    VALUE self;
-    VALUE proc;
+readline_s_set_completion_proc(VALUE self, VALUE proc)
 {
     rb_secure(4);
     if (!rb_respond_to(proc, rb_intern("call")))
@@ -132,35 +146,28 @@ readline_s_set_completion_proc(self, proc)
 }
 
 static VALUE
-readline_s_get_completion_proc(self)
-    VALUE self;
+readline_s_get_completion_proc(VALUE self)
 {
     rb_secure(4);
     return rb_attr_get(mReadline, completion_proc);
 }
 
 static VALUE
-readline_s_set_completion_case_fold(self, val)
-    VALUE self;
-    VALUE val;
+readline_s_set_completion_case_fold(VALUE self, VALUE val)
 {
     rb_secure(4);
     return rb_ivar_set(mReadline, completion_case_fold, val);
 }
 
 static VALUE
-readline_s_get_completion_case_fold(self)
-    VALUE self;
+readline_s_get_completion_case_fold(VALUE self)
 {
     rb_secure(4);
     return rb_attr_get(mReadline, completion_case_fold);
 }
 
 static char **
-readline_attempted_completion_function(text, start, end)
-    const char *text;
-    int start;
-    int end;
+readline_attempted_completion_function(const char *text, int start, int end)
 {
     VALUE proc, ary, temp;
     char **result;
@@ -177,14 +184,14 @@ readline_attempted_completion_function(text, start, end)
     ary = rb_funcall(proc, rb_intern("call"), 1, rb_tainted_str_new2(text));
     if (TYPE(ary) != T_ARRAY)
 	ary = rb_Array(ary);
-    matches = RARRAY(ary)->len;
+    matches = RARRAY_LEN(ary);
     if (matches == 0)
 	return NULL;
     result = ALLOC_N(char *, matches + 2);
     for (i = 0; i < matches; i++) {
-	temp = rb_obj_as_string(RARRAY(ary)->ptr[i]);
-	result[i + 1] = ALLOC_N(char, RSTRING(temp)->len + 1);
-	strcpy(result[i + 1], RSTRING(temp)->ptr);
+	temp = rb_obj_as_string(RARRAY_PTR(ary)[i]);
+	result[i + 1] = ALLOC_N(char, RSTRING_LEN(temp) + 1);
+	strcpy(result[i + 1], RSTRING_PTR(temp));
     }
     result[matches + 1] = NULL;
 
@@ -224,8 +231,7 @@ readline_attempted_completion_function(text, start, end)
 }
 
 static VALUE
-readline_s_vi_editing_mode(self)
-    VALUE self;
+readline_s_vi_editing_mode(VALUE self)
 {
 #ifdef HAVE_RL_VI_EDITING_MODE
     rb_secure(4);
@@ -238,8 +244,7 @@ readline_s_vi_editing_mode(self)
 }
 
 static VALUE
-readline_s_emacs_editing_mode(self)
-    VALUE self;
+readline_s_emacs_editing_mode(VALUE self)
 {
 #ifdef HAVE_RL_EMACS_EDITING_MODE
     rb_secure(4);
@@ -252,8 +257,7 @@ readline_s_emacs_editing_mode(self)
 }
 
 static VALUE
-readline_s_set_completion_append_character(self, str)
-    VALUE self, str;
+readline_s_set_completion_append_character(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_COMPLETION_APPEND_CHARACTER
     rb_secure(4);
@@ -262,10 +266,10 @@ readline_s_set_completion_append_character(self, str)
     }
     else {
 	SafeStringValue(str);
-	if (RSTRING(str)->len == 0) {
+	if (RSTRING_LEN(str) == 0) {
 	    rl_completion_append_character = '\0';
 	} else {
-	    rl_completion_append_character = RSTRING(str)->ptr[0];
+	    rl_completion_append_character = RSTRING_PTR(str)[0];
 	}
     }
     return self;
@@ -276,8 +280,7 @@ readline_s_set_completion_append_character(self, str)
 }
 
 static VALUE
-readline_s_get_completion_append_character(self)
-    VALUE self;
+readline_s_get_completion_append_character(VALUE self)
 {
 #ifdef HAVE_RL_COMPLETION_APPEND_CHARACTER
     VALUE str;
@@ -287,7 +290,7 @@ readline_s_get_completion_append_character(self)
 	return Qnil;
 
     str = rb_str_new("", 1);
-    RSTRING(str)->ptr[0] = rl_completion_append_character;
+    RSTRING_PTR(str)[0] = rl_completion_append_character;
     return str;
 #else
     rb_notimplement();
@@ -296,8 +299,7 @@ readline_s_get_completion_append_character(self)
 }
 
 static VALUE
-readline_s_set_basic_word_break_characters(self, str)
-    VALUE self, str;
+readline_s_set_basic_word_break_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_BASIC_WORD_BREAK_CHARACTERS
     static char *basic_word_break_characters = NULL;
@@ -306,14 +308,14 @@ readline_s_set_basic_word_break_characters(self, str)
     SafeStringValue(str);
     if (basic_word_break_characters == NULL) {
 	basic_word_break_characters =
-	    ALLOC_N(char, RSTRING(str)->len + 1);
+	    ALLOC_N(char, RSTRING_LEN(str) + 1);
     }
     else {
-	REALLOC_N(basic_word_break_characters, char, RSTRING(str)->len + 1);
+	REALLOC_N(basic_word_break_characters, char, RSTRING_LEN(str) + 1);
     }
     strncpy(basic_word_break_characters,
-	    RSTRING(str)->ptr, RSTRING(str)->len);
-    basic_word_break_characters[RSTRING(str)->len] = '\0';
+	    RSTRING_PTR(str), RSTRING_LEN(str));
+    basic_word_break_characters[RSTRING_LEN(str)] = '\0';
     rl_basic_word_break_characters = basic_word_break_characters;
     return self;
 #else
@@ -323,8 +325,7 @@ readline_s_set_basic_word_break_characters(self, str)
 }
 
 static VALUE
-readline_s_get_basic_word_break_characters(self, str)
-    VALUE self, str;
+readline_s_get_basic_word_break_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_BASIC_WORD_BREAK_CHARACTERS
     rb_secure(4);
@@ -338,8 +339,7 @@ readline_s_get_basic_word_break_characters(self, str)
 }
 
 static VALUE
-readline_s_set_completer_word_break_characters(self, str)
-    VALUE self, str;
+readline_s_set_completer_word_break_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_COMPLETER_WORD_BREAK_CHARACTERS
     static char *completer_word_break_characters = NULL;
@@ -348,14 +348,14 @@ readline_s_set_completer_word_break_characters(self, str)
     SafeStringValue(str);
     if (completer_word_break_characters == NULL) {
 	completer_word_break_characters =
-	    ALLOC_N(char, RSTRING(str)->len + 1);
+	    ALLOC_N(char, RSTRING_LEN(str) + 1);
     }
     else {
-	REALLOC_N(completer_word_break_characters, char, RSTRING(str)->len + 1);
+	REALLOC_N(completer_word_break_characters, char, RSTRING_LEN(str) + 1);
     }
     strncpy(completer_word_break_characters,
-	    RSTRING(str)->ptr, RSTRING(str)->len);
-    completer_word_break_characters[RSTRING(str)->len] = '\0';
+	    RSTRING_PTR(str), RSTRING_LEN(str));
+    completer_word_break_characters[RSTRING_LEN(str)] = '\0';
     rl_completer_word_break_characters = completer_word_break_characters;
     return self;
 #else
@@ -365,8 +365,7 @@ readline_s_set_completer_word_break_characters(self, str)
 }
 
 static VALUE
-readline_s_get_completer_word_break_characters(self, str)
-    VALUE self, str;
+readline_s_get_completer_word_break_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_COMPLETER_WORD_BREAK_CHARACTERS
     rb_secure(4);
@@ -380,8 +379,7 @@ readline_s_get_completer_word_break_characters(self, str)
 }
 
 static VALUE
-readline_s_set_basic_quote_characters(self, str)
-    VALUE self, str;
+readline_s_set_basic_quote_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_BASIC_QUOTE_CHARACTERS
     static char *basic_quote_characters = NULL;
@@ -390,14 +388,14 @@ readline_s_set_basic_quote_characters(self, str)
     SafeStringValue(str);
     if (basic_quote_characters == NULL) {
 	basic_quote_characters =
-	    ALLOC_N(char, RSTRING(str)->len + 1);
+	    ALLOC_N(char, RSTRING_LEN(str) + 1);
     }
     else {
-	REALLOC_N(basic_quote_characters, char, RSTRING(str)->len + 1);
+	REALLOC_N(basic_quote_characters, char, RSTRING_LEN(str) + 1);
     }
     strncpy(basic_quote_characters,
-	    RSTRING(str)->ptr, RSTRING(str)->len);
-    basic_quote_characters[RSTRING(str)->len] = '\0';
+	    RSTRING_PTR(str), RSTRING_LEN(str));
+    basic_quote_characters[RSTRING_LEN(str)] = '\0';
     rl_basic_quote_characters = basic_quote_characters;
 
     return self;
@@ -408,8 +406,7 @@ readline_s_set_basic_quote_characters(self, str)
 }
 
 static VALUE
-readline_s_get_basic_quote_characters(self, str)
-    VALUE self, str;
+readline_s_get_basic_quote_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_BASIC_QUOTE_CHARACTERS
     rb_secure(4);
@@ -423,8 +420,7 @@ readline_s_get_basic_quote_characters(self, str)
 }
 
 static VALUE
-readline_s_set_completer_quote_characters(self, str)
-    VALUE self, str;
+readline_s_set_completer_quote_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_COMPLETER_QUOTE_CHARACTERS
     static char *completer_quote_characters = NULL;
@@ -433,14 +429,13 @@ readline_s_set_completer_quote_characters(self, str)
     SafeStringValue(str);
     if (completer_quote_characters == NULL) {
 	completer_quote_characters =
-	    ALLOC_N(char, RSTRING(str)->len + 1);
+	    ALLOC_N(char, RSTRING_LEN(str) + 1);
     }
     else {
-	REALLOC_N(completer_quote_characters, char, RSTRING(str)->len + 1);
+	REALLOC_N(completer_quote_characters, char, RSTRING_LEN(str) + 1);
     }
-    strncpy(completer_quote_characters,
-	    RSTRING(str)->ptr, RSTRING(str)->len);
-    completer_quote_characters[RSTRING(str)->len] = '\0';
+    strncpy(completer_quote_characters, RSTRING_PTR(str), RSTRING_LEN(str));
+    completer_quote_characters[RSTRING_LEN(str)] = '\0';
     rl_completer_quote_characters = completer_quote_characters;
 
     return self;
@@ -451,8 +446,7 @@ readline_s_set_completer_quote_characters(self, str)
 }
 
 static VALUE
-readline_s_get_completer_quote_characters(self, str)
-    VALUE self, str;
+readline_s_get_completer_quote_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_COMPLETER_QUOTE_CHARACTERS
     rb_secure(4);
@@ -466,8 +460,7 @@ readline_s_get_completer_quote_characters(self, str)
 }
 
 static VALUE
-readline_s_set_filename_quote_characters(self, str)
-    VALUE self, str;
+readline_s_set_filename_quote_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_FILENAME_QUOTE_CHARACTERS
     static char *filename_quote_characters = NULL;
@@ -476,14 +469,13 @@ readline_s_set_filename_quote_characters(self, str)
     SafeStringValue(str);
     if (filename_quote_characters == NULL) {
 	filename_quote_characters =
-	    ALLOC_N(char, RSTRING(str)->len + 1);
+	    ALLOC_N(char, RSTRING_LEN(str) + 1);
     }
     else {
-	REALLOC_N(filename_quote_characters, char, RSTRING(str)->len + 1);
+	REALLOC_N(filename_quote_characters, char, RSTRING_LEN(str) + 1);
     }
-    strncpy(filename_quote_characters,
-	    RSTRING(str)->ptr, RSTRING(str)->len);
-    filename_quote_characters[RSTRING(str)->len] = '\0';
+    strncpy(filename_quote_characters, RSTRING_PTR(str), RSTRING_LEN(str));
+    filename_quote_characters[RSTRING_LEN(str)] = '\0';
     rl_filename_quote_characters = filename_quote_characters;
 
     return self;
@@ -494,8 +486,7 @@ readline_s_set_filename_quote_characters(self, str)
 }
 
 static VALUE
-readline_s_get_filename_quote_characters(self, str)
-    VALUE self, str;
+readline_s_get_filename_quote_characters(VALUE self, VALUE str)
 {
 #ifdef HAVE_RL_FILENAME_QUOTE_CHARACTERS
     rb_secure(4);
@@ -509,16 +500,13 @@ readline_s_get_filename_quote_characters(self, str)
 }
 
 static VALUE
-hist_to_s(self)
-    VALUE self;
+hist_to_s(VALUE self)
 {
     return rb_str_new2("HISTORY");
 }
 
 static VALUE
-hist_get(self, index)
-    VALUE self;
-    VALUE index;
+hist_get(VALUE self, VALUE index)
 {
     HIST_ENTRY *entry;
     int i;
@@ -536,10 +524,7 @@ hist_get(self, index)
 }
 
 static VALUE
-hist_set(self, index, str)
-    VALUE self;
-    VALUE index;
-    VALUE str;
+hist_set(VALUE self, VALUE index, VALUE str)
 {
 #ifdef HAVE_REPLACE_HISTORY_ENTRY
     HIST_ENTRY *entry;
@@ -551,7 +536,7 @@ hist_set(self, index, str)
     if (i < 0) {
         i += history_length;
     }
-    entry = replace_history_entry(i, RSTRING(str)->ptr, NULL);
+    entry = replace_history_entry(i, RSTRING_PTR(str), NULL);
     if (entry == NULL) {
 	rb_raise(rb_eIndexError, "invalid index");
     }
@@ -563,21 +548,16 @@ hist_set(self, index, str)
 }
 
 static VALUE
-hist_push(self, str)
-    VALUE self;
-    VALUE str;
+hist_push(VALUE self, VALUE str)
 {
     rb_secure(4);
     SafeStringValue(str);
-    add_history(RSTRING(str)->ptr);
+    add_history(RSTRING_PTR(str));
     return self;
 }
 
 static VALUE
-hist_push_method(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
+hist_push_method(int argc, VALUE *argv, VALUE self)
 {
     VALUE str;
     
@@ -585,14 +565,13 @@ hist_push_method(argc, argv, self)
     while (argc--) {
 	str = *argv++;
 	SafeStringValue(str);
-	add_history(RSTRING(str)->ptr);
+	add_history(RSTRING_PTR(str));
     }
     return self;
 }
 
 static VALUE
-rb_remove_history(index)
-    int index;
+rb_remove_history(int index)
 {
 #ifdef HAVE_REMOVE_HISTORY
     HIST_ENTRY *entry;
@@ -614,8 +593,7 @@ rb_remove_history(index)
 }
 
 static VALUE
-hist_pop(self)
-    VALUE self;
+hist_pop(VALUE self)
 {
     rb_secure(4);
     if (history_length > 0) {
@@ -626,8 +604,7 @@ hist_pop(self)
 }
 
 static VALUE
-hist_shift(self)
-    VALUE self;
+hist_shift(VALUE self)
 {
     rb_secure(4);
     if (history_length > 0) {
@@ -638,12 +615,13 @@ hist_shift(self)
 }
 
 static VALUE
-hist_each(self)
-    VALUE self;
+hist_each(VALUE self)
 {
     HIST_ENTRY *entry;
     int i;
 
+    RETURN_ENUMERATOR(self, 0, 0);
+
     rb_secure(4);
     for (i = 0; i < history_length; i++) {
         entry = history_get(history_base + i);
@@ -655,25 +633,21 @@ hist_each(self)
 }
 
 static VALUE
-hist_length(self)
-    VALUE self;
+hist_length(VALUE self)
 {
     rb_secure(4);
     return INT2NUM(history_length);
 }
 
 static VALUE
-hist_empty_p(self)
-    VALUE self;
+hist_empty_p(VALUE self)
 {
     rb_secure(4);
     return history_length == 0 ? Qtrue : Qfalse;
 }
 
 static VALUE
-hist_delete_at(self, index)
-    VALUE self;
-    VALUE index;
+hist_delete_at(VALUE self, VALUE index)
 {
     int i;
 
@@ -688,9 +662,7 @@ hist_delete_at(self, index)
 }
 
 static VALUE
-filename_completion_proc_call(self, str)
-    VALUE self;
-    VALUE str;
+filename_completion_proc_call(VALUE self, VALUE str)
 {
     VALUE result;
     char **matches;
@@ -705,7 +677,7 @@ filename_completion_proc_call(self, str)
 	    free(matches[i]);
 	}
 	free(matches);
-	if (RARRAY(result)->len >= 2)
+	if (RARRAY_LEN(result) >= 2)
 	    rb_ary_shift(result);
     }
     else {
@@ -715,9 +687,7 @@ filename_completion_proc_call(self, str)
 }
 
 static VALUE
-username_completion_proc_call(self, str)
-    VALUE self;
-    VALUE str;
+username_completion_proc_call(VALUE self, VALUE str)
 {
     VALUE result;
     char **matches;
@@ -732,7 +702,7 @@ username_completion_proc_call(self, str)
 	    free(matches[i]);
 	}
 	free(matches);
-	if (RARRAY(result)->len >= 2)
+	if (RARRAY_LEN(result) >= 2)
 	    rb_ary_shift(result);
     }
     else {
