@@ -8,11 +8,11 @@
  */
 
 #ifndef lint
-/*char sdbm_rcsid[] = "$Id: _sdbm.c,v 1.7 2006/02/13 04:53:19 matz Exp $";*/
+/*char sdbm_rcsid[] = "$Id: _sdbm.c 12799 2007-07-15 13:24:39Z nobu $";*/
 #endif
 
 #include "sdbm.h"
-#include "config.h"
+#include "ruby/config.h"
 
 /*
  * sdbm - ndbm work-alike hashed database library
@@ -71,7 +71,7 @@ static int   duppair proto((char *, datum));
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef MSDOS
+#ifdef DOSISH
 #include <io.h>
 #endif
 #include <sys/types.h>
@@ -598,7 +598,7 @@ getnext(register DBM *db)
  */
 
 #ifndef lint
-/*char pair_rcsid[] = "$Id: _sdbm.c,v 1.7 2006/02/13 04:53:19 matz Exp $";*/
+/*char pair_rcsid[] = "$Id: _sdbm.c 12799 2007-07-15 13:24:39Z nobu $";*/
 #endif
 
 #ifndef BSD42
