@@ -2,8 +2,7 @@
 
   sdbminit.c -
 
-  $Author: shyouhei $
-  $Date: 2007-02-13 08:01:19 +0900 (火, 13  2月 2007) $
+  $Author: ko1 $
   created at: Fri May  7 08:34:24 JST 1999
 
   Copyright (C) 1995-2001 Yukihiro Matsumoto
@@ -45,7 +44,7 @@ free_sdbm(struct dbmdata *dbmp)
 {
 
     if (dbmp->di_dbm) sdbm_close(dbmp->di_dbm);
-    free(dbmp);
+    ruby_xfree(dbmp);
 }
 
 static VALUE
@@ -487,6 +486,8 @@ fsdbm_each_value(VALUE obj)
     struct dbmdata *dbmp;
     DBM *dbm;
 
+    RETURN_ENUMERATOR(obj, 0, 0);
+
     GetDBM2(obj, dbmp, dbm);
     for (key = sdbm_firstkey(dbm); key.dptr; key = sdbm_nextkey(dbm)) {
 	val = sdbm_fetch(dbm, key);
@@ -503,6 +504,8 @@ fsdbm_each_key(VALUE obj)
     struct dbmdata *dbmp;
     DBM *dbm;
 
+    RETURN_ENUMERATOR(obj, 0, 0);
+
     GetDBM2(obj, dbmp, dbm);
     for (key = sdbm_firstkey(dbm); key.dptr; key = sdbm_nextkey(dbm)) {
 	rb_yield(rb_tainted_str_new(key.dptr, key.dsize));
@@ -519,6 +522,8 @@ fsdbm_each_pair(VALUE obj)
     struct dbmdata *dbmp;
     VALUE keystr, valstr;
 
+    RETURN_ENUMERATOR(obj, 0, 0);
+
     GetDBM2(obj, dbmp, dbm);
     for (key = sdbm_firstkey(dbm); key.dptr; key = sdbm_nextkey(dbm)) {
 	val = sdbm_fetch(dbm, key);
