@@ -2,8 +2,7 @@
 
   socket.c -
 
-  $Author: matz $
-  $Date: 2007-08-25 12:29:39 +0900 (土, 25  8月 2007) $
+  $Author: nobu $
   created at: Thu Mar 31 12:21:29 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -25,6 +24,10 @@
 #include <sys/uio.h>
 #endif
 
+#ifdef HAVE_XTI_H
+#include <xti.h>
+#endif
+
 #ifndef _WIN32
 #if defined(__BEOS__)
 # include <net/socket.h>
@@ -737,7 +740,7 @@ bsock_recv(int argc, VALUE *argv, VALUE sock)
  * to _recv_nonblock_ fails. 
  *
  * BasicSocket#recv_nonblock may raise any error corresponding to recvfrom(2) failure,
- * including Errno::EAGAIN.
+ * including Errno::EWOULDBLOCK.
  *
  * === See
  * * Socket#recvfrom
@@ -763,9 +766,9 @@ bsock_do_not_rev_lookup_set(VALUE self, VALUE val)
     return val;
 }
 
-NORETURN(static void raise_socket_error(char *, int));
+NORETURN(static void raise_socket_error(const char *, int));
 static void
-raise_socket_error(char *reason, int error)
+raise_socket_error(const char *reason, int error)
 {
 #ifdef EAI_SYSTEM
     if (error == EAI_SYSTEM) rb_sys_fail(reason);
@@ -813,7 +816,7 @@ str_isnumber(const char *p)
     if (!p || *p == '\0')
        return 0;
     ep = NULL;
-    (void)strtoul(p, &ep, 10);
+    (void)STRTOUL(p, &ep, 10);
     if (ep && *ep == '\0')
        return 1;
     else
@@ -1236,7 +1239,7 @@ init_inetsock_internal(struct inetsock_arg *arg)
     int type = arg->type;
     struct addrinfo *res;
     int fd, status = 0;
-    char *syscall;
+    const char *syscall;
 
     arg->remote.res = sock_addrinfo(arg->remote.host, arg->remote.serv, SOCK_STREAM,
 				    (type == INET_SERVER) ? AI_PASSIVE : 0);
@@ -1454,6 +1457,24 @@ tcp_svr_init(int argc, VALUE *argv, VALUE sock)
 	return init_inetsock(sock, Qnil, arg1, Qnil, Qnil, INET_SERVER);
 }
 
+static void
+make_fd_nonblock(int fd)
+{
+    int flags;
+#ifdef F_GETFL
+    flags = fcntl(fd, F_GETFL);
+    if (flags == -1) {
+        rb_sys_fail(0);
+    }
+#else
+    flags = 0;
+#endif
+    flags |= O_NONBLOCK;
+    if (fcntl(fd, F_SETFL, flags) == -1) {
+        rb_sys_fail(0);
+    }
+}
+
 static VALUE
 s_accept_nonblock(VALUE klass, rb_io_t *fptr, struct sockaddr *sockaddr, socklen_t *len)
 {
@@ -1465,6 +1486,7 @@ s_accept_nonblock(VALUE klass, rb_io_t *fptr, struct sockaddr *sockaddr, socklen
     if (fd2 < 0) {
         rb_sys_fail("accept(2)");
     }
+    make_fd_nonblock(fd2);
     return init_sock(rb_obj_alloc(klass), fd2);
 }
 
@@ -1529,7 +1551,7 @@ tcp_accept(VALUE sock)
  * 	serv = TCPServer.new(2202)
  * 	begin
  * 	  sock = serv.accept_nonblock
- * 	rescue Errno::EAGAIN, Errno::ECONNABORTED, Errno::EPROTO, Errno::EINTR
+ * 	rescue Errno::EAGAIN, Errno::EWOULDBLOCK, Errno::ECONNABORTED, Errno::EPROTO, Errno::EINTR
  * 	  IO.select([serv])
  * 	  retry
  * 	end
@@ -1539,7 +1561,7 @@ tcp_accept(VALUE sock)
  * to TCPServer#accept_nonblock fails. 
  *
  * TCPServer#accept_nonblock may raise any error corresponding to accept(2) failure,
- * including Errno::EAGAIN.
+ * including Errno::EWOULDBLOCK.
  * 
  * === See
  * * TCPServer#accept
@@ -1830,7 +1852,7 @@ udp_send(int argc, VALUE *argv, VALUE sock)
  * to _recvfrom_nonblock_ fails. 
  *
  * UDPSocket#recvfrom_nonblock may raise any error corresponding to recvfrom(2) failure,
- * including Errno::EAGAIN.
+ * including Errno::EWOULDBLOCK.
  *
  * === See
  * * Socket#recvfrom
@@ -1848,7 +1870,7 @@ unix_init(VALUE sock, VALUE path)
     return init_unixsock(sock, path, 0);
 }
 
-static char*
+static const char*
 unixpath(struct sockaddr_un *sockaddr, socklen_t len)
 {
     if (sockaddr->sun_path < (char*)sockaddr + len)
@@ -2093,7 +2115,7 @@ unix_accept(VALUE sock)
  * 	serv = UNIXServer.new("/tmp/sock")
  * 	begin
  * 	  sock = serv.accept_nonblock
- * 	rescue Errno::EAGAIN, Errno::ECONNABORTED, Errno::EPROTO, Errno::EINTR
+ * 	rescue Errno::EAGAIN, Errno::EWOULDBLOCK, Errno::ECONNABORTED, Errno::EPROTO, Errno::EINTR
  * 	  IO.select([serv])
  * 	  retry
  * 	end
@@ -2103,7 +2125,7 @@ unix_accept(VALUE sock)
  * to UNIXServer#accept_nonblock fails. 
  *
  * UNIXServer#accept_nonblock may raise any error corresponding to accept(2) failure,
- * including Errno::EAGAIN.
+ * including Errno::EWOULDBLOCK.
  * 
  * === See
  * * UNIXServer#accept
@@ -2822,7 +2844,7 @@ sock_recvfrom(int argc, VALUE *argv, VALUE sock)
  * 	client, client_sockaddr = socket.accept
  * 	begin
  * 	  pair = client.recvfrom_nonblock(20)
- * 	rescue Errno::EAGAIN
+ * 	rescue Errno::EAGAIN, Errno::EWOULDBLOCK
  * 	  IO.select([client])
  * 	  retry
  * 	end
@@ -2844,7 +2866,7 @@ sock_recvfrom(int argc, VALUE *argv, VALUE sock)
  * to _recvfrom_nonblock_ fails. 
  *
  * Socket#recvfrom_nonblock may raise any error corresponding to recvfrom(2) failure,
- * including Errno::EAGAIN.
+ * including Errno::EWOULDBLOCK.
  *
  * === See
  * * Socket#recvfrom
@@ -2890,7 +2912,7 @@ sock_accept(VALUE sock)
  * 	socket.listen(5)
  * 	begin
  * 	  client_socket, client_sockaddr = socket.accept_nonblock
- * 	rescue Errno::EAGAIN, Errno::ECONNABORTED, Errno::EPROTO, Errno::EINTR
+ * 	rescue Errno::EAGAIN, Errno::EWOULDBLOCK, Errno::ECONNABORTED, Errno::EPROTO, Errno::EINTR
  * 	  IO.select([socket])
  * 	  retry
  * 	end
@@ -2912,7 +2934,7 @@ sock_accept(VALUE sock)
  * to _accept_nonblock_ fails. 
  *
  * Socket#accept_nonblock may raise any error corresponding to accept(2) failure,
- * including Errno::EAGAIN.
+ * including Errno::EWOULDBLOCK.
  * 
  * === See
  * * Socket#accept
@@ -3142,7 +3164,7 @@ sock_s_getservbyname(int argc, VALUE *argv)
 	char *s = RSTRING_PTR(service);
 	char *end;
 
-	port = strtoul(s, &end, 0);
+	port = STRTOUL(s, &end, 0);
 	if (*end != '\0') {
 	    rb_raise(rb_eSocket, "no such service %s/%s", s, RSTRING_PTR(proto));
 	}
@@ -3440,7 +3462,7 @@ static VALUE
 sock_s_unpack_sockaddr_un(VALUE self, VALUE addr)
 {
     struct sockaddr_un * sockaddr;
-    char *sun_path;
+    const char *sun_path;
     VALUE path;
 
     sockaddr = (struct sockaddr_un*)StringValuePtr(addr);
@@ -3466,7 +3488,7 @@ sock_s_unpack_sockaddr_un(VALUE self, VALUE addr)
 static VALUE mConst;
 
 static void
-sock_define_const(char *name, int value)
+sock_define_const(const char *name, int value)
 {
     rb_define_const(rb_cSocket, name, INT2FIX(value));
     rb_define_const(mConst, name, INT2FIX(value));
