@@ -2,13 +2,13 @@
 /*
  * rubyext.c
  *
- * $Author: matz $
- * $Date: 2007-12-22 21:14:37 +0900 (Sat, 22 Dec 2007) $
+ * $Author: naruse $
  *
  * Copyright (C) 2003-2005 why the lucky stiff
  */
 
 #include "ruby/ruby.h"
+#include "ruby/encoding.h"
 #include "syck.h"
 #include <sys/types.h>
 #include <time.h>
@@ -512,6 +512,7 @@ yaml_org_handler( SyckNode *n, VALUE *ref )
             else if ( strcmp( type_id, "str" ) == 0 )
             {
                 obj = rb_str_new( n->data.str->ptr, n->data.str->len );
+                rb_enc_associate(obj, rb_utf8_encoding());
             }
             else
             {
@@ -885,7 +886,6 @@ syck_parser_set_resolver(VALUE self, VALUE resolver)
 static VALUE
 syck_resolver_initialize(VALUE self)
 {
-    VALUE tags = rb_hash_new();
     rb_ivar_set(self, s_tags, rb_hash_new());
     return self;
 }
@@ -917,7 +917,6 @@ syck_resolver_use_types_at(VALUE self, VALUE hsh)
 VALUE
 syck_resolver_detect_implicit(VALUE self, VALUE val)
 {
-    char *type_id;
     return rb_str_new2( "" );
 }
 
@@ -1240,6 +1239,7 @@ syck_genericresolver_node_import(VALUE self, VALUE node)
         case syck_str_kind:
         {
             v = rb_str_new( n->data.str->ptr, n->data.str->len );
+	    rb_enc_associate(v, rb_utf8_encoding());
             if ( n->data.str->style == scalar_1quote )
             {
                 style = sym_1quote;
@@ -1947,7 +1947,6 @@ VALUE
 syck_emitter_emit(int argc, VALUE *argv, VALUE self)
 {
     VALUE oid, proc;
-    char *anchor_name;
     SyckEmitter *emitter;
     struct emitter_xtra *bonus;
     SYMID symple;
