@@ -4,11 +4,11 @@
  * <amos+ruby@utdallas.edu>
  *
  * $RoughId: syslog.c,v 1.21 2002/02/25 12:21:17 knu Exp $
- * $Id: syslog.c,v 1.9 2004/03/29 07:54:25 matz Exp $
+ * $Id: syslog.c 12501 2007-06-10 03:06:15Z nobu $
  */
 
-#include "ruby.h"
-#include "util.h"
+#include "ruby/ruby.h"
+#include "ruby/util.h"
 #include <syslog.h>
 
 /* Syslog class */
@@ -32,7 +32,7 @@ static void syslog_write(int pri, int argc, VALUE *argv)
 
     str = rb_f_sprintf(argc, argv);
 
-    syslog(pri, "%s", RSTRING(str)->ptr);
+    syslog(pri, "%s", RSTRING_PTR(str));
 }
 
 /* Syslog module methods */
@@ -70,7 +70,7 @@ static VALUE mSyslog_open(int argc, VALUE *argv, VALUE self)
 #else
     Check_SafeStr(ident);
 #endif
-    syslog_ident = strdup(RSTRING(ident)->ptr);
+    syslog_ident = strdup(RSTRING_PTR(ident));
 
     if (NIL_P(opt)) {
 	syslog_options = LOG_PID | LOG_CONS;
