@@ -3,13 +3,15 @@
   id.h - 
 
   $Author: ko1 $
-  $Date: $
   created at: Thu Jul 12 04:38:07 2007
 
   Copyright (C) 2007 Koichi Sasada
 
 **********************************************************************/
 
+#ifndef RUBY_ID_H
+#define RUBY_ID_H
+
 extern VALUE symIFUNC;
 extern VALUE symCFUNC;
 
@@ -48,5 +50,6 @@ extern ID idBitblt;
 extern ID idAnswer;
 extern ID idSend;
 extern ID id__send__;
-
-
+extern ID idRespond_to;
+extern ID idInitialize;
+#endif /* RUBY_ID_H */
