@@ -2,8 +2,7 @@
 
   defines.h -
 
-  $Author: matz $
-  $Date: 2007-08-29 11:15:03 +0900 (水, 29  8月 2007) $
+  $Author: nobu $
   created at: Wed May 18 00:21:44 JST 1994
 
 ************************************************/
@@ -64,6 +63,11 @@ void *xrealloc(void*,size_t);
 void *xrealloc2(void*,size_t,size_t);
 void xfree(void*);
 
+#define STRINGIZE(expr) STRINGIZE0(expr)
+#ifndef STRINGIZE0
+#define STRINGIZE0(expr) #expr
+#endif
+
 #if SIZEOF_LONG_LONG > 0
 # define LONG_LONG long long
 #elif SIZEOF___INT64 > 0
@@ -106,12 +110,13 @@ void xfree(void*);
 #endif
 #endif
 
-/* define RUBY_USE_EUC/SJIS for default kanji-code */
-#ifndef DEFAULT_KCODE
-#if defined(DOSISH) || defined(__CYGWIN__) || defined(__MACOS__) || defined(OS2)
-#define DEFAULT_KCODE KCODE_SJIS
-#else
-#define DEFAULT_KCODE KCODE_EUC
+#if defined(__NeXT__) || defined(__APPLE__)
+/* Do not trust WORDS_BIGENDIAN from configure since -arch compiler flag may
+   result in a different endian.  Instead trust __BIG_ENDIAN__ and
+   __LITTLE_ENDIAN__ which are set correctly by -arch. */
+#undef WORDS_BIGENDIAN
+#ifdef __BIG_ENDIAN__
+#define WORDS_BIGENDIAN
 #endif
 #endif
 
@@ -168,13 +173,6 @@ void xfree(void*);
 #ifndef S_ISREG
 #define S_ISREG(mode)  (((mode) & (0170000)) == (0100000))
 #endif
-/* Do not trust WORDS_BIGENDIAN from configure since -arch compiler flag may
-   result in a different endian.  Instead trust __BIG_ENDIAN__ and
-   __LITTLE_ENDIAN__ which are set correctly by -arch. */
-#undef WORDS_BIGENDIAN
-#ifdef __BIG_ENDIAN__
-#define WORDS_BIGENDIAN
-#endif
 #ifndef __APPLE__
 /* NextStep, OpenStep (but not Rhapsody) */
 #ifndef GETPGRP_VOID
@@ -264,6 +262,14 @@ void rb_ia64_flushrs(void);
 #define ENV_IGNORECASE
 #endif
 
+#ifndef CASEFOLD_FILESYSTEM
+# if defined DOSISH || defined __VMS
+#   define CASEFOLD_FILESYSTEM 1
+# else
+#   define CASEFOLD_FILESYSTEM 0
+# endif
+#endif
+
 #ifndef DLEXT_MAXLEN
 #define DLEXT_MAXLEN 4
 #endif
