@@ -2,8 +2,7 @@
 
   rubyio.h -
 
-  $Author: matz $
-  $Date: 2007-08-25 12:29:39 +0900 (土, 25  8月 2007) $
+  $Author: akr $
   created at: Fri Nov 12 16:47:09 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -45,7 +44,9 @@ typedef struct rb_io_t {
     int rbuf_off;
     int rbuf_len;
     int rbuf_capa;
+    VALUE tied_io_for_writing;
     rb_encoding *enc;
+    rb_encoding *enc2;
 } rb_io_t;
 
 #define HAVE_RB_IO_T 1
@@ -88,6 +89,9 @@ typedef struct rb_io_t {
     fp->rbuf_off = 0;\
     fp->rbuf_len = 0;\
     fp->rbuf_capa = 0;\
+    fp->tied_io_for_writing = 0;\
+    fp->enc = 0;\
+    fp->enc2 = 0;\
 } while (0)
 
 FILE *rb_io_stdio_file(rb_io_t *fptr);
