@@ -4,7 +4,6 @@
   	      for missing timeval struct
 
   $Author: akr $
-  $Date: 2007-11-19 18:09:38 +0900 (Mon, 19 Nov 2007) $
   created at: Sat May 11 23:46:03 JST 2002
 
 ************************************************/
@@ -80,6 +79,18 @@ extern double erf(double);
 extern double erfc(double);
 #endif
 
+#ifndef HAVE_TGAMMA
+extern double tgamma(double);
+#endif
+
+#ifndef HAVE_LGAMMA_R
+extern double lgamma_r(double, int *);
+#endif
+
+#ifndef HAVE_CBRT
+extern double cbrt(double);
+#endif
+
 #ifndef isinf
 # ifndef HAVE_ISINF
 #  if defined(HAVE_FINITE) && defined(HAVE_ISNAN)
@@ -110,14 +121,6 @@ extern double modf(double, double *);
 #endif
 */
 
-#ifndef HAVE_STRCASECMP
-extern int strcasecmp(const char *, const char *);
-#endif
-
-#ifndef HAVE_STRNCASECMP
-extern int strncasecmp(const char *, const char *, size_t);
-#endif
-
 #ifndef HAVE_STRCHR
 extern char *strchr(const char *, int);
 extern char *strrchr(const char *, int);
@@ -141,10 +144,6 @@ extern long strtol(const char *, char **, int);
 #endif
 */
 
-#ifndef HAVE_STRTOUL
-extern unsigned long strtoul(const char *, char **, int);
-#endif
-
 #ifndef HAVE_VSNPRINTF
 # include <stdarg.h>
 extern int snprintf(char *, size_t n, char const *, ...);
