@@ -1,18 +1,18 @@
 /**********************************************************************
 
-  ruby.h -
+  ruby/ruby.h -
 
   $Author: akr $
   created at: Thu Jun 10 14:26:32 JST 1993
 
-  Copyright (C) 1993-2007 Yukihiro Matsumoto
+  Copyright (C) 1993-2008 Yukihiro Matsumoto
   Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
   Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
 **********************************************************************/
 
-#ifndef RUBY_H
-#define RUBY_H 1
+#ifndef RUBY_RUBY_H
+#define RUBY_RUBY_H 1
 
 #if defined(__cplusplus)
 extern "C" {
@@ -21,10 +21,12 @@ extern "C" {
 #endif
 #endif
 
+#ifndef RUBY_LIB
 #include "ruby/config.h"
 #ifdef RUBY_EXTCONF_H
 #include RUBY_EXTCONF_H
 #endif
+#endif
 
 #define NORETURN_STYLE_NEW 1
 #ifndef NORETURN
@@ -58,25 +60,15 @@ extern "C" {
 # include <intrinsics.h>
 #endif
 
+#ifdef HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
+
 #include <stddef.h>
 #include <stdio.h>
 
 #include "defines.h"
 
-/* need to include <ctype.h> to use these macros */
-#ifndef ISPRINT
-#define ISASCII(c) isascii((int)(unsigned char)(c))
-#undef ISPRINT
-#define ISPRINT(c) (ISASCII(c) && isprint((int)(unsigned char)(c)))
-#define ISSPACE(c) (ISASCII(c) && isspace((int)(unsigned char)(c)))
-#define ISUPPER(c) (ISASCII(c) && isupper((int)(unsigned char)(c)))
-#define ISLOWER(c) (ISASCII(c) && islower((int)(unsigned char)(c)))
-#define ISALNUM(c) (ISASCII(c) && isalnum((int)(unsigned char)(c)))
-#define ISALPHA(c) (ISASCII(c) && isalpha((int)(unsigned char)(c)))
-#define ISDIGIT(c) (ISASCII(c) && isdigit((int)(unsigned char)(c)))
-#define ISXDIGIT(c) (ISASCII(c) && isxdigit((int)(unsigned char)(c)))
-#endif
-
 #if defined(HAVE_ALLOCA_H)
 #include <alloca.h>
 #else
@@ -95,12 +87,24 @@ typedef unsigned long VALUE;
 typedef unsigned long ID;
 # define SIGNED_VALUE long
 # define SIZEOF_VALUE SIZEOF_LONG
+# define PRIdVALUE "ld"
+# define PRIiVALUE "li"
+# define PRIoVALUE "lo"
+# define PRIuVALUE "lu"
+# define PRIxVALUE "lx"
+# define PRIXVALUE "lX"
 #elif SIZEOF_LONG_LONG == SIZEOF_VOIDP
 typedef unsigned LONG_LONG VALUE;
 typedef unsigned LONG_LONG ID;
 # define SIGNED_VALUE LONG_LONG
 # define LONG_LONG_VALUE 1
 # define SIZEOF_VALUE SIZEOF_LONG_LONG
+# define PRIdVALUE "lld"
+# define PRIiVALUE "lli"
+# define PRIoVALUE "llo"
+# define PRIuVALUE "llu"
+# define PRIxVALUE "llx"
+# define PRIXVALUE "llX"
 #else
 # error ---->> ruby requires sizeof(void*) == sizeof(long) to be compiled. <<----
 #endif
@@ -181,6 +185,14 @@ VALUE rb_ull2inum(unsigned LONG_LONG);
 # define OFFT2NUM(v) INT2NUM(v)
 #endif
 
+#if SIZEOF_SIZE_T > SIZEOF_LONG && defined(HAVE_LONG_LONG)
+# define SIZET2NUM(v) ULL2NUM(v)
+#elif SIZEOF_SIZE_T == SIZEOF_LONG
+# define SIZET2NUM(v) ULONG2NUM(v)
+#else
+# define SIZET2NUM(v) UINT2NUM(v)
+#endif
+
 #ifndef PIDT2NUM
 #define PIDT2NUM(v) LONG2NUM(v)
 #endif
@@ -203,7 +215,7 @@ VALUE rb_ull2inum(unsigned LONG_LONG);
 #define FIX2LONG(x) RSHIFT((SIGNED_VALUE)x,1)
 #define FIX2ULONG(x) ((((VALUE)(x))>>1)&LONG_MAX)
 #define FIXNUM_P(f) (((SIGNED_VALUE)(f))&FIXNUM_FLAG)
-#define POSFIXABLE(f) ((f) <= FIXNUM_MAX)
+#define POSFIXABLE(f) ((f) < FIXNUM_MAX+1)
 #define NEGFIXABLE(f) ((f) >= FIXNUM_MIN)
 #define FIXABLE(f) (POSFIXABLE(f) && NEGFIXABLE(f))
 
@@ -241,61 +253,62 @@ enum ruby_special_consts {
 
 enum ruby_value_type {
     RUBY_T_NONE   = 0x00,
-#define T_NONE   RUBY_T_NONE
 
-    RUBY_T_NIL    = 0x01,
-#define T_NIL    RUBY_T_NIL
-    RUBY_T_OBJECT = 0x02,
-#define T_OBJECT RUBY_T_OBJECT
-    RUBY_T_CLASS  = 0x03,
-#define T_CLASS  RUBY_T_CLASS
-    RUBY_T_ICLASS = 0x04,
-#define T_ICLASS RUBY_T_ICLASS
-    RUBY_T_MODULE = 0x05,
-#define T_MODULE RUBY_T_MODULE
-    RUBY_T_FLOAT  = 0x06,
-#define T_FLOAT  RUBY_T_FLOAT
-    RUBY_T_STRING = 0x07,
-#define T_STRING RUBY_T_STRING
-    RUBY_T_REGEXP = 0x08,
-#define T_REGEXP RUBY_T_REGEXP
-    RUBY_T_ARRAY  = 0x09,
-#define T_ARRAY  RUBY_T_ARRAY
-    RUBY_T_FIXNUM = 0x0a,
-#define T_FIXNUM RUBY_T_FIXNUM
-    RUBY_T_HASH   = 0x0b,
-#define T_HASH   RUBY_T_HASH
-    RUBY_T_STRUCT = 0x0c,
-#define T_STRUCT RUBY_T_STRUCT
-    RUBY_T_BIGNUM = 0x0d,
-#define T_BIGNUM RUBY_T_BIGNUM
-    RUBY_T_FILE   = 0x0e,
-#define T_FILE   RUBY_T_FILE
+    RUBY_T_OBJECT = 0x01,
+    RUBY_T_CLASS  = 0x02,
+    RUBY_T_MODULE = 0x03,
+    RUBY_T_FLOAT  = 0x04,
+    RUBY_T_STRING = 0x05,
+    RUBY_T_REGEXP = 0x06,
+    RUBY_T_ARRAY  = 0x07,
+    RUBY_T_HASH   = 0x08,
+    RUBY_T_STRUCT = 0x09,
+    RUBY_T_BIGNUM = 0x0a,
+    RUBY_T_FILE   = 0x0b,
+    RUBY_T_DATA   = 0x0c,
+    RUBY_T_MATCH  = 0x0d,
+    RUBY_T_COMPLEX  = 0x0e,
+    RUBY_T_RATIONAL = 0x0f,
 
-    RUBY_T_TRUE   = 0x10,
-#define T_TRUE   RUBY_T_TRUE
-    RUBY_T_FALSE  = 0x11,
-#define T_FALSE  RUBY_T_FALSE
-    RUBY_T_DATA   = 0x12,
-#define T_DATA   RUBY_T_DATA
-    RUBY_T_MATCH  = 0x13,
-#define T_MATCH  RUBY_T_MATCH
+    RUBY_T_NIL    = 0x11,
+    RUBY_T_TRUE   = 0x12,
+    RUBY_T_FALSE  = 0x13,
     RUBY_T_SYMBOL = 0x14,
-#define T_SYMBOL RUBY_T_SYMBOL
+    RUBY_T_FIXNUM = 0x15,
 
-    RUBY_T_VALUES = 0x1a,
-#define T_VALUES RUBY_T_VALUES
-    RUBY_T_BLOCK  = 0x1b,
-#define T_BLOCK  RUBY_T_BLOCK
-    RUBY_T_UNDEF  = 0x1c,
-#define T_UNDEF  RUBY_T_UNDEF
-    RUBY_T_NODE   = 0x1f,
-#define T_NODE   RUBY_T_NODE
+    RUBY_T_UNDEF  = 0x1b,
+    RUBY_T_NODE   = 0x1c,
+    RUBY_T_ICLASS = 0x1d,
 
     RUBY_T_MASK   = 0x1f,
-#define T_MASK   RUBY_T_MASK
 };
 
+#define T_NONE   RUBY_T_NONE
+#define T_NIL    RUBY_T_NIL
+#define T_OBJECT RUBY_T_OBJECT
+#define T_CLASS  RUBY_T_CLASS
+#define T_ICLASS RUBY_T_ICLASS
+#define T_MODULE RUBY_T_MODULE
+#define T_FLOAT  RUBY_T_FLOAT
+#define T_STRING RUBY_T_STRING
+#define T_REGEXP RUBY_T_REGEXP
+#define T_ARRAY  RUBY_T_ARRAY
+#define T_HASH   RUBY_T_HASH
+#define T_STRUCT RUBY_T_STRUCT
+#define T_BIGNUM RUBY_T_BIGNUM
+#define T_FILE   RUBY_T_FILE
+#define T_FIXNUM RUBY_T_FIXNUM
+#define T_TRUE   RUBY_T_TRUE
+#define T_FALSE  RUBY_T_FALSE
+#define T_DATA   RUBY_T_DATA
+#define T_MATCH  RUBY_T_MATCH
+#define T_SYMBOL RUBY_T_SYMBOL
+#define T_RATIONAL RUBY_T_RATIONAL
+#define T_COMPLEX RUBY_T_COMPLEX
+#define T_UNDEF  RUBY_T_UNDEF
+#define T_NODE   RUBY_T_NODE
+#define T_MASK   RUBY_T_MASK
+
 #define BUILTIN_TYPE(x) (((struct RBasic*)(x))->flags & T_MASK)
 
 #define TYPE(x) rb_type((VALUE)(x))
@@ -326,6 +339,9 @@ void rb_check_safe_str(VALUE);
 VALUE rb_get_path(VALUE);
 #define FilePathValue(v) ((v) = rb_get_path(v))
 
+VALUE rb_get_path_no_checksafe(VALUE);
+#define FilePathStringValue(v) ((v) = rb_get_path_no_checksafe(v))
+
 void rb_secure(int);
 int rb_safe_level(void);
 void rb_set_safe_level(int);
@@ -368,6 +384,12 @@ unsigned LONG_LONG rb_num2ull(VALUE);
 # define NUM2OFFT(x) NUM2LONG(x)
 #endif
 
+#if defined(HAVE_LONG_LONG) && SIZEOF_SIZE_T > SIZEOF_LONG
+# define NUM2SIZET(x) ((size_t)NUM2ULL(x))
+#else
+# define NUM2SIZET(x) NUM2ULONG(x)
+#endif
+
 double rb_num2dbl(VALUE);
 #define NUM2DBL(x) rb_num2dbl((VALUE)(x))
 
@@ -402,29 +424,57 @@ struct RBasic {
     VALUE klass;
 };
 
+#define ROBJECT_EMBED_LEN_MAX 3
 struct RObject {
     struct RBasic basic;
-    struct st_table *iv_tbl;
+    union {
+	struct {
+	    long numiv;
+	    VALUE *ivptr;
+            struct st_table *iv_index_tbl; /* shortcut for RCLASS_IV_INDEX_TBL(rb_obj_class(obj)) */
+	} heap;
+	VALUE ary[ROBJECT_EMBED_LEN_MAX];
+    } as;
 };
+#define ROBJECT_EMBED FL_USER1
+#define ROBJECT_NUMIV(o) \
+    ((RBASIC(o)->flags & ROBJECT_EMBED) ? \
+     ROBJECT_EMBED_LEN_MAX : \
+     ROBJECT(o)->as.heap.numiv)
+#define ROBJECT_IVPTR(o) \
+    ((RBASIC(o)->flags & ROBJECT_EMBED) ? \
+     ROBJECT(o)->as.ary : \
+     ROBJECT(o)->as.heap.ivptr)
+#define ROBJECT_IV_INDEX_TBL(o) \
+    ((RBASIC(o)->flags & ROBJECT_EMBED) ? \
+     RCLASS_IV_INDEX_TBL(rb_obj_class(o)) : \
+     ROBJECT(o)->as.heap.iv_index_tbl)
 
-struct RValues {
-    struct RBasic basic;
-    VALUE v1;
-    VALUE v2;
-    VALUE v3;
-};
+typedef struct {
+    VALUE super;
+    struct st_table *iv_tbl;
+} rb_classext_t;
 
 struct RClass {
     struct RBasic basic;
-    struct st_table *iv_tbl;
+    rb_classext_t *ptr;
     struct st_table *m_tbl;
-    VALUE super;
+    struct st_table *iv_index_tbl;
 };
+#define RCLASS_IV_TBL(c) (RCLASS(c)->ptr->iv_tbl)
+#define RCLASS_M_TBL(c) (RCLASS(c)->m_tbl)
+#define RCLASS_SUPER(c) (RCLASS(c)->ptr->super)
+#define RCLASS_IV_INDEX_TBL(c) (RCLASS(c)->iv_index_tbl)
+#define RMODULE_IV_TBL(m) RCLASS_IV_TBL(m)
+#define RMODULE_M_TBL(m) RCLASS_M_TBL(m)
+#define RMODULE_SUPER(m) RCLASS_SUPER(m)
 
 struct RFloat {
     struct RBasic basic;
-    double value;
+    double float_value;
 };
+#define RFLOAT_VALUE(v) (RFLOAT(v)->float_value)
+#define DOUBLE2NUM(dbl)  rb_float_new(dbl)
 
 #define ELTS_SHARED FL_USER2
 
@@ -494,6 +544,18 @@ struct RFile {
     struct rb_io_t *fptr;
 };
 
+struct RRational {
+    struct RBasic basic;
+    VALUE num;
+    VALUE den;
+};
+
+struct RComplex {
+    struct RBasic basic;
+    VALUE real;
+    VALUE image;
+};
+
 struct RData {
     struct RBasic basic;
     void (*dmark)(void*);
@@ -547,12 +609,39 @@ struct RStruct {
      RSTRUCT(st)->as.ary : \
      RSTRUCT(st)->as.heap.ptr)
 
+#define RBIGNUM_EMBED_LEN_MAX ((sizeof(VALUE)*3)/sizeof(BDIGIT))
 struct RBignum {
     struct RBasic basic;
-    char sign; /* positive:0, negative:1 */
-    long len;
-    void *digits;
+    union {
+        struct {
+            long len;
+            BDIGIT *digits;
+        } heap;
+        BDIGIT ary[RBIGNUM_EMBED_LEN_MAX];
+    } as;
 };
+#define RBIGNUM_SIGN_BIT FL_USER1
+/* sign: positive:1, negative:0 */
+#define RBIGNUM_SIGN(b) ((RBASIC(b)->flags & RBIGNUM_SIGN_BIT) != 0)
+#define RBIGNUM_SET_SIGN(b,sign) \
+  ((sign) ? (RBASIC(b)->flags |= RBIGNUM_SIGN_BIT) \
+          : (RBASIC(b)->flags &= ~RBIGNUM_SIGN_BIT))
+#define RBIGNUM_POSITIVE_P(b) RBIGNUM_SIGN(b)
+#define RBIGNUM_NEGATIVE_P(b) (!RBIGNUM_SIGN(b))
+
+#define RBIGNUM_EMBED_FLAG FL_USER2
+#define RBIGNUM_EMBED_LEN_MASK (FL_USER5|FL_USER4|FL_USER3)
+#define RBIGNUM_EMBED_LEN_SHIFT (FL_USHIFT+3)
+#define RBIGNUM_LEN(b) \
+    ((RBASIC(b)->flags & RBIGNUM_EMBED_FLAG) ? \
+     (long)((RBASIC(b)->flags >> RBIGNUM_EMBED_LEN_SHIFT) & \
+            (RBIGNUM_EMBED_LEN_MASK >> RBIGNUM_EMBED_LEN_SHIFT)) : \
+     RBIGNUM(b)->as.heap.len)
+/* LSB:RBIGNUM_DIGITS(b)[0], MSB:RBIGNUM_DIGITS(b)[RBIGNUM_LEN(b)-1] */
+#define RBIGNUM_DIGITS(b) \
+    ((RBASIC(b)->flags & RBIGNUM_EMBED_FLAG) ? \
+     RBIGNUM(b)->as.ary : \
+     RBIGNUM(b)->as.heap.digits)
 
 #define R_CAST(st)   (struct st*)
 #define RBASIC(obj)  (R_CAST(RBasic)(obj))
@@ -568,70 +657,40 @@ struct RBignum {
 #define RSTRUCT(obj) (R_CAST(RStruct)(obj))
 #define RBIGNUM(obj) (R_CAST(RBignum)(obj))
 #define RFILE(obj)   (R_CAST(RFile)(obj))
-#define RVALUES(obj) (R_CAST(RValues)(obj))
+#define RRATIONAL(obj) (R_CAST(RRational)(obj))
+#define RCOMPLEX(obj) (R_CAST(RComplex)(obj))
 
-enum ruby_value_flags {
-    RUBY_FL_MARK      = (1<<5),
-#define FL_MARK      RUBY_FL_MARK
-    RUBY_FL_RESERVED  = (1<<6)	/* will be used in the future GC */,
-#define FL_RESERVED  RUBY_FL_RESERVED
-    RUBY_FL_FINALIZE  = (1<<7),
-#define FL_FINALIZE  RUBY_FL_FINALIZE
-    RUBY_FL_TAINT     = (1<<8),
-#define FL_TAINT     RUBY_FL_TAINT
-    RUBY_FL_EXIVAR    = (1<<9),
-#define FL_EXIVAR    RUBY_FL_EXIVAR
-    RUBY_FL_FREEZE    = (1<<10),
-#define FL_FREEZE    RUBY_FL_FREEZE
-    RUBY_FL_SINGLETON = (1<<11),
-#define FL_SINGLETON RUBY_FL_SINGLETON
+#define FL_SINGLETON FL_USER0
+#define FL_MARK      (((VALUE)1)<<5)
+#define FL_RESERVED  (((VALUE)1)<<6) /* will be used in the future GC */
+#define FL_FINALIZE  (((VALUE)1)<<7)
+#define FL_TAINT     (((VALUE)1)<<8)
+#define FL_EXIVAR    (((VALUE)1)<<9)
+#define FL_FREEZE    (((VALUE)1)<<10)
 
-    RUBY_FL_USHIFT    = 11,
-#define FL_USHIFT    RUBY_FL_USHIFT
+#define FL_USHIFT    11
 
-    RUBY_FL_USER0     = (1<<(FL_USHIFT+0)),
-#define FL_USER0     RUBY_FL_USER0
-    RUBY_FL_USER1     = (1<<(FL_USHIFT+1)),
-#define FL_USER1     RUBY_FL_USER1
-    RUBY_FL_USER2     = (1<<(FL_USHIFT+2)),
-#define FL_USER2     RUBY_FL_USER2
-    RUBY_FL_USER3     = (1<<(FL_USHIFT+3)),
-#define FL_USER3     RUBY_FL_USER3
-    RUBY_FL_USER4     = (1<<(FL_USHIFT+4)),
-#define FL_USER4     RUBY_FL_USER4
-    RUBY_FL_USER5     = (1<<(FL_USHIFT+5)),
-#define FL_USER5     RUBY_FL_USER5
-    RUBY_FL_USER6     = (1<<(FL_USHIFT+6)),
-#define FL_USER6     RUBY_FL_USER6
-    RUBY_FL_USER7     = (1<<(FL_USHIFT+7)),
-#define FL_USER7     RUBY_FL_USER7
-    RUBY_FL_USER8     = (1<<(FL_USHIFT+8)),
-#define FL_USER8     RUBY_FL_USER8
-    RUBY_FL_USER9     = (1<<(FL_USHIFT+9)),
-#define FL_USER9     RUBY_FL_USER9
-    RUBY_FL_USER10     = (1<<(FL_USHIFT+10)),
-#define FL_USER10     RUBY_FL_USER10
-    RUBY_FL_USER11     = (1<<(FL_USHIFT+11)),
-#define FL_USER11     RUBY_FL_USER11
-    RUBY_FL_USER12     = (1<<(FL_USHIFT+12)),
-#define FL_USER12     RUBY_FL_USER12
-    RUBY_FL_USER13     = (1<<(FL_USHIFT+13)),
-#define FL_USER13     RUBY_FL_USER13
-    RUBY_FL_USER14     = (1<<(FL_USHIFT+14)),
-#define FL_USER14     RUBY_FL_USER14
-    RUBY_FL_USER15     = (1<<(FL_USHIFT+15)),
-#define FL_USER15     RUBY_FL_USER15
-    RUBY_FL_USER16     = (1<<(FL_USHIFT+16)),
-#define FL_USER16     RUBY_FL_USER16
-    RUBY_FL_USER17     = (1<<(FL_USHIFT+17)),
-#define FL_USER17     RUBY_FL_USER17
-    RUBY_FL_USER18     = (1<<(FL_USHIFT+18)),
-#define FL_USER18     RUBY_FL_USER18
-    RUBY_FL_USER19     = (1<<(FL_USHIFT+19)),
-#define FL_USER19     RUBY_FL_USER19
-    RUBY_FL_USER20     = (1<<(FL_USHIFT+20)),
-#define FL_USER20     RUBY_FL_USER20
-};
+#define FL_USER0     (((VALUE)1)<<(FL_USHIFT+0))
+#define FL_USER1     (((VALUE)1)<<(FL_USHIFT+1))
+#define FL_USER2     (((VALUE)1)<<(FL_USHIFT+2))
+#define FL_USER3     (((VALUE)1)<<(FL_USHIFT+3))
+#define FL_USER4     (((VALUE)1)<<(FL_USHIFT+4))
+#define FL_USER5     (((VALUE)1)<<(FL_USHIFT+5))
+#define FL_USER6     (((VALUE)1)<<(FL_USHIFT+6))
+#define FL_USER7     (((VALUE)1)<<(FL_USHIFT+7))
+#define FL_USER8     (((VALUE)1)<<(FL_USHIFT+8))
+#define FL_USER9     (((VALUE)1)<<(FL_USHIFT+9))
+#define FL_USER10    (((VALUE)1)<<(FL_USHIFT+10))
+#define FL_USER11    (((VALUE)1)<<(FL_USHIFT+11))
+#define FL_USER12    (((VALUE)1)<<(FL_USHIFT+12))
+#define FL_USER13    (((VALUE)1)<<(FL_USHIFT+13))
+#define FL_USER14    (((VALUE)1)<<(FL_USHIFT+14))
+#define FL_USER15    (((VALUE)1)<<(FL_USHIFT+15))
+#define FL_USER16    (((VALUE)1)<<(FL_USHIFT+16))
+#define FL_USER17    (((VALUE)1)<<(FL_USHIFT+17))
+#define FL_USER18    (((VALUE)1)<<(FL_USHIFT+18))
+#define FL_USER19    (((VALUE)1)<<(FL_USHIFT+19))
+#define FL_USER20    (((VALUE)1)<<(FL_USHIFT+20))
 
 #define SPECIAL_CONST_P(x) (IMMEDIATE_P(x) || !RTEST(x))
 
@@ -705,8 +764,26 @@ const char *rb_id2name(ID);
 ID rb_to_id(VALUE);
 VALUE rb_id2str(ID);
 
-char *rb_class2name(VALUE);
-char *rb_obj_classname(VALUE);
+#define CONST_ID_CACHE(result, str)			\
+    {							\
+	static ID rb_intern_id_cache;			\
+	if (!rb_intern_id_cache)			\
+	    rb_intern_id_cache = rb_intern2(str, strlen(str));	\
+	result rb_intern_id_cache;			\
+    }
+#define CONST_ID(var, str) \
+    do CONST_ID_CACHE(var =, str) while (0)
+#ifdef __GNUC__
+/* __builtin_constant_p and statement expression is available
+ * since gcc-2.7.2.3 at least. */
+#define rb_intern(str) \
+    (__builtin_constant_p(str) ? \
+        (CONST_ID_CACHE(/**/, str)) : \
+        rb_intern(str))
+#endif
+
+const char *rb_class2name(VALUE);
+const char *rb_obj_classname(VALUE);
 
 void rb_p(VALUE);
 
@@ -726,7 +803,10 @@ VALUE rb_iv_set(VALUE, const char*, VALUE);
 
 VALUE rb_equal(VALUE,VALUE);
 
-RUBY_EXTERN VALUE ruby_verbose, ruby_debug;
+VALUE *rb_ruby_verbose_ptr(void);
+VALUE *rb_ruby_debug_ptr(void);
+#define ruby_verbose (*rb_ruby_verbose_ptr())
+#define ruby_debug   (*rb_ruby_debug_ptr())
 
 PRINTF_ARGS(NORETURN(void rb_raise(VALUE, const char*, ...)), 2, 3);
 PRINTF_ARGS(NORETURN(void rb_fatal(const char*, ...)), 1, 2);
@@ -744,9 +824,12 @@ PRINTF_ARGS(void rb_sys_warning(const char*, ...), 1, 2);
 PRINTF_ARGS(void rb_warn(const char*, ...), 1, 2);
 PRINTF_ARGS(void rb_compile_warn(const char *, int, const char*, ...), 3, 4);
 
+typedef VALUE rb_block_call_func(VALUE, VALUE, int, VALUE*);
+
 VALUE rb_each(VALUE);
 VALUE rb_yield(VALUE);
 VALUE rb_yield_values(int n, ...);
+VALUE rb_yield_values2(int n, const VALUE *argv);
 VALUE rb_yield_splat(VALUE);
 int rb_block_given_p(void);
 void rb_need_block(void);
@@ -756,21 +839,21 @@ VALUE rb_rescue(VALUE(*)(ANYARGS),VALUE,VALUE(*)(ANYARGS),VALUE);
 VALUE rb_rescue2(VALUE(*)(ANYARGS),VALUE,VALUE(*)(ANYARGS),VALUE,...);
 VALUE rb_ensure(VALUE(*)(ANYARGS),VALUE,VALUE(*)(ANYARGS),VALUE);
 VALUE rb_catch(const char*,VALUE(*)(ANYARGS),VALUE);
+VALUE rb_catch_obj(VALUE,VALUE(*)(ANYARGS),VALUE);
 NORETURN(void rb_throw(const char*,VALUE));
+NORETURN(void rb_throw_obj(VALUE,VALUE));
 
 VALUE rb_require(const char*);
 
 #ifdef __ia64
 void ruby_init_stack(VALUE*, void*);
-#define RUBY_INIT_STACK \
-    VALUE variable_in_this_stack_frame; \
-    ruby_init_stack(&variable_in_this_stack_frame, rb_ia64_bsp());
+#define ruby_init_stack(addr) ruby_init_stack(addr, rb_ia64_bsp())
 #else
 void ruby_init_stack(VALUE*);
+#endif
 #define RUBY_INIT_STACK \
     VALUE variable_in_this_stack_frame; \
     ruby_init_stack(&variable_in_this_stack_frame);
-#endif
 void ruby_init(void);
 void *ruby_options(int, char**);
 int ruby_run_node(void *);
@@ -795,6 +878,7 @@ RUBY_EXTERN VALUE rb_cCont;
 RUBY_EXTERN VALUE rb_cDir;
 RUBY_EXTERN VALUE rb_cData;
 RUBY_EXTERN VALUE rb_cFalseClass;
+RUBY_EXTERN VALUE rb_cEnumerator;
 RUBY_EXTERN VALUE rb_cFile;
 RUBY_EXTERN VALUE rb_cFixnum;
 RUBY_EXTERN VALUE rb_cFloat;
@@ -809,6 +893,8 @@ RUBY_EXTERN VALUE rb_cNilClass;
 RUBY_EXTERN VALUE rb_cNumeric;
 RUBY_EXTERN VALUE rb_cProc;
 RUBY_EXTERN VALUE rb_cRange;
+RUBY_EXTERN VALUE rb_cRational;
+RUBY_EXTERN VALUE rb_cComplex;
 RUBY_EXTERN VALUE rb_cRegexp;
 RUBY_EXTERN VALUE rb_cStat;
 RUBY_EXTERN VALUE rb_cString;
@@ -902,9 +988,86 @@ rb_special_const_p(VALUE obj)
 static char *dln_libs_to_be_linked[] = { EXTLIB, 0 };
 #endif
 
+#if (defined(__APPLE__) || defined(__NeXT__)) && defined(__MACH__)
+/* to link startup code with ObjC support */
+#define RUBY_GLOBAL_SETUP static void objcdummyfunction(void) {objc_msgSend();}
+#else
+#define RUBY_GLOBAL_SETUP
+#endif
+
+void ruby_sysinit(int *, char ***);
+
 #define RUBY_VM 1 /* YARV */
 #define HAVE_NATIVETHREAD
-int is_ruby_native_thread(void);
+int ruby_native_thread_p(void);
+
+#define RUBY_EVENT_NONE     0x00
+#define RUBY_EVENT_LINE     0x01
+#define RUBY_EVENT_CLASS    0x02
+#define RUBY_EVENT_END      0x04
+#define RUBY_EVENT_CALL     0x08
+#define RUBY_EVENT_RETURN   0x10
+#define RUBY_EVENT_C_CALL   0x20
+#define RUBY_EVENT_C_RETURN 0x40
+#define RUBY_EVENT_RAISE    0x80
+#define RUBY_EVENT_ALL      0xff
+#define RUBY_EVENT_VM      0x100
+#define RUBY_EVENT_SWITCH  0x200
+
+typedef unsigned int rb_event_flag_t;
+typedef void (*rb_event_hook_func_t)(rb_event_flag_t, VALUE data, VALUE, ID, VALUE klass);
+
+typedef struct rb_event_hook_struct {
+    rb_event_flag_t flag;
+    rb_event_hook_func_t func;
+    VALUE data;
+    struct rb_event_hook_struct *next;
+} rb_event_hook_t;
+
+void rb_add_event_hook(rb_event_hook_func_t func, rb_event_flag_t events,
+		       VALUE data);
+int rb_remove_event_hook(rb_event_hook_func_t func);
+
+/* locale insensitive functions */
+
+#define rb_isascii(c) ((unsigned long)(c) < 128)
+int rb_isalnum(int c);
+int rb_isalpha(int c);
+int rb_isblank(int c);
+int rb_iscntrl(int c);
+int rb_isdigit(int c);
+int rb_isgraph(int c);
+int rb_islower(int c);
+int rb_isprint(int c);
+int rb_ispunct(int c);
+int rb_isspace(int c);
+int rb_isupper(int c);
+int rb_isxdigit(int c);
+int rb_tolower(int c);
+int rb_toupper(int c);
+
+#ifndef ISPRINT
+#define ISASCII(c) rb_isascii((unsigned char)(c))
+#undef ISPRINT
+#define ISPRINT(c) rb_isprint((unsigned char)(c))
+#define ISSPACE(c) rb_isspace((unsigned char)(c))
+#define ISUPPER(c) rb_isupper((unsigned char)(c))
+#define ISLOWER(c) rb_islower((unsigned char)(c))
+#define ISALNUM(c) rb_isalnum((unsigned char)(c))
+#define ISALPHA(c) rb_isalpha((unsigned char)(c))
+#define ISDIGIT(c) rb_isdigit((unsigned char)(c))
+#define ISXDIGIT(c) rb_isxdigit((unsigned char)(c))
+#endif
+#define TOUPPER(c) rb_toupper((unsigned char)(c))
+#define TOLOWER(c) rb_tolower((unsigned char)(c))
+
+int st_strcasecmp(const char *s1, const char *s2);
+int st_strncasecmp(const char *s1, const char *s2, size_t n);
+#define STRCASECMP(s1, s2) (st_strcasecmp(s1, s2))
+#define STRNCASECMP(s1, s2, n) (st_strncasecmp(s1, s2, n))
+
+unsigned long ruby_strtoul(const char *str, char **endptr, int base);
+#define STRTOUL(str, endptr, base) (ruby_strtoul(str, endptr, base))
 
 #if defined(__cplusplus)
 #if 0
@@ -912,5 +1075,4 @@ int is_ruby_native_thread(void);
 #endif
 }  /* extern "C" { */
 #endif
-
-#endif /* RUBY_H */
+#endif /* RUBY_RUBY_H */
