@@ -2,16 +2,15 @@
 
   insnhelper.h - helper macros to implement each instructions
 
-  $Author: ko1 $
-  $Date: 2007-11-14 07:13:04 +0900 (Wed, 14 Nov 2007) $
+  $Author: nobu $
   created at: 04/01/01 15:50:34 JST
 
   Copyright (C) 2004-2007 Koichi Sasada
 
 **********************************************************************/
 
-#ifndef _INSNHELPER_H_INCLUDED_
-#define _INSNHELPER_H_INCLUDED_
+#ifndef RUBY_INSNHELPER_H
+#define RUBY_INSNHELPER_H
 
 #include "ruby/ruby.h"
 #include "ruby/node.h"
@@ -146,4 +145,4 @@
     CALL_METHOD(num, 0, 0, id, rb_method_node(klass, id), recv, CLASS_OF(recv)); \
 } while (0)
 
-#endif /* _INSNHELPER_H_INCLUDED_ */
+#endif /* RUBY_INSNHELPER_H */
