@@ -2,8 +2,7 @@
 
   iseq.c -
 
-  $Author: ko1 $
-  $Date: 2007-08-17 01:25:30 +0900 (金, 17  8月 2007) $
+  $Author: nobu $
   created at: 2006-07-11(Tue) 09:00:03 +0900
 
   Copyright (C) 2006 Koichi Sasada
@@ -82,8 +81,8 @@ iseq_mark(void *ptr)
 	RUBY_MARK_UNLESS_NULL(iseq->filename);
 	RUBY_MARK_UNLESS_NULL((VALUE)iseq->cref_stack);
 	RUBY_MARK_UNLESS_NULL(iseq->klass);
-	RUBY_MARK_UNLESS_NULL((VALUE)iseq->node);
-	RUBY_MARK_UNLESS_NULL(iseq->cached_special_block);
+/* 	RUBY_MARK_UNLESS_NULL((VALUE)iseq->node); */
+/*	RUBY_MARK_UNLESS_NULL(iseq->cached_special_block); */
 
 	if (iseq->compile_data != 0) {
 	    RUBY_MARK_UNLESS_NULL(iseq->compile_data->mark_ary);
@@ -164,9 +163,12 @@ prepare_iseq_build(rb_iseq_t *iseq,
     iseq->arg_rest = -1;
     iseq->arg_block = -1;
     iseq->klass = 0;
-    iseq->special_block_builder = GC_GUARDED_PTR_REF(block_opt);
-    iseq->cached_special_block_builder = 0;
-    iseq->cached_special_block = 0;
+
+    /*
+     * iseq->special_block_builder = GC_GUARDED_PTR_REF(block_opt);
+     * iseq->cached_special_block_builder = 0;
+     * iseq->cached_special_block = 0;
+     */
 
     iseq->compile_data = ALLOC(struct iseq_compile_data);
     MEMZERO(iseq->compile_data, struct iseq_compile_data, 1);
@@ -238,6 +240,10 @@ make_compile_option(rb_compile_option_t *option, VALUE opt)
   { VALUE flag = rb_hash_aref(h, ID2SYM(rb_intern(#mem))); \
       if (flag == Qtrue)  { o->mem = 1; } \
       else if (flag == Qfalse)  { o->mem = 0; } \
+  }
+#define SET_COMPILE_OPTION_NUM(o, h, mem) \
+  { VALUE num = rb_hash_aref(opt, ID2SYM(rb_intern(#mem))); \
+      if (!NIL_P(num)) o->mem = NUM2INT(num); \
   }
 	SET_COMPILE_OPTION(option, opt, inline_const_cache);
 	SET_COMPILE_OPTION(option, opt, peephole_optimization);
@@ -247,7 +253,9 @@ make_compile_option(rb_compile_option_t *option, VALUE opt)
 	SET_COMPILE_OPTION(option, opt, instructions_unification);
 	SET_COMPILE_OPTION(option, opt, stack_caching);
 	SET_COMPILE_OPTION(option, opt, trace_instruction);
+	SET_COMPILE_OPTION_NUM(option, opt, debug_level);
 #undef SET_COMPILE_OPTION
+#undef SET_COMPILE_OPTION_NUM
     }
     else {
 	rb_raise(rb_eTypeError, "Compile option must be Hash/true/false/nil");
@@ -260,6 +268,8 @@ make_compile_option_value(rb_compile_option_t *option)
     VALUE opt = rb_hash_new();
 #define SET_COMPILE_OPTION(o, h, mem) \
   rb_hash_aset(h, ID2SYM(rb_intern(#mem)), o->mem ? Qtrue : Qfalse)
+#define SET_COMPILE_OPTION_NUM(o, h, mem) \
+  rb_hash_aset(h, ID2SYM(rb_intern(#mem)), INT2NUM(o->mem))
     {
 	SET_COMPILE_OPTION(option, opt, inline_const_cache);
 	SET_COMPILE_OPTION(option, opt, peephole_optimization);
@@ -268,8 +278,10 @@ make_compile_option_value(rb_compile_option_t *option)
 	SET_COMPILE_OPTION(option, opt, operands_unification);
 	SET_COMPILE_OPTION(option, opt, instructions_unification);
 	SET_COMPILE_OPTION(option, opt, stack_caching);
+	SET_COMPILE_OPTION_NUM(option, opt, debug_level);
     }
 #undef SET_COMPILE_OPTION
+#undef SET_COMPILE_OPTION_NUM
     return opt;
 }
 
@@ -321,48 +333,48 @@ VALUE iseq_build_from_ary(rb_iseq_t *iseq, VALUE locals, VALUE args,
 #define CHECK_ARRAY(v)   rb_convert_type(v, T_ARRAY, "Array", "to_ary")
 #define CHECK_STRING(v)  rb_convert_type(v, T_STRING, "String", "to_str")
 #define CHECK_SYMBOL(v)  rb_convert_type(v, T_SYMBOL, "Symbol", "to_sym")
-#define CHECK_INTEGER(v) (NUM2LONG(v), v)
+static inline VALUE CHECK_INTEGER(VALUE v) {NUM2LONG(v); return v;}
 VALUE
 iseq_load(VALUE self, VALUE data, VALUE parent, VALUE opt)
 {
     VALUE iseqval = iseq_alloc(rb_cISeq);
 
     VALUE magic, version1, version2, format_type, misc;
-    VALUE name, filename, line;
+    VALUE name, filename;
     VALUE type, body, locals, args, exception;
 
     VALUE iseq_type;
     struct st_table *type_map = 0;
     rb_iseq_t *iseq;
     rb_compile_option_t option;
+    int i = 0;
 
     /* [magic, major_version, minor_version, format_type, misc,
-     *  name, filename, line,
+     *  name, filename,
      *  type, locals, args, exception_table, body]
      */
 
     data        = CHECK_ARRAY(data);
 
-    magic       = CHECK_STRING(rb_ary_entry(data, 0));
-    version1    = CHECK_INTEGER(rb_ary_entry(data, 1));
-    version2    = CHECK_INTEGER(rb_ary_entry(data, 2));
-    format_type = CHECK_INTEGER(rb_ary_entry(data, 3));
-    misc        = rb_ary_entry(data, 4); /* TODO */
+    magic       = CHECK_STRING(rb_ary_entry(data, i++));
+    version1    = CHECK_INTEGER(rb_ary_entry(data, i++));
+    version2    = CHECK_INTEGER(rb_ary_entry(data, i++));
+    format_type = CHECK_INTEGER(rb_ary_entry(data, i++));
+    misc        = rb_ary_entry(data, i++); /* TODO */
 
-    name        = CHECK_STRING(rb_ary_entry(data, 5));
-    filename    = CHECK_STRING(rb_ary_entry(data, 6));
-    line        = CHECK_ARRAY(rb_ary_entry(data, 7));
+    name        = CHECK_STRING(rb_ary_entry(data, i++));
+    filename    = CHECK_STRING(rb_ary_entry(data, i++));
 
-    type        = CHECK_SYMBOL(rb_ary_entry(data, 8));
-    locals      = CHECK_ARRAY(rb_ary_entry(data, 9));
+    type        = CHECK_SYMBOL(rb_ary_entry(data, i++));
+    locals      = CHECK_ARRAY(rb_ary_entry(data, i++));
 
-    args        = rb_ary_entry(data, 10);
+    args        = rb_ary_entry(data, i++);
     if (FIXNUM_P(args) || (args = CHECK_ARRAY(args))) {
 	/* */
     }
 
-    exception   = CHECK_ARRAY(rb_ary_entry(data, 11));
-    body        = CHECK_ARRAY(rb_ary_entry(data, 12));
+    exception   = CHECK_ARRAY(rb_ary_entry(data, i++));
+    body        = CHECK_ARRAY(rb_ary_entry(data, i++));
 
     GetISeqPtr(iseqval, iseq);
     iseq->self = iseqval;
@@ -376,6 +388,7 @@ iseq_load(VALUE self, VALUE data, VALUE parent, VALUE opt)
 	st_insert(type_map, ID2SYM(rb_intern("rescue")), ISEQ_TYPE_RESCUE);
 	st_insert(type_map, ID2SYM(rb_intern("ensure")), ISEQ_TYPE_ENSURE);
 	st_insert(type_map, ID2SYM(rb_intern("eval")), ISEQ_TYPE_EVAL);
+	st_insert(type_map, ID2SYM(rb_intern("defined_guard")), ISEQ_TYPE_DEFINED_GUARD);
     }
 
     if (st_lookup(type_map, type, &iseq_type) == 0) {
@@ -426,11 +439,11 @@ VALUE
 rb_iseq_compile_with_option(VALUE src, VALUE file, VALUE line, VALUE opt)
 {
     rb_compile_option_t option;
-    NODE *node = compile_string(src, file, line);
+    NODE *node = compile_string(StringValue(src), file, line);
     rb_thread_t *th = GET_THREAD();
     make_compile_option(&option, opt);
 
-    if (th->base_block) {
+    if (th->base_block && th->base_block->iseq) {
 	return rb_iseq_new_with_opt(node, th->base_block->iseq->name,
 				    file, th->base_block->iseq->self,
 				    ISEQ_TYPE_EVAL, &option);
@@ -452,6 +465,8 @@ iseq_s_compile(int argc, VALUE *argv, VALUE self)
 {
     VALUE src, file = Qnil, line = INT2FIX(1), opt = Qnil;
 
+    rb_secure(1);
+
     rb_scan_args(argc, argv, "13", &src, &file, &line, &opt);
     file = file == Qnil ? rb_str_new2("<compiled>") : file;
     line = line == Qnil ? INT2FIX(1) : line;
@@ -469,6 +484,7 @@ iseq_s_compile_file(int argc, VALUE *argv, VALUE self)
     const char *fname;
     rb_compile_option_t option;
 
+    rb_secure(1);
     rb_scan_args(argc, argv, "11", &file, &opt);
     fname = StringValueCStr(file);
 
@@ -485,6 +501,7 @@ static VALUE
 iseq_s_compile_option_set(VALUE self, VALUE opt)
 {
     rb_compile_option_t option;
+    rb_secure(1);
     make_compile_option(&option, opt);
     COMPILE_OPTION_DEFAULT = option;
     return opt;
@@ -510,6 +527,7 @@ iseq_check(VALUE val)
 static VALUE
 iseq_eval(VALUE self)
 {
+    rb_secure(1);
     return rb_iseq_eval(self);
 }
 
@@ -531,27 +549,40 @@ static VALUE
 iseq_to_a(VALUE self)
 {
     rb_iseq_t *iseq = iseq_check(self);
+    rb_secure(1);
     return iseq_data_to_ary(iseq);
 }
 
-/*
-  now, search algorithm is brute force. but this should be binary search.
- */
-static unsigned short
-find_line_no(rb_iseq_t *iseqdat, unsigned long pos)
+/* TODO: search algorithm is brute force.
+         this should be binary search or so. */
+
+static struct iseq_insn_info_entry *
+get_insn_info(const rb_iseq_t *iseq, const unsigned long pos)
 {
-    unsigned long i, size = iseqdat->insn_info_size;
-    struct iseq_insn_info_entry *iiary = iseqdat->insn_info_table;
+    unsigned long i, size = iseq->insn_info_size;
+    struct iseq_insn_info_entry *table = iseq->insn_info_table;
 
     for (i = 0; i < size; i++) {
-	if (iiary[i].position == pos) {
-	    return iiary[i].line_no;
+	if (table[i].position == pos) {
+	    return &table[i];
 	}
     }
-    /* rb_bug("find_line_no: can't find %lu", pos); */
+
     return 0;
 }
 
+static unsigned short
+find_line_no(rb_iseq_t *iseq, unsigned long pos)
+{
+    struct iseq_insn_info_entry *entry = get_insn_info(iseq, pos);
+    if (entry) {
+	return entry->line_no;
+    }
+    else {
+	return 0;
+    }
+}
+
 static unsigned short
 find_prev_line_no(rb_iseq_t *iseqdat, unsigned long pos)
 {
@@ -569,8 +600,6 @@ find_prev_line_no(rb_iseq_t *iseqdat, unsigned long pos)
 	}
     }
 
-    /* rb_bug("find_prev_line_no: can't find - %lu", pos); */
-
     return 0;
 }
 
@@ -579,7 +608,7 @@ insn_operand_intern(rb_iseq_t *iseq,
 		    int insn, int op_no, VALUE op,
 		    int len, int pos, VALUE *pnop, VALUE child)
 {
-    char *types = insn_op_types(insn);
+    const char *types = insn_op_types(insn);
     char type = types[op_no];
     VALUE ret;
     char buff[0x100];
@@ -631,6 +660,7 @@ insn_operand_intern(rb_iseq_t *iseq,
       }
       case TS_ID:		/* ID (symbol) */
 	op = ID2SYM(op);
+
       case TS_VALUE:		/* VALUE */
 	ret = rb_inspect(op);
 	if (CLASS_OF(op) == rb_cISeq) {
@@ -655,7 +685,7 @@ insn_operand_intern(rb_iseq_t *iseq,
       case TS_GENTRY:
 	{
 	    struct global_entry *entry = (struct global_entry *)op;
-	    ret = rb_str_new2(rb_id2name(entry->id));
+	    ret = rb_str_dup(rb_id2str(entry->id));
 	}
 	break;
 
@@ -688,7 +718,7 @@ ruby_iseq_disasm_insn(VALUE ret, VALUE *iseq, int pos,
     int insn = iseq[pos];
     int len = insn_len(insn);
     int i, j;
-    char *types = insn_op_types(insn);
+    const char *types = insn_op_types(insn);
     VALUE str = rb_str_new(0, 0);
     char buff[0x100];
     char insn_name_buff[0x100];
@@ -706,7 +736,7 @@ ruby_iseq_disasm_insn(VALUE ret, VALUE *iseq, int pos,
     rb_str_cat2(str, buff);
 
     for (j = 0; types[j]; j++) {
-	char *types = insn_op_types(insn);
+	const char *types = insn_op_types(insn);
 	VALUE opstr = insn_operand_intern(iseqdat, insn, j, iseq[pos + j + 1],
 					  len, pos, &iseq[pos + j + 2],
 					  child);
@@ -717,7 +747,7 @@ ruby_iseq_disasm_insn(VALUE ret, VALUE *iseq, int pos,
 	}
     }
 
-    {
+    if (1) {
 	int line_no = find_line_no(iseqdat, pos);
 	int prev = find_prev_line_no(iseqdat, pos);
 	if (line_no && line_no != prev) {
@@ -726,6 +756,14 @@ ruby_iseq_disasm_insn(VALUE ret, VALUE *iseq, int pos,
 	    str = rb_str_new2(buff);
 	}
     }
+    else {
+	/* for debug */
+	struct iseq_insn_info_entry *entry = get_insn_info(iseqdat, pos);
+	snprintf(buff, sizeof(buff), "%-60s(line: %d, sp: %d)",
+		 RSTRING_PTR(str), entry->line_no, entry->sp);
+	str = rb_str_new2(buff);
+    }
+
     if (ret) {
 	rb_str_cat2(str, "\n");
 	rb_str_concat(ret, str);
@@ -736,7 +774,7 @@ ruby_iseq_disasm_insn(VALUE ret, VALUE *iseq, int pos,
     return len;
 }
 
-static char *
+static const char *
 catch_type(int type)
 {
     switch (type) {
@@ -771,6 +809,8 @@ ruby_iseq_disasm(VALUE self)
     char buff[0x200];
     enum {header_minlen = 72};
 
+    rb_secure(1);
+
     iseq = iseqdat->iseq;
     size = iseqdat->iseq_size;
 
@@ -863,6 +903,25 @@ ruby_iseq_disasm(VALUE self)
     return str;
 }
 
+static VALUE
+iseq_s_disasm(VALUE klass, VALUE body)
+{
+    extern NODE *rb_method_body(VALUE body);
+    NODE *node;
+    VALUE ret = Qnil;
+
+    rb_secure(1);
+
+    if ((node = rb_method_body(body)) != 0) {
+	if (nd_type(node) == RUBY_VM_METHOD_NODE) {
+	    VALUE iseqval = (VALUE)node->nd_body;
+	    ret = ruby_iseq_disasm(iseqval);
+	}
+    }
+
+    return ret;
+}
+
 const char *
 ruby_node_name(int node)
 {
@@ -897,12 +956,12 @@ exception_type2symbol(VALUE type)
 {
     ID id;
     switch(type) {
-      case CATCH_TYPE_RESCUE: id = rb_intern("rescue"); break;
-      case CATCH_TYPE_ENSURE: id = rb_intern("ensure"); break;
-      case CATCH_TYPE_RETRY:  id = rb_intern("retry");  break;
-      case CATCH_TYPE_BREAK:  id = rb_intern("break");  break;
-      case CATCH_TYPE_REDO:   id = rb_intern("redo");   break;
-      case CATCH_TYPE_NEXT:   id = rb_intern("next");   break;
+      case CATCH_TYPE_RESCUE: CONST_ID(id, "rescue"); break;
+      case CATCH_TYPE_ENSURE: CONST_ID(id, "ensure"); break;
+      case CATCH_TYPE_RETRY:  CONST_ID(id, "retry");  break;
+      case CATCH_TYPE_BREAK:  CONST_ID(id, "break");  break;
+      case CATCH_TYPE_REDO:   CONST_ID(id, "redo");   break;
+      case CATCH_TYPE_NEXT:   CONST_ID(id, "next");   break;
       default:
 	rb_bug("...");
     }
@@ -942,6 +1001,7 @@ iseq_data_to_ary(rb_iseq_t *iseq)
     DECL_SYMBOL(rescue);
     DECL_SYMBOL(ensure);
     DECL_SYMBOL(eval);
+    DECL_SYMBOL(defined_guard);
 
     if (sym_top == 0) {
 	int i;
@@ -955,6 +1015,7 @@ iseq_data_to_ary(rb_iseq_t *iseq)
 	INIT_SYMBOL(rescue);
 	INIT_SYMBOL(ensure);
 	INIT_SYMBOL(eval);
+	INIT_SYMBOL(defined_guard);
     }
 
     /* type */
@@ -966,6 +1027,7 @@ iseq_data_to_ary(rb_iseq_t *iseq)
       case ISEQ_TYPE_RESCUE: type = sym_rescue; break;
       case ISEQ_TYPE_ENSURE: type = sym_ensure; break;
       case ISEQ_TYPE_EVAL:   type = sym_eval;   break;
+      case ISEQ_TYPE_DEFINED_GUARD: type = sym_defined_guard; break;
       default: rb_bug("unsupported iseq type");
     };
 
@@ -1232,16 +1294,20 @@ Init_ISeq(void)
     rb_define_alloc_func(rb_cISeq, iseq_alloc);
     rb_define_method(rb_cISeq, "inspect", iseq_inspect, 0);
     rb_define_method(rb_cISeq, "disasm", ruby_iseq_disasm, 0);
+    rb_define_method(rb_cISeq, "disassemble", ruby_iseq_disasm, 0);
     rb_define_method(rb_cISeq, "to_a", iseq_to_a, 0);
     rb_define_method(rb_cISeq, "eval", iseq_eval, 0);
 
-    rb_define_singleton_method(rb_cISeq, "load", iseq_s_load, -1);
+    /* disable this feature because there is no verifier. */
+    /* rb_define_singleton_method(rb_cISeq, "load", iseq_s_load, -1); */
+    (void)iseq_s_load;
+
     rb_define_singleton_method(rb_cISeq, "compile", iseq_s_compile, -1);
     rb_define_singleton_method(rb_cISeq, "new", iseq_s_compile, -1);
     rb_define_singleton_method(rb_cISeq, "compile_file", iseq_s_compile_file, -1);
-    rb_define_singleton_method(rb_cISeq, "compile_option",
-			       iseq_s_compile_option_get, 0);
-    rb_define_singleton_method(rb_cISeq, "compile_option=",
-			       iseq_s_compile_option_set, 1);
+    rb_define_singleton_method(rb_cISeq, "compile_option", iseq_s_compile_option_get, 0);
+    rb_define_singleton_method(rb_cISeq, "compile_option=", iseq_s_compile_option_set, 1);
+    rb_define_singleton_method(rb_cISeq, "disasm", iseq_s_disasm, 1);
+    rb_define_singleton_method(rb_cISeq, "disassemble", iseq_s_disasm, 1);
 }
 
