@@ -187,12 +187,10 @@ class CSV
       @header_row = header_row
       
       # handle extra headers or fields
-      larger, smaller, transform = headers.size > fields.size ?
-                                   [headers, fields, :to_a]   :
-                                   [fields, headers, :reverse]
-      @row              = Array.new
-      larger.each_with_index do |e, i|
-        @row << [e, smaller[i]].send(transform)
+      @row = if headers.size > fields.size
+        headers.each_with_index.map { |header, i| [header, fields[i]] }
+      else
+        fields.each_with_index.map { |field, i| [headers[i], field] }
       end
     end
     
@@ -815,7 +813,7 @@ class CSV
   # <b><tt>:all</tt></b>::        All built-in converters.  A combination of 
   #                               <tt>:date_time</tt> and <tt>:numeric</tt>.
   # 
-  # This Hash is intetionally left unfrozen and users should feel free to add
+  # This Hash is intentionally left unfrozen and users should feel free to add
   # values to it that can be accessed by all CSV objects.
   # 
   # To add a combo field, the value should be an Array of names.  Combo fields
@@ -1150,7 +1148,7 @@ class CSV
   # Hash containing any +options+ CSV::new() understands.
   # 
   # This method works like Ruby's open() call, in that it will pass a CSV object
-  # to a provided block and close it when the block termminates, or it will
+  # to a provided block and close it when the block terminates, or it will
   # return the CSV object when no block is provided.  (*Note*: This is different
   # from the Ruby 1.8 CSV library which passed rows to the block.  Use
   # CSV::foreach() for that behavior.)
@@ -1315,7 +1313,7 @@ class CSV
   #                                       unconverted_fields() method will be
   #                                       added to all returned rows (Array or
   #                                       CSV::Row) that will return the fields
-  #                                       as they were before convertion.  Note
+  #                                       as they were before conversion.  Note
   #                                       that <tt>:headers</tt> supplied by
   #                                       Array or String were not fields of the
   #                                       document and thus will have an empty
@@ -1526,7 +1524,7 @@ class CSV
     # 
     loop do
       # add another read to the line
-      line  += @io.gets(@row_sep) rescue return nil
+      (line += @io.gets(@row_sep)) rescue return nil
       # copy the line so we can chop it up in parsing
       parse = line.dup
       parse.sub!(@parsers[:line_end], "")
@@ -1746,7 +1744,7 @@ class CSV
 
     instance_variable_set("@#{field_name}", Array.new)
     
-    # find the correct method to add the coverters
+    # find the correct method to add the converters
     convert = method(field_name.to_s.sub(/ers\Z/, ""))
     
     # load converters
@@ -1814,8 +1812,7 @@ class CSV
     # see if we are converting headers or fields
     converters = headers ? @header_converters : @converters
     
-    converted = Array.new
-    fields.each_with_index do |field, index|
+    fields.each_with_index.map do |field, index|  # map_with_index
       converters.each do |converter|
         field = if converter.arity == 1  # straight field converter
           converter[field]
@@ -1825,9 +1822,8 @@ class CSV
         end
         break unless field.is_a? String  # short-curcuit pipeline for speed
       end
-      converted << field  # final state of each field, converted or original
+      field  # final state of each field, converted or original
     end
-    converted
   end
   
   # 
