@@ -1,5 +1,5 @@
-# format.rb: Written by Tadayoshi Funaba 1999-2007
-# $Id: format.rb,v 2.40 2007-09-09 08:28:03+09 tadf Exp $
+# format.rb: Written by Tadayoshi Funaba 1999-2008
+# $Id: format.rb,v 2.43 2008-01-17 20:16:31+09 tadf Exp $
 
 require 'rational'
 
@@ -215,9 +215,9 @@ class Date
 	  :emit_a, :emit_ad, :emit_au
 
   def strftime(fmt='%F')
-    fmt.gsub(/%([-_0^#]+)?(\d+)?([EO]?(?::{1,3}z|.))/m) do |m|
+    fmt.gsub(/%([-_0^#]+)?(\d+)?([EO]?(?::{1,3}z|.))/m) do
       f = {}
-      a = $&
+      m = $&
       s, w, c = $1, $2, $3
       if s
 	s.scan(/./) do |k|
@@ -257,25 +257,23 @@ class Date
       when 'j'; emit_n(yday, 3, f)
       when 'k'; emit_a(hour, 2, f)
       when 'L'
-	emit_n((sec_fraction / (1.to_r/(10**3))).round, 3, f)
+	emit_n((sec_fraction / MILLISECONDS_IN_SECOND).floor, 3, f)
       when 'l'; emit_a((hour % 12).nonzero? || 12, 2, f)
       when 'M', 'OM'; emit_n(min, 2, f)
       when 'm', 'Om'; emit_n(mon, 2, f)
       when 'N'
-	emit_n((sec_fraction / (1.to_r/(10**9))).round, 9, f)
+	emit_n((sec_fraction / NANOSECONDS_IN_SECOND).floor, 9, f)
       when 'n'; "\n"
       when 'P'; emit_ad(strftime('%p').downcase, 0, f)
       when 'p'; emit_au(if hour < 12 then 'AM' else 'PM' end, 0, f)
       when 'Q'
-	d = ajd - jd_to_ajd(self.class::UNIXEPOCH, 0)
-	s = (d * 86400*10**3).to_i
+	s = ((ajd - UNIX_EPOCH_IN_AJD) / MILLISECONDS_IN_DAY).round
 	emit_sn(s, 1, f)
       when 'R'; emit_a(strftime('%H:%M'), 0, f)
       when 'r'; emit_a(strftime('%I:%M:%S %p'), 0, f)
       when 'S', 'OS'; emit_n(sec, 2, f)
       when 's'
-	d = ajd - jd_to_ajd(self.class::UNIXEPOCH, 0)
-	s = (d * 86400).to_i
+	s = ((ajd - UNIX_EPOCH_IN_AJD) / SECONDS_IN_DAY).round
 	emit_sn(s, 1, f)
       when 'T'
 	if m == '%T'
@@ -299,9 +297,9 @@ class Date
 	t = $1.size
 	sign = if offset < 0 then -1 else +1 end
 	fr = offset.abs
-	hh, fr = fr.divmod(1.to_r/24)
-	mm, fr = fr.divmod(1.to_r/1440)
-	ss, fr = fr.divmod(1.to_r/86400)
+	ss = fr.div(SECONDS_IN_DAY) # 4p
+	hh, ss = ss.divmod(3600)
+	mm, ss = ss.divmod(60)
 	if t == 3
 	  if    ss.nonzero? then t =  2
 	  elsif mm.nonzero? then t =  1
@@ -329,7 +327,7 @@ class Date
       when '%'; emit_a('%', 0, f)
       when '+'; emit_a(strftime('%a %b %e %H:%M:%S %Z %Y'), 0, f)
       else
-	a
+	m
       end
     end
   end
@@ -372,12 +370,12 @@ class Date
 
 =begin
   def beat(n=0)
-    i, f = (new_offset(1.to_r/24).day_fraction * 1000).divmod(1)
+    i, f = (new_offset(HOURS_IN_DAY).day_fraction * 1000).divmod(1)
     ('@%03d' % i) +
       if n < 1
 	''
       else
-	'.%0*d' % [n, (f / (1.to_r/(10**n))).round]
+	'.%0*d' % [n, (f / Rational(1, 10**n)).round]
       end
   end
 =end
@@ -456,8 +454,8 @@ class Date
 				 then /\A([-+]?\d{1,3})/
 				 else /\A([-+]?\d{1,})/
 				 end, '')
-#	  val = $1.to_i.to_r / (10**3)
-	  val = $1.to_i.to_r / (10**$1.size)
+#	  val = Rational($1.to_i, 10**3)
+	  val = Rational($1.to_i, 10**$1.size)
 	  e.sec_fraction = val
 	when 'M', 'OM'
 	  return unless str.sub!(/\A(\d{1,2})/, '')
@@ -474,8 +472,8 @@ class Date
 				 then /\A([-+]?\d{1,9})/
 				 else /\A([-+]?\d{1,})/
 				 end, '')
-#	  val = $1.to_i.to_r / (10**9)
-	  val = $1.to_i.to_r / (10**$1.size)
+#	  val = Rational($1.to_i, 10**9)
+	  val = Rational($1.to_i, 10**$1.size)
 	  e.sec_fraction = val
 	when 'n', 't'
 	  return unless _strptime_i(str, "\s", e)
@@ -484,7 +482,7 @@ class Date
 	  e._merid = if $1.downcase == 'a' then 0 else 12 end
 	when 'Q'
 	  return unless str.sub!(/\A(-?\d{1,})/, '')
-	  val = $1.to_i.to_r / 10**3
+	  val = Rational($1.to_i, 10**3)
 	  e.seconds = val
 	when 'R'
 	  return unless _strptime_i(str, '%H:%M', e)
@@ -723,7 +721,7 @@ class Date
       e.hour = $1.to_i
       e.min = $2.to_i if $2
       e.sec = $3.to_i if $3
-      e.sec_fraction = $4.to_i.to_r / (10**$4.size) if $4
+      e.sec_fraction = Rational($4.to_i, 10**$4.size) if $4
 
       if $5
 	e.hour %= 12
@@ -738,9 +736,9 @@ class Date
 =begin
   def self._parse_beat(str, e) # :nodoc:
     if str.sub!(/@\s*(\d+)(?:[,.](\d*))?/, ' ')
-      beat = $1.to_i.to_r
-      beat += $2.to_i.to_r / (10**$2.size) if $2
-      secs = beat.to_r / 1000
+      beat = Rational($1.to_i)
+      beat += Rational($2.to_i, 10**$2.size) if $2
+      secs = Rational(beat, 1000)
       h, min, s, fr = self.day_fraction_to_time(secs)
       e.hour = h
       e.min = min
@@ -823,9 +821,6 @@ class Date
 	str.sub!(/\b-(\d{3})\b/n, ' ')
       e.yday = $1.to_i
       true
-    elsif e._date && str.sub!(/('?[-+]?\d+)-('?-?\d+)/n, ' ')
-      s3e(e, nil, $1, $2)
-      true
     end
   end
 
@@ -840,11 +835,6 @@ class Date
       e.mon = $3.to_i
       e.mday = $4.to_i
       true
-    elsif e._jis && str.sub!(/(\d+)\.(\d+)\.(\d+)/in, ' ')
-      e.year = $1.to_i + 1988
-      e.mon = $2.to_i
-      e.mday = $3.to_i
-      true
     end
   end
 
@@ -860,54 +850,17 @@ class Date
     end
   end
 
-  def self._parse_sla_jp(str, e) # :nodoc:
+  def self._parse_sla(str, e) # :nodoc:
     if str.sub!(%r|('?-?\d+)/\s*('?\d+)(?:\D\s*('?-?\d+))?|n, ' ') # '
       s3e(e, $1, $2, $3)
       true
     end
   end
 
-  def self._parse_sla_eu(str, e) # :nodoc:
-    if str.sub!(%r|('?-?\d+)/\s*('?\d+)(?:\D\s*('?-?\d+))?|n, ' ') # '
-      s3e(e, $3, $2, $1)
-      true
-    end
-  end
-
-  def self._parse_sla_us(str, e) # :nodoc:
-    if str.sub!(%r|('?-?\d+)/\s*('?\d+)(?:\D\s*('?-?\d+))?|n, ' ') # '
-      s3e(e, $3, $1, $2)
-      true
-    end
-  end
-
-  def self._parse_dot_jp(str, e) # :nodoc:
+  def self._parse_dot(str, e) # :nodoc:
     if str.sub!(%r|('?-?\d+)\.\s*('?\d+)\.\s*('?-?\d+)|n, ' ') # '
       s3e(e, $1, $2, $3)
       true
-    elsif e._date && str.sub!(%r|('?-?\d+)\.\s*('?\d+)|n, ' ') # '
-      s3e(e, nil, $1, $2)
-      true
-    end
-  end
-
-  def self._parse_dot_eu(str, e) # :nodoc:
-    if str.sub!(%r|('?-?\d+)\.\s*('?\d+)\.\s*('?-?\d+)|n, ' ') # '
-      s3e(e, $3, $2, $1)
-      true
-    elsif e._date && str.sub!(%r|('?-?\d+)\.\s*('?\d+)|n, ' ') # '
-      s3e(e, nil, $2, $1)
-      true
-    end
-  end
-
-  def self._parse_dot_us(str, e) # :nodoc:
-    if str.sub!(%r|('?-?\d+)\.\s*('?\d+)\.\s*('?-?\d+)|n, ' ') # '
-      s3e(e, $3, $1, $2)
-      true
-    elsif e._date && str.sub!(%r|('?-?\d+)\.\s*('?\d+)|n, ' ') # '
-      s3e(e, nil, $1, $2)
-      true
     end
   end
 
@@ -958,24 +911,15 @@ class Date
 	if $3.nil? && $4
 	  e.sec  = $2[-2, 2].to_i
 	else
-	  if e._time
-	    e.hour = $2[ 0, 2].to_i
-	  else
-	    e.mday = $2[ 0, 2].to_i
-	  end
+	  e.mday = $2[ 0, 2].to_i
 	end
       when 4
 	if $3.nil? && $4
 	  e.sec  = $2[-2, 2].to_i
 	  e.min  = $2[-4, 2].to_i
 	else
-	  if e._time
-	    e.hour = $2[ 0, 2].to_i
-	    e.min  = $2[ 2, 2].to_i
-	  else
-	    e.mon  = $2[ 0, 2].to_i
-	    e.mday = $2[ 2, 2].to_i
-	  end
+	  e.mon  = $2[ 0, 2].to_i
+	  e.mday = $2[ 2, 2].to_i
 	end
       when 6
 	if $3.nil? && $4
@@ -983,15 +927,9 @@ class Date
 	  e.min  = $2[-4, 2].to_i
 	  e.hour = $2[-6, 2].to_i
 	else
-	  if e._time || e._ofx
-	    e.hour = $2[ 0, 2].to_i
-	    e.min  = $2[ 2, 2].to_i
-	    e.sec  = $2[ 4, 2].to_i
-	  else
-	    e.year = ($1 + $2[ 0, 2]).to_i
-	    e.mon  = $2[ 2, 2].to_i
-	    e.mday = $2[ 4, 2].to_i
-	  end
+	  e.year = ($1 + $2[ 0, 2]).to_i
+	  e.mon  = $2[ 2, 2].to_i
+	  e.mday = $2[ 4, 2].to_i
 	end
       when 8, 10, 12, 14
 	if $3.nil? && $4
@@ -1063,7 +1001,7 @@ class Date
 	end
       end
       if $4
-	e.sec_fraction = $4.to_i.to_r / (10**$4.size)
+	e.sec_fraction = Rational($4.to_i, 10**$4.size)
       end
       if $5
 	e.zone = $5
@@ -1082,116 +1020,33 @@ class Date
 
   private_class_method :_parse_day, :_parse_time, # :_parse_beat,
 	:_parse_eu, :_parse_us, :_parse_iso, :_parse_iso2,
-	:_parse_jis, :_parse_vms,
-	:_parse_sla_jp, :_parse_sla_eu, :_parse_sla_us,
-	:_parse_dot_jp, :_parse_dot_eu, :_parse_dot_us,
+	:_parse_jis, :_parse_vms, :_parse_sla, :_parse_dot,
 	:_parse_year, :_parse_mon, :_parse_mday, :_parse_ddd
 
-  def self._parse(str, hints={})
+  def self._parse(str, comp=true)
     str = str.dup
 
     e = Format::Bag.new
 
-    e._comp = true
-
-    unless Hash === hints
-      e._comp = hints
-    else
-      hints.each do |k, v|
-	case k
-	when :comp, :complete
-	  e._comp = v
-	when :style, :endian, :endianness, :order
-	  v = {
-	    :jp     => :jp,
-	    :eu     => :eu,
-	    :us     => :us,
-	    :big    => :jp,
-	    :little => :eu,
-	    :middle => :us,
-	    :ymd    => :jp,
-	    :dmy    => :eu,
-	    :mdy    => :us
-	  }[v]
-	  e._style = v
-	when :date
-	  e._date = v
-	when :time
-	  e._time = v
-	when :jis
-	  e._jis = v
-	when :ofx
-	  e._ofx = v
-	else
-	  raise ArgumentError, 'unknown hint'
-	end
-      end
-    end
+    e._comp = comp
 
     str.gsub!(/[^-+',.\/:@[:alnum:]\[\]\x80-\xff]+/n, ' ')
 
-    case e._style
-    when :jp
-      _parse_time(str, e) # || _parse_beat(str, e)
-      _parse_eu(str, e)     ||
-      _parse_us(str, e)     ||
-      _parse_iso(str, e)    ||
-      _parse_jis(str, e)    ||
-      _parse_vms(str, e)    ||
-      _parse_sla_jp(str, e) ||
-      _parse_dot_jp(str, e) ||
-      _parse_iso2(str, e)   ||
-      _parse_year(str, e)   ||
-      _parse_mon(str, e)    ||
-      _parse_mday(str, e)   ||
-      _parse_ddd(str, e)
-      _parse_day(str, e)
-    when :eu
-      _parse_time(str, e) # || _parse_beat(str, e)
-      _parse_eu(str, e)     ||
-      _parse_iso(str, e)    ||
-      _parse_jis(str, e)    ||
-      _parse_vms(str, e)    ||
-      _parse_sla_eu(str, e) ||
-      _parse_dot_eu(str, e) ||
-      _parse_iso2(str, e)   ||
-      _parse_year(str, e)   ||
-      _parse_mon(str, e)    ||
-      _parse_mday(str, e)   ||
-      _parse_ddd(str, e)    ||
-      _parse_us(str, e)
-      _parse_day(str, e)
-    when :us
-      _parse_time(str, e) # || _parse_beat(str, e)
-      _parse_eu(str, e)     ||
-      _parse_us(str, e)     ||
-      _parse_iso(str, e)    ||
-      _parse_jis(str, e)    ||
-      _parse_vms(str, e)    ||
-      _parse_sla_us(str, e) ||
-      _parse_dot_us(str, e) ||
-      _parse_iso2(str, e)   ||
-      _parse_year(str, e)   ||
-      _parse_mon(str, e)    ||
-      _parse_mday(str, e)   ||
-      _parse_ddd(str, e)
-      _parse_day(str, e)
-    else
-      _parse_time(str, e) # || _parse_beat(str, e)
-      _parse_eu(str, e)     ||
-      _parse_us(str, e)     ||
-      _parse_iso(str, e)    ||
-      _parse_jis(str, e)    ||
-      _parse_vms(str, e)    ||
-      _parse_sla_jp(str, e) ||
-      _parse_dot_jp(str, e) ||
-      _parse_iso2(str, e)   ||
-      _parse_year(str, e)   ||
-      _parse_mon(str, e)    ||
-      _parse_mday(str, e)   ||
-      _parse_ddd(str, e)
-      _parse_day(str, e)
-    end
+    _parse_time(str, e) # || _parse_beat(str, e)
+    _parse_day(str, e)
+
+    _parse_eu(str, e)     ||
+    _parse_us(str, e)     ||
+    _parse_iso(str, e)    ||
+    _parse_jis(str, e)    ||
+    _parse_vms(str, e)    ||
+    _parse_sla(str, e)    ||
+    _parse_dot(str, e)    ||
+    _parse_iso2(str, e)   ||
+    _parse_year(str, e)   ||
+    _parse_mon(str, e)    ||
+    _parse_mday(str, e)   ||
+    _parse_ddd(str, e)
 
     if str.sub!(/\b(bc\b|bce\b|b\.c\.|b\.c\.e\.)/in, ' ')
       if e.year
@@ -1269,17 +1124,46 @@ class Date
   end
 
   def self._xmlschema(str) # :nodoc:
-    if /\A\s*-?\d{4}-\d{2}-\d{2}
-	(t
-	\d{2}:\d{2}:\d{2}(?:\.\d+)?)?
+    if /\A\s*(-?\d{4,})(?:-(\d{2})(?:-(\d{2}))?)?
+	(?:t
+	  (\d{2}):(\d{2}):(\d{2})(?:\.(\d+))?)?
 	(z|[-+]\d{2}:\d{2})?\s*\z/inx =~ str
-      if $1.nil? && $2
-	str = str.sub(/(z|[-+]\d{2}:\d{2})/in, 'T00:00:00\1')
+      e = Format::Bag.new
+      e.year = $1.to_i
+      e.mon = $2.to_i if $2
+      e.mday = $3.to_i if $3
+      e.hour = $4.to_i if $4
+      e.min = $5.to_i if $5
+      e.sec = $6.to_i if $6
+      e.sec_fraction = Rational($7.to_i, 10**$7.size) if $7
+      if $8
+	e.zone = $8
+	e.offset = zone_to_diff($8)
       end
-      _parse(str)
-    elsif /\A\s*\d{2}:\d{2}:\d{2}(\.\d+)?
+      e.to_hash
+    elsif /\A\s*(\d{2}):(\d{2}):(\d{2})(?:\.(\d+))?
 	(z|[-+]\d{2}:\d{2})?\s*\z/inx =~ str
-      _parse(str)
+      e = Format::Bag.new
+      e.hour = $1.to_i if $1
+      e.min = $2.to_i if $2
+      e.sec = $3.to_i if $3
+      e.sec_fraction = Rational($4.to_i, 10**$4.size) if $4
+      if $5
+	e.zone = $5
+	e.offset = zone_to_diff($5)
+      end
+      e.to_hash
+    elsif /\A\s*(?:--(\d{2})(?:-(\d{2}))?|---(\d{2}))
+	(z|[-+]\d{2}:\d{2})?\s*\z/inx =~ str
+      e = Format::Bag.new
+      e.mon = $1.to_i if $1
+      e.mday = $2.to_i if $2
+      e.mday = $3.to_i if $3
+      if $4
+	e.zone = $4
+	e.offset = zone_to_diff($4)
+      end
+      e.to_hash
     end
   end
 
@@ -1290,7 +1174,7 @@ class Date
 	-?(\d{2,})\s+ # allow minus, anyway
 	\d{2}:\d{2}(:\d{2})?\s*
 	(?:[-+]\d{4}|ut|gmt|e[sd]t|c[sd]t|m[sd]t|p[sd]t|[a-ik-z])\s*\z/inox =~ str
-      e = _parse(str, :comp=>false)
+      e = _parse(str, false)
       if $1.size < 4
 	if e[:year] < 50
 	  e[:year] += 2000
@@ -1333,7 +1217,11 @@ class Date
 	(t
 	(\d{2}:\d{2}(:\d{2}([,.]\d*)?)?
 	(z|[-+]\d{2}(:?\d{2})?)?)?)?\s*\z/inx =~ str
-      _parse(str, :jis=>true)
+      if /\A\s*\d/ =~ str
+	_parse(str.sub(/\A\s*(\d)/, 'h\1'))
+      else
+	_parse(str)
+      end
     else
       _iso8601(str)
     end
@@ -1359,7 +1247,7 @@ class Date
 	  hour, min, sec, = zone.split(':')
 	elsif zone.include?(',') || zone.include?('.')
 	  hour, fr, = zone.split(/[,.]/)
-	  min = fr.to_i.to_r / (10**fr.size) * 60
+	  min = Rational(fr.to_i, 10**fr.size) * 60
 	else
 	  case zone.size
 	  when 3
@@ -1399,7 +1287,7 @@ class DateTime < Date
 	     if n < 1
 	       ''
 	     else
-	       '.%0*d' % [n, (sec_fraction / (1.to_r/(10**n))).round]
+	       '.%0*d' % [n, (sec_fraction / Rational(1, 10**n)).round]
 	     end +
 	     '%:z')
   end
