@@ -236,7 +236,7 @@
 # Rails, the web application framework, uses ERB to create views.
 #
 class ERB
-  Revision = '$Date: 2006/01/10 15:21:45 $' 	#'
+  Revision = '$Date:: 2008-06-04 18:37:38 +0900#$' 	#'
 
   # Returns revision information for the erb.rb module.
   def self.version
@@ -304,11 +304,11 @@ class ERB
       def scan(&block)
 	@stag = nil
 	if @percent
-	  @src.each do |line|
+	  @src.each_line do |line|
 	    percent_line(line, &block)
 	  end
 	else
-	  @src.each do |line|
+	  @src.each_line do |line|
 	    @scan_line.call(line, &block)
 	  end
 	end
@@ -392,7 +392,7 @@ class ERB
 
     class SimpleScanner < Scanner # :nodoc:
       def scan
-	@src.each do |line|
+	@src.each_line do |line|
 	  line.split(SplitRegexp).each do |token|
 	    next if token.empty?
 	    yield(token)
@@ -658,7 +658,7 @@ class ERB
   #    
   #    def build
   #      b = binding
-  #      # create and run templates, filling member data variebles
+  #      # create and run templates, filling member data variables
   #      ERB.new(<<-'END_PRODUCT'.gsub(/^\s+/, ""), 0, "", "@product").result b
   #        <%= PRODUCT[:name] %>
   #        <%= PRODUCT[:desc] %>
@@ -799,7 +799,9 @@ class ERB
     # 	Programming%20Ruby%3A%20%20The%20Pragmatic%20Programmer%27s%20Guide
     #
     def url_encode(s)
-      s.to_s.gsub(/[^a-zA-Z0-9_\-.]/n){ sprintf("%%%02X", $&.unpack("C")[0]) }
+      s.to_s.dup.force_encoding("ASCII-8BIT").gsub(/[^a-zA-Z0-9_\-.]/n) {
+        sprintf("%%%02X", $&.unpack("C")[0])
+      }
     end
     alias u url_encode
     module_function :u
