@@ -1,8 +1,7 @@
 #
 #   irb/init.rb - irb initialize module
 #   	$Release Version: 0.9.5$
-#   	$Revision: 1.11 $
-#   	$Date: 2005/06/07 15:02:31 $
+#   	$Revision: 14912 $
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
@@ -121,8 +120,11 @@ module IRB
     @CONF[:LC_MESSAGES].load("irb/error.rb")
   end
 
+  FEATURE_IOPT_CHANGE_VERSION = "1.9.0"
+
   # option analyzing
   def IRB.parse_opts
+    load_path = []
     while opt = ARGV.shift
       case opt
       when "-f"
@@ -136,9 +138,7 @@ module IRB
 	@CONF[:LOAD_MODULES].push opt if opt
       when /^-I(.+)?/
         opt = $1 || ARGV.shift
-        $LOAD_PATH.push opt if opt
-      when /^-K(.)/
-	$KCODE = $1
+	load_path.concat(opt.split(File::PATH_SEPARATOR)) if opt
       when "--inspect"
 	@CONF[:INSPECT_MODE] = true
       when "--noinspect"
@@ -189,6 +189,12 @@ module IRB
 	break
       end
     end
+    if RUBY_VERSION >= FEATURE_IOPT_CHANGE_VERSION
+      load_path.collect! do |path|
+	/\A\.\// =~ path ? path : File.expand_path(path)
+      end
+    end
+    $LOAD_PATH.unshift(*load_path)
   end
 
   # running config
@@ -197,7 +203,7 @@ module IRB
       begin
 	load rc_file
       rescue LoadError, Errno::ENOENT
-      rescue
+      rescue # StandardError, ScriptError
 	print "load error: #{rc_file}\n"
 	print $!.class, ": ", $!, "\n"
 	for err in $@[0, $@.size - 2]
@@ -224,7 +230,7 @@ module IRB
   # enumerate possible rc-file base name generators
   def IRB.rc_file_generators
     if irbrc = ENV["IRBRC"]
-      yield proc{|rc| irbrc}
+      yield proc{|rc| rc == "rc" ? irbrc : irbrc+rc}
     end
     if home = ENV["HOME"]
       yield proc{|rc| home+"/.irb#{rc}"} 
@@ -241,7 +247,7 @@ module IRB
     for m in @CONF[:LOAD_MODULES]
       begin
 	require m
-      rescue
+      rescue # StandardError, ScriptError
 	print $@[0], ":", $!.class, ": ", $!, "\n"
       end
     end
