@@ -2,7 +2,6 @@
 #   mathn.rb - 
 #   	$Release Version: 0.5 $
 #   	$Revision: 1.1.1.1.4.1 $
-#   	$Date: 1998/01/16 12:36:05 $
 #   	by Keiju ISHITSUKA(SHL Japan Inc.)
 #
 # --
@@ -16,14 +15,6 @@ require "matrix.rb"
 
 class Integer
 
-  remove_method(:gcd2)
-  def gcd2(other)
-    min = self.abs
-    max = other.abs
-    min, max = max % min, min while min > 0
-    max
-  end
-
   def Integer.from_prime_division(pd)
     value = 1
     for prime, index in pd
@@ -33,6 +24,7 @@ class Integer
   end
   
   def prime_division
+    raise ZeroDivisionError if self == 0
     ps = Prime.new
     value = self
     pv = []
@@ -109,6 +101,7 @@ class Prime
   alias next succ
 
   def each
+    return to_enum(:each) unless block_given?
     loop do
       yield succ
     end
@@ -116,27 +109,25 @@ class Prime
 end
 
 class Fixnum
+  remove_method :/
   alias / quo
 end
 
 class Bignum
+  remove_method :/
   alias / quo
 end
 
 class Rational
   Unify = true
 
-  remove_method(:inspect)
-  def inspect
-    format "%s/%s", numerator.inspect, denominator.inspect
-  end
-
   alias power! **
 
   def ** (other)
     if other.kind_of?(Rational)
+      other2 = other
       if self < 0
-	return Complex.new!(self, 0) ** other
+	return Complex.__send__(:new!, self, 0) ** other
       elsif other == 0
 	return Rational(1,1)
       elsif self == 0
@@ -155,7 +146,7 @@ class Rational
       for elm in npd
 	elm[1] = elm[1] * other
 	if !elm[1].kind_of?(Integer) and elm[1].denominator != 1
-	  return Float(self) ** other
+         return Float(self) ** other2
 	end
 	elm[1] = elm[1].to_i
       end
@@ -163,7 +154,7 @@ class Rational
       for elm in dpd
 	elm[1] = elm[1] * other
 	if !elm[1].kind_of?(Integer) and elm[1].denominator != 1
-	  return Float(self) ** other
+         return Float(self) ** other2
 	end
 	elm[1] = elm[1].to_i
       end
@@ -184,7 +175,7 @@ class Rational
 	num = 1
 	den = 1
       end
-      Rational.new!(num, den)
+      Rational(num, den)
     elsif other.kind_of?(Float)
       Float(self) ** other
     else
@@ -196,7 +187,7 @@ class Rational
   def power2(other)
     if other.kind_of?(Rational)
       if self < 0
-	return Complex(self, 0) ** other
+	return Complex.__send__(:new!, self, 0) ** other
       elsif other == 0
 	return Rational(1,1)
       elsif self == 0
@@ -210,7 +201,7 @@ class Rational
       neard = self.denominator.to_f ** (1.0/other.denominator.to_f)
       loop do
 	if (neard**other.denominator == self.denominator)
-	  dem = neaed
+	  dem = neard
 	  break
 	end
       end
@@ -228,7 +219,7 @@ class Rational
 	num = 1
 	den = 1
       end
-      Rational.new!(num, den)
+      Rational(num, den)
     elsif other.kind_of?(Float)
       Float(self) ** other
     else
@@ -315,4 +306,3 @@ end
 class Complex
   Unify = true
 end
-
