@@ -51,6 +51,9 @@ $archdir = CONFIG["archdir"]
 $sitedir = CONFIG["sitedir"]
 $sitelibdir = CONFIG["sitelibdir"]
 $sitearchdir = CONFIG["sitearchdir"]
+$vendordir = CONFIG["vendordir"]
+$vendorlibdir = CONFIG["vendorlibdir"]
+$vendorarchdir = CONFIG["vendorarchdir"]
 
 $mswin = /mswin/ =~ RUBY_PLATFORM
 $bccwin = /bccwin/ =~ RUBY_PLATFORM
@@ -63,6 +66,8 @@ $beos = /beos/ =~ RUBY_PLATFORM
 $solaris = /solaris/ =~ RUBY_PLATFORM
 $dest_prefix_pattern = (File::PATH_SEPARATOR == ';' ? /\A([[:alpha:]]:)?/ : /\A/)
 
+# :stopdoc:
+
 def config_string(key, config = CONFIG)
   s = config[key] and !s.empty? and block_given? ? yield(s) : s
 end
@@ -71,20 +76,35 @@ def dir_re(dir)
   Regexp.new('\$(?:\('+dir+'\)|\{'+dir+'\})(?:\$(?:\(target_prefix\)|\{target_prefix\}))?')
 end
 
+def relative_from(path, base)
+  dir = File.join(path, "")
+  if File.expand_path(dir) == File.expand_path(dir, base)
+    path
+  else
+    File.join(base, path)
+  end
+end
+
 INSTALL_DIRS = [
   [dir_re('commondir'), "$(RUBYCOMMONDIR)"],
   [dir_re('sitedir'), "$(RUBYCOMMONDIR)"],
+  [dir_re('vendordir'), "$(RUBYCOMMONDIR)"],
   [dir_re('rubylibdir'), "$(RUBYLIBDIR)"],
   [dir_re('archdir'), "$(RUBYARCHDIR)"],
   [dir_re('sitelibdir'), "$(RUBYLIBDIR)"],
+  [dir_re('vendorlibdir'), "$(RUBYLIBDIR)"],
   [dir_re('sitearchdir'), "$(RUBYARCHDIR)"],
+  [dir_re('vendorarchdir'), "$(RUBYARCHDIR)"],
   [dir_re('rubyhdrdir'), "$(RUBYHDRDIR)"],
   [dir_re('sitehdrdir'), "$(SITEHDRDIR)"],
+  [dir_re('vendorhdrdir'), "$(VENDORHDRDIR)"],
+  [dir_re('bindir'), "$(BINDIR)"],
 ]
 
 def install_dirs(target_prefix = nil)
   if $extout
     dirs = [
+      ['BINDIR',        '$(extout)/bin'],
       ['RUBYCOMMONDIR', '$(extout)/common'],
       ['RUBYLIBDIR',    '$(RUBYCOMMONDIR)$(target_prefix)'],
       ['RUBYARCHDIR',   '$(extout)/$(arch)$(target_prefix)'],
@@ -95,14 +115,25 @@ def install_dirs(target_prefix = nil)
     ]
   elsif $extmk
     dirs = [
+      ['BINDIR',        '$(bindir)'],
       ['RUBYCOMMONDIR', '$(rubylibdir)'],
       ['RUBYLIBDIR',    '$(rubylibdir)$(target_prefix)'],
       ['RUBYARCHDIR',   '$(archdir)$(target_prefix)'],
       ['HDRDIR',        '$(rubyhdrdir)/ruby$(target_prefix)'],
       ['ARCHHDRDIR',    '$(rubyhdrdir)/$(arch)/ruby$(target_prefix)'],
     ]
+  elsif $configure_args.has_key?('--vendor')
+    dirs = [
+      ['BINDIR',        '$(bindir)'],
+      ['RUBYCOMMONDIR', '$(vendordir)$(target_prefix)'],
+      ['RUBYLIBDIR',    '$(vendorlibdir)$(target_prefix)'],
+      ['RUBYARCHDIR',   '$(vendorarchdir)$(target_prefix)'],
+      ['HDRDIR',        '$(rubyhdrdir)/ruby$(target_prefix)'],
+      ['ARCHHDRDIR',    '$(rubyhdrdir)/$(arch)/ruby$(target_prefix)'],
+    ]
   else
     dirs = [
+      ['BINDIR',        '$(bindir)'],
       ['RUBYCOMMONDIR', '$(sitedir)$(target_prefix)'],
       ['RUBYLIBDIR',    '$(sitelibdir)$(target_prefix)'],
       ['RUBYARCHDIR',   '$(sitearchdir)$(target_prefix)'],
@@ -121,7 +152,8 @@ end
 
 topdir = File.dirname(libdir = File.dirname(__FILE__))
 extdir = File.expand_path("ext", topdir)
-$extmk = File.expand_path($0)[0, extdir.size+1] == extdir+"/"
+path = File.expand_path($0)
+$extmk = path[0, topdir.size+1] == topdir+"/" && %r"\A(ext|enc|tool)\z" =~ File.dirname(path[topdir.size+1..-1])
 if not $extmk and File.exist?(($hdrdir = RbConfig::CONFIG["rubyhdrdir"]) + "/ruby/ruby.h")
   $topdir = $hdrdir
   $top_srcdir = $hdrdir
@@ -135,16 +167,24 @@ else
 end
 
 OUTFLAG = CONFIG['OUTFLAG']
+COUTFLAG = CONFIG['COUTFLAG']
 CPPOUTFILE = CONFIG['CPPOUTFILE']
 
 CONFTEST_C = "conftest.c".freeze
 
 class String
+  # Wraps a string in escaped quotes if it contains whitespace.
   def quote
     /\s/ =~ self ? "\"#{self}\"" : "#{self}"
   end
+
+  # Generates a string used as cpp macro name.
+  def tr_cpp
+    strip.upcase.tr_s("^A-Z0-9_", "_")
+  end
 end
 class Array
+  # Wraps all strings in escaped quotes if they contain whitespace.
   def quote
     map {|s| s.quote}
   end
@@ -154,6 +194,8 @@ def rm_f(*files)
   FileUtils.rm_f(Dir[*files])
 end
 
+# Returns time stamp of the +target+ file if it exists and is newer
+# than or equal to all of +times+.
 def modified?(target, times)
   (t = File.mtime(target)) rescue return nil
   Array === times or times = [times]
@@ -176,6 +218,12 @@ def merge_libs(*libs)
   end
 end
 
+# This is a custom logging module. It generates an mkmf.log file when you
+# run your extconf.rb script. This can be useful for debugging unexpected
+# failures.
+#
+# This module and its associated methods are meant for internal use only.
+#
 module Logging
   @log = nil
   @logfile = 'mkmf.log'
@@ -233,6 +281,12 @@ module Logging
 end
 
 def xsystem command
+  varpat = /\$\((\w+)\)|\$\{(\w+)\}/
+  if varpat =~ command
+    vars = Hash.new {|h, k| h[k] = ''; ENV[k]}
+    command = command.dup
+    nil while command.gsub!(varpat) {vars[$1||$2]}
+  end
   Logging::open do
     puts command.quote
     system(command)
@@ -268,8 +322,16 @@ end
 def create_tmpsrc(src)
   src = yield(src) if block_given?
   src = src.gsub(/[ \t]+$/, '').gsub(/\A\n+|^\n+$/, '').sub(/[^\n]\z/, "\\&\n")
-  open(CONFTEST_C, "wb") do |cfile|
-    cfile.print src
+  count = 0
+  begin
+    open(CONFTEST_C, "wb") do |cfile|
+      cfile.print src
+    end
+  rescue Errno::EACCES
+    if (count += 1) < 5
+      sleep 0.2
+      retry
+    end
   end
   src
 end
@@ -460,7 +522,7 @@ def try_var(var, headers = nil, &b)
 #{headers}
 /*top*/
 int main() { return 0; }
-int t() { const volatile void *volatile p; p = (void *)&#{var}; return 0; }
+int t() { const volatile void *volatile p; p = &(&#{var})[0]; return 0; }
 SRC
 end
 
@@ -490,6 +552,7 @@ ensure
   log_src(src)
 end
 
+# This is used internally by the have_macro? method.
 def macro_defined?(macro, src, opt = "", &b)
   src = src.sub(/[^\n]\z/, "\\&\n")
   try_compile(src + <<"SRC", opt, &b)
@@ -513,13 +576,14 @@ end
 
 def install_files(mfile, ifiles, map = nil, srcprefix = nil)
   ifiles or return
+  ifiles.empty? and return
   srcprefix ||= '$(srcdir)'
   RbConfig::expand(srcdir = srcprefix.dup)
   dirs = []
   path = Hash.new {|h, i| h[i] = dirs.push([i])[-1]}
   ifiles.each do |files, dir, prefix|
     dir = map_dir(dir, map)
-    prefix = %r|\A#{Regexp.quote(prefix)}/?| if prefix
+    prefix &&= %r|\A#{Regexp.quote(prefix)}/?|
     if /\A\.\// =~ files
       # install files which are in current working directory.
       files = files[2..-1]
@@ -533,6 +597,10 @@ def install_files(mfile, ifiles, map = nil, srcprefix = nil)
     Dir.glob(files) do |fx|
       f = fx
       f[0..len] = "" if len
+      case File.basename(f)
+      when *$NONINSTALLFILES
+        next
+      end
       d = File.dirname(f)
       d.sub!(prefix, "") if prefix
       d = (d.empty? || d == ".") ? dir : File.join(dir, d)
@@ -553,7 +621,7 @@ def install_rb(mfile, dest, srcdir = nil)
   install_files(mfile, [["lib/**/*.rb", dest, "lib"]], nil, srcdir)
 end
 
-def append_library(libs, lib)
+def append_library(libs, lib) # :no-doc:
   format(LIBARG, lib) + " " + libs
 end
 
@@ -564,6 +632,11 @@ def message(*s)
   end
 end
 
+# This emits a string to stdout that allows users to see the results of the
+# various have* and find* methods as they are tested.
+#
+# Internal use only.
+#
 def checking_for(m, fmt = nil)
   f = caller[0][/in `(.*)'$/, 1] and f << ": " #` for vim #'
   m = "checking #{/\Acheck/ =~ f ? '' : 'for '}#{m}... "
@@ -593,6 +666,8 @@ def checking_message(target, place = nil, opt = nil)
   end
 end
 
+# :startdoc:
+
 # Returns whether or not +macro+ is defined either in the common header
 # files or within any +headers+ you provide.
 #
@@ -612,7 +687,7 @@ end
 # If +headers+ are provided, it will include those header files as the
 # header files it looks in when searching for +func+.
 #
-# Real name of the library to be linked can be altered by
+# The real name of the library to be linked can be altered by
 # '--with-FOOlib' configuration option.
 #
 def have_library(lib, func = nil, headers = nil, &b)
@@ -673,7 +748,7 @@ end
 def have_func(func, headers = nil, &b)
   checking_for checking_message("#{func}()", headers) do
     if try_func(func, $libs, headers, &b)
-      $defs.push(format("-DHAVE_%s", func.upcase))
+      $defs.push(format("-DHAVE_%s", func.tr_cpp))
       true
     else
       false
@@ -692,7 +767,7 @@ end
 def have_var(var, headers = nil, &b)
   checking_for checking_message(var, headers) do
     if try_var(var, headers, &b)
-      $defs.push(format("-DHAVE_%s", var.upcase))
+      $defs.push(format("-DHAVE_%s", var.tr_cpp))
       true
     else
       false
@@ -751,10 +826,12 @@ end
 # (in addition to the common header files).
 #
 # If found, a macro is passed as a preprocessor constant to the compiler using
-# the member name, in uppercase, prepended with 'HAVE_ST_'.
+# the type name and the member name, in uppercase, prepended with 'HAVE_'.
 #
-# For example, if have_struct_member('foo', 'bar') returned true, then the
-# HAVE_ST_BAR preprocessor macro would be passed to the compiler.
+# For example, if have_struct_member('struct foo', 'bar') returned true, then the
+# HAVE_STRUCT_FOO_BAR preprocessor macro would be passed to the compiler.
+#
+# HAVE_ST_BAR is also defined for backward compatibility.
 # 
 def have_struct_member(type, member, headers = nil, &b)
   checking_for checking_message("#{type}.#{member}", headers) do
@@ -765,7 +842,8 @@ def have_struct_member(type, member, headers = nil, &b)
 int main() { return 0; }
 int s = (char *)&((#{type}*)0)->#{member} - (char *)0;
 SRC
-      $defs.push(format("-DHAVE_ST_%s", member.upcase))
+      $defs.push(format("-DHAVE_%s_%s", type.tr_cpp, member.tr_cpp))
+      $defs.push(format("-DHAVE_ST_%s", member.tr_cpp)) # backward compatibility
       true
     else
       false
@@ -781,7 +859,7 @@ def try_type(type, headers = nil, opt = "", &b)
 typedef #{type} conftest_type;
 int conftestval[sizeof(conftest_type)?1:-1];
 SRC
-    $defs.push(format("-DHAVE_TYPE_%s", type.strip.upcase.tr_s("^A-Z0-9_", "_")))
+    $defs.push(format("-DHAVE_TYPE_%s", type.tr_cpp))
     true
   else
     false
@@ -836,7 +914,7 @@ def try_const(const, headers = nil, opt = "", &b)
 typedef #{type || 'int'} conftest_type;
 conftest_type conftestval = #{type ? '' : '(int)'}#{const};
 SRC
-    $defs.push(format("-DHAVE_CONST_%s", const.strip.upcase.tr_s("^A-Z0-9_", "_")))
+    $defs.push(format("-DHAVE_CONST_%s", const.tr_cpp))
     true
   else
     false
@@ -883,12 +961,16 @@ def check_sizeof(type, headers = nil, &b)
   end
   checking_for checking_message("size of #{type}", headers), fmt do
     if size = try_constant(expr, headers, &b)
-      $defs.push(format("-DSIZEOF_%s=%d", type.upcase.tr_s("^A-Z0-9_", "_"), size))
+      $defs.push(format("-DSIZEOF_%s=%d", type.tr_cpp, size))
       size
     end
   end
 end
 
+# :stopdoc:
+
+# Used internally by the what_type? method to determine if +type+ is a scalar
+# pointer.
 def scalar_ptr_type?(type, member = nil, headers = nil, &b)
   try_compile(<<"SRC", &b)   # pointer
 #{COMMON_HEADERS}
@@ -900,6 +982,8 @@ int t() {return (int)(1-*(conftestval#{member ? ".#{member}" : ""}));}
 SRC
 end
 
+# Used internally by the what_type? method to determine if +type+ is a scalar
+# pointer.
 def scalar_type?(type, member = nil, headers = nil, &b)
   try_compile(<<"SRC", &b)   # pointer
 #{COMMON_HEADERS}
@@ -943,6 +1027,10 @@ def what_type?(type, member = nil, headers = nil, &b)
   end
 end
 
+# This method is used internally by the find_executable method.
+#
+# Internal use only.
+#
 def find_executable0(bin, path = nil)
   ext = config_string('EXEEXT')
   if File.expand_path(bin) == bin
@@ -963,12 +1051,25 @@ def find_executable0(bin, path = nil)
   nil
 end
 
+# :startdoc:
+
+# Searches for the executable +bin+ on +path+. The default path is your
+# PATH environment variable. If that isn't defined, it will resort to
+# searching /usr/local/bin, /usr/ucb, /usr/bin and /bin.
+#
+# If found, it will return the full path, including the executable name,
+# of where it was found.
+#
+# Note that this method does not actually affect the generated Makefile.
+#
 def find_executable(bin, path = nil)
   checking_for checking_message(bin, path) do
     find_executable0(bin, path)
   end
 end
 
+# :stopdoc:
+
 def arg_config(config, default=nil, &block)
   $arg_config << [config, default]
   defaults = []
@@ -980,6 +1081,20 @@ def arg_config(config, default=nil, &block)
   $configure_args.fetch(config.tr('_', '-'), *defaults, &block)
 end
 
+# :startdoc:
+
+# Tests for the presence of a --with-<tt>config</tt> or --without-<tt>config</tt>
+# option. Returns true if the with option is given, false if the without
+# option is given, and the default value otherwise.
+#
+# This can be useful for adding custom definitions, such as debug information.
+#
+# Example:
+#
+#    if with_config("debug")
+#       $defs.push("-DOSSL_DEBUG") unless $defs.include? "-DOSSL_DEBUG"
+#    end
+#
 def with_config(config, default=nil)
   config = config.sub(/^--with[-_]/, '')
   val = arg_config("--with-"+config) do
@@ -1001,6 +1116,18 @@ def with_config(config, default=nil)
   end
 end
 
+# Tests for the presence of an --enable-<tt>config</tt> or
+# --disable-<tt>config</tt> option. Returns true if the enable option is given,
+# false if the disable option is given, and the default value otherwise.
+#
+# This can be useful for adding custom definitions, such as debug information.
+#
+# Example:
+#
+#    if enable_config("debug")
+#       $defs.push("-DOSSL_DEBUG") unless $defs.include? "-DOSSL_DEBUG"
+#    end
+#
 def enable_config(config, default=nil)
   if arg_config("--enable-"+config)
     true
@@ -1013,6 +1140,32 @@ def enable_config(config, default=nil)
   end
 end
 
+# Generates a header file consisting of the various macro definitions generated
+# by other methods such as have_func and have_header. These are then wrapped in
+# a custom #ifndef based on the +header+ file name, which defaults to
+# 'extconf.h'.
+#
+# For example:
+# 
+#    # extconf.rb
+#    require 'mkmf'
+#    have_func('realpath')
+#    have_header('sys/utime.h')
+#    create_header
+#    create_makefile('foo')
+#
+# The above script would generate the following extconf.h file:
+#
+#    #ifndef EXTCONF_H
+#    #define EXTCONF_H
+#    #define HAVE_REALPATH 1
+#    #define HAVE_SYS_UTIME_H 1
+#    #endif
+#
+# Given that the create_header method generates a file based on definitions
+# set earlier in your extconf.rb file, you will probably want to make this
+# one of the last methods you call in your script.
+#
 def create_header(header = "extconf.h")
   message "creating %s\n", header
     sym = header.tr("a-z./\055", "A-Z___")
@@ -1080,6 +1233,10 @@ def dir_config(target, idefault=nil, ldefault=nil)
   [idir, ldir]
 end
 
+# :stopdoc:
+
+# Handles meta information about installed libraries. Uses your platform's
+# pkg-config program if it has one.
 def pkg_config(pkg)
   if pkgconfig = with_config("#{pkg}-config") and find_executable0(pkgconfig)
     # iff package specific config command is given
@@ -1117,6 +1274,10 @@ def with_destdir(dir)
   /\A\$[\(\{]/ =~ dir ? dir : "$(DESTDIR)"+dir
 end
 
+# Converts forward slashes to backslashes. Aimed at MS Windows.
+#
+# Internal use only.
+#
 def winsep(s)
   s.tr('/', '\\')
 end
@@ -1151,7 +1312,7 @@ arch_hdrdir = #{$arch_hdrdir}
 VPATH = #{vpath.join(CONFIG['PATH_SEPARATOR'])}
 }
   if $extmk
-    mk << "RUBYLIB = -\nRUBYOPT = -rpurelib.rb\n"
+    mk << "RUBYLIB = -\nRUBYOPT = -r$(top_srcdir)/ext/purelib.rb\n"
   end
   if destdir = CONFIG["prefix"][$dest_prefix_pattern, 1]
     mk << "\nDESTDIR = #{destdir}\n"
@@ -1180,13 +1341,23 @@ LIBRUBY = #{CONFIG['LIBRUBY']}
 LIBRUBY_A = #{CONFIG['LIBRUBY_A']}
 LIBRUBYARG_SHARED = #$LIBRUBYARG_SHARED
 LIBRUBYARG_STATIC = #$LIBRUBYARG_STATIC
+OUTFLAG = #{OUTFLAG}
+COUTFLAG = #{COUTFLAG}
 
 RUBY_EXTCONF_H = #{$extconf_h}
+cflags   = #{CONFIG['cflags']}
+optflags = #{CONFIG['optflags']}
+debugflags = #{CONFIG['debugflags']}
+warnflags = #{CONFIG['warnflags']}
 CFLAGS   = #{$static ? '' : CONFIG['CCDLFLAGS']} #$CFLAGS #$ARCH_FLAG
 INCFLAGS = -I. #$INCFLAGS
+DEFS     = #{CONFIG['DEFS']}
 CPPFLAGS = #{extconf_h}#{$CPPFLAGS}
 CXXFLAGS = $(CFLAGS) #{CONFIG['CXXFLAGS']}
-DLDFLAGS = #$LDFLAGS #$DLDFLAGS #$ARCH_FLAG
+ldflags  = #{$LDFLAGS}
+dldflags = #{$DLDFLAGS}
+archflag = #{$ARCH_FLAG}
+DLDFLAGS = $(ldflags) $(dldflags) $(archflag)
 LDSHARED = #{CONFIG['LDSHARED']}
 LDSHAREDXX = #{config_string('LDSHAREDXX') || '$(LDSHARED)'}
 AR = #{CONFIG['AR']}
@@ -1221,6 +1392,7 @@ preload = #{defined?($preload) && $preload ? $preload.join(' ') : ''}
   end
   mk
 end
+# :startdoc:
 
 def dummy_makefile(srcdir)
   configuration(srcdir) << <<RULES << CLEANINGS
@@ -1232,6 +1404,62 @@ all install static install-so install-rb: Makefile
 RULES
 end
 
+def depend_rules(depend)
+  suffixes = []
+  depout = []
+  cont = implicit = nil
+  impconv = proc do
+    COMPILE_RULES.each {|rule| depout << (rule % implicit[0]) << implicit[1]}
+    implicit = nil
+  end
+  ruleconv = proc do |line|
+    if implicit
+      if /\A\t/ =~ line
+        implicit[1] << line
+        next
+      else
+        impconv[]
+      end
+    end
+    if m = /\A\.(\w+)\.(\w+)(?:\s*:)/.match(line)
+      suffixes << m[1] << m[2]
+      implicit = [[m[1], m[2]], [m.post_match]]
+      next
+    elsif RULE_SUBST and /\A(?!\s*\w+\s*=)[$\w][^#]*:/ =~ line
+      line.gsub!(%r"(\s)(?!\.)([^$(){}+=:\s\/\\,]+)(?=\s|\z)") {$1 + RULE_SUBST % $2}
+    end
+    depout << line
+  end
+  depend.each_line do |line|
+    line.gsub!(/\.o\b/, ".#{$OBJEXT}")
+    line.gsub!(/\$\((?:hdr|top)dir\)\/config.h/, $config_h) if $config_h
+    line.gsub!(%r"\$\(hdrdir\)/(?!ruby(?![^:;/\s]))(?=[-\w]+\.h)", '\&ruby/')
+    if $nmake && /\A\s*\$\(RM|COPY\)/ =~ line
+      line.gsub!(%r"[-\w\./]{2,}"){$&.tr("/", "\\")}
+      line.gsub!(/(\$\((?!RM|COPY)[^:)]+)(?=\))/, '\1:/=\\')
+    end
+    if /(?:^|[^\\])(?:\\\\)*\\$/ =~ line
+      (cont ||= []) << line
+      next
+    elsif cont
+      line = (cont << line).join
+      cont = nil
+    end
+    ruleconv.call(line)
+  end
+  if cont
+    ruleconv.call(cont.join)
+  elsif implicit
+    impconv.call
+  end
+  unless suffixes.empty?
+    depout.unshift(".SUFFIXES: ." + suffixes.uniq.join(" .") + "\n\n")
+  end
+  depout.unshift("$(OBJS): $(RUBY_EXTCONF_H)\n\n") if $extconf_h
+  depout.flatten!
+  depout
+end
+
 # Generates the Makefile for your extension, passing along any options and
 # preprocessor constants that you may have generated through other methods.
 #
@@ -1249,6 +1477,41 @@ end
 # load the file within a Ruby program later, that directory structure will
 # have to be followed, e.g. "require 'test/foo'".
 #
+# The +srcprefix+ should be used when your source files are not in the same
+# directory as your build script. This will not only eliminate the need for
+# you to manually copy the source files into the same directory as your build
+# script, but it also sets the proper +target_prefix+ in the generated
+# Makefile.
+#
+# Setting the +target_prefix+ will, in turn, install the generated binary in
+# a directory under your Config::CONFIG['sitearchdir'] that mimics your local
+# filesystem when you run 'make install'.
+#
+# For example, given the following file tree:
+#
+#    ext/
+#       extconf.rb
+#       test/
+#          foo.c
+#
+# And given the following code:
+#
+#    create_makefile('test/foo', 'test')
+#
+# That will set the +target_prefix+ in the generated Makefile to 'test'. That,
+# in turn, will create the following file tree when installed via the
+# 'make install' command:
+#
+#    /path/to/ruby/sitearchdir/test/foo.so
+#
+# It is recommended that you use this approach to generate your makefiles,
+# instead of copying files around manually, because some third party
+# libraries may depend on the +target_prefix+ being set properly.
+#
+# The +srcprefix+ argument can be used to override the default source
+# directory, i.e. the current directory . It is included as part of the VPATH
+# and added to the list of INCFLAGS.
+#
 def create_makefile(target, srcprefix = nil)
   $target = target
   libpath = $DEFLIBPATH|$LIBPATH
@@ -1352,11 +1615,11 @@ static:		$(STATIC_LIB)#{$extout ? " install-rb" : ""}
   dirs = []
   mfile.print "install: install-so install-rb\n\n"
   sodir = (dir = "$(RUBYARCHDIR)").dup
-  mfile.print("install-so: #{dir}\n")
+  mfile.print("install-so: ")
   if target
     f = "$(DLLIB)"
     dest = "#{dir}/#{f}"
-    mfile.print "install-so: #{dest}\n"
+    mfile.puts dir, "install-so: #{dest}"
     unless $extout
       mfile.print "#{dest}: #{f}\n"
       if (sep = config_string('BUILD_FILE_SEPARATOR'))
@@ -1373,6 +1636,8 @@ static:		$(STATIC_LIB)#{$extout ? " install-rb" : ""}
 	mfile.print "\t@echo #{dir}/#{File.basename(f)}>>$(INSTALLED_LIST)\n"
       end
     end
+  else
+    mfile.puts "Makefile"
   end
   mfile.print("install-rb: pre-install-rb install-rb-default\n")
   mfile.print("install-rb-default: pre-install-rb-default\n")
@@ -1384,11 +1649,13 @@ static:		$(STATIC_LIB)#{$extout ? " install-rb" : ""}
       unless dirs.include?(dir)
 	dirs << dir
 	mfile.print "pre-install-rb#{sfx}: #{dir}\n"
-      end
+      end if $nmake
       for f in files
 	dest = "#{dir}/#{File.basename(f)}"
 	mfile.print("install-rb#{sfx}: #{dest}\n")
-	mfile.print("#{dest}: #{f}\n\t$(#{$extout ? 'COPY' : 'INSTALL_DATA'}) ")
+        mfile.print("#{dest}: #{f}\n")
+        mfile.print("\t$(MAKEDIRS) $(@D)\n") unless $nmake
+        mfile.print("\t$(#{$extout ? 'COPY' : 'INSTALL_DATA'}) ")
 	sep = config_string('BUILD_FILE_SEPARATOR')
 	if sep
 	  f = f.gsub("/", sep)
@@ -1406,7 +1673,7 @@ static:		$(STATIC_LIB)#{$extout ? " install-rb" : ""}
     end
   end
   dirs.unshift(sodir) if target and !dirs.include?(sodir)
-  dirs.each {|d| mfile.print "#{d}:\n\t$(MAKEDIRS) $@\n"}
+  dirs.each {|d| mfile.print "#{d}:\n\t$(MAKEDIRS) $@\n" if $nmake || d == sodir}
 
   mfile.print <<-SITEINSTALL
 
@@ -1460,57 +1727,7 @@ site-install-rb: install-rb
 
   depend = File.join(srcdir, "depend")
   if File.exist?(depend)
-    suffixes = []
-    depout = []
-    open(depend, "r") do |dfile|
-      mfile.printf "###\n"
-      cont = implicit = nil
-      impconv = proc do
-	COMPILE_RULES.each {|rule| depout << (rule % implicit[0]) << implicit[1]}
-	implicit = nil
-      end
-      ruleconv = proc do |line|
-	if implicit
-	  if /\A\t/ =~ line
-	    implicit[1] << line
-	    next
-	  else
-	    impconv[]
-	  end
-	end
-	if m = /\A\.(\w+)\.(\w+)(?:\s*:)/.match(line)
-          suffixes << m[1] << m[2]
-	  implicit = [[m[1], m[2]], [m.post_match]]
-	  next
-	elsif RULE_SUBST and /\A(?!\s*\w+\s*=)[$\w][^#]*:/ =~ line
-	  line.gsub!(%r"(?<=\s)(?!\.)([^$(){}+=:\s\/\\,]+)(?=\s|\z)", &RULE_SUBST.method(:%))
-	end
-	depout << line
-      end
-      while line = dfile.gets()
-	line.gsub!(/\.o\b/, ".#{$OBJEXT}")
-	line.gsub!(/\$\((?:hdr|top)dir\)\/config.h/, $config_h) if $config_h
-	line.gsub!(%r"\$\(hdrdir\)/(?!ruby/)", '\&ruby/')
-	if /(?:^|[^\\])(?:\\\\)*\\$/ =~ line
-	  (cont ||= []) << line
-	  next
-	elsif cont
-	  line = (cont << line).join
-	  cont = nil
-	end
-	ruleconv.call(line)
-      end
-      if cont
-	ruleconv.call(cont.join)
-      elsif implicit
-	impconv.call
-      end
-    end
-    unless suffixes.empty?
-      mfile.print ".SUFFIXES: .", suffixes.uniq.join(" ."), "\n\n"
-    end
-    mfile.print "$(OBJS): $(RUBY_EXTCONF_H)\n\n" if $extconf_h
-    mfile.print(*depout.flatten)
+    mfile.print("###\n", *depend_rules(File.read(depend)))
   else
     headers = %w[ruby.h defines.h]
     if RULE_SUBST
@@ -1526,6 +1743,8 @@ ensure
   mfile.close if mfile
 end
 
+# :stopdoc:
+
 def init_mkmf(config = CONFIG)
   $makefile_created = false
   $arg_config = []
@@ -1547,7 +1766,8 @@ def init_mkmf(config = CONFIG)
   $DEFLIBPATH = $extmk ? ["$(topdir)"] : CROSS_COMPILING ? [] : ["$(libdir)"]
   $DEFLIBPATH.unshift(".")
   $LIBPATH = []
-  $INSTALLFILES = nil
+  $INSTALLFILES = []
+  $NONINSTALLFILES = [/~\z/, /\A#.*#\z/, /\A\.#/, /\.bak\z/i, /\.orig\z/, /\.rej\z/, /\.l[ao]\z/, /\.o\z/]
   $VPATH = %w[$(srcdir) $(arch_hdrdir)/ruby $(hdrdir)/ruby]
 
   $objs = nil
@@ -1578,6 +1798,11 @@ details.  You may need configuration options.
 Provided configuration options:
 MESSAGE
 
+# Returns whether or not the Makefile was successfully generated. If not,
+# the script will abort with an error message.
+#
+# Internal use only.
+#
 def mkmf_failed(path)
   unless $makefile_created or File.exist?("Makefile")
     opts = $arg_config.collect {|t, n| "\t#{t}#{n ? "=#{n}" : ""}\n"}
@@ -1585,6 +1810,8 @@ def mkmf_failed(path)
   end
 end
 
+# :startdoc:
+
 init_mkmf
 
 $make = with_config("make-prog", ENV["MAKE"] || "make")
@@ -1630,8 +1857,8 @@ COMMON_LIBS = config_string('COMMON_LIBS', &split) || []
 
 COMPILE_RULES = config_string('COMPILE_RULES', &split) || %w[.%s.%s:]
 RULE_SUBST = config_string('RULE_SUBST')
-COMPILE_C = config_string('COMPILE_C') || '$(CC) $(INCFLAGS) $(CPPFLAGS) $(CFLAGS) -c $<'
-COMPILE_CXX = config_string('COMPILE_CXX') || '$(CXX) $(INCFLAGS) $(CPPFLAGS) $(CXXFLAGS) -c $<'
+COMPILE_C = config_string('COMPILE_C') || '$(CC) $(INCFLAGS) $(CPPFLAGS) $(CFLAGS) $(COUTFLAG)$@ -c $<'
+COMPILE_CXX = config_string('COMPILE_CXX') || '$(CXX) $(INCFLAGS) $(CPPFLAGS) $(CXXFLAGS) $(COUTFLAG)$@ -c $<'
 TRY_LINK = config_string('TRY_LINK') ||
   "$(CC) #{OUTFLAG}conftest $(INCFLAGS) $(CPPFLAGS) " \
   "$(CFLAGS) $(src) $(LIBPATH) $(LDFLAGS) $(ARCH_FLAG) $(LOCAL_LIBS) $(LIBS)"
