@@ -12,9 +12,9 @@
 # modify this program under the same terms as Ruby itself.
 # 
 # NOTE: You can find Japanese version of this document at:
-# http://www.ruby-lang.org/ja/man/index.cgi?cmd=view;name=net%2Fsmtp.rb
+# http://www.ruby-lang.org/ja/man/html/net_smtp.html
 # 
-# $Id: smtp.rb 13244 2007-08-24 09:57:53Z matz $
+# $Id: smtp.rb 16033 2008-04-15 08:12:30Z kazu $
 #
 # See Net::SMTP for documentation. 
 # 
@@ -172,7 +172,7 @@ module Net
   #
   class SMTP
 
-    Revision = %q$Revision: 13244 $.split[1]
+    Revision = %q$Revision: 16033 $.split[1]
 
     # The default SMTP port number, 25.
     def SMTP.default_port
@@ -578,6 +578,9 @@ module Net
       logging "TLS connection started"
       s.sync_close = true
       s.connect
+      if @ssl_context.verify_mode != OpenSSL::SSL::VERIFY_NONE
+        s.post_connection_check(@address)
+      end
       s
     end
 
@@ -719,7 +722,7 @@ module Net
     def authenticate(user, secret, authtype = DEFAULT_AUTH_TYPE)
       check_auth_method authtype
       check_auth_args user, secret
-      send! auth_method(authtype), user, secret
+      send auth_method(authtype), user, secret
     end
 
     def auth_plain(user, secret)
