@@ -1,59 +1,3 @@
-#= open-uri.rb
-#
-#open-uri.rb is easy-to-use wrapper for net/http, net/https and net/ftp.
-#
-#== Example
-#
-#It is possible to open http/https/ftp URL as usual a file:
-#
-#  open("http://www.ruby-lang.org/") {|f|
-#    f.each_line {|line| p line}
-#  }
-#
-#The opened file has several methods for meta information as follows since
-#it is extended by OpenURI::Meta.
-#
-#  open("http://www.ruby-lang.org/en") {|f|
-#    f.each_line {|line| p line}
-#    p f.base_uri         # <URI::HTTP:0x40e6ef2 URL:http://www.ruby-lang.org/en/>
-#    p f.content_type     # "text/html"
-#    p f.charset          # "iso-8859-1"
-#    p f.content_encoding # []
-#    p f.last_modified    # Thu Dec 05 02:45:02 UTC 2002
-#  }
-#
-#Additional header fields can be specified by an optional hash argument.
-#
-#  open("http://www.ruby-lang.org/en/",
-#    "User-Agent" => "Ruby/#{RUBY_VERSION}",
-#    "From" => "foo@bar.invalid",
-#    "Referer" => "http://www.ruby-lang.org/") {|f|
-#    ...
-#  }
-#
-#The environment variables such as http_proxy, https_proxy and ftp_proxy
-#are in effect by default.  :proxy => nil disables proxy.
-#
-#  open("http://www.ruby-lang.org/en/raa.html",
-#    :proxy => nil) {|f|
-#    ...
-#  }
-#
-#URI objects can be opened in similar way.
-#
-#  uri = URI.parse("http://www.ruby-lang.org/en/")
-#  uri.open {|f|
-#    ...
-#  }
-#
-#URI objects can be read directly.
-#The returned string is also extended by OpenURI::Meta.
-#
-#  str = uri.read
-#  p str.base_uri
-#
-#Author:: Tanaka Akira <akr@m17n.org>
-
 require 'uri'
 require 'stringio'
 require 'time'
@@ -61,6 +5,9 @@ require 'time'
 module Kernel
   private
   alias open_uri_original_open open # :nodoc:
+  class << self
+    alias open_uri_original_open open # :nodoc:
+  end
 
   # makes possible to open various resources including URIs.
   # If the first argument respond to `open' method,
@@ -91,6 +38,59 @@ module Kernel
   module_function :open
 end
 
+# OpenURI is an easy-to-use wrapper for net/http, net/https and net/ftp.
+#
+#== Example
+#
+# It is possible to open http/https/ftp URL as usual like opening a file:
+#
+#   open("http://www.ruby-lang.org/") {|f|
+#     f.each_line {|line| p line}
+#   }
+#
+# The opened file has several methods for meta information as follows since
+# it is extended by OpenURI::Meta.
+#
+#   open("http://www.ruby-lang.org/en") {|f|
+#     f.each_line {|line| p line}
+#     p f.base_uri         # <URI::HTTP:0x40e6ef2 URL:http://www.ruby-lang.org/en/>
+#     p f.content_type     # "text/html"
+#     p f.charset          # "iso-8859-1"
+#     p f.content_encoding # []
+#     p f.last_modified    # Thu Dec 05 02:45:02 UTC 2002
+#   }
+#
+# Additional header fields can be specified by an optional hash argument.
+#
+#   open("http://www.ruby-lang.org/en/",
+#     "User-Agent" => "Ruby/#{RUBY_VERSION}",
+#     "From" => "foo@bar.invalid",
+#     "Referer" => "http://www.ruby-lang.org/") {|f|
+#     # ...
+#   }
+#
+# The environment variables such as http_proxy, https_proxy and ftp_proxy
+# are in effect by default.  :proxy => nil disables proxy.
+#
+#   open("http://www.ruby-lang.org/en/raa.html", :proxy => nil) {|f|
+#     # ...
+#   }
+#
+# URI objects can be opened in a similar way.
+#
+#   uri = URI.parse("http://www.ruby-lang.org/en/")
+#   uri.open {|f|
+#     # ...
+#   }
+#
+# URI objects can be read directly. The returned string is also extended by
+# OpenURI::Meta.
+#
+#   str = uri.read
+#   p str.base_uri
+#
+# Author:: Tanaka Akira <akr@m17n.org>
+
 module OpenURI
   Options = {
     :proxy => true,
@@ -101,6 +101,8 @@ module OpenURI
     :read_timeout => true,
     :ssl_ca_cert => nil,
     :ssl_verify_mode => nil,
+    :ftp_active_mode => false,
+    :redirect => true,
   }
 
   def OpenURI.check_options(options) # :nodoc:
@@ -201,6 +203,9 @@ module OpenURI
           # URI.  It is converted to absolute URI using uri as a base URI.
           redirect = uri + redirect
         end
+        if !options.fetch(:redirect, true)
+          raise HTTPRedirect.new(buf.io.status.join(' '), buf.io, redirect)
+        end
         unless OpenURI.redirectable?(uri, redirect)
           raise "redirection forbidden: #{uri} -> #{redirect}"
         end
@@ -224,6 +229,9 @@ module OpenURI
   def OpenURI.redirectable?(uri1, uri2) # :nodoc:
     # This test is intended to forbid a redirection from http://... to
     # file:///etc/passwd.
+    # https to http redirect is also forbidden intentionally.
+    # It avoids sending secure cookie or referer by non-secure HTTP protocol.
+    # (RFC 2109 4.3.1, RFC 2965 3.3, RFC 2616 15.1.3)
     # However this is ad hoc.  It should be extensible/configurable.
     uri1.scheme.downcase == uri2.scheme.downcase ||
     (/\A(?:http|ftp)\z/i =~ uri1.scheme && /\A(?:http|ftp)\z/i =~ uri2.scheme)
@@ -282,7 +290,6 @@ module OpenURI
       else
         store.set_default_paths
       end
-      store.set_default_paths
       http.cert_store = store
     end
     if options.include? :read_timeout
@@ -291,16 +298,6 @@ module OpenURI
 
     resp = nil
     http.start {
-      if target.class == URI::HTTPS
-        # xxx: information hiding violation
-        sock = http.instance_variable_get(:@socket)
-        if sock.respond_to?(:io)
-          sock = sock.io # 1.9
-        else
-          sock = sock.instance_variable_get(:@socket) # 1.8
-        end
-        sock.post_connection_check(target_host)
-      end
       req = Net::HTTP::Get.new(request_uri, header)
       if options.include? :http_basic_authentication
         user, pass = options[:http_basic_authentication]
@@ -333,7 +330,12 @@ module OpenURI
          Net::HTTPFound, # 302
          Net::HTTPSeeOther, # 303
          Net::HTTPTemporaryRedirect # 307
-      throw :open_uri_redirect, URI.parse(resp['location'])
+      begin
+        loc_uri = URI.parse(resp['location'])
+      rescue URI::InvalidURIError
+        raise OpenURI::HTTPError.new(io.status.join(' ') + ' (Invalid Location URI)', io)
+      end
+      throw :open_uri_redirect, loc_uri
     else
       raise OpenURI::HTTPError.new(io.status.join(' '), io)
     end
@@ -347,6 +349,14 @@ module OpenURI
     attr_reader :io
   end
 
+  class HTTPRedirect < HTTPError
+    def initialize(message, io, uri)
+      super(message, io)
+      @uri = uri
+    end
+    attr_reader :uri
+  end
+
   class Buffer # :nodoc:
     def initialize
       @io = StringIO.new
@@ -362,14 +372,14 @@ module OpenURI
         require 'tempfile'
         io = Tempfile.new('open-uri')
         io.binmode
-        Meta.init io, @io if @io.respond_to? :meta
+        Meta.init io, @io if Meta === @io
         io << @io.string
         @io = io
       end
     end
 
     def io
-      Meta.init @io unless @io.respond_to? :meta
+      Meta.init @io unless Meta === @io
       @io
     end
   end
@@ -402,8 +412,29 @@ module OpenURI
     # The Hash keys are downcased for canonicalization.
     attr_reader :meta
 
+    def meta_setup_encoding # :nodoc:
+      charset = self.charset
+      enc = nil
+      if charset
+        begin
+          enc = Encoding.find(charset)
+        rescue ArgumentError
+        end
+      end
+      enc = Encoding::ASCII_8BIT unless enc
+      if self.respond_to? :force_encoding
+        self.force_encoding(enc)
+      elsif self.respond_to? :string
+        self.string.force_encoding(enc)
+      else # Tempfile
+        self.set_encoding enc
+      end
+    end
+
     def meta_add_field(name, value) # :nodoc:
-      @meta[name.downcase] = value
+      name = name.downcase
+      @meta[name] = value
+      meta_setup_encoding if name == 'content-type'
     end
 
     # returns a Time which represents Last-Modified field.
@@ -428,7 +459,7 @@ module OpenURI
         subtype = $2.downcase
         parameters = []
         $3.scan(/;#{RE_LWS}?(#{RE_TOKEN})#{RE_LWS}?=#{RE_LWS}?(?:(#{RE_TOKEN})|(#{RE_QUOTED_STRING}))/no) {|att, val, qval|
-          val = qval.gsub(/[\r\n\t !#-\[\]-~\x80-\xff]+|(\\[\x00-\x7f])/) { $1 ? $1[1,1] : $& } if qval
+          val = qval.gsub(/[\r\n\t !#-\[\]-~\x80-\xff]+|(\\[\x00-\x7f])/n) { $1 ? $1[1,1] : $& } if qval
           parameters << [att.downcase, val]
         }
         ["#{type}/#{subtype}", *parameters]
@@ -610,6 +641,24 @@ module OpenURI
     # OpenURI::OpenRead#open returns an IO like object if block is not given.
     # Otherwise it yields the IO object and return the value of the block.
     # The IO object is extended with OpenURI::Meta.
+    #
+    # [:ftp_active_mode]
+    #  Synopsis:
+    #    :ftp_active_mode=>bool
+    #
+    # :ftp_active_mode=>true is used to make ftp active mode.
+    # Note that the active mode is default in Ruby 1.8 or prior.
+    # Ruby 1.9 uses passive mode by default.
+    #
+    # [:redirect]
+    #  Synopsis:
+    #    :redirect=>bool
+    #
+    # :redirect=>false is used to disable HTTP redirects at all.
+    # OpenURI::HTTPRedirect exception raised on redirection.
+    # It is true by default.
+    # The true means redirections between http and ftp is permitted.
+    #
     def open(*rest, &block)
       OpenURI.open_uri(self, *rest, &block)
     end
@@ -663,7 +712,7 @@ module URI
             proxy_uri = nil
           end
         else # http_proxy is safe to use because ENV is case sensitive.
-          proxy_uri = ENV[name]
+          proxy_uri = ENV.to_hash[name]
         end
         if !proxy_uri
           # Use CGI_HTTP_PROXY.  cf. libwww-perl.
@@ -746,6 +795,7 @@ module URI
 
       # The access sequence is defined by RFC 1738
       ftp = Net::FTP.open(self.host)
+      ftp.passive = true if !options[:ftp_active_mode]
       # todo: extract user/passwd from .netrc.
       user = 'anonymous'
       passwd = nil
