@@ -15,6 +15,10 @@
 # == Pathname
 #
 # Pathname represents a pathname which locates a file in a filesystem.
+# The pathname depends on OS: Unix, Windows, etc.
+# Pathname library works with pathnames of local OS.
+# However non-Unix pathnames are supported experimentally.
+#
 # It does not represent the file itself.
 # A Pathname can be relative or absolute.  It's not until you try to
 # reference the file that it even matters whether the file exists or not.
@@ -182,12 +186,22 @@
 # information.  In some cases, a brief description will follow.
 #
 class Pathname
+
+  # :stopdoc:
+  if RUBY_VERSION < "1.9"
+    TO_PATH = :to_str
+  else
+    # to_path is implemented so Pathname objects are usable with File.open, etc.
+    TO_PATH = :to_path
+  end
+  # :startdoc:
+
   #
   # Create a Pathname object from the given String (or String-like object).
   # If +path+ contains a NUL character (<tt>\0</tt>), an ArgumentError is raised.
   #
   def initialize(path)
-    path = path.to_path if path.respond_to? :to_path
+    path = path.__send__(TO_PATH) if path.respond_to? TO_PATH
     @path = path.dup
 
     if /\0/ =~ @path
@@ -229,7 +243,7 @@ class Pathname
   end
 
   # to_path is implemented so Pathname objects are usable with File.open, etc.
-  alias to_path to_s
+  alias_method TO_PATH, :to_s
 
   def inspect # :nodoc:
     "#<#{self.class}:#{@path}>"
@@ -241,11 +255,22 @@ class Pathname
   end
 
   if File::ALT_SEPARATOR
-    SEPARATOR_PAT = /[#{Regexp.quote File::ALT_SEPARATOR}#{Regexp.quote File::SEPARATOR}]/
+    SEPARATOR_LIST = "#{Regexp.quote File::ALT_SEPARATOR}#{Regexp.quote File::SEPARATOR}"
+    SEPARATOR_PAT = /[#{SEPARATOR_LIST}]/
   else
+    SEPARATOR_LIST = "#{Regexp.quote File::SEPARATOR}"
     SEPARATOR_PAT = /#{Regexp.quote File::SEPARATOR}/
   end
 
+  # Return a pathname which the extension of the basename is substituted by
+  # <i>repl</i>.
+  #
+  # If self has no extension part, <i>repl</i> is appended.
+  def sub_ext(repl)
+    ext = File.extname(@path)
+    self.class.new(@path.chomp(ext) + repl)
+  end
+
   # chop_basename(path) -> [pre-basename, basename] or nil
   def chop_basename(path)
     base = File.basename(path)
@@ -491,9 +516,10 @@ class Pathname
   #   Pathname.new("/usr/bin/ruby").each_filename {|filename| ... }
   #     # yields "usr", "bin", and "ruby".
   #
-  def each_filename # :yield: s
+  def each_filename # :yield: filename
     prefix, names = split_names(@path)
     names.each {|filename| yield filename }
+    nil
   end
 
   # Iterates over and yields a new Pathname object
@@ -514,6 +540,8 @@ class Pathname
   #
   # It doesn't access actual filesystem.
   #
+  # This method is available since 1.8.5.
+  #
   def descend
     vs = []
     ascend {|v| vs << v }
@@ -539,6 +567,8 @@ class Pathname
   #
   # It doesn't access actual filesystem.
   #
+  # This method is available since 1.8.5.
+  #
   def ascend
     path = @path
     yield self
@@ -1031,3 +1061,13 @@ class Pathname    # * mixed *
     end
   end
 end
+
+module Kernel
+  # create a pathname object.
+  #
+  # This method is available since 1.8.5.
+  def Pathname(path) # :doc:
+    Pathname.new(path)
+  end
+  private :Pathname
+end
