@@ -82,7 +82,7 @@ class RDoc::Generator::HTML
     @classes    = []
 
     write_style_sheet
-    gen_sub_directories
+    gen_sub_directories()
     build_indices
     generate_html
   end
@@ -157,7 +157,6 @@ class RDoc::Generator::HTML
     # the individual descriptions for files and classes
     gen_into(@files)
     gen_into(@classes)
-
     # and the index files
     gen_file_index
     gen_class_index
@@ -169,21 +168,14 @@ class RDoc::Generator::HTML
   end
 
   def gen_into(list)
-    @file_list ||= index_to_links @files
-    @class_list ||= index_to_links @classes
-    @method_list ||= index_to_links RDoc::Generator::Method.all_methods
-
     list.each do |item|
-      next unless item.document_self
-
-      op_file = item.path
-
-      FileUtils.mkdir_p File.dirname(op_file)
-
-      open op_file, 'w' do |io|
-        item.write_on io, @file_list, @class_list, @method_list
+      if item.document_self
+        op_file = item.path
+        FileUtils.mkdir_p(File.dirname(op_file))
+        open(op_file, "w") { |file| item.write_on(file) }
       end
     end
+
   end
 
   def gen_file_index
@@ -229,23 +221,9 @@ class RDoc::Generator::HTML
   # line.
 
   def gen_main_index
-    if @template.const_defined? :FRAMELESS then
-      main = @files.find do |file|
-        @main_page == file.name
-      end
-
-      if main.nil? then
-        main = @classes.find do |klass|
-          main_page == klass.context.full_name
-        end
-      end
-    else
-      main = RDoc::TemplatePage.new @template::INDEX
-    end
+    template = RDoc::TemplatePage.new @template::INDEX
 
     open 'index.html', 'w'  do |f|
-      style_url = style_url '', @options.css
-
       classes = @classes.sort.map { |klass| klass.value_hash }
 
       values = {
@@ -259,31 +237,18 @@ class RDoc::Generator::HTML
 
       values['inline_source'] = @options.inline_source
 
-      if main.respond_to? :write_on then
-        main.write_on f, @file_list, @class_list, @method_list, values
-      else
-        main.write_html_on f, values
-      end
+      template.write_html_on f, values
     end
   end
 
-  def index_to_links(collection)
-    collection.sort.map do |f|
-      next unless f.document_self
-      { "href" => f.path, "name" => f.index_name }
-    end.compact
-  end
-
   ##
   # Returns the url of the main page
 
   def main_url
     @main_page = @options.main_page
     @main_page_ref = nil
-
-    if @main_page then
+    if @main_page
       @main_page_ref = RDoc::Generator::AllReferences[@main_page]
-
       if @main_page_ref then
         @main_page_path = @main_page_ref.path
       else
@@ -386,8 +351,15 @@ class RDoc::Generator::HTMLInOne < RDoc::Generator::HTML
   end
 
   def gen_an_index(collection, title)
+    res = []
+    collection.sort.each do |f|
+      if f.document_self
+        res << { "href" => f.path, "name" => f.index_name }
+      end
+    end
+
     return {
-      "entries" => index_to_links(collection),
+      "entries" => res,
       'list_title' => title,
       'index_url'  => main_url,
     }
@@ -395,3 +367,4 @@ class RDoc::Generator::HTMLInOne < RDoc::Generator::HTML
 
 end
 
+
