@@ -3,7 +3,6 @@
 
 require "rdoc/code_objects"
 require "rdoc/parsers/parserfactory"
-require "rdoc/options"
 require "rdoc/rdoc"
 
 module RDoc
@@ -37,6 +36,7 @@ module RDoc
     "rb_cTime"             => "Time",
     "rb_cTrueClass"        => "TrueClass",
     "rb_cStruct"           => "Struct",
+    "rb_cVM"               => "VM",
     "rb_eException"        => "Exception",
     "rb_eStandardError"    => "StandardError",
     "rb_eSystemExit"       => "SystemExit",
@@ -117,10 +117,10 @@ module RDoc
   # method in the HTML output. This method must be in the same source file
   # as the rb_define_method.
   #
-  # C classes can be diagramed (see /tc/dl/ruby/ruby/error.c), and RDoc
+  # C classes can be diagrammed (see /tc/dl/ruby/ruby/error.c), and RDoc
   # integrates C and Ruby source into one tree
   #
-  # The comment blocks may include special direcives:
+  # The comment blocks may include special directives:
   #
   # [Document-class: <i>name</i>]
   #   This comment block is documentation for the given class. Use this
@@ -131,10 +131,10 @@ module RDoc
   #   automatically find the method from it's declaration
   #
   # [call-seq:  <i>text up to an empty line</i>]
-  #   Because C source doesn't give descripive names to Ruby-level parameters,
+  #   Because C source doesn't give descriptive names to Ruby-level parameters,
   #   you need to document the calling sequence explicitly
   #
-  # In additon, RDoc assumes by default that the C method implementing a 
+  # In addition, RDoc assumes by default that the C method implementing a 
   # Ruby function is in the same source file as the rb_define_method call.
   # If this isn't the case, add the comment 
   #
@@ -177,13 +177,13 @@ module RDoc
     # prepare to parse a C file
     def initialize(top_level, file_name, body, options, stats)
       @known_classes = KNOWN_CLASSES.dup
-      @body = handle_tab_width(handle_ifdefs_in(body))
       @options = options
+      @body = handle_tab_width(handle_ifdefs_in(body))
       @stats   = stats
       @top_level = top_level
       @classes = Hash.new
       @file_dir = File.dirname(file_name)
-      @progress = $stderr unless options.quiet
+      @progress = $stderr unless @options.quiet
     end
 
     # Extract the classes/modules and methods from a C file
@@ -627,8 +627,8 @@ module RDoc
     # Find the C code corresponding to a Ruby method
     def find_body(meth_name, meth_obj, body, quiet = false)
       case body
-      when %r{((?>/\*.*?\*/\s*))(?:static\s+)?VALUE\s+#{meth_name}
-              \s*(\([^)]*\))\s*\{.*?^\}}xm
+      when %r"((?>/\*.*?\*/\s*))(?:static\s+)?VALUE\s+#{meth_name}
+              \s*(\([^)]*\))\s*\{.*?^\}"xm
         comment, params = $1, $2
         body_text = $&
 
@@ -752,7 +752,7 @@ module RDoc
 
     def handle_tab_width(body)
       if /\t/ =~ body
-        tab_width = Options.instance.tab_width
+        tab_width = @options.tab_width
         body.split(/\n/).map do |line|
           1 while line.gsub!(/\t+/) { ' ' * (tab_width*$&.length - $`.length % tab_width)}  && $~ #`
           line
@@ -766,7 +766,7 @@ module RDoc
     # Removes #ifdefs that would otherwise confuse us
     
     def handle_ifdefs_in(body)
-      body.gsub(/^#ifdef HAVE_PROTOTYPES.*?#else.*?\n(.*?)#endif.*?\n/m) { $1 }
+      body.gsub(/^#ifdef HAVE_PROTOTYPES.*?#else.*?\n(.*?)#endif.*?\n/m, '\1')
     end
     
   end
