@@ -28,7 +28,6 @@ require 'thread'
 # 
 # * NIS is not supported.
 # * /etc/nsswitch.conf is not supported.
-# * IPv6 is not supported.
 
 class Resolv
 
@@ -161,7 +160,7 @@ class Resolv
   # DNS::Hosts is a hostname resolver that uses the system hosts file.
 
   class Hosts
-    if /mswin32|cygwin|mingw|bccwin/ =~ RUBY_PLATFORM
+    if /mswin32|mingw|bccwin/ =~ RUBY_PLATFORM
       require 'win32/resolv'
       DefaultFileName = Win32::Resolv.get_hosts_path
     else
@@ -253,7 +252,7 @@ class Resolv
     end
 
     ##
-    # Iterates over all hostnames for +address+ retrived from the hosts file.
+    # Iterates over all hostnames for +address+ retrieved from the hosts file.
 
     def each_name(address, &proc)
       lazy_initialize
@@ -389,6 +388,7 @@ class Resolv
 
     def each_address(name)
       each_resource(name, Resource::IN::A) {|resource| yield resource.address}
+      each_resource(name, Resource::IN::AAAA) {|resource| yield resource.address}
     end
 
     ##
@@ -445,7 +445,6 @@ class Resolv
     # * Resolv::DNS::Resource::IN::A
     # * Resolv::DNS::Resource::IN::AAAA
     # * Resolv::DNS::Resource::IN::ANY
-    # * Resolv::DNS::Resource::IN::ANY
     # * Resolv::DNS::Resource::IN::CNAME
     # * Resolv::DNS::Resource::IN::HINFO
     # * Resolv::DNS::Resource::IN::MINFO
@@ -778,6 +777,7 @@ class Resolv
           config_hash = Config.parse_resolv_conf(filename)
         else
           if /mswin32|cygwin|mingw|bccwin/ =~ RUBY_PLATFORM
+            require 'win32/resolv'
             search, nameserver = Win32::Resolv.get_resolv_info
             config_hash = {}
             config_hash[:nameserver] = nameserver if nameserver
@@ -1032,7 +1032,7 @@ class Resolv
 
       def ==(other) # :nodoc:
         return false unless Name === other
-        return @labels == other.to_a && @absolute == other.absolute?
+        return @labels.join == other.to_a.join && @absolute == other.absolute?
       end
 
       alias eql? == # :nodoc:
@@ -1262,7 +1262,7 @@ class Resolv
         end
 
         def put_label(d)
-          self.put_string(d.string)
+          self.put_string(d.to_s)
         end
       end
 
@@ -2026,7 +2026,7 @@ class Resolv
     end
 
     ##
-    # A String reperesentation of this IPv4 address.
+    # A String representation of this IPv4 address.
 
     ##
     # The raw IPv4 address as a String.
@@ -2191,7 +2191,7 @@ class Resolv
 
     def to_name
       return DNS::Name.new(
-        @address.unpack("H32")[0].split(//).reverse + ['ip6', 'int'])
+        @address.unpack("H32")[0].split(//).reverse + ['ip6', 'arpa'])
     end
 
     def ==(other) # :nodoc:
