@@ -53,7 +53,7 @@ require "rss/xml-stylesheet"
 
 module RSS
 
-  VERSION = "0.1.9"
+  VERSION = "0.2.4"
 
   URI = "http://purl.org/rss/1.0/"
 
@@ -208,7 +208,7 @@ EOC
                              disp_name=nil)
       name ||= tag_name
       disp_name ||= name
-      self::ELEMENTS << name
+      self::ELEMENTS << name unless self::ELEMENTS.include?(name)
       add_need_initialize_variable(name)
       install_model(tag_name, uri, occurs, name)
 
@@ -361,6 +361,12 @@ EOC
     end
 
     def csv_attr_reader(*attrs)
+      separator = nil
+      if attrs.last.is_a?(Hash)
+        options = attrs.pop
+        separator = options[:separator]
+      end
+      separator ||= ", "
       attrs.each do |attr|
         attr = attr.id2name if attr.kind_of?(Integer)
         module_eval(<<-EOC, __FILE__, __LINE__ + 1)
@@ -369,7 +375,7 @@ EOC
             if @#{attr}.nil?
               @#{attr}
             else
-              @#{attr}.join(", ")
+              @#{attr}.join(#{separator.dump})
             end
           end
         EOC
@@ -531,6 +537,14 @@ EOC
       EOC
     end
 
+    def csv_integer_writer(name, disp_name=name)
+      module_eval(<<-EOC, __FILE__, __LINE__ + 1)
+        def #{name}=(new_value)
+          @#{name} = Utils::CSV.parse(new_value) {|v| Integer(v)}
+        end
+      EOC
+    end
+
     def def_children_accessor(accessor_name, plural_name)
       module_eval(<<-EOC, *get_file_and_line_from_caller(2))
       def #{plural_name}
@@ -720,6 +734,8 @@ EOC
           yes_other_writer name, disp_name
         when :csv
           csv_writer name
+        when :csv_integer
+          csv_integer_writer name
         else
           attr_writer name
         end
@@ -737,6 +753,8 @@ EOC
           yes_other_attr_reader name
         when :csv
           csv_attr_reader name
+        when :csv_integer
+          csv_attr_reader name, :separator => ","
         else
           convert_attr_reader name
         end
@@ -1183,7 +1201,7 @@ EOC
         __send__(self.class.xml_getter).to_s
       else
         _content = content
-        _content = Base64.encode64(_content) if need_base64_encode?
+        _content = [_content].pack("m").delete("\n") if need_base64_encode?
         h(_content)
       end
     end
