@@ -11,9 +11,9 @@ require 'rubygems/remote_fetcher'
 #   @fetcher = Gem::FakeFetcher.new
 #   @fetcher.data['http://gems.example.com/yaml'] = source_index.to_yaml
 #   Gem::RemoteFetcher.fetcher = @fetcher
-#
+#   
 #   # invoke RubyGems code
-#
+#   
 #   paths = @fetcher.paths
 #   assert_equal 'http://gems.example.com/yaml', paths.shift
 #   assert paths.empty?, paths.join(', ')
@@ -23,7 +23,6 @@ require 'rubygems/remote_fetcher'
 class Gem::FakeFetcher
 
   attr_reader :data
-  attr_reader :last_request
   attr_accessor :paths
 
   def initialize
@@ -31,73 +30,34 @@ class Gem::FakeFetcher
     @paths = []
   end
 
-  def find_data(path)
+  def fetch_path(path)
     path = path.to_s
     @paths << path
-    raise ArgumentError, 'need full URI' unless path =~ %r'^https?://'
+    raise ArgumentError, 'need full URI' unless path =~ %r'^http://'
+    data = @data[path]
 
-    unless @data.key? path then
-      raise Gem::RemoteFetcher::FetchError.new("no data for #{path}", path)
+    if data.nil? then
+      raise Gem::RemoteFetcher::FetchError.new('no data', path)
     end
 
-    @data[path]
-  end
-
-  def fetch_path path, mtime = nil
-    data = find_data(path)
-
-    if data.respond_to?(:call) then
-      data.call
-    else
-      if path.to_s =~ /gz$/ and not data.nil? and not data.empty? then
-        data = Gem.gunzip data
-      end
-
-      data
-    end
-  end
-
-  # Thanks, FakeWeb!
-  def open_uri_or_path(path)
-    data = find_data(path)
-    body, code, msg = data
-
-    response = Net::HTTPResponse.send(:response_class, code.to_s).new("1.0", code.to_s, msg)
-    response.instance_variable_set(:@body, body)
-    response.instance_variable_set(:@read, true)
-    response
-  end
-
-  def request(uri, request_class, last_modified = nil)
-    data = find_data(uri)
-    body, code, msg = data
-
-    @last_request = request_class.new uri.request_uri
-    yield @last_request if block_given?
-
-    response = Net::HTTPResponse.send(:response_class, code.to_s).new("1.0", code.to_s, msg)
-    response.instance_variable_set(:@body, body)
-    response.instance_variable_set(:@read, true)
-    response
+    data.respond_to?(:call) ? data.call : data
   end
 
   def fetch_size(path)
     path = path.to_s
     @paths << path
-
     raise ArgumentError, 'need full URI' unless path =~ %r'^http://'
-
-    unless @data.key? path then
-      raise Gem::RemoteFetcher::FetchError.new("no data for #{path}", path)
-    end
-
     data = @data[path]
 
+    if data.nil? then
+      raise Gem::RemoteFetcher::FetchError.new("no data for #{path}", nil)
+    end
+
     data.respond_to?(:call) ? data.call : data.length
   end
 
   def download spec, source_uri, install_dir = Gem.dir
-    name = spec.file_name
+    name = "#{spec.full_name}.gem"
     path = File.join(install_dir, 'cache', name)
 
     Gem.ensure_gem_subdirectories install_dir
@@ -131,17 +91,30 @@ end
 #--
 # This class was added to flush out problems in Rubinius' IO implementation.
 
-class TempIO < Tempfile
+class TempIO
+
+  @@count = 0
+
   def initialize(string = '')
-    super "TempIO"
-    binmode
-    write string
-    rewind
+    @tempfile = Tempfile.new "TempIO-#{@@count += 1}"
+    @tempfile.binmode
+    @tempfile.write string
+    @tempfile.rewind
+  end
+
+  def method_missing(meth, *args, &block)
+    @tempfile.send(meth, *args, &block)
+  end
+
+  def respond_to?(meth)
+    @tempfile.respond_to? meth
   end
 
   def string
-    flush
-    Gem.read_binary path
+    @tempfile.flush
+
+    Gem.read_binary @tempfile.path
   end
+
 end
 
