@@ -1,10 +1,9 @@
 # scanf for Ruby
 #
 # $Release Version: 1.1.2 $
-# $Revision: 1.7 $
-# $Id: scanf.rb,v 1.7 2004/03/06 02:14:33 dblack Exp $
-# $Author: dblack $
-# $Date: 2004/03/06 02:14:33 $
+# $Revision: 14912 $
+# $Id: scanf.rb 14912 2008-01-06 15:49:38Z akr $
+# $Author: akr $
 #
 # A product of the Austin Ruby Codefest (Austin, Texas, August 2002)
 
@@ -467,11 +466,11 @@ module Scanf
     end
 
     def letter
-      /%\*?\d*([a-z\[])/.match(@spec_string).to_a[1]
+      @spec_string[/%\*?\d*([a-z\[])/, 1]
     end
 
     def width
-      w = /%\*?(\d+)/.match(@spec_string).to_a[1]
+      w = @spec_string[/%\*?(\d+)/, 1]
       w && w.to_i
     end
 
@@ -542,7 +541,8 @@ module Scanf
       @string_left = str
       @matched_count = 0
 
-      @specs.each_with_index do |spec,@i|
+      @specs.each_with_index do |spec,i|
+        @i=i
         @last_spec_tried = spec
         @last_match_tried = spec.match(@string_left)
         break unless @last_match_tried
