@@ -2,14 +2,14 @@
 #--
 # set.rb - defines the Set class
 #++
-# Copyright (c) 2002 Akinori MUSHA <knu@iDaemons.org>
+# Copyright (c) 2002-2008 Akinori MUSHA <knu@iDaemons.org>
 #
 # Documentation by Akinori MUSHA and Gavin Sinclair. 
 #
 # All rights reserved.  You can redistribute and/or modify it under the same
 # terms as Ruby.
 #
-#   $Id: set.rb,v 1.26 2005/05/29 04:55:48 akr Exp $
+#   $Id: set.rb 17045 2008-06-09 08:58:30Z knu $
 #
 # == Overview 
 # 
@@ -28,14 +28,13 @@
 # This is a hybrid of Array's intuitive inter-operation facilities and
 # Hash's fast lookup.
 #
-# Several methods accept any Enumerable object (implementing +each+)
-# for greater flexibility: new, replace, merge, subtract, |, &, -, ^.
-#
 # The equality of each couple of elements is determined according to
 # Object#eql? and Object#hash, since Set uses Hash as storage.
 #
-# Finally, if you are using class Set, you can also use Enumerable#to_set
-# for convenience.
+# Set is easy to use with Enumerable objects (implementing +each+).
+# Most of the initializer methods and binary operators accept generic
+# Enumerable objects besides sets and arrays.  An Enumerable object
+# can be converted to Set using the +to_set+ method.
 #
 # == Example
 #
@@ -48,6 +47,10 @@
 #   s1.subset? s2                         # -> false
 #   s2.subset? s1                         # -> true
 #
+# == Contact
+#
+#   - Akinori MUSHA <knu@iDaemons.org> (current maintainer)
+#
 class Set
   include Enumerable
 
@@ -78,6 +81,24 @@ class Set
     @hash = orig.instance_eval{@hash}.dup
   end
 
+  def freeze	# :nodoc:
+    super
+    @hash.freeze
+    self
+  end
+
+  def taint	# :nodoc:
+    super
+    @hash.taint
+    self
+  end
+
+  def untaint	# :nodoc:
+    super
+    @hash.untaint
+    self
+  end
+
   # Returns the number of elements.
   def size
     @hash.size
@@ -101,7 +122,6 @@ class Set
     if enum.class == self.class
       @hash.replace(enum.instance_eval { @hash })
     else
-      enum.is_a?(Enumerable) or raise ArgumentError, "value must be enumerable"
       clear
       enum.each { |o| add(o) }
     end
@@ -184,16 +204,18 @@ class Set
   end
 
   # Calls the given block once for each element in the set, passing
-  # the element as parameter.
+  # the element as parameter.  Returns an enumerator if no block is
+  # given.
   def each
+    block_given? or return enum_for(__method__)
     @hash.each_key { |o| yield(o) }
     self
   end
 
   # Adds the given object to the set and returns self.  Use +merge+ to
-  # add several elements at once.
+  # add many elements at once.
   def add(o)
-    @hash[o] = o
+    @hash[o] = true
     self
   end
   alias << add
@@ -209,7 +231,7 @@ class Set
   end
 
   # Deletes the given object from the set and returns self.  Use +subtract+ to
-  # delete several items at once.
+  # delete many items at once.
   def delete(o)
     @hash.delete(o)
     self
@@ -228,12 +250,14 @@ class Set
   # Deletes every element of the set for which block evaluates to
   # true, and returns self.
   def delete_if
-    @hash.delete_if { |o,| yield(o) }
+    block_given? or return enum_for(__method__)
+    to_a.each { |o| @hash.delete(o) if yield(o) }
     self
   end
 
-  # Do collect() destructively.
+  # Replaces the elements with ones returned by collect().
   def collect!
+    block_given? or return enum_for(__method__)
     set = self.class.new
     each { |o| set << yield(o) }
     replace(set)
@@ -243,6 +267,7 @@ class Set
   # Equivalent to Set#delete_if, but returns nil if no changes were
   # made.
   def reject!
+    block_given? or return enum_for(__method__)
     n = size
     delete_if { |o| yield(o) }
     size == n ? nil : self
@@ -251,10 +276,9 @@ class Set
   # Merges the elements of the given enumerable object to the set and
   # returns self.
   def merge(enum)
-    if enum.class == self.class
+    if enum.is_a?(Set)
       @hash.update(enum.instance_eval { @hash })
     else
-      enum.is_a?(Enumerable) or raise ArgumentError, "value must be enumerable"
       enum.each { |o| add(o) }
     end
 
@@ -264,7 +288,6 @@ class Set
   # Deletes every element that appears in the given enumerable object
   # and returns self.
   def subtract(enum)
-    enum.is_a?(Enumerable) or raise ArgumentError, "value must be enumerable"
     enum.each { |o| delete(o) }
     self
   end
@@ -272,7 +295,6 @@ class Set
   # Returns a new set built by merging the set and the elements of the
   # given enumerable object.
   def |(enum)
-    enum.is_a?(Enumerable) or raise ArgumentError, "value must be enumerable"
     dup.merge(enum)
   end
   alias + |		##
@@ -281,28 +303,25 @@ class Set
   # Returns a new set built by duplicating the set, removing every
   # element that appears in the given enumerable object.
   def -(enum)
-    enum.is_a?(Enumerable) or raise ArgumentError, "value must be enumerable"
     dup.subtract(enum)
   end
   alias difference -	##
 
-  # Returns a new array containing elements common to the set and the
+  # Returns a new set containing elements common to the set and the
   # given enumerable object.
   def &(enum)
-    enum.is_a?(Enumerable) or raise ArgumentError, "value must be enumerable"
     n = self.class.new
-    enum.each { |o| include?(o) and n.add(o) }
+    enum.each { |o| n.add(o) if include?(o) }
     n
   end
   alias intersection &	##
 
-  # Returns a new array containing elements exclusive between the set
+  # Returns a new set containing elements exclusive between the set
   # and the given enumerable object.  (set ^ enum) is equivalent to
   # ((set | enum) - (set & enum)).
   def ^(enum)
-    enum.is_a?(Enumerable) or raise ArgumentError, "value must be enumerable"
-    n = dup
-    enum.each { |o| if n.include?(o) then n.delete(o) else n.add(o) end }
+    n = Set.new(enum)
+    each { |o| if n.include?(o) then n.delete(o) else n.add(o) end }
     n
   end
 
@@ -313,7 +332,8 @@ class Set
 
     set.is_a?(Set) && size == set.size or return false
 
-    set.all? { |o| @hash.value?(o) }
+    hash = @hash.dup
+    set.all? { |o| hash.include?(o) }
   end
 
   def hash	# :nodoc:
@@ -321,7 +341,8 @@ class Set
   end
 
   def eql?(o)	# :nodoc:
-    @hash.hash == o.hash
+    return false unless o.is_a?(Set)
+    @hash.eql?(o.instance_eval{@hash})
   end
 
   # Classifies the set by the return value of the given block and
@@ -338,6 +359,8 @@ class Set
   #             #     2001=>#<Set: {"c.rb", "d.rb", "e.rb"}>,
   #             #     2002=>#<Set: {"f.rb"}>}
   def classify # :yields: o
+    block_given? or return enum_for(__method__)
+
     h = {}
 
     each { |i|
@@ -365,6 +388,8 @@ class Set
   #             #            #<Set: {3, 4}>,
   #             #            #<Set: {6}>}>
   def divide(&func)
+    func or return enum_for(__method__)
+
     if func.arity == 2
       require 'tsort'
 
@@ -471,7 +496,7 @@ class SortedSet < Set
 
 	  def add(o)
 	    @keys = nil
-	    @hash[o] = o
+	    @hash[o] = true
 	    self
 	  end
 	  alias << add
@@ -483,8 +508,9 @@ class SortedSet < Set
 	  end
 
 	  def delete_if
+            block_given? or return enum_for(__method__)
 	    n = @hash.size
-	    @hash.delete_if { |o,| yield(o) }
+	    super
 	    @keys = nil if @hash.size != n
 	    self
 	  end
@@ -495,7 +521,9 @@ class SortedSet < Set
 	  end
 
 	  def each
+	    block_given? or return enum_for(__method__)
 	    to_a.each { |o| yield(o) }
+	    self
 	  end
 
 	  def to_a
@@ -517,6 +545,7 @@ end
 
 module Enumerable
   # Makes a set from the enumerable object with given arguments.
+  # Needs to +require "set"+ to use this method.
   def to_set(klass = Set, *args, &block)
     klass.new(self, *args, &block)
   end
@@ -556,7 +585,7 @@ end
 #     if @proc.arity == 2
 #       instance_eval %{
 # 	def add(o)
-# 	  @hash[o] = o if @proc.call(self, o)
+# 	  @hash[o] = true if @proc.call(self, o)
 # 	  self
 # 	end
 # 	alias << add
@@ -565,13 +594,12 @@ end
 # 	  if include?(o) || !@proc.call(self, o)
 # 	    nil
 # 	  else
-# 	    @hash[o] = o
+# 	    @hash[o] = true
 # 	    self
 # 	  end
 # 	end
 # 
 # 	def replace(enum)
-# 	  enum.is_a?(Enumerable) or raise ArgumentError, "value must be enumerable"
 # 	  clear
 # 	  enum.each { |o| add(o) }
 # 
@@ -579,7 +607,6 @@ end
 # 	end
 # 
 # 	def merge(enum)
-# 	  enum.is_a?(Enumerable) or raise ArgumentError, "value must be enumerable"
 # 	  enum.each { |o| add(o) }
 # 
 # 	  self
@@ -588,7 +615,9 @@ end
 #     else
 #       instance_eval %{
 # 	def add(o)
-# 	  @hash[o] = o if @proc.call(o)
+#         if @proc.call(o)
+# 	    @hash[o] = true 
+#         end
 # 	  self
 # 	end
 # 	alias << add
@@ -597,7 +626,7 @@ end
 # 	  if include?(o) || !@proc.call(o)
 # 	    nil
 # 	  else
-# 	    @hash[o] = o
+# 	    @hash[o] = true
 # 	    self
 # 	  end
 # 	end
@@ -644,12 +673,11 @@ class TC_Set < Test::Unit::TestCase
       Set.new([])
       Set.new([1,2])
       Set.new('a'..'c')
-      Set.new('XYZ')
     }
-    assert_raises(ArgumentError) {
+    assert_raises(NoMethodError) {
       Set.new(false)
     }
-    assert_raises(ArgumentError) {
+    assert_raises(NoMethodError) {
       Set.new(1)
     }
     assert_raises(ArgumentError) {
@@ -909,9 +937,11 @@ class TC_Set < Test::Unit::TestCase
     ary = [1,3,5,7,10,20]
     set = Set.new(ary)
 
-    assert_raises(LocalJumpError) {
-      set.each
-    }
+    ret = set.each { |o| }
+    assert_same(set, ret)
+
+    e = set.each
+    assert_instance_of(Enumerable::Enumerator, e)
 
     assert_nothing_raised {
       set.each { |o|
@@ -1044,6 +1074,13 @@ class TC_Set < Test::Unit::TestCase
     assert_equal(Set[2,4], ret)
   end
 
+  def test_xor
+    set = Set[1,2,3,4]
+    ret = set ^ [2,4,5,5]
+    assert_not_same(set, ret)
+    assert_equal(Set[1,3,5], ret)
+  end
+
   def test_eq
     set1 = Set[2,3,1]
     set2 = Set[1,2,3]
@@ -1149,11 +1186,33 @@ class TC_SortedSet < Test::Unit::TestCase
     assert_equal([-10,-8,-6,-4,-2], s.to_a)
 
     prev = nil
-    s.each { |o| assert(prev < o) if prev; prev = o }
+    ret = s.each { |o| assert(prev < o) if prev; prev = o }
     assert_not_nil(prev)
+    assert_same(s, ret)
 
     s = SortedSet.new([2,1,3]) { |o| o * -2 }
     assert_equal([-6,-4,-2], s.to_a)
+
+    s = SortedSet.new(['one', 'two', 'three', 'four'])
+    a = []
+    ret = s.delete_if { |o| a << o; o.start_with?('t') }
+    assert_same(s, ret)
+    assert_equal(['four', 'one'], s.to_a)
+    assert_equal(['four', 'one', 'three', 'two'], a)
+
+    s = SortedSet.new(['one', 'two', 'three', 'four'])
+    a = []
+    ret = s.reject! { |o| a << o; o.start_with?('t') }
+    assert_same(s, ret)
+    assert_equal(['four', 'one'], s.to_a)
+    assert_equal(['four', 'one', 'three', 'two'], a)
+
+    s = SortedSet.new(['one', 'two', 'three', 'four'])
+    a = []
+    ret = s.reject! { |o| a << o; false }
+    assert_same(nil, ret)
+    assert_equal(['four', 'one', 'three', 'two'], s.to_a)
+    assert_equal(['four', 'one', 'three', 'two'], a)
   end
 end
 
