@@ -1,9 +1,8 @@
 #
 #   shell/command-controller.rb - 
-#   	$Release Version: 0.6.0 $
-#   	$Revision: 1.10 $
-#   	$Date: 2004/04/18 23:19:47 $
-#   	by Keiju ISHITSUKA(Nippon Rational Inc.)
+#   	$Release Version: 0.7 $
+#   	$Revision: 14912 $
+#   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
 #
@@ -11,7 +10,6 @@
 #
 
 require "e2mmap"
-require "ftools"
 require "thread"
 
 require "shell/error"
@@ -26,7 +24,13 @@ class Shell
     #
     # initialize of Shell and related classes.
     #
-    NoDelegateMethods = ["initialize", "expand_path"]
+    m = [:initialize, :expand_path]
+    if Object.methods.first.kind_of?(String)
+      NoDelegateMethods = m.collect{|m| m.id2name}
+    else
+      NoDelegateMethods = m
+    end
+
     def self.initialize
 
       install_builtin_commands
@@ -111,13 +115,20 @@ class Shell
     #	  Dir#open  (when path is directory)
     #	mode has an effect only when path is a file
     #
-    def open(path, mode)
+    def open(path, mode = nil, perm = 0666, &b)
       path = expand_path(path)
       if File.directory?(path)
-	Dir.open(path)
+	Dir.open(path, &b)
       else
-	effect_umask do
-	  File.open(path, mode)
+	if @shell.umask
+	  f = File.open(path, mode, perm)
+	  File.chmod(perm & ~@shell.umask, path)
+	  if block_given?
+	    f.each &b
+	  end
+	  f
+	else
+	  f = File.open(path, mode, perm, &b)
 	end
       end
     end
@@ -130,12 +141,15 @@ class Shell
     #	  File#unlink (when path is file)
     #
     def unlink(path)
+      @shell.check_point
+
       path = expand_path(path)
       if File.directory?(path)
 	Dir.unlink(path)
       else
 	IO.unlink(path)
       end
+      Void.new(@shell)
     end
 
     #
@@ -155,6 +169,7 @@ class Shell
     #	  sh[:exists?, "foo"]
     #	  sh["exists?", "foo"]
     #	  
+    alias top_level_test test
     def test(command, file1, file2=nil)
       file1 = expand_path(file1)
       file2 = expand_path(file2) if file2
@@ -162,7 +177,11 @@ class Shell
 
       case command
       when Integer
-	top_level_test(command, file1, file2)
+	if file2
+	  top_level_test(command, file1, file2)
+	else
+	  top_level_test(command, file1)
+	end
       when String
 	if command.size == 1
 	  if file2
@@ -194,9 +213,23 @@ class Shell
     #	same as Dir.mkdir()
     #	  
     def mkdir(*path)
-      for dir in path
-	Dir.mkdir(expand_path(dir))
+      @shell.check_point
+      notify("mkdir #{path.join(' ')}")
+      
+      perm = nil
+      if path.last.kind_of?(Integer)
+	perm = path.pop
       end
+      for dir in path
+	d = expand_path(dir)
+	if perm
+	  Dir.mkdir(d, perm)
+	else
+	  Dir.mkdir(d)
+	end
+	File.chmod(d, 0666 & ~@shell.umask) if @shell.umask
+      end
+      Void.new(@shell)
     end
 
     #
@@ -205,9 +238,13 @@ class Shell
     #	same as Dir.rmdir()
     #	  
     def rmdir(*path)
+      @shell.check_point
+      notify("rmdir #{path.join(' ')}")
+
       for dir in path
 	Dir.rmdir(expand_path(dir))
       end
+      Void.new(@shell)
     end
 
     #
@@ -299,35 +336,17 @@ class Shell
 
     # %pwd, %cwd -> @pwd
     def notify(*opts, &block)
-      Thread.exclusive do
-	Shell.notify(*opts) {|mes|
-	  yield mes if iterator?
+      Shell.notify(*opts) {|mes|
+	yield mes if iterator?
 	
-	  mes.gsub!("%pwd", "#{@cwd}")
-	  mes.gsub!("%cwd", "#{@cwd}")
-	}
-      end
+	mes.gsub!("%pwd", "#{@cwd}")
+	mes.gsub!("%cwd", "#{@cwd}")
+      }
     end
 
     #
     # private functions
     #
-    def effect_umask
-      if @shell.umask
-	Thread.critical = true
-	save = File.umask
-	begin
-	  yield
-	ensure
-	  File.umask save
-	  Thread.critical = false
-	end
-      else
-	yield
-      end
-    end
-    private :effect_umask
-
     def find_system_command(command)
       return command if /^\// =~ command
       case path = @system_commands[command]
@@ -343,7 +362,7 @@ class Shell
 
       for p in @shell.system_path
 	path = join(p, command)
-	if FileTest.exists?(path)
+	if FileTest.exist?(path)
 	  @system_commands[command] = path
 	  return path
 	end
@@ -454,7 +473,7 @@ class Shell
 		 end]
 	Shell.notify "Define #{meth}(#{arg_str})", Shell.debug?
 	Shell.notify("Definition of #{meth}: ", d, 
-	     Shell.debug.kind_of?(Integer) && Shell.debug > 1)
+		     Shell.debug.kind_of?(Integer) && Shell.debug > 1)
 	eval d
       end
     end
@@ -568,24 +587,6 @@ class Shell
       def_builtin_commands(FileTest, 
 		   FileTest.singleton_methods(false).collect{|m| [m, ["FILENAME"]]})
 
-      # method related ftools
-      normal_delegation_ftools_methods = [
-	["syscopy", ["FILENAME_FROM", "FILENAME_TO"]],
-	["copy", ["FILENAME_FROM", "FILENAME_TO"]],
-	["move", ["FILENAME_FROM", "FILENAME_TO"]],
-	["compare", ["FILENAME_FROM", "FILENAME_TO"]],
-	["safe_unlink", ["*FILENAMES"]],
-	["makedirs", ["*FILENAMES"]],
-	#    ["chmod", ["mode", "*FILENAMES"]],
-	["install", ["FILENAME_FROM", "FILENAME_TO", "mode"]],
-      ]
-      def_builtin_commands(File,
-		   normal_delegation_ftools_methods)
-      alias_method :cmp, :compare
-      alias_method :mv, :move
-      alias_method :cp, :copy
-      alias_method :rm_f, :safe_unlink
-      alias_method :mkpath, :makedirs
     end
 
   end
