@@ -66,7 +66,7 @@ class Time
       'N' => -1, 'O' => -2, 'P' => -3, 'Q' => -4,  'R' => -5,  'S' => -6, 
       'T' => -7, 'U' => -8, 'V' => -9, 'W' => -10, 'X' => -11, 'Y' => -12,
     }
-    def zone_offset(zone, year=Time.now.year)
+    def zone_offset(zone, year=self.now.year)
       off = nil
       zone = zone.upcase
       if /\A([+-])(\d\d):?(\d\d)\z/ =~ zone
@@ -75,9 +75,9 @@ class Time
         off = zone.to_i * 3600
       elsif ZoneOffset.include?(zone)
         off = ZoneOffset[zone] * 3600
-      elsif ((t = Time.local(year, 1, 1)).zone.upcase == zone rescue false)
+      elsif ((t = self.local(year, 1, 1)).zone.upcase == zone rescue false)
         off = t.utc_offset
-      elsif ((t = Time.local(year, 7, 1)).zone.upcase == zone rescue false)
+      elsif ((t = self.local(year, 7, 1)).zone.upcase == zone rescue false)
         off = t.utc_offset
       end
       off
@@ -177,7 +177,7 @@ class Time
       if off
         year, mon, day, hour, min, sec =
           apply_offset(year, mon, day, hour, min, sec, off)
-        t = Time.utc(year, mon, day, hour, min, sec, usec)
+        t = self.utc(year, mon, day, hour, min, sec, usec)
         t.localtime if !zone_utc?(zone)
         t
       else
@@ -236,7 +236,7 @@ class Time
     #
     # A failure for Time.parse should be checked, though.
     #
-    def parse(date, now=Time.now)
+    def parse(date, now=self.now)
       d = Date._parse(date, false)
       year = d[:year]
       year = yield(year) if year && block_given?
@@ -250,7 +250,7 @@ class Time
     # block.  For example:
     #
     #     Time.strptime(...) {|y| y < 100 ? (y >= 69 ? y + 1900 : y + 2000) : y}
-    def strptime(date, format, now=Time.now)
+    def strptime(date, format, now=self.now)
       d = Date._strptime(date, format)
       raise ArgumentError, "invalid strptime format - `#{format}'" unless d
       year = d[:year]
@@ -344,7 +344,7 @@ class Time
         else
           year += 1900
         end
-        Time.utc(year, $2, $1.to_i, $4.to_i, $5.to_i, $6.to_i)
+        self.utc(year, $2, $1.to_i, $4.to_i, $5.to_i, $6.to_i)
       elsif /\A\s*
              (?:Mon|Tue|Wed|Thu|Fri|Sat|Sun)\x20
              (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\x20
@@ -374,7 +374,7 @@ class Time
           (-?\d+)-(\d\d)-(\d\d)
           T
           (\d\d):(\d\d):(\d\d)
-          (\.\d*)?
+          (\.\d+)?
           (Z|[+-]\d\d:\d\d)?
           \s*\z/ix =~ date
         year = $1.to_i
@@ -384,14 +384,16 @@ class Time
         min = $5.to_i
         sec = $6.to_i
         usec = 0
-        usec = $7.to_f * 1000000 if $7
+        if $7
+          usec = Rational($7) * 1000000
+        end
         if $8
           zone = $8
           year, mon, day, hour, min, sec =
             apply_offset(year, mon, day, hour, min, sec, zone_offset(zone))
-          Time.utc(year, mon, day, hour, min, sec, usec)
+          self.utc(year, mon, day, hour, min, sec, usec)
         else
-          Time.local(year, mon, day, hour, min, sec, usec)
+          self.local(year, mon, day, hour, min, sec, usec)
         end
       else
         raise ArgumentError.new("invalid date: #{date.inspect}")
@@ -622,6 +624,7 @@ if __FILE__ == $0
                    Time.xmlschema("2000-01-12T12:13:14Z"))
       assert_equal(Time.utc(2001, 4, 17, 19, 23, 17, 300000),
                    Time.xmlschema("2001-04-17T19:23:17.3Z"))
+      assert_raise(ArgumentError) { Time.xmlschema("2000-01-01T00:00:00.+00:00") }
     end
 
     def test_encode_xmlschema
@@ -645,6 +648,8 @@ if __FILE__ == $0
         t = Time.utc(1960, 12, 31, 23, 0, 0, 123456)
         assert_equal("1960-12-31T23:00:00.123456Z", t.xmlschema(6))
       end
+
+      assert_equal(249, Time.xmlschema("2008-06-05T23:49:23.000249+09:00").usec)
     end
 
     def test_completion
@@ -808,6 +813,10 @@ if __FILE__ == $0
       assert_equal(t, Time.xmlschema("1998-12-31T23:00:00-01:00"))
     end
 
+    def test_xmlschema_fraction
+      assert_equal(500000, Time.xmlschema("2000-01-01T00:00:00.5+00:00").tv_usec)
+    end
+
     def test_ruby_talk_152866
       t = Time::xmlschema('2005-08-30T22:48:00-07:00')
       assert_equal(31, t.day)
