@@ -3,7 +3,7 @@
 #
 # Author:: Akira Yamada <akira@ruby-lang.org>
 # License:: You can redistribute it and/or modify it under the same term as Ruby.
-# Revision:: $Id: generic.rb,v 1.17 2005/02/23 06:08:01 akira Exp $
+# Revision:: $Id: generic.rb 15456 2008-02-13 07:26:52Z naruse $
 #
 
 require 'uri/common'
@@ -12,6 +12,7 @@ module URI
   
   #
   # Base class for all URI classes.
+  # Implements generic URI syntax as per RFC 2396.
   #
   class Generic
     include URI
@@ -303,19 +304,19 @@ module URI
       end
       check_userinfo(*userinfo)
       set_userinfo(*userinfo)
-      userinfo
+      # returns userinfo
     end
 
     def user=(user)
       check_user(user)
       set_user(user)
-      user
+      # returns user
     end
     
     def password=(password)
       check_password(password)
       set_password(password)
-      password
+      # returns password
     end
 
     def set_userinfo(user, password = nil)
@@ -336,8 +337,8 @@ module URI
     protected :set_user
 
     def set_password(v)
-      set_userinfo(@user, v)
-      v
+      @password = v
+      # returns v
     end
     protected :set_password
 
@@ -355,7 +356,9 @@ module URI
     private :escape_userpass
 
     def userinfo
-      if !@password
+      if @user.nil?
+        nil
+      elsif @password.nil?
         @user
       else
         @user + ':' + @password
@@ -613,65 +616,65 @@ module URI
     private :split_path
 
     def merge_path(base, rel)
+
       # RFC2396, Section 5.2, 5)
-      if rel[0] == ?/ #/
-        # RFC2396, Section 5.2, 5)
-        return rel
+      # RFC2396, Section 5.2, 6)
+      base_path = split_path(base)
+      rel_path  = split_path(rel)
 
-      else
-        # RFC2396, Section 5.2, 6)
-        base_path = split_path(base)
-        rel_path  = split_path(rel)
-
-        # RFC2396, Section 5.2, 6), a)
-	base_path << '' if base_path.last == '..'
-	while i = base_path.index('..')
-	  base_path.slice!(i - 1, 2)
-        end
-        if base_path.empty?
-          base_path = [''] # keep '/' for root directory
-        else
-	  base_path.pop
-        end
-
-        # RFC2396, Section 5.2, 6), c)
-        # RFC2396, Section 5.2, 6), d)
-        rel_path.push('') if rel_path.last == '.' || rel_path.last == '..'
-        rel_path.delete('.')
-
-        # RFC2396, Section 5.2, 6), e)
-        tmp = []
-        rel_path.each do |x|
-          if x == '..' &&
-              !(tmp.empty? || tmp.last == '..')
-            tmp.pop
-          else
-            tmp << x
-          end
-        end
-
-        add_trailer_slash = true
-        while x = tmp.shift
-          if x == '..' && base_path.size > 1
-            # RFC2396, Section 4
-            # a .. or . in an absolute path has no special meaning
-            base_path.pop
-          else
-            # if x == '..'
-            #   valid absolute (but abnormal) path "/../..."
-            # else
-            #   valid absolute path
-            # end
-            base_path << x
-            tmp.each {|t| base_path << t}
-            add_trailer_slash = false
-            break
-          end
-        end
-        base_path.push('') if add_trailer_slash
-
-        return base_path.join('/')
+      # RFC2396, Section 5.2, 6), a)
+      base_path << '' if base_path.last == '..'
+      while i = base_path.index('..')
+        base_path.slice!(i - 1, 2)
       end
+
+      if (first = rel_path.first) and first.empty?
+        base_path.clear
+        rel_path.shift
+      end
+
+      # RFC2396, Section 5.2, 6), c)
+      # RFC2396, Section 5.2, 6), d)
+      rel_path.push('') if rel_path.last == '.' || rel_path.last == '..'
+      rel_path.delete('.')
+
+      # RFC2396, Section 5.2, 6), e)
+      tmp = []
+      rel_path.each do |x|
+        if x == '..' &&
+            !(tmp.empty? || tmp.last == '..')
+          tmp.pop
+        else
+          tmp << x
+        end
+      end
+
+      add_trailer_slash = !tmp.empty?
+      if base_path.empty?
+        base_path = [''] # keep '/' for root directory
+      elsif add_trailer_slash
+        base_path.pop
+      end
+      while x = tmp.shift
+        if x == '..'
+          # RFC2396, Section 4
+          # a .. or . in an absolute path has no special meaning
+          base_path.pop if base_path.size > 1
+        else
+          # if x == '..'
+          #   valid absolute (but abnormal) path "/../..."
+          # else
+          #   valid absolute path
+          # end
+          base_path << x
+          tmp.each {|t| base_path << t}
+          add_trailer_slash = false
+          break
+        end
+      end
+      base_path.push('') if add_trailer_slash
+
+      return base_path.join('/')
     end
     private :merge_path
 
@@ -689,10 +692,10 @@ module URI
     #
     #   require 'uri'
     #
-    #   uri = URI.parse("http://my.rubysite.com")
+    #   uri = URI.parse("http://my.example.com")
     #   uri.merge!("/main.rbx?page=1")
     #   p uri
-    #   # =>  #<URI::HTTP:0x2021f3b0 URL:http://my.rubysite.com/main.rbx?page=1>
+    #   # =>  #<URI::HTTP:0x2021f3b0 URL:http://my.example.com/main.rbx?page=1>
     #
     def merge!(oth)
       t = merge(oth)
@@ -718,9 +721,9 @@ module URI
     #
     #   require 'uri'
     #
-    #   uri = URI.parse("http://my.rubysite.com")
+    #   uri = URI.parse("http://my.example.com")
     #   p uri.merge("/main.rbx?page=1")
-    #   # =>  #<URI::HTTP:0x2021f3b0 URL:http://my.rubysite.com/main.rbx?page=1>
+    #   # =>  #<URI::HTTP:0x2021f3b0 URL:http://my.example.com/main.rbx?page=1>
     #
     def merge(oth)
       begin
@@ -904,8 +907,8 @@ module URI
     #
     #   require 'uri'
     #
-    #   uri = URI.parse('http://my.rubysite.com/main.rbx?page=1')
-    #   p uri.route_from('http://my.rubysite.com')
+    #   uri = URI.parse('http://my.example.com/main.rbx?page=1')
+    #   p uri.route_from('http://my.example.com')
     #   #=> #<URI::Generic:0x20218858 URL:/main.rbx?page=1>
     #
     def route_from(oth)
@@ -944,8 +947,8 @@ module URI
     #
     #   require 'uri'
     #
-    #   uri = URI.parse('http://my.rubysite.com')
-    #   p uri.route_to('http://my.rubysite.com/main.rbx?page=1')
+    #   uri = URI.parse('http://my.example.com')
+    #   p uri.route_to('http://my.example.com/main.rbx?page=1')
     #   #=> #<URI::Generic:0x2020c2f6 URL:/main.rbx?page=1>
     #    
     def route_to(oth)
@@ -1085,9 +1088,9 @@ module URI
     #
     #   require 'uri'
     #
-    #   uri = URI.parse('http://myuser:mypass@my.rubysite.com/test.rbx')
+    #   uri = URI.parse('http://myuser:mypass@my.example.com/test.rbx')
     #   p uri.select(:userinfo, :host, :path)
-    #   # => ["myuser:mypass", "my.rubysite.com", "/test.rbx"]
+    #   # => ["myuser:mypass", "my.example.com", "/test.rbx"]
     #
     def select(*components)
       components.collect do |c|
@@ -1100,8 +1103,9 @@ module URI
       end
     end
 
+    @@to_s = Kernel.instance_method(:to_s)
     def inspect
-      sprintf("#<%s:0x%x URL:%s>", self.class.to_s, self.object_id, self.to_s)
+      @@to_s.bind(self).call.sub!(/>\z/) {" URL:#{self}>"}
     end
 
     def coerce(oth)
