@@ -3,7 +3,7 @@
 #
 # Author:: Akira Yamada <akira@ruby-lang.org>
 # License:: You can redistribute it and/or modify it under the same term as Ruby.
-# Revision:: $Id: mailto.rb,v 1.9 2005/08/24 05:08:00 ryan Exp $
+# Revision:: $Id: mailto.rb 14565 2007-12-24 01:51:49Z drbrain $
 #
 
 require 'uri/generic'
@@ -55,7 +55,7 @@ module URI
         (#{PATTERN::FRAGMENT})                        (?# 3: fragment)
       )?
       \\z
-    ", Regexp::EXTENDED, 'N').freeze
+    ", Regexp::EXTENDED).freeze
     # :startdoc:
 
     #
