@@ -3,7 +3,7 @@
 # 
 # Copyright (C) 2001, 2002, 2003 by Michael Neumann (mneumann@ntecs.de)
 #
-# $Id: create.rb,v 1.2 2004/11/15 23:26:20 mneumann Exp $
+# $Id: create.rb 11816 2007-02-23 03:42:01Z knu $
 #
 
 require "date"
@@ -194,12 +194,12 @@ module XMLRPC
 	when TrueClass, FalseClass
 	  @writer.tag("boolean", param ? "1" : "0")
 
-	when String 
-	  @writer.tag("string", param)
-
 	when Symbol 
 	  @writer.tag("string", param.to_s)
 
+	when String 
+	  @writer.tag("string", param)
+
         when NilClass
           if Config::ENABLE_NIL_CREATE
             @writer.ele("nil")
@@ -241,12 +241,7 @@ module XMLRPC
 	    @writer.ele("data", *a)
 	  )
 
-	when Date
-	  t = param
-	  @writer.tag("dateTime.iso8601", 
-	    format("%.4d%02d%02dT00:00:00", t.year, t.month, t.day))
-
-	when Time
+	when Time, Date, ::DateTime
 	  @writer.tag("dateTime.iso8601", param.strftime("%Y%m%dT%H:%M:%S"))  
 
 	when XMLRPC::DateTime
