@@ -6,7 +6,7 @@
 # 
 # Copyright (C) 2001, 2002, 2003 by Michael Neumann (mneumann@ntecs.de)
 #
-# $Id: utils.rb,v 1.4 2005/06/19 15:47:31 mneumann Exp $ 
+# $Id: utils.rb 13767 2007-10-24 18:46:08Z jeg2 $ 
 #
 
 module XMLRPC
@@ -157,7 +157,7 @@ module XMLRPC
   module ParseContentType
     def parse_content_type(str)
       a, *b = str.split(";")
-      return a.strip, *b
+      return a.strip.downcase, *b
     end
   end
 
