@@ -23,11 +23,10 @@ static const char *const loadable_ext[] = {
     0
 };
 
-VALUE rb_load_path;		/* to be moved to VM */
-static VALUE
-get_load_path(void)
+VALUE
+rb_get_load_path(void)
 {
-    VALUE load_path = rb_load_path;
+    VALUE load_path = GET_VM()->load_path;
     VALUE ary = rb_ary_new2(RARRAY_LEN(load_path));
     long i;
 
@@ -37,6 +36,12 @@ get_load_path(void)
     return ary;
 }
 
+static VALUE
+load_path_getter(ID id, rb_vm_t *vm)
+{
+    return vm->load_path;
+}
+
 static VALUE
 get_loaded_features(void)
 {
@@ -126,7 +131,7 @@ rb_feature_p(const char *feature, const char *ext, int rb, int expanded, const c
 	if ((n = RSTRING_LEN(v)) < len) continue;
 	if (strncmp(f, feature, len) != 0) {
 	    if (expanded) continue;
-	    if (!load_path) load_path = get_load_path();
+	    if (!load_path) load_path = rb_get_load_path();
 	    if (!(p = loaded_feature_path(f, n, feature, len, type, load_path)))
 		continue;
 	    f += RSTRING_LEN(p) + 1;
@@ -151,7 +156,7 @@ rb_feature_p(const char *feature, const char *ext, int rb, int expanded, const c
 	    fs.name = feature;
 	    fs.len = len;
 	    fs.type = type;
-	    fs.load_path = load_path ? load_path : get_load_path();
+	    fs.load_path = load_path ? load_path : rb_get_load_path();
 	    fs.result = 0;
 	    st_foreach(loading_tbl, loaded_feature_path_i, (st_data_t)&fs);
 	    if ((f = fs.result) != 0) {
@@ -166,18 +171,22 @@ rb_feature_p(const char *feature, const char *ext, int rb, int expanded, const c
 	    return !IS_RBEXT(ext) ? 's' : 'r';
 	}
 	else {
+	    VALUE bufstr;
 	    char *buf;
 
 	    if (ext && *ext) return 0;
-	    buf = ALLOCA_N(char, len + DLEXT_MAXLEN + 1);
+	    bufstr = rb_str_tmp_new(len + DLEXT_MAXLEN);
+	    buf = RSTRING_PTR(bufstr);
 	    MEMCPY(buf, feature, char, len);
 	    for (i = 0; (e = loadable_ext[i]) != 0; i++) {
 		strncpy(buf + len, e, DLEXT_MAXLEN + 1);
 		if (st_get_key(loading_tbl, (st_data_t)buf, &data)) {
+		    rb_str_resize(bufstr, 0);
 		    if (fn) *fn = (const char*)data;
 		    return i ? 's' : 'r';
 		}
 	    }
+	    rb_str_resize(bufstr, 0);
 	}
     }
     return 0;
@@ -187,7 +196,13 @@ int
 rb_provided(const char *feature)
 {
     const char *ext = strrchr(feature, '.');
+    volatile VALUE fullpath = 0;
 
+    if (*feature == '.' &&
+	(feature[1] == '/' || strncmp(feature+1, "./", 2) == 0)) {
+	fullpath = rb_file_expand_path(rb_str_new2(feature), Qnil);
+	feature = RSTRING_PTR(fullpath);
+    }
     if (ext && !strchr(ext, '/')) {
 	if (IS_RBEXT(ext)) {
 	    if (rb_feature_p(feature, ext, Qtrue, Qfalse, 0)) return Qtrue;
@@ -357,7 +372,7 @@ load_unlock(const char *ftptr)
 	st_table *loading_tbl = get_loading_table();
 
 	if (st_delete(loading_tbl, &key, &data)) {
-	    free((char *)key);
+	    xfree((char *)key);
 	    rb_barrier_release((VALUE)data);
 	}
     }
@@ -510,8 +525,6 @@ rb_require_safe(VALUE fname, int safe)
     } volatile saved;
     char *volatile ftptr = 0;
 
-    FilePathValue(fname);
-    RB_GC_GUARD(fname) = rb_str_new4(fname);
     PUSH_TAG();
     saved.safe = rb_safe_level();
     if ((state = EXEC_TAG()) == 0) {
@@ -520,6 +533,8 @@ rb_require_safe(VALUE fname, int safe)
 	int found;
 
 	rb_set_safe_level_force(safe);
+	FilePathValue(fname);
+	RB_GC_GUARD(fname) = rb_str_new4(fname);
 	found = search_required(fname, &path);
 	if (found) {
 	    if (!path || !(ftptr = load_lock(RSTRING_PTR(path)))) {
@@ -635,7 +650,7 @@ rb_mod_autoload_p(VALUE mod, VALUE sym)
 static VALUE
 rb_f_autoload(VALUE obj, VALUE sym, VALUE file)
 {
-    VALUE klass = ruby_cbase();
+    VALUE klass = rb_vm_cbase();
     if (NIL_P(klass)) {
 	rb_raise(rb_eTypeError, "Can not set autoload on singleton class");
     }
@@ -649,8 +664,8 @@ rb_f_autoload(VALUE obj, VALUE sym, VALUE file)
 static VALUE
 rb_f_autoload_p(VALUE obj, VALUE sym)
 {
-    /* use ruby_cbase() as same as rb_f_autoload. */
-    VALUE klass = ruby_cbase();
+    /* use rb_vm_cbase() as same as rb_f_autoload. */
+    VALUE klass = rb_vm_cbase();
     if (NIL_P(klass)) {
 	return Qnil;
     }
@@ -660,14 +675,18 @@ rb_f_autoload_p(VALUE obj, VALUE sym)
 void
 Init_load()
 {
-    rb_define_readonly_variable("$:", &rb_load_path);
-    rb_define_readonly_variable("$-I", &rb_load_path);
-    rb_define_readonly_variable("$LOAD_PATH", &rb_load_path);
-    rb_load_path = rb_ary_new();
+    rb_vm_t *vm = GET_VM();
+    const char *var_load_path = "$:";
+    ID id_load_path = rb_intern(var_load_path);
+
+    rb_define_hooked_variable(var_load_path, (VALUE*)GET_VM(), load_path_getter, 0);
+    rb_alias_variable((rb_intern)("$-I"), id_load_path);
+    rb_alias_variable((rb_intern)("$LOAD_PATH"), id_load_path);
+    vm->load_path = rb_ary_new();
 
     rb_define_virtual_variable("$\"", get_loaded_features, 0);
     rb_define_virtual_variable("$LOADED_FEATURES", get_loaded_features, 0);
-    GET_VM()->loaded_features = rb_ary_new();
+    vm->loaded_features = rb_ary_new();
 
     rb_define_global_function("load", rb_f_load, -1);
     rb_define_global_function("require", rb_f_require, 1);
