@@ -2,18 +2,18 @@
 
   marshal.c -
 
-  $Author: akr $
-  $Date: 2006/02/05 14:40:01 $
+  $Author: nobu $
   created at: Thu Apr 27 16:30:01 JST 1995
 
-  Copyright (C) 1993-2003 Yukihiro Matsumoto
+  Copyright (C) 1993-2007 Yukihiro Matsumoto
 
 **********************************************************************/
 
-#include "ruby.h"
-#include "rubyio.h"
-#include "st.h"
-#include "util.h"
+#include "ruby/ruby.h"
+#include "ruby/io.h"
+#include "ruby/st.h"
+#include "ruby/util.h"
+#include "ruby/encoding.h"
 
 #include <math.h>
 #ifdef HAVE_FLOAT_H
@@ -80,7 +80,56 @@ shortlen(long len, BDIGIT *ds)
 
 static ID s_dump, s_load, s_mdump, s_mload;
 static ID s_dump_data, s_load_data, s_alloc;
-static ID s_getc, s_read, s_write, s_binmode;
+static ID s_getbyte, s_read, s_write, s_binmode;
+
+ID rb_id_encoding(void);
+
+typedef struct {
+    VALUE newclass;
+    VALUE oldclass;
+    VALUE (*dumper)(VALUE);
+    VALUE (*loader)(VALUE, VALUE);
+} marshal_compat_t;
+
+static st_table *compat_allocator_tbl;
+static VALUE compat_allocator_tbl_wrapper;
+
+static int
+mark_marshal_compat_i(st_data_t key, st_data_t value)
+{
+    marshal_compat_t *p = (marshal_compat_t *)value;
+    rb_gc_mark(p->newclass);
+    rb_gc_mark(p->oldclass);
+    return ST_CONTINUE;
+}
+
+static void
+mark_marshal_compat_t(void *tbl)
+{
+    if (!tbl) return;
+    st_foreach(tbl, mark_marshal_compat_i, 0);
+}
+
+void
+rb_marshal_define_compat(VALUE newclass, VALUE oldclass, VALUE (*dumper)(VALUE), VALUE (*loader)(VALUE, VALUE))
+{
+    marshal_compat_t *compat;
+    rb_alloc_func_t allocator = rb_get_alloc_func(newclass);
+
+    if (!allocator) {
+        rb_raise(rb_eTypeError, "no allocator");
+    }
+
+    compat = ALLOC(marshal_compat_t);
+    compat->newclass = Qnil;
+    compat->oldclass = Qnil;
+    compat->newclass = newclass;
+    compat->oldclass = oldclass;
+    compat->dumper = dumper;
+    compat->loader = loader;
+
+    st_insert(compat_allocator_tbl, (st_data_t)allocator, (st_data_t)compat);
+}
 
 struct dump_arg {
     VALUE obj;
@@ -88,6 +137,9 @@ struct dump_arg {
     st_table *symbols;
     st_table *data;
     int taint;
+    st_table *compat_tbl;
+    VALUE wrapper;
+    st_table *encodings;
 };
 
 struct dump_call_arg {
@@ -96,11 +148,29 @@ struct dump_call_arg {
     int limit;
 };
 
+static void
+check_dump_arg(struct dump_arg *arg)
+{
+    if (!DATA_PTR(arg->wrapper)) {
+	rb_raise(rb_eRuntimeError, "Marshal.dump reentered");
+    }
+}
+
+static void
+mark_dump_arg(void *ptr)
+{
+    struct dump_arg *p = ptr;
+    if (!ptr)
+        return;
+    rb_mark_set(p->data);
+    rb_mark_hash(p->compat_tbl);
+}
+
 static VALUE
 class2path(VALUE klass)
 {
     VALUE path = rb_class_path(klass);
-    char *n = RSTRING(path)->ptr;
+    char *n = RSTRING_PTR(path);
 
     if (n[0] == '#') {
 	rb_raise(rb_eTypeError, "can't dump anonymous %s %s",
@@ -120,7 +190,7 @@ w_nbyte(const char *s, int n, struct dump_arg *arg)
 {
     VALUE buf = arg->str;
     rb_str_buf_cat(buf, s, n);
-    if (arg->dest && RSTRING(buf)->len >= BUFSIZ) {
+    if (arg->dest && RSTRING_LEN(buf) >= BUFSIZ) {
 	if (arg->taint) OBJ_TAINT(buf);
 	rb_io_write(arg->dest, buf);
 	rb_str_resize(buf, 0);
@@ -143,8 +213,8 @@ w_bytes(const char *s, int n, struct dump_arg *arg)
 static void
 w_short(int x, struct dump_arg *arg)
 {
-    w_byte((x >> 0) & 0xff, arg);
-    w_byte((x >> 8) & 0xff, arg);
+    w_byte((char)((x >> 0) & 0xff), arg);
+    w_byte((char)((x >> 8) & 0xff), arg);
 }
 
 static void
@@ -165,11 +235,11 @@ w_long(long x, struct dump_arg *arg)
 	return;
     }
     if (0 < x && x < 123) {
-	w_byte(x + 5, arg);
+	w_byte((char)(x + 5), arg);
 	return;
     }
     if (-124 < x && x < 0) {
-	w_byte((x - 5)&0xff, arg);
+	w_byte((char)((x - 5)&0xff), arg);
 	return;
     }
     for (i=1;i<sizeof(long)+1;i++) {
@@ -276,7 +346,7 @@ load_mantissa(double d, const char *buf, int len)
 static void
 w_float(double d, struct dump_arg *arg)
 {
-    char buf[100];
+    char buf[FLOAT_DIG + (DECIMAL_MANT + 7) / 8 + 10];
 
     if (isinf(d)) {
 	if (d < 0) strcpy(buf, "-inf");
@@ -293,7 +363,7 @@ w_float(double d, struct dump_arg *arg)
 	int len;
 
 	/* xxx: should not use system's sprintf(3) */
-	sprintf(buf, "%.*g", FLOAT_DIG, d);
+	snprintf(buf, sizeof(buf), "%.*g", FLOAT_DIG, d);
 	len = strlen(buf);
 	w_bytes(buf, len + save_mantissa(d, buf + len), arg);
 	return;
@@ -304,7 +374,7 @@ w_float(double d, struct dump_arg *arg)
 static void
 w_symbol(ID id, struct dump_arg *arg)
 {
-    char *sym = rb_id2name(id);
+    const char *sym;
     st_data_t num;
 
     if (st_lookup(arg->symbols, id, &num)) {
@@ -312,6 +382,10 @@ w_symbol(ID id, struct dump_arg *arg)
 	w_long((long)num, arg);
     }
     else {
+	sym = rb_id2name(id);
+	if (!sym) {
+	    rb_raise(rb_eTypeError, "can't dump anonymous ID %ld", id);
+	}
 	w_byte(TYPE_SYMBOL, arg);
 	w_bytes(sym, strlen(sym), arg);
 	st_add_direct(arg->symbols, id, arg->symbols->num_entries);
@@ -340,65 +414,115 @@ hash_each(VALUE key, VALUE value, struct dump_call_arg *arg)
 static void
 w_extended(VALUE klass, struct dump_arg *arg, int check)
 {
-    char *path;
+    const char *path;
 
-    if (FL_TEST(klass, FL_SINGLETON)) {
-	if (check && RCLASS(klass)->m_tbl->num_entries ||
-	    (RCLASS(klass)->iv_tbl && RCLASS(klass)->iv_tbl->num_entries > 1)) {
+    if (check && FL_TEST(klass, FL_SINGLETON)) {
+	if (RCLASS_M_TBL(klass)->num_entries ||
+	    (RCLASS_IV_TBL(klass) && RCLASS_IV_TBL(klass)->num_entries > 1)) {
 	    rb_raise(rb_eTypeError, "singleton can't be dumped");
 	}
-	klass = RCLASS(klass)->super;
+	klass = RCLASS_SUPER(klass);
     }
     while (BUILTIN_TYPE(klass) == T_ICLASS) {
 	path = rb_class2name(RBASIC(klass)->klass);
 	w_byte(TYPE_EXTENDED, arg);
 	w_unique(path, arg);
-	klass = RCLASS(klass)->super;
+	klass = RCLASS_SUPER(klass);
     }
 }
 
 static void
 w_class(char type, VALUE obj, struct dump_arg *arg, int check)
 {
+    volatile VALUE p;
     char *path;
+    st_data_t real_obj;
+    VALUE klass;
 
-    VALUE klass = CLASS_OF(obj);
+    if (st_lookup(arg->compat_tbl, (st_data_t)obj, &real_obj)) {
+        obj = (VALUE)real_obj;
+    }
+    klass = CLASS_OF(obj);
     w_extended(klass, arg, check);
     w_byte(type, arg);
-    path = RSTRING(class2path(rb_class_real(klass)))->ptr;
+    p = class2path(rb_class_real(klass));
+    path = RSTRING_PTR(p);
     w_unique(path, arg);
 }
 
 static void
-w_uclass(VALUE obj, VALUE base_klass, struct dump_arg *arg)
+w_uclass(VALUE obj, VALUE super, struct dump_arg *arg)
 {
     VALUE klass = CLASS_OF(obj);
 
     w_extended(klass, arg, Qtrue);
     klass = rb_class_real(klass);
-    if (klass != base_klass) {
+    if (klass != super) {
 	w_byte(TYPE_UCLASS, arg);
-	w_unique(RSTRING(class2path(klass))->ptr, arg);
+	w_unique(RSTRING_PTR(class2path(klass)), arg);
     }
 }
 
 static int
 w_obj_each(ID id, VALUE value, struct dump_call_arg *arg)
 {
+    if (id == rb_id_encoding()) return ST_CONTINUE;
     w_symbol(id, arg->arg);
     w_object(value, arg->arg, arg->limit);
     return ST_CONTINUE;
 }
 
 static void
-w_ivar(st_table *tbl, struct dump_call_arg *arg)
+w_encoding(VALUE obj, long num, struct dump_call_arg *arg)
 {
+    int encidx = rb_enc_get_index(obj);
+    rb_encoding *enc = 0;
+    st_data_t name;
+
+    if (encidx <= 0 || !(enc = rb_enc_from_index(encidx))) {
+	w_long(num, arg->arg);
+	return;
+    }
+    w_long(num + 1, arg->arg);
+    w_symbol(rb_id_encoding(), arg->arg);
+    do {
+	if (!arg->arg->encodings)
+	    arg->arg->encodings = st_init_strcasetable();
+	else if (st_lookup(arg->arg->encodings, (st_data_t)rb_enc_name(enc), &name))
+	    break;
+	name = (st_data_t)rb_str_new2(rb_enc_name(enc));
+	st_insert(arg->arg->encodings, (st_data_t)rb_enc_name(enc), name);
+    } while (0);
+    w_object(name, arg->arg, arg->limit);
+}
+
+static void
+w_ivar(VALUE obj, st_table *tbl, struct dump_call_arg *arg)
+{
+    long num = tbl ? tbl->num_entries : 0;
+
+    w_encoding(obj, num, arg);
     if (tbl) {
-	w_long(tbl->num_entries, arg->arg);
 	st_foreach_safe(tbl, w_obj_each, (st_data_t)arg);
     }
-    else {
-	w_long(0, arg->arg);
+}
+
+static void
+w_objivar(VALUE obj, struct dump_call_arg *arg)
+{
+    VALUE *ptr;
+    long i, len, num;
+
+    len = ROBJECT_NUMIV(obj);
+    ptr = ROBJECT_IVPTR(obj);
+    num = 0;
+    for (i = 0; i < len; i++)
+        if (ptr[i] != Qundef)
+            num += 1;
+
+    w_encoding(obj, num, arg);
+    if (num != 0) {
+        rb_ivar_foreach(obj, w_obj_each, (st_data_t)arg);
     }
 }
 
@@ -408,6 +532,9 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
     struct dump_call_arg c_arg;
     st_table *ivtbl = 0;
     st_data_t num;
+    int hasiv = 0;
+#define has_ivars(obj, ivtbl) ((ivtbl = rb_generic_ivar_table(obj)) != 0 || \
+			       (!SPECIAL_CONST_P(obj) && !ENCODING_IS_ASCII8BIT(obj)))
 
     if (limit == 0) {
 	rb_raise(rb_eArgError, "exceed depth limit");
@@ -423,7 +550,7 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
 	return;
     }
 
-    if (ivtbl = rb_generic_ivar_table(obj)) {
+    if ((hasiv = has_ivars(obj, ivtbl)) != 0) {
 	w_byte(TYPE_IVAR, arg);
     }
     if (obj == Qnil) {
@@ -455,34 +582,58 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
     else {
 	if (OBJ_TAINTED(obj)) arg->taint = Qtrue;
 
-	st_add_direct(arg->data, obj, arg->data->num_entries);
 	if (rb_respond_to(obj, s_mdump)) {
-	    VALUE v;
+	    volatile VALUE v;
+
+            st_add_direct(arg->data, obj, arg->data->num_entries);
 
 	    v = rb_funcall(obj, s_mdump, 0, 0);
+	    check_dump_arg(arg);
 	    w_class(TYPE_USRMARSHAL, obj, arg, Qfalse);
 	    w_object(v, arg, limit);
-	    if (ivtbl) w_ivar(0, &c_arg);
+	    if (hasiv) w_ivar(obj, 0, &c_arg);
 	    return;
 	}
 	if (rb_respond_to(obj, s_dump)) {
 	    VALUE v;
+            st_table *ivtbl2 = 0;
+            int hasiv2;
 
 	    v = rb_funcall(obj, s_dump, 1, INT2NUM(limit));
+	    check_dump_arg(arg);
 	    if (TYPE(v) != T_STRING) {
 		rb_raise(rb_eTypeError, "_dump() must return string");
 	    }
-	    if (!ivtbl && (ivtbl = rb_generic_ivar_table(v))) {
+	    if ((hasiv2 = has_ivars(v, ivtbl2)) != 0 && !hasiv) {
 		w_byte(TYPE_IVAR, arg);
 	    }
 	    w_class(TYPE_USERDEF, obj, arg, Qfalse);
-	    w_bytes(RSTRING(v)->ptr, RSTRING(v)->len, arg);
-	    if (ivtbl) {
-		w_ivar(ivtbl, &c_arg);
+	    w_bytes(RSTRING_PTR(v), RSTRING_LEN(v), arg);
+            if (hasiv2) {
+		w_ivar(v, ivtbl2, &c_arg);
+            }
+            else if (hasiv) {
+		w_ivar(obj, ivtbl, &c_arg);
 	    }
+            st_add_direct(arg->data, obj, arg->data->num_entries);
 	    return;
 	}
 
+        st_add_direct(arg->data, obj, arg->data->num_entries);
+
+        {
+            st_data_t compat_data;
+            rb_alloc_func_t allocator = rb_get_alloc_func(RBASIC(obj)->klass);
+            if (st_lookup(compat_allocator_tbl,
+                          (st_data_t)allocator,
+                          &compat_data)) {
+                marshal_compat_t *compat = (marshal_compat_t*)compat_data;
+                VALUE real_obj = obj;
+                obj = compat->dumper(real_obj);
+                st_insert(arg->compat_tbl, (st_data_t)obj, (st_data_t)real_obj);
+            }
+        }
+
 	switch (BUILTIN_TYPE(obj)) {
 	  case T_CLASS:
 	    if (FL_TEST(obj, FL_SINGLETON)) {
@@ -490,8 +641,8 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
 	    }
 	    w_byte(TYPE_CLASS, arg);
 	    {
-		VALUE path = class2path(obj);
-		w_bytes(RSTRING(path)->ptr, RSTRING(path)->len, arg);
+		volatile VALUE path = class2path(obj);
+		w_bytes(RSTRING_PTR(path), RSTRING_LEN(path), arg);
 	    }
 	    break;
 
@@ -499,21 +650,21 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
 	    w_byte(TYPE_MODULE, arg);
 	    {
 		VALUE path = class2path(obj);
-		w_bytes(RSTRING(path)->ptr, RSTRING(path)->len, arg);
+		w_bytes(RSTRING_PTR(path), RSTRING_LEN(path), arg);
 	    }
 	    break;
 
 	  case T_FLOAT:
 	    w_byte(TYPE_FLOAT, arg);
-	    w_float(RFLOAT(obj)->value, arg);
+	    w_float(RFLOAT_VALUE(obj), arg);
 	    break;
 
 	  case T_BIGNUM:
 	    w_byte(TYPE_BIGNUM, arg);
 	    {
-		char sign = RBIGNUM(obj)->sign ? '+' : '-';
-		long len = RBIGNUM(obj)->len;
-		BDIGIT *d = RBIGNUM(obj)->digits;
+		char sign = RBIGNUM_SIGN(obj) ? '+' : '-';
+		long len = RBIGNUM_LEN(obj);
+		BDIGIT *d = RBIGNUM_DIGITS(obj);
 
 		w_byte(sign, arg);
 		w_long(SHORTLEN(len), arg); /* w_short? */
@@ -538,27 +689,28 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
 	  case T_STRING:
 	    w_uclass(obj, rb_cString, arg);
 	    w_byte(TYPE_STRING, arg);
-	    w_bytes(RSTRING(obj)->ptr, RSTRING(obj)->len, arg);
+	    w_bytes(RSTRING_PTR(obj), RSTRING_LEN(obj), arg);
 	    break;
 
 	  case T_REGEXP:
 	    w_uclass(obj, rb_cRegexp, arg);
 	    w_byte(TYPE_REGEXP, arg);
 	    w_bytes(RREGEXP(obj)->str, RREGEXP(obj)->len, arg);
-	    w_byte(rb_reg_options(obj), arg);
+	    w_byte((char)rb_reg_options(obj), arg);
 	    break;
 
 	  case T_ARRAY:
 	    w_uclass(obj, rb_cArray, arg);
 	    w_byte(TYPE_ARRAY, arg);
 	    {
-		long len = RARRAY(obj)->len;
-		VALUE *ptr = RARRAY(obj)->ptr;
+		long i, len = RARRAY_LEN(obj);
 
 		w_long(len, arg);
-		while (len--) {
-		    w_object(*ptr, arg, limit);
-		    ptr++;
+		for (i=0; i<RARRAY_LEN(obj); i++) {
+		    w_object(RARRAY_PTR(obj)[i], arg, limit);
+		    if (len != RARRAY_LEN(obj)) {
+			rb_raise(rb_eRuntimeError, "array modified during dump");
+		    }
 		}
 	    }
 	    break;
@@ -575,7 +727,7 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
 	    else {
 		w_byte(TYPE_HASH_DEF, arg);
 	    }
-	    w_long(RHASH(obj)->tbl->num_entries, arg);
+	    w_long(RHASH_SIZE(obj), arg);
 	    rb_hash_foreach(obj, hash_each, (st_data_t)&c_arg);
 	    if (!NIL_P(RHASH(obj)->ifnone)) {
 		w_object(RHASH(obj)->ifnone, arg, limit);
@@ -592,7 +744,7 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
 		w_long(len, arg);
 		mem = rb_struct_members(obj);
 		for (i=0; i<len; i++) {
-		    w_symbol(SYM2ID(RARRAY(mem)->ptr[i]), arg);
+		    w_symbol(SYM2ID(RARRAY_PTR(mem)[i]), arg);
 		    w_object(RSTRUCT_PTR(obj)[i], arg, limit);
 		}
 	    }
@@ -600,20 +752,21 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
 
 	  case T_OBJECT:
 	    w_class(TYPE_OBJECT, obj, arg, Qtrue);
-	    w_ivar(ROBJECT(obj)->iv_tbl, &c_arg);
+	    w_objivar(obj, &c_arg);
 	    break;
 
 	  case T_DATA:
 	    {
 		VALUE v;
 
-		w_class(TYPE_DATA, obj, arg, Qtrue);
 		if (!rb_respond_to(obj, s_dump_data)) {
 		    rb_raise(rb_eTypeError,
 			     "no marshal_dump is defined for class %s",
 			     rb_obj_classname(obj));
 		}
 		v = rb_funcall(obj, s_dump_data, 0);
+		check_dump_arg(arg);
+		w_class(TYPE_DATA, obj, arg, Qtrue);
 		w_object(v, arg, limit);
 	    }
 	    break;
@@ -624,8 +777,8 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
 	    break;
 	}
     }
-    if (ivtbl) {
-	w_ivar(ivtbl, &c_arg);
+    if (hasiv) {
+	w_ivar(obj, ivtbl, &c_arg);
     }
 }
 
@@ -643,8 +796,12 @@ dump(struct dump_call_arg *arg)
 static VALUE
 dump_ensure(struct dump_arg *arg)
 {
+    if (!DATA_PTR(arg->wrapper)) return 0;
     st_free_table(arg->symbols);
     st_free_table(arg->data);
+    st_free_table(arg->compat_tbl);
+    DATA_PTR(arg->wrapper) = 0;
+    arg->wrapper = 0;
     if (arg->taint) {
 	OBJ_TAINT(arg->str);
     }
@@ -717,6 +874,9 @@ marshal_dump(int argc, VALUE *argv)
     arg.symbols = st_init_numtable();
     arg.data    = st_init_numtable();
     arg.taint   = Qfalse;
+    arg.compat_tbl = st_init_numtable();
+    arg.wrapper = Data_Wrap_Struct(rb_cData, mark_dump_arg, 0, &arg);
+    arg.encodings = 0;
     c_arg.obj   = obj;
     c_arg.arg   = &arg;
     c_arg.limit = limit;
@@ -736,9 +896,21 @@ struct load_arg {
     VALUE data;
     VALUE proc;
     int taint;
+    st_table *compat_tbl;
+    VALUE compat_tbl_wrapper;
 };
 
+static void
+check_load_arg(struct load_arg *arg)
+{
+    if (!DATA_PTR(arg->compat_tbl_wrapper)) {
+	rb_raise(rb_eRuntimeError, "Marshal.load reentered");
+    }
+}
+
+static VALUE r_entry(VALUE v, struct load_arg *arg);
 static VALUE r_object(struct load_arg *arg);
+static VALUE path2class(const char *path);
 
 static int
 r_byte(struct load_arg *arg)
@@ -746,8 +918,8 @@ r_byte(struct load_arg *arg)
     int c;
 
     if (TYPE(arg->src) == T_STRING) {
-	if (RSTRING(arg->src)->len > arg->offset) {
-	    c = (unsigned char)RSTRING(arg->src)->ptr[arg->offset++];
+	if (RSTRING_LEN(arg->src) > arg->offset) {
+	    c = (unsigned char)RSTRING_PTR(arg->src)[arg->offset++];
 	}
 	else {
 	    rb_raise(rb_eArgError, "marshal data too short");
@@ -755,9 +927,10 @@ r_byte(struct load_arg *arg)
     }
     else {
 	VALUE src = arg->src;
-	VALUE v = rb_funcall2(src, s_getc, 0, 0);
+	VALUE v = rb_funcall2(src, s_getbyte, 0, 0);
+	check_load_arg(arg);
 	if (NIL_P(v)) rb_eof_error();
-	c = (unsigned char)FIX2INT(v);
+	c = (unsigned char)NUM2CHR(v);
     }
     return c;
 }
@@ -765,7 +938,7 @@ r_byte(struct load_arg *arg)
 static void
 long_toobig(int size)
 {
-    rb_raise(rb_eTypeError, "long too big for this architecture (size %d, given %d)",
+    rb_raise(rb_eTypeError, "long too big for this architecture (size %zd, given %d)",
 	     sizeof(long), size);
 }
 
@@ -819,8 +992,8 @@ r_bytes0(long len, struct load_arg *arg)
 
     if (len == 0) return rb_str_new(0, 0);
     if (TYPE(arg->src) == T_STRING) {
-	if (RSTRING(arg->src)->len > arg->offset) {
-	    str = rb_str_new(RSTRING(arg->src)->ptr+arg->offset, len);
+	if (RSTRING_LEN(arg->src) - arg->offset >= len) {
+	    str = rb_str_new(RSTRING_PTR(arg->src)+arg->offset, len);
 	    arg->offset += len;
 	}
 	else {
@@ -832,9 +1005,10 @@ r_bytes0(long len, struct load_arg *arg)
 	VALUE src = arg->src;
 	VALUE n = LONG2NUM(len);
 	str = rb_funcall2(src, s_read, 1, &n);
+	check_load_arg(arg);
 	if (NIL_P(str)) goto too_short;
 	StringValue(str);
-	if (RSTRING(str)->len != len) goto too_short;
+	if (RSTRING_LEN(str) != len) goto too_short;
 	if (OBJ_TAINTED(str)) arg->taint = Qtrue;
     }
     return str;
@@ -855,9 +1029,9 @@ r_symlink(struct load_arg *arg)
 static ID
 r_symreal(struct load_arg *arg)
 {
-    ID id;
+    volatile VALUE s = r_bytes(arg);
+    ID id = rb_intern(RSTRING_PTR(s));
 
-    id = rb_intern(RSTRING(r_bytes(arg))->ptr);
     st_insert(arg->symbols, arg->symbols->num_entries, id);
 
     return id;
@@ -866,13 +1040,20 @@ r_symreal(struct load_arg *arg)
 static ID
 r_symbol(struct load_arg *arg)
 {
-    if (r_byte(arg) == TYPE_SYMLINK) {
+    int type;
+
+    switch ((type = r_byte(arg))) {
+      case TYPE_SYMBOL:
+	return r_symreal(arg);
+      case TYPE_SYMLINK:
 	return r_symlink(arg);
+      default:
+	rb_raise(rb_eArgError, "dump format error(0x%x)", type);
+	break;
     }
-    return r_symreal(arg);
 }
 
-static char*
+static const char*
 r_unique(struct load_arg *arg)
 {
     return rb_id2name(r_symbol(arg));
@@ -887,8 +1068,40 @@ r_string(struct load_arg *arg)
 static VALUE
 r_entry(VALUE v, struct load_arg *arg)
 {
-    rb_hash_aset(arg->data, INT2FIX(RHASH(arg->data)->tbl->num_entries), v);
-    if (arg->taint) OBJ_TAINT(v);
+    st_data_t real_obj = (VALUE)Qundef;
+    if (st_lookup(arg->compat_tbl, v, &real_obj)) {
+        rb_hash_aset(arg->data, INT2FIX(RHASH_SIZE(arg->data)), (VALUE)real_obj);
+    }
+    else {
+        rb_hash_aset(arg->data, INT2FIX(RHASH_SIZE(arg->data)), v);
+    }
+    if (arg->taint) {
+        OBJ_TAINT(v);
+        if ((VALUE)real_obj != Qundef)
+            OBJ_TAINT((VALUE)real_obj);
+    }
+    return v;
+}
+
+static VALUE
+r_leave(VALUE v, struct load_arg *arg)
+{
+    st_data_t data;
+    if (st_lookup(arg->compat_tbl, v, &data)) {
+        VALUE real_obj = (VALUE)data;
+        rb_alloc_func_t allocator = rb_get_alloc_func(CLASS_OF(real_obj));
+        st_data_t key = v;
+        if (st_lookup(compat_allocator_tbl, (st_data_t)allocator, &data)) {
+            marshal_compat_t *compat = (marshal_compat_t*)data;
+            compat->loader(real_obj, v);
+        }
+        st_delete(arg->compat_tbl, &key, 0);
+        v = real_obj;
+    }
+    if (arg->proc) {
+	v = rb_funcall(arg->proc, rb_intern("call"), 1, v);
+	check_load_arg(arg);
+    }
     return v;
 }
 
@@ -902,7 +1115,13 @@ r_ivar(VALUE obj, struct load_arg *arg)
 	while (len--) {
 	    ID id = r_symbol(arg);
 	    VALUE val = r_object(arg);
-	    rb_ivar_set(obj, id, val);
+	    if (id == rb_id_encoding()) {
+		int idx = rb_enc_find_index(StringValueCStr(val));
+		if (idx > 0) rb_enc_associate_index(obj, idx);
+	    }
+	    else {
+		rb_ivar_set(obj, id, val);
+	    }
 	}
     }
 }
@@ -930,7 +1149,28 @@ path2module(const char *path)
 }
 
 static VALUE
-r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
+obj_alloc_by_path(const char *path, struct load_arg *arg)
+{
+    VALUE klass;
+    st_data_t data;
+    rb_alloc_func_t allocator;
+
+    klass = path2class(path);
+
+    allocator = rb_get_alloc_func(klass);
+    if (st_lookup(compat_allocator_tbl, (st_data_t)allocator, &data)) {
+        marshal_compat_t *compat = (marshal_compat_t*)data;
+        VALUE real_obj = rb_obj_alloc(klass);
+        VALUE obj = rb_obj_alloc(compat->oldclass);
+        st_insert(arg->compat_tbl, (st_data_t)obj, (st_data_t)real_obj);
+        return obj;
+    }
+
+    return rb_obj_alloc(klass);
+}
+
+static VALUE
+r_object0(struct load_arg *arg, int *ivp, VALUE extmod)
 {
     VALUE v = Qnil;
     int type = r_byte(arg);
@@ -940,16 +1180,21 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
       case TYPE_LINK:
 	id = r_long(arg);
 	v = rb_hash_aref(arg->data, LONG2FIX(id));
+	check_load_arg(arg);
 	if (NIL_P(v)) {
 	    rb_raise(rb_eArgError, "dump format error (unlinked)");
 	}
-	return v;
+	if (arg->proc) {
+	    v = rb_funcall(arg->proc, rb_intern("call"), 1, v);
+	    check_load_arg(arg);
+	}
+	break;
 
       case TYPE_IVAR:
         {
 	    int ivar = Qtrue;
 
-	    v = r_object0(arg, 0, &ivar, extmod);
+	    v = r_object0(arg, &ivar, extmod);
 	    if (ivar) r_ivar(v, arg);
 	}
 	break;
@@ -961,8 +1206,8 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
             if (NIL_P(extmod)) extmod = rb_ary_new2(0);
             rb_ary_push(extmod, m);
 
-	    v = r_object0(arg, 0, 0, extmod);
-            while (RARRAY(extmod)->len > 0) {
+	    v = r_object0(arg, 0, extmod);
+            while (RARRAY_LEN(extmod) > 0) {
                 m = rb_ary_pop(extmod);
                 rb_extend_object(v, m);
             }
@@ -976,7 +1221,7 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 	    if (FL_TEST(c, FL_SINGLETON)) {
 		rb_raise(rb_eTypeError, "singleton can't be loaded");
 	    }
-	    v = r_object0(arg, 0, 0, extmod);
+	    v = r_object0(arg, 0, extmod);
 	    if (rb_special_const_p(v) || TYPE(v) == T_OBJECT || TYPE(v) == T_CLASS) {
 	      format_error:
 		rb_raise(rb_eArgError, "dump format error (user class)");
@@ -992,14 +1237,17 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 
       case TYPE_NIL:
 	v = Qnil;
+	v = r_leave(v, arg);
 	break;
 
       case TYPE_TRUE:
 	v = Qtrue;
+	v = r_leave(v, arg);
 	break;
 
       case TYPE_FALSE:
 	v = Qfalse;
+	v = r_leave(v, arg);
 	break;
 
       case TYPE_FIXNUM:
@@ -1007,13 +1255,14 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 	    long i = r_long(arg);
 	    v = LONG2FIX(i);
 	}
+	v = r_leave(v, arg);
 	break;
 
       case TYPE_FLOAT:
 	{
 	    double d, t = 0.0;
 	    VALUE str = r_bytes(arg);
-	    const char *ptr = RSTRING(str)->ptr;
+	    const char *ptr = RSTRING_PTR(str);
 
 	    if (strcmp(ptr, "nan") == 0) {
 		d = t / t;
@@ -1027,10 +1276,11 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 	    else {
 		char *e;
 		d = strtod(ptr, &e);
-		d = load_mantissa(d, e, RSTRING(str)->len - (e - ptr));
+		d = load_mantissa(d, e, RSTRING_LEN(str) - (e - ptr));
 	    }
-	    v = rb_float_new(d);
-	    r_entry(v, arg);
+	    v = DOUBLE2NUM(d);
+	    v = r_entry(v, arg);
+            v = r_leave(v, arg);
 	}
 	break;
 
@@ -1042,21 +1292,21 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 
 	    NEWOBJ(big, struct RBignum);
 	    OBJSETUP(big, rb_cBignum, T_BIGNUM);
-	    big->sign = (r_byte(arg) == '+');
+	    RBIGNUM_SET_SIGN(big, (r_byte(arg) == '+'));
 	    len = r_long(arg);
 	    data = r_bytes0(len * 2, arg);
 #if SIZEOF_BDIGITS == SIZEOF_SHORT
-	    big->len = len;
+            rb_big_resize((VALUE)big, len);
 #else
-	    big->len = (len + 1) * 2 / sizeof(BDIGIT);
+            rb_big_resize((VALUE)big, (len + 1) * 2 / sizeof(BDIGIT));
 #endif
-	    big->digits = digits = ALLOC_N(BDIGIT, big->len);
-	    MEMCPY(digits, RSTRING(data)->ptr, char, len * 2);
+            digits = RBIGNUM_DIGITS(big);
+	    MEMCPY(digits, RSTRING_PTR(data), char, len * 2);
 #if SIZEOF_BDIGITS > SIZEOF_SHORT
 	    MEMZERO((char *)digits + len * 2, char,
-		    big->len * sizeof(BDIGIT) - len * 2);
+		    RBIGNUM_LEN(big) * sizeof(BDIGIT) - len * 2);
 #endif
-	    len = big->len;
+	    len = RBIGNUM_LEN(big);
 	    while (len > 0) {
 		unsigned char *p = (unsigned char *)digits;
 		BDIGIT num = 0;
@@ -1075,19 +1325,22 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 		len--;
 	    }
 	    v = rb_big_norm((VALUE)big);
-	    r_entry(v, arg);
+	    v = r_entry(v, arg);
+            v = r_leave(v, arg);
 	}
 	break;
 
       case TYPE_STRING:
 	v = r_entry(r_string(arg), arg);
+        v = r_leave(v, arg);
 	break;
 
       case TYPE_REGEXP:
 	{
 	    volatile VALUE str = r_bytes(arg);
 	    int options = r_byte(arg);
-	    v = r_entry(rb_reg_new(RSTRING(str)->ptr, RSTRING(str)->len, options), arg);
+	    v = r_entry(rb_reg_new_str(str, options), arg);
+            v = r_leave(v, arg);
 	}
 	break;
 
@@ -1096,10 +1349,11 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 	    volatile long len = r_long(arg); /* gcc 2.7.2.3 -O2 bug?? */
 
 	    v = rb_ary_new2(len);
-	    r_entry(v, arg);
+	    v = r_entry(v, arg);
 	    while (len--) {
 		rb_ary_push(v, r_object(arg));
 	    }
+            v = r_leave(v, arg);
 	}
 	break;
 
@@ -1109,7 +1363,7 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 	    long len = r_long(arg);
 
 	    v = rb_hash_new();
-	    r_entry(v, arg);
+	    v = r_entry(v, arg);
 	    while (len--) {
 		VALUE key = r_object(arg);
 		VALUE value = r_object(arg);
@@ -1118,40 +1372,46 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 	    if (type == TYPE_HASH_DEF) {
 		RHASH(v)->ifnone = r_object(arg);
 	    }
+            v = r_leave(v, arg);
 	}
 	break;
 
       case TYPE_STRUCT:
 	{
-	    VALUE klass, mem, values;
+	    VALUE klass, mem;
+            VALUE values;
 	    volatile long i;	/* gcc 2.7.2.3 -O2 bug?? */
 	    long len;
 	    ID slot;
 
 	    klass = path2class(r_unique(arg));
-	    mem = rb_struct_s_members(klass);
-	    if (mem == Qnil) {
-		rb_raise(rb_eTypeError, "uninitialized struct");
-	    }
 	    len = r_long(arg);
 
-	    values = rb_ary_new2(len);
-	    for (i=0; i<len; i++) {
-		rb_ary_push(values, Qnil);
+            v = rb_obj_alloc(klass);
+	    if (TYPE(v) != T_STRUCT) {
+		rb_raise(rb_eTypeError, "class %s not a struct", rb_class2name(klass));
 	    }
-	    v = rb_struct_alloc(klass, values);
-	    r_entry(v, arg);
+	    mem = rb_struct_s_members(klass);
+            if (RARRAY_LEN(mem) != len) {
+                rb_raise(rb_eTypeError, "struct %s not compatible (struct size differs)",
+                         rb_class2name(klass));
+            }
+
+	    v = r_entry(v, arg);
+	    values = rb_ary_new2(len);
 	    for (i=0; i<len; i++) {
 		slot = r_symbol(arg);
 
-		if (RARRAY(mem)->ptr[i] != ID2SYM(slot)) {
+		if (RARRAY_PTR(mem)[i] != ID2SYM(slot)) {
 		    rb_raise(rb_eTypeError, "struct %s not compatible (:%s for :%s)",
 			     rb_class2name(klass),
 			     rb_id2name(slot),
-			     rb_id2name(SYM2ID(RARRAY(mem)->ptr[i])));
+			     rb_id2name(SYM2ID(RARRAY_PTR(mem)[i])));
 		}
-		rb_struct_aset(v, LONG2FIX(i), r_object(arg));
+                rb_ary_push(values, r_object(arg));
 	    }
+            rb_struct_initialize(v, values);
+            v = r_leave(v, arg);
 	}
 	break;
 
@@ -1170,7 +1430,9 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 		*ivp = Qfalse;
 	    }
 	    v = rb_funcall(klass, s_load, 1, data);
-	    r_entry(v, arg);
+	    check_load_arg(arg);
+	    v = r_entry(v, arg);
+            v = r_leave(v, arg);
 	}
         break;
 
@@ -1180,8 +1442,8 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 	    VALUE data;
 
 	    v = rb_obj_alloc(klass);
-            if (! NIL_P(extmod)) {
-                while (RARRAY(extmod)->len > 0) {
+            if (!NIL_P(extmod)) {
+                while (RARRAY_LEN(extmod) > 0) {
                     VALUE m = rb_ary_pop(extmod);
                     rb_extend_object(v, m);
                 }
@@ -1190,22 +1452,23 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 		rb_raise(rb_eTypeError, "instance of %s needs to have method `marshal_load'",
 			 rb_class2name(klass));
 	    }
-	    r_entry(v, arg);
+	    v = r_entry(v, arg);
 	    data = r_object(arg);
 	    rb_funcall(v, s_mload, 1, data);
+	    check_load_arg(arg);
+            v = r_leave(v, arg);
 	}
         break;
 
       case TYPE_OBJECT:
 	{
-	    VALUE klass = path2class(r_unique(arg));
-
-	    v = rb_obj_alloc(klass);
+            v = obj_alloc_by_path(r_unique(arg), arg);
 	    if (TYPE(v) != T_OBJECT) {
 		rb_raise(rb_eArgError, "dump format error");
 	    }
-	    r_entry(v, arg);
+	    v = r_entry(v, arg);
 	    r_ivar(v, arg);
+	    v = r_leave(v, arg);
 	}
 	break;
 
@@ -1219,6 +1482,7 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
 		   warn = Qfalse;
 	       }
 	       v = rb_funcall(klass, s_alloc, 0);
+	       check_load_arg(arg);
            }
 	   else {
 	       v = rb_obj_alloc(klass);
@@ -1226,13 +1490,15 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
            if (TYPE(v) != T_DATA) {
                rb_raise(rb_eArgError, "dump format error");
            }
-           r_entry(v, arg);
+           v = r_entry(v, arg);
            if (!rb_respond_to(v, s_load_data)) {
                rb_raise(rb_eTypeError,
                         "class %s needs to have instance method `_load_data'",
                         rb_class2name(klass));
            }
-           rb_funcall(v, s_load_data, 1, r_object0(arg, 0, 0, extmod));
+           rb_funcall(v, s_load_data, 1, r_object0(arg, 0, extmod));
+	   check_load_arg(arg);
+           v = r_leave(v, arg);
        }
        break;
 
@@ -1240,8 +1506,9 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
         {
 	    volatile VALUE str = r_bytes(arg);
 
-	    v = rb_path2class(RSTRING(str)->ptr);
-	    r_entry(v, arg);
+	    v = rb_path2class(RSTRING_PTR(str));
+	    v = r_entry(v, arg);
+            v = r_leave(v, arg);
 	}
 	break;
 
@@ -1249,8 +1516,9 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
         {
 	    volatile VALUE str = r_bytes(arg);
 
-	    v = path2class(RSTRING(str)->ptr);
-	    r_entry(v, arg);
+	    v = path2class(RSTRING_PTR(str));
+	    v = r_entry(v, arg);
+            v = r_leave(v, arg);
 	}
 	break;
 
@@ -1258,32 +1526,32 @@ r_object0(struct load_arg *arg, VALUE proc, int *ivp, VALUE extmod)
         {
 	    volatile VALUE str = r_bytes(arg);
 
-	    v = path2module(RSTRING(str)->ptr);
-	    r_entry(v, arg);
+	    v = path2module(RSTRING_PTR(str));
+	    v = r_entry(v, arg);
+            v = r_leave(v, arg);
 	}
 	break;
 
       case TYPE_SYMBOL:
 	v = ID2SYM(r_symreal(arg));
+	v = r_leave(v, arg);
 	break;
 
       case TYPE_SYMLINK:
-	return ID2SYM(r_symlink(arg));
+	v = ID2SYM(r_symlink(arg));
+	break;
 
       default:
 	rb_raise(rb_eArgError, "dump format error(0x%x)", type);
 	break;
     }
-    if (proc) {
-	rb_funcall(proc, rb_intern("call"), 1, v);
-    }
     return v;
 }
 
 static VALUE
 r_object(struct load_arg *arg)
 {
-    return r_object0(arg, arg->proc, 0, Qnil);
+    return r_object0(arg, 0, Qnil);
 }
 
 static VALUE
@@ -1295,7 +1563,11 @@ load(struct load_arg *arg)
 static VALUE
 load_ensure(struct load_arg *arg)
 {
+    if (!DATA_PTR(arg->compat_tbl_wrapper)) return 0;
     st_free_table(arg->symbols);
+    st_free_table(arg->compat_tbl);
+    DATA_PTR(arg->compat_tbl_wrapper) = 0;
+    arg->compat_tbl_wrapper = 0;
     return 0;
 }
 
@@ -1319,11 +1591,12 @@ marshal_load(int argc, VALUE *argv)
     struct load_arg arg;
 
     rb_scan_args(argc, argv, "11", &port, &proc);
-    if (rb_respond_to(port, rb_intern("to_str"))) {
+    v = rb_check_string_type(port);
+    if (!NIL_P(v)) {
 	arg.taint = OBJ_TAINTED(port); /* original taintedness */
-	StringValue(port);	       /* possible conversion */
+	port = v;
     }
-    else if (rb_respond_to(port, s_getc) && rb_respond_to(port, s_read)) {
+    else if (rb_respond_to(port, s_getbyte) && rb_respond_to(port, s_read)) {
 	if (rb_respond_to(port, s_binmode)) {
 	    rb_funcall2(port, s_binmode, 0, 0);
 	}
@@ -1334,6 +1607,8 @@ marshal_load(int argc, VALUE *argv)
     }
     arg.src = port;
     arg.offset = 0;
+    arg.compat_tbl = st_init_numtable();
+    arg.compat_tbl_wrapper = Data_Wrap_Struct(rb_cData, rb_mark_tbl, 0, arg.compat_tbl);
 
     major = r_byte(&arg);
     minor = r_byte(&arg);
@@ -1350,6 +1625,7 @@ marshal_load(int argc, VALUE *argv)
 
     arg.symbols = st_init_numtable();
     arg.data   = rb_hash_new();
+    RBASIC(arg.data)->klass = 0;
     if (NIL_P(proc)) arg.proc = 0;
     else             arg.proc = proc;
     v = rb_ensure(load, (VALUE)&arg, load_ensure, (VALUE)&arg);
@@ -1372,9 +1648,9 @@ marshal_load(int argc, VALUE *argv)
  * first two bytes of marshaled data.
  *
  *     str = Marshal.dump("thing")
- *     RUBY_VERSION   #=> "1.8.0"
- *     str[0]         #=> 4
- *     str[1]         #=> 8
+ *     RUBY_VERSION   #=> "1.9.0"
+ *     str[0].ord     #=> 4
+ *     str[1].ord     #=> 8
  *
  * Some objects cannot be dumped: if the objects to be dumped include
  * bindings, procedure or method objects, instances of class IO, or
@@ -1393,6 +1669,8 @@ marshal_load(int argc, VALUE *argv)
 void
 Init_marshal(void)
 {
+#undef rb_intern
+
     VALUE rb_mMarshal = rb_define_module("Marshal");
 
     s_dump = rb_intern("_dump");
@@ -1402,7 +1680,7 @@ Init_marshal(void)
     s_dump_data = rb_intern("_dump_data");
     s_load_data = rb_intern("_load_data");
     s_alloc = rb_intern("_alloc");
-    s_getc = rb_intern("getc");
+    s_getbyte = rb_intern("getbyte");
     s_read = rb_intern("read");
     s_write = rb_intern("write");
     s_binmode = rb_intern("binmode");
@@ -1413,6 +1691,11 @@ Init_marshal(void)
 
     rb_define_const(rb_mMarshal, "MAJOR_VERSION", INT2FIX(MARSHAL_MAJOR));
     rb_define_const(rb_mMarshal, "MINOR_VERSION", INT2FIX(MARSHAL_MINOR));
+
+    compat_allocator_tbl = st_init_numtable();
+    rb_gc_register_address(&compat_allocator_tbl_wrapper);
+    compat_allocator_tbl_wrapper =
+	Data_Wrap_Struct(rb_cData, mark_marshal_compat_t, 0, compat_allocator_tbl);
 }
 
 VALUE
