@@ -5,11 +5,10 @@
 ;;; C/C++ mode style for Ruby.
 ;;;
 ;;;  $Author: nobu $
-;;;  $Date: 2007-10-26 15:00:52 +0900 (Fri, 26 Oct 2007) $
 ;;;  created at: Thu Apr 26 13:54:01 JST 2007
 ;;;
 
-(defconst ruby-style-revision "$Revision: 13784 $"
+(defconst ruby-style-revision "$Revision: 16153 $"
   "Ruby style revision string.")
 
 (defconst ruby-style-version
@@ -20,6 +19,7 @@
 
 (defun ruby-style-case-indent (x)
   (save-excursion
+    (back-to-indentation)
     (unless (progn (backward-up-list) (back-to-indentation)
 		   (> (point) (cdr x)))
       (goto-char (cdr x))
@@ -27,8 +27,9 @@
 
 (defun ruby-style-label-indent (x)
   (save-excursion
+    (back-to-indentation)
     (unless (progn (backward-up-list) (back-to-indentation)
-		   (> (point) (cdr x)))
+		   (>= (point) (cdr x)))
       (goto-char (cdr x))
       (condition-case ()
 	  (progn
