@@ -2,8 +2,7 @@
 
   object.c -
 
-  $Author: matz $
-  $Date: 2007-08-25 12:29:39 +0900 (土, 25  8月 2007) $
+  $Author: nobu $
   created at: Thu Jul 15 12:01:24 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -20,6 +19,7 @@
 #include <errno.h>
 #include <ctype.h>
 #include <math.h>
+#include <float.h>
 
 VALUE rb_cBasicObject;
 VALUE rb_mKernel;
@@ -32,7 +32,7 @@ VALUE rb_cNilClass;
 VALUE rb_cTrueClass;
 VALUE rb_cFalseClass;
 
-static ID id_eq, id_eql, id_inspect, id_init_copy;
+static ID id_eq, id_eql, id_match, id_inspect, id_init_copy;
 
 /*
  *  call-seq:
@@ -89,18 +89,47 @@ rb_eql(VALUE obj1, VALUE obj2)
  *     1.eql? 1.0   #=> false
  */
 
-static VALUE
+VALUE
 rb_obj_equal(VALUE obj1, VALUE obj2)
 {
     if (obj1 == obj2) return Qtrue;
     return Qfalse;
 }
 
+/*
+ *  call-seq:
+ *     !obj    => true or false
+ *
+ *  Boolean negate.
+ */
+
+VALUE
+rb_obj_not(VALUE obj)
+{
+    return RTEST(obj) ? Qfalse : Qtrue;
+}
+
+/*
+ *  call-seq:
+ *     obj != other        => true or false
+ *
+ *  Returns true if two objects are not-equal, otherwise false.
+ */
+
+VALUE
+rb_obj_not_equal(VALUE obj1, VALUE obj2)
+{
+    VALUE result = rb_funcall(obj1, id_eq, 1, obj2);
+    return RTEST(result) ? Qfalse : Qtrue;
+}
+
 VALUE
 rb_class_real(VALUE cl)
 {
-    while (FL_TEST(cl, FL_SINGLETON) || TYPE(cl) == T_ICLASS) {
-	cl = RCLASS(cl)->super;
+    if (cl == 0)
+        return 0;
+    while ((RBASIC(cl)->flags & FL_SINGLETON) || BUILTIN_TYPE(cl) == T_ICLASS) {
+	cl = RCLASS_SUPER(cl);
     }
     return cl;
 }
@@ -137,15 +166,36 @@ init_copy(VALUE dest, VALUE obj)
     rb_gc_copy_finalizer(dest, obj);
     switch (TYPE(obj)) {
       case T_OBJECT:
+        if (!(RBASIC(dest)->flags & ROBJECT_EMBED) && ROBJECT_IVPTR(dest)) {
+            xfree(ROBJECT_IVPTR(dest));
+            ROBJECT(dest)->as.heap.ivptr = 0;
+            ROBJECT(dest)->as.heap.numiv = 0;
+            ROBJECT(dest)->as.heap.iv_index_tbl = 0;
+        }
+        if (RBASIC(obj)->flags & ROBJECT_EMBED) {
+            MEMCPY(ROBJECT(dest)->as.ary, ROBJECT(obj)->as.ary, VALUE, ROBJECT_EMBED_LEN_MAX);
+            RBASIC(dest)->flags |= ROBJECT_EMBED;
+        }
+        else {
+            long len = ROBJECT(obj)->as.heap.numiv;
+            VALUE *ptr = ALLOC_N(VALUE, len);
+            MEMCPY(ptr, ROBJECT(obj)->as.heap.ivptr, VALUE, len);
+            ROBJECT(dest)->as.heap.ivptr = ptr;
+            ROBJECT(dest)->as.heap.numiv = len;
+            ROBJECT(dest)->as.heap.iv_index_tbl = ROBJECT(obj)->as.heap.iv_index_tbl;
+            RBASIC(dest)->flags &= ~ROBJECT_EMBED;
+        }
+        break;
       case T_CLASS:
       case T_MODULE:
-	if (ROBJECT(dest)->iv_tbl) {
-	    st_free_table(ROBJECT(dest)->iv_tbl);
-	    ROBJECT(dest)->iv_tbl = 0;
+	if (RCLASS_IV_TBL(dest)) {
+	    st_free_table(RCLASS_IV_TBL(dest));
+	    RCLASS_IV_TBL(dest) = 0;
 	}
-	if (ROBJECT(obj)->iv_tbl) {
-	    ROBJECT(dest)->iv_tbl = st_copy(ROBJECT(obj)->iv_tbl);
+	if (RCLASS_IV_TBL(obj)) {
+	    RCLASS_IV_TBL(dest) = st_copy(RCLASS_IV_TBL(obj));
 	}
+        break;
     }
     rb_funcall(dest, id_init_copy, 1, obj);
 }
@@ -248,7 +298,7 @@ rb_obj_init_copy(VALUE obj, VALUE orig)
 VALUE
 rb_any_to_s(VALUE obj)
 {
-    char *cname = rb_obj_classname(obj);
+    const char *cname = rb_obj_classname(obj);
     VALUE str;
 
     str = rb_sprintf("#<%s:%p>", cname, (void*)obj);
@@ -296,7 +346,7 @@ inspect_obj(VALUE obj, VALUE str, int recur)
 	rb_str_cat2(str, " ...");
     }
     else {
-	st_foreach_safe(ROBJECT(obj)->iv_tbl, inspect_i, str);
+	rb_ivar_foreach(obj, inspect_i, str);
     }
     rb_str_cat2(str, ">");
     RSTRING_PTR(str)[0] = '#';
@@ -314,22 +364,34 @@ inspect_obj(VALUE obj, VALUE str, int recur)
  *  generate the string.
  *     
  *     [ 1, 2, 3..4, 'five' ].inspect   #=> "[1, 2, 3..4, \"five\"]"
- *     Time.new.inspect                 #=> "Wed Apr 09 08:54:39 CDT 2003"
+ *     Time.new.inspect                 #=> "2008-03-08 19:43:39 +0900"
  */
 
 
 static VALUE
 rb_obj_inspect(VALUE obj)
 {
-    if (TYPE(obj) == T_OBJECT
-	&& ROBJECT(obj)->iv_tbl
-	&& ROBJECT(obj)->iv_tbl->num_entries > 0) {
-	VALUE str;
-	char *c;
 
-	c = rb_obj_classname(obj);
-	str = rb_sprintf("-<%s:%p", c, (void*)obj);
-	return rb_exec_recursive(inspect_obj, obj, str);
+    if (TYPE(obj) == T_OBJECT) {
+        int has_ivar = 0;
+        VALUE *ptr = ROBJECT_IVPTR(obj);
+        long len = ROBJECT_NUMIV(obj);
+        long i;
+
+        for (i = 0; i < len; i++) {
+            if (ptr[i] != Qundef) {
+                has_ivar = 1;
+                break;
+            }
+        }
+
+        if (has_ivar) {
+            VALUE str;
+            const char *c = rb_obj_classname(obj);
+
+            str = rb_sprintf("-<%s:%p", c, (void*)obj);
+            return rb_exec_recursive(inspect_obj, obj, str);
+        }
     }
     return rb_funcall(obj, rb_intern("to_s"), 0, 0);
 }
@@ -402,9 +464,9 @@ rb_obj_is_kind_of(VALUE obj, VALUE c)
     }
 
     while (cl) {
-	if (cl == c || RCLASS(cl)->m_tbl == RCLASS(c)->m_tbl)
+	if (cl == c || RCLASS_M_TBL(cl) == RCLASS_M_TBL(c))
 	    return Qtrue;
-	cl = RCLASS(cl)->super;
+	cl = RCLASS_SUPER(cl);
     }
     return Qfalse;
 }
@@ -414,9 +476,15 @@ rb_obj_is_kind_of(VALUE obj, VALUE c)
  *  call-seq:
  *     obj.tap{|x|...}    => obj
  *  
- *  Returns the receiver after executing the block given.  Its main
- *  purpose is to be inseted in the method chain.
- *     
+ *  Yields <code>x</code> to the block, and then returns <code>x</code>.
+ *  The primary purpose of this method is to "tap into" a method chain,
+ *  in order to perform operations on intermediate results within the chain.
+ *
+ *	(1..10)                .tap {|x| puts "original: #{x.inspect}"}
+ *	  .to_a                .tap {|x| puts "array: #{x.inspect}"}
+ *	  .select {|x| x%2==0} .tap {|x| puts "evens: #{x.inspect}"}
+ *	  .map { |x| x*x }     .tap {|x| puts "squares: #{x.inspect}"}
+ *
  */
 
 VALUE
@@ -630,6 +698,7 @@ rb_obj_infect(VALUE obj1, VALUE obj2)
     OBJ_INFECT(obj1, obj2);
 }
 
+static st_table *immediate_frozen_tbl = 0;
 
 /*
  *  call-seq:
@@ -658,6 +727,12 @@ rb_obj_freeze(VALUE obj)
 	    rb_raise(rb_eSecurityError, "Insecure: can't freeze object");
 	}
 	OBJ_FREEZE(obj);
+	if (SPECIAL_CONST_P(obj)) {
+	    if (!immediate_frozen_tbl) {
+		immediate_frozen_tbl = st_init_numtable();
+	    }
+	    st_insert(immediate_frozen_tbl, obj, (st_data_t)Qtrue);
+	}
     }
     return obj;
 }
@@ -673,10 +748,14 @@ rb_obj_freeze(VALUE obj)
  *     a.frozen?   #=> true
  */
 
-static VALUE
+VALUE
 rb_obj_frozen_p(VALUE obj)
 {
     if (OBJ_FROZEN(obj)) return Qtrue;
+    if (SPECIAL_CONST_P(obj)) {
+	if (!immediate_frozen_tbl) return Qfalse;
+	if (st_lookup(immediate_frozen_tbl, obj, 0)) return Qtrue;
+    }
     return Qfalse;
 }
 
@@ -715,7 +794,7 @@ nil_to_i(VALUE obj)
 static VALUE
 nil_to_f(VALUE obj)
 {
-    return rb_float_new(0.0);
+    return DOUBLE2NUM(0.0);
 }
 
 /*
@@ -728,7 +807,7 @@ nil_to_f(VALUE obj)
 static VALUE
 nil_to_s(VALUE obj)
 {
-    return rb_str_new2("");
+    return rb_usascii_str_new(0, 0);
 }
 
 /*
@@ -742,16 +821,6 @@ nil_to_s(VALUE obj)
  *     nil.to_a   #=> []
  */
 
-/*
- * Document-method: to_splat
- *
- *  call-seq:
- *     nil.to_splat   => []
- *  
- *  Always returns an empty array.
- *     
- */
-
 static VALUE
 nil_to_a(VALUE obj)
 {
@@ -768,30 +837,9 @@ nil_to_a(VALUE obj)
 static VALUE
 nil_inspect(VALUE obj)
 {
-    return rb_str_new2("nil");
+    return rb_usascii_str_new2("nil");
 }
 
-#ifdef NIL_PLUS
-static VALUE
-nil_plus(VALUE x, VALUE y)
-{
-    switch (TYPE(y)) {
-      case T_NIL:
-      case T_FIXNUM:
-      case T_FLOAT:
-      case T_BIGNUM:
-      case T_STRING:
-      case T_ARRAY:
-	return y;
-      default:
-	rb_raise(rb_eTypeError, "tried to add %s(%s) to nil",
-		 RSTRING_PTR(rb_inspect(y)),
-		 rb_obj_classname(y));
-    }
-    /* not reached */
-}
-#endif
-
 /***********************************************************************
  *  Document-class: TrueClass
  *
@@ -812,7 +860,7 @@ nil_plus(VALUE x, VALUE y)
 static VALUE
 true_to_s(VALUE obj)
 {
-    return rb_str_new2("true");
+    return rb_usascii_str_new2("true");
 }
 
 
@@ -889,7 +937,7 @@ true_xor(VALUE obj, VALUE obj2)
 static VALUE
 false_to_s(VALUE obj)
 {
-    return rb_str_new2("false");
+    return rb_usascii_str_new2("false");
 }
 
 /*
@@ -982,11 +1030,25 @@ rb_false(VALUE obj)
  */
 
 static VALUE
-rb_obj_pattern_match(VALUE obj1, VALUE obj2)
+rb_obj_match(VALUE obj1, VALUE obj2)
 {
     return Qnil;
 }
 
+/*
+ *  call-seq:
+ *     obj !~ other  => nil
+ *  
+ *  Returns true if two objects does not match, using <i>=~</i> method.
+ */
+
+static VALUE
+rb_obj_not_match(VALUE obj1, VALUE obj2)
+{
+    VALUE result = rb_funcall(obj1, id_match, 1, obj2);
+    return RTEST(result) ? Qfalse : Qtrue;
+}
+
 
 /***********************************************************************
  *
@@ -1011,8 +1073,8 @@ rb_obj_pattern_match(VALUE obj1, VALUE obj2)
  *       end
  *     end
  *     Mod.class              #=> Module
- *     Mod.constants          #=> ["E", "PI", "CONST"]
- *     Mod.instance_methods   #=> ["meth"]
+ *     Mod.constants          #=> [:CONST, :PI, :E]
+ *     Mod.instance_methods   #=> [:meth]
  *     
  */
 
@@ -1029,7 +1091,7 @@ static VALUE
 rb_mod_to_s(VALUE klass)
 {
     if (FL_TEST(klass, FL_SINGLETON)) {
-	VALUE s = rb_str_new2("#<");
+	VALUE s = rb_usascii_str_new2("#<");
 	VALUE v = rb_iv_get(klass, "__attached__");
 
 	rb_str_cat2(s, "Class:");
@@ -1058,7 +1120,7 @@ rb_mod_to_s(VALUE klass)
 static VALUE
 rb_mod_freeze(VALUE mod)
 {
-    rb_mod_to_s(mod);
+    rb_class_name(mod);
     return rb_obj_freeze(mod);
 }
 
@@ -1104,15 +1166,15 @@ rb_class_inherited_p(VALUE mod, VALUE arg)
 	rb_raise(rb_eTypeError, "compared with non class/module");
     }
     while (mod) {
-	if (RCLASS(mod)->m_tbl == RCLASS(arg)->m_tbl)
+	if (RCLASS_M_TBL(mod) == RCLASS_M_TBL(arg))
 	    return Qtrue;
-	mod = RCLASS(mod)->super;
+	mod = RCLASS_SUPER(mod);
     }
     /* not mod < arg; check if mod > arg */
     while (arg) {
-	if (RCLASS(arg)->m_tbl == RCLASS(start)->m_tbl)
+	if (RCLASS_M_TBL(arg) == RCLASS_M_TBL(start))
 	    return Qfalse;
-	arg = RCLASS(arg)->super;
+	arg = RCLASS_SUPER(arg);
     }
     return Qnil;
 }
@@ -1251,11 +1313,11 @@ rb_class_s_alloc(VALUE klass)
  *     a.meth2          #=> "bye"
  */
 
-extern VALUE rb_mod_module_exec(int argc, VALUE *argv, VALUE mod);
-
 static VALUE
 rb_mod_initialize(VALUE module)
 {
+    extern VALUE rb_mod_module_exec(int argc, VALUE *argv, VALUE mod);
+
     if (rb_block_given_p()) {
 	rb_mod_module_exec(1, &module, module);
     }
@@ -1277,16 +1339,17 @@ rb_class_initialize(int argc, VALUE *argv, VALUE klass)
 {
     VALUE super;
 
-    if (RCLASS(klass)->super != 0) {
+    if (RCLASS_SUPER(klass) != 0) {
 	rb_raise(rb_eTypeError, "already initialized class");
     }
-    if (rb_scan_args(argc, argv, "01", &super) == 0) {
+    if (argc == 0) {
 	super = rb_cObject;
     }
     else {
+	rb_scan_args(argc, argv, "01", &super);
 	rb_check_inheritable(super);
     }
-    RCLASS(klass)->super = super;
+    RCLASS_SUPER(klass) = super;
     rb_make_metaclass(klass, RBASIC(super)->klass);
     rb_class_inherited(super, klass);
     rb_mod_initialize(klass);
@@ -1298,8 +1361,21 @@ rb_class_initialize(int argc, VALUE *argv, VALUE klass)
  *  call-seq:
  *     class.allocate()   =>   obj
  *  
- *  Allocates space for a new object of <i>class</i>'s class. The
- *  returned object must be an instance of <i>class</i>.
+ *  Allocates space for a new object of <i>class</i>'s class and does not
+ *  call initialize on the new instance. The returned object must be an
+ *  instance of <i>class</i>.
+ *  
+ *      klass = Class.new do
+ *        def initialize(*args)
+ *          @initialized = true
+ *        end
+ *      
+ *        def initialized?
+ *          @initialized || false
+ *        end
+ *      end
+ *      
+ *      klass.allocate.initialized? #=> false
  *     
  */
 
@@ -1308,7 +1384,7 @@ rb_obj_alloc(VALUE klass)
 {
     VALUE obj;
 
-    if (RCLASS(klass)->super == 0 && klass != rb_cBasicObject) {
+    if (RCLASS_SUPER(klass) == 0 && klass != rb_cBasicObject) {
 	rb_raise(rb_eTypeError, "can't instantiate uninitialized class");
     }
     if (FL_TEST(klass, FL_SINGLETON)) {
@@ -1358,22 +1434,24 @@ rb_class_new_instance(int argc, VALUE *argv, VALUE klass)
  *  
  *  Returns the superclass of <i>class</i>, or <code>nil</code>.
  *     
- *     File.superclass     #=> IO
- *     IO.superclass       #=> Object
- *     Object.superclass   #=> nil
+ *     File.superclass          #=> IO
+ *     IO.superclass            #=> Object
+ *     Object.superclass        #=> BasicObject
+ *     BasicObject.superclass   #=> nil
  *     
  */
 
 static VALUE
 rb_class_superclass(VALUE klass)
 {
-    VALUE super = RCLASS(klass)->super;
+    VALUE super = RCLASS_SUPER(klass);
 
     if (!super) {
+	if (klass == rb_cBasicObject) return Qnil;
 	rb_raise(rb_eTypeError, "uninitialized class");
     }
     while (TYPE(super) == T_ICLASS) {
-	super = RCLASS(super)->super;
+	super = RCLASS_SUPER(super);
     }
     if (!super) {
 	return Qnil;
@@ -1444,7 +1522,7 @@ rb_mod_attr_writer(int argc, VALUE *argv, VALUE klass)
  *     module Mod
  *       attr_accessor(:one, :two)
  *     end
- *     Mod.instance_methods.sort   #=> ["one", "one=", "two", "two="]
+ *     Mod.instance_methods.sort   #=> [:one, :one=, :two, :two=]
  */
 
 static VALUE
@@ -1560,8 +1638,8 @@ rb_mod_const_defined(int argc, VALUE *argv, VALUE mod)
  *     end
  *     k = Klass.new
  *     k.methods[0..9]    #=> ["kMethod", "freeze", "nil?", "is_a?", 
- *                             "class", "instance_variable_set",
- *                              "methods", "extend", "__send__", "instance_eval"]
+ *                        #    "class", "instance_variable_set",
+ *                        #    "methods", "extend", "__send__", "instance_eval"]
  *     k.methods.length   #=> 42
  */
 
@@ -1850,7 +1928,7 @@ rb_convert_type(VALUE val, int type, const char *tname, const char *method)
     if (TYPE(val) == type) return val;
     v = convert_type(val, tname, method, Qtrue);
     if (TYPE(v) != type) {
-	char *cname = rb_obj_classname(val);
+	const char *cname = rb_obj_classname(val);
 	rb_raise(rb_eTypeError, "can't convert %s to %s (%s#%s gives %s)",
 		 cname, tname, cname, method, rb_obj_classname(v));
     }
@@ -1867,7 +1945,7 @@ rb_check_convert_type(VALUE val, int type, const char *tname, const char *method
     v = convert_type(val, tname, method, Qfalse);
     if (NIL_P(v)) return Qnil;
     if (TYPE(v) != type) {
-	char *cname = rb_obj_classname(val);
+	const char *cname = rb_obj_classname(val);
 	rb_raise(rb_eTypeError, "can't convert %s to %s (%s#%s gives %s)",
 		 cname, tname, cname, method, rb_obj_classname(v));
     }
@@ -1878,9 +1956,12 @@ rb_check_convert_type(VALUE val, int type, const char *tname, const char *method
 static VALUE
 rb_to_integer(VALUE val, const char *method)
 {
-    VALUE v = convert_type(val, "Integer", method, Qtrue);
+    VALUE v;
+
+    if (FIXNUM_P(val)) return val;
+    v = convert_type(val, "Integer", method, Qtrue);
     if (!rb_obj_is_kind_of(v, rb_cInteger)) {
-	char *cname = rb_obj_classname(val);
+	const char *cname = rb_obj_classname(val);
 	rb_raise(rb_eTypeError, "can't convert %s to Integer (%s#%s gives %s)",
 		 cname, cname, method, rb_obj_classname(v));
     }
@@ -1890,7 +1971,10 @@ rb_to_integer(VALUE val, const char *method)
 VALUE
 rb_check_to_integer(VALUE val, const char *method)
 {
-    VALUE v = convert_type(val, "Integer", method, Qfalse);
+    VALUE v;
+
+    if (FIXNUM_P(val)) return val;
+    v = convert_type(val, "Integer", method, Qfalse);
     if (!rb_obj_is_kind_of(v, rb_cInteger)) {
 	return Qnil;
     }
@@ -1910,11 +1994,11 @@ rb_Integer(VALUE val)
 
     switch (TYPE(val)) {
       case T_FLOAT:
-	if (RFLOAT(val)->value <= (double)FIXNUM_MAX
-	    && RFLOAT(val)->value >= (double)FIXNUM_MIN) {
+	if (RFLOAT_VALUE(val) <= (double)FIXNUM_MAX
+	    && RFLOAT_VALUE(val) >= (double)FIXNUM_MIN) {
 	    break;
 	}
-	return rb_dbl2big(RFLOAT(val)->value);
+	return rb_dbl2big(RFLOAT_VALUE(val));
 
       case T_FIXNUM:
       case T_BIGNUM:
@@ -1951,7 +2035,7 @@ rb_Integer(VALUE val)
  *     
  *     Integer(123.999)    #=> 123
  *     Integer("0x1a")     #=> 26
- *     Integer(Time.new)   #=> 1049896590
+ *     Integer(Time.new)   #=> 1204973019
  */
 
 static VALUE
@@ -1972,43 +2056,49 @@ rb_cstr_to_dbl(const char *p, int badcheck)
 
     if (!p) return 0.0;
     q = p;
-	while (ISSPACE(*p)) p++;
+    while (ISSPACE(*p)) p++;
     d = strtod(p, &end);
     if (errno == ERANGE) {
 	OutOfRange();
-	rb_warn("Float %.*s%s out of range", w, p, ellipsis);
+	rb_warning("Float %.*s%s out of range", w, p, ellipsis);
 	errno = 0;
     }
     if (p == end) {
-	  bad:
 	if (badcheck) {
+	  bad:
 	    rb_invalid_str(q, "Float()");
 	}
 	return d;
     }
     if (*end) {
-	char *buf = ALLOCA_N(char, strlen(p)+1);
+	char buf[DBL_DIG * 4 + 10];
 	char *n = buf;
+	char *e = buf + sizeof(buf) - 1;
+	char prev = 0;
 
-	while (p < end) *n++ = *p++;
+	while (p < end && n < e) prev = *n++ = *p++;
 	while (*p) {
 	    if (*p == '_') {
 		/* remove underscores between digits */
-		    if (n == buf || !ISDIGIT(n[-1])) goto bad;
+		if (badcheck) {
+		    if (n == buf || !ISDIGIT(prev)) goto bad;
+		    ++p;
+		    if (!ISDIGIT(*p)) goto bad;
+		}
+		else {
 		    while (*++p == '_');
-		if (!ISDIGIT(*p)) {
-		    if (badcheck) goto bad;
-		    break;
+		    continue;
 		}
 	    }
-	    *n++ = *p++;
+	    prev = *p++;
+	    if (n < e) *n++ = prev;
 	}
 	*n = '\0';
 	p = buf;
 	d = strtod(p, &end);
 	if (errno == ERANGE) {
 	    OutOfRange();
-	    rb_warn("Float %.*s%s out of range", w, p, ellipsis);
+	    rb_warning("Float %.*s%s out of range", w, p, ellipsis);
 	    errno = 0;
 	}
 	if (badcheck) {
@@ -2054,29 +2144,23 @@ rb_Float(VALUE val)
 {
     switch (TYPE(val)) {
       case T_FIXNUM:
-	return rb_float_new((double)FIX2LONG(val));
+	return DOUBLE2NUM((double)FIX2LONG(val));
 
       case T_FLOAT:
 	return val;
 
       case T_BIGNUM:
-	return rb_float_new(rb_big2dbl(val));
+	return DOUBLE2NUM(rb_big2dbl(val));
 
       case T_STRING:
-	return rb_float_new(rb_str_to_dbl(val, Qtrue));
+	return DOUBLE2NUM(rb_str_to_dbl(val, Qtrue));
 
       case T_NIL:
 	rb_raise(rb_eTypeError, "can't convert nil into Float");
 	break;
 
       default:
-      {
-	  VALUE f = rb_convert_type(val, T_FLOAT, "Float", "to_f");
-	  if (isnan(RFLOAT(f)->value)) {
-	      rb_raise(rb_eArgError, "invalid value for Float()");
-	  }
-	  return f;
-      }
+	return rb_convert_type(val, T_FLOAT, "Float", "to_f");
     }
 }
 
@@ -2103,7 +2187,7 @@ rb_num2dbl(VALUE val)
 {
     switch (TYPE(val)) {
       case T_FLOAT:
-	return RFLOAT(val)->value;
+	return RFLOAT_VALUE(val);
 
       case T_STRING:
 	rb_raise(rb_eTypeError, "no implicit conversion to float from string");
@@ -2117,7 +2201,7 @@ rb_num2dbl(VALUE val)
 	break;
     }
 
-    return RFLOAT(rb_Float(val))->value;
+    return RFLOAT_VALUE(rb_Float(val));
 }
 
 char*
@@ -2281,6 +2365,8 @@ boot_defclass(const char *name, VALUE super)
 void
 Init_Object(void)
 {
+#undef rb_intern
+
     VALUE metaclass;
 
     rb_cBasicObject = boot_defclass("BasicObject", 0);
@@ -2297,6 +2383,12 @@ Init_Object(void)
     rb_define_alloc_func(rb_cBasicObject, rb_class_allocate_instance);
     rb_define_method(rb_cBasicObject, "==", rb_obj_equal, 1);
     rb_define_method(rb_cBasicObject, "equal?", rb_obj_equal, 1);
+    rb_define_method(rb_cBasicObject, "!", rb_obj_not, 0);
+    rb_define_method(rb_cBasicObject, "!=", rb_obj_not_equal, 1);
+
+    rb_define_private_method(rb_cBasicObject, "singleton_method_added", rb_obj_dummy, 1);
+    rb_define_private_method(rb_cBasicObject, "singleton_method_removed", rb_obj_dummy, 1);
+    rb_define_private_method(rb_cBasicObject, "singleton_method_undefined", rb_obj_dummy, 1);
 
     rb_mKernel = rb_define_module("Kernel");
     rb_include_module(rb_cObject, rb_mKernel);
@@ -2309,12 +2401,11 @@ Init_Object(void)
 
     rb_define_method(rb_mKernel, "nil?", rb_false, 0);
     rb_define_method(rb_mKernel, "===", rb_equal, 1); 
-    rb_define_method(rb_mKernel, "=~", rb_obj_pattern_match, 1);
-
+    rb_define_method(rb_mKernel, "=~", rb_obj_match, 1);
+    rb_define_method(rb_mKernel, "!~", rb_obj_not_match, 1);
     rb_define_method(rb_mKernel, "eql?", rb_obj_equal, 1);
 
     rb_define_method(rb_mKernel, "class", rb_obj_class, 0);
-
     rb_define_method(rb_mKernel, "clone", rb_obj_clone, 0);
     rb_define_method(rb_mKernel, "dup", rb_obj_dup, 0);
     rb_define_method(rb_mKernel, "initialize_copy", rb_obj_init_copy, 1);
@@ -2344,10 +2435,6 @@ Init_Object(void)
     rb_define_method(rb_mKernel, "is_a?", rb_obj_is_kind_of, 1);
     rb_define_method(rb_mKernel, "tap", rb_obj_tap, 0);
 
-    rb_define_private_method(rb_mKernel, "singleton_method_added", rb_obj_dummy, 1);
-    rb_define_private_method(rb_mKernel, "singleton_method_removed", rb_obj_dummy, 1);
-    rb_define_private_method(rb_mKernel, "singleton_method_undefined", rb_obj_dummy, 1);
-
     rb_define_global_function("sprintf", rb_f_sprintf, -1); /* in sprintf.c */
     rb_define_global_function("format", rb_f_sprintf, -1);  /* in sprintf.c */
 
@@ -2362,7 +2449,6 @@ Init_Object(void)
     rb_define_method(rb_cNilClass, "to_f", nil_to_f, 0);
     rb_define_method(rb_cNilClass, "to_s", nil_to_s, 0);
     rb_define_method(rb_cNilClass, "to_a", nil_to_a, 0);
-    rb_define_method(rb_cNilClass, "to_splat", nil_to_a, 0);
     rb_define_method(rb_cNilClass, "inspect", nil_inspect, 0);
     rb_define_method(rb_cNilClass, "&", false_and, 1);
     rb_define_method(rb_cNilClass, "|", false_or, 1);
@@ -2402,8 +2488,6 @@ Init_Object(void)
 		     rb_class_protected_instance_methods, -1); /* in class.c */
     rb_define_method(rb_cModule, "private_instance_methods", 
 		     rb_class_private_instance_methods, -1);   /* in class.c */
-    rb_define_method(rb_cModule, "local_methods", 
-		     rb_class_local_methods, 0);               /* in class.c */
 
     rb_define_method(rb_cModule, "constants", rb_mod_constants, -1); /* in variable.c */
     rb_define_method(rb_cModule, "const_get", rb_mod_const_get, -1);
@@ -2453,6 +2537,7 @@ Init_Object(void)
 
     id_eq = rb_intern("==");
     id_eql = rb_intern("eql?");
+    id_match = rb_intern("=~");
     id_inspect = rb_intern("inspect");
     id_init_copy = rb_intern("initialize_copy");
 }
