@@ -2,8 +2,7 @@
 
   pack.c -
 
-  $Author: usa $
-  $Date: 2007-12-25 03:12:24 +0900 (Tue, 25 Dec 2007) $
+  $Author: ko1 $
   created at: Thu Feb 10 15:17:05 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -13,6 +12,7 @@
 #include "ruby/ruby.h"
 #include <sys/types.h>
 #include <ctype.h>
+#include <errno.h>
 
 #define SIZE16 2
 #define SIZE32 4
@@ -67,8 +67,8 @@ TOKEN_PASTE(swap,x)(xtype z)		\
 	t[sizeof(xtype)-i-1] = s[i];	\
     }					\
     r = *(xtype *)t;			\
-    free(t);				\
-    free(zp);				\
+    xfree(t);				\
+    xfree(zp);				\
     return r;				\
 }
 
@@ -475,7 +475,7 @@ pack_pack(VALUE ary, VALUE fmt)
 	    continue;
 	}
         if (*p == '_' || *p == '!') {
-	    const char *natstr = "sSiIlL";
+	    static const char natstr[] = "sSiIlL";
 
 	    if (strchr(natstr, type)) {
 #ifdef NATINT_PACK
@@ -492,7 +492,11 @@ pack_pack(VALUE ary, VALUE fmt)
 	    p++;
 	}
 	else if (ISDIGIT(*p)) {
-	    len = strtoul(p, (char**)&p, 10);
+	    errno = 0;
+	    len = STRTOUL(p, (char**)&p, 10);
+	    if (errno) {
+		rb_raise(rb_eRangeError, "pack length too big");
+	    }
 	}
 	else {
 	    len = 1;
@@ -1331,7 +1335,7 @@ pack_unpack(VALUE str, VALUE fmt)
 	}
 	star = 0;
 	if (*p == '_' || *p == '!') {
-	    const char *natstr = "sSiIlL";
+	    static const char natstr[] = "sSiIlL";
 
 	    if (strchr(natstr, type)) {
 #ifdef NATINT_PACK
@@ -1351,7 +1355,11 @@ pack_unpack(VALUE str, VALUE fmt)
 	    p++;
 	}
 	else if (ISDIGIT(*p)) {
-	    len = strtoul(p, (char**)&p, 10);
+	    errno = 0;
+	    len = STRTOUL(p, (char**)&p, 10);
+	    if (errno) {
+		rb_raise(rb_eRangeError, "pack length too big");
+	    }
 	}
 	else {
 	    len = (type != '@');
@@ -1872,8 +1880,8 @@ pack_unpack(VALUE str, VALUE fmt)
 
 	  case 'P':
 	    if (sizeof(char *) <= send - s) {
+		VALUE tmp = Qnil;
 		char *t;
-		VALUE tmp;
 
 		memcpy(&t, s, sizeof(char *));
 		s += sizeof(char *);
@@ -1903,9 +1911,6 @@ pack_unpack(VALUE str, VALUE fmt)
 			rb_raise(rb_eArgError, "non associated pointer");
 		    }
 		}
-		else {
-		    tmp = Qnil;
-		}
 		UNPACK_PUSH(tmp);
 	    }
 	    break;
@@ -1917,7 +1922,7 @@ pack_unpack(VALUE str, VALUE fmt)
 		if (send - s < sizeof(char *))
 		    break;
 		else {
-		    VALUE tmp;
+		    VALUE tmp = Qnil;
 		    char *t;
 
 		    memcpy(&t, s, sizeof(char *));
@@ -1942,9 +1947,6 @@ pack_unpack(VALUE str, VALUE fmt)
 			    rb_raise(rb_eArgError, "non associated pointer");
 			}
 		    }
-		    else {
-			tmp = Qnil;
-		    }
 		    UNPACK_PUSH(tmp);
 		}
 	    }
