@@ -2,17 +2,16 @@
 
   random.c -
 
-  $Author: ocean $
-  $Date: 2005/09/12 10:44:20 $
+  $Author: nobu $
   created at: Fri Dec 24 16:39:21 JST 1993
 
-  Copyright (C) 1993-2003 Yukihiro Matsumoto
+  Copyright (C) 1993-2007 Yukihiro Matsumoto
 
 **********************************************************************/
 
 /* 
 This is based on trimmed version of MT19937.  To get the original version,
-contact <http://www.math.keio.ac.jp/~matumoto/emt.html>.
+contact <http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html>.
 
 The original copyright notice follows.
 
@@ -176,7 +175,7 @@ genrand_real(void)
 
 /* These real versions are due to Isaku Wada, 2002/01/09 added */
 
-#include "ruby.h"
+#include "ruby/ruby.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -188,7 +187,19 @@ genrand_real(void)
 #include <fcntl.h>
 #endif
 
-static int first = 1;
+unsigned long
+rb_genrand_int32(void)
+{
+    return genrand_int32();
+}
+
+double
+rb_genrand_real(void)
+{
+    return genrand_real();
+}
+
+static int seed_initialized = 0;
 static VALUE saved_seed = INT2FIX(0);
 
 static VALUE
@@ -205,7 +216,7 @@ rand_init(VALUE vseed)
           len = sizeof(VALUE);
           break;
       case T_BIGNUM:
-          len = RBIGNUM(seed)->len * SIZEOF_BDIGITS;
+          len = RBIGNUM_LEN(seed) * SIZEOF_BDIGITS;
           if (len == 0)
               len = 4;
           break;
@@ -224,12 +235,12 @@ rand_init(VALUE vseed)
     }
     else {
         int i, j;
-        for (i = RBIGNUM(seed)->len-1; 0 <= i; i--) {
+        for (i = RBIGNUM_LEN(seed)-1; 0 <= i; i--) {
             j = i * SIZEOF_BDIGITS / 4;
 #if SIZEOF_BDIGITS < 4
             buf[j] <<= SIZEOF_BDIGITS * 8;
 #endif
-            buf[j] |= ((BDIGIT *)RBIGNUM(seed)->digits)[i];
+            buf[j] |= RBIGNUM_DIGITS(seed)[i];
         }
     }
     while (1 < len && buf[len-1] == 0) {
@@ -243,34 +254,27 @@ rand_init(VALUE vseed)
             len--;
         init_by_array(buf, len);
     }
-    first = 0;
     old = saved_seed;
     saved_seed = seed;
-    free(buf);
+    xfree(buf);
     return old;
 }
 
-static VALUE
-random_seed(void)
+#define DEFAULT_SEED_LEN (4 * sizeof(long))
+
+static void
+fill_random_seed(char *ptr)
 {
     static int n = 0;
+    unsigned long *seed;
     struct timeval tv;
     int fd;
     struct stat statbuf;
+    char *buf = (char*)ptr;
 
-    int seed_len;
-    BDIGIT *digits;
-    unsigned long *seed;
-    NEWOBJ(big, struct RBignum);
-    OBJSETUP(big, rb_cBignum, T_BIGNUM);
+    seed = (unsigned long *)buf;
 
-    seed_len = 4 * sizeof(long);
-    big->sign = 1;
-    big->len = seed_len / SIZEOF_BDIGITS + 1;
-    digits = big->digits = ALLOC_N(BDIGIT, big->len);
-    seed = (unsigned long *)big->digits;
-
-    memset(digits, 0, big->len * SIZEOF_BDIGITS);
+    memset(buf, 0, DEFAULT_SEED_LEN);
 
 #ifdef S_ISCHR
     if ((fd = open("/dev/urandom", O_RDONLY
@@ -285,7 +289,7 @@ random_seed(void)
 #endif
             )) >= 0) {
         if (fstat(fd, &statbuf) == 0 && S_ISCHR(statbuf.st_mode)) {
-            read(fd, seed, seed_len);
+            read(fd, seed, DEFAULT_SEED_LEN);
         }
         close(fd);
     }
@@ -296,13 +300,35 @@ random_seed(void)
     seed[1] ^= tv.tv_sec;
     seed[2] ^= getpid() ^ (n++ << 16);
     seed[3] ^= (unsigned long)&seed;
+}
+
+static VALUE
+make_seed_value(char *ptr)
+{
+    BDIGIT *digits;
+    NEWOBJ(big, struct RBignum);
+    OBJSETUP(big, rb_cBignum, T_BIGNUM);
+
+    RBIGNUM_SET_SIGN(big, 1);
+    rb_big_resize((VALUE)big, DEFAULT_SEED_LEN / SIZEOF_BDIGITS + 1);
+    digits = RBIGNUM_DIGITS(big);
+
+    MEMCPY((char *)RBIGNUM_DIGITS(big), ptr, char, DEFAULT_SEED_LEN);
 
     /* set leading-zero-guard if need. */
-    digits[big->len-1] = digits[big->len-2] <= 1 ? 1 : 0;
+    digits[RBIGNUM_LEN(big)-1] = digits[RBIGNUM_LEN(big)-2] <= 1 ? 1 : 0;
 
     return rb_big_norm((VALUE)big);
 }
 
+static VALUE
+random_seed(void)
+{
+    char buf[DEFAULT_SEED_LEN];
+    fill_random_seed(buf);
+    return make_seed_value(buf);
+}
+
 /*
  *  call-seq:
  *     srand(number=0)    => old_seed
@@ -323,9 +349,12 @@ rb_f_srand(int argc, VALUE *argv, VALUE obj)
     VALUE seed, old;
 
     rb_secure(4);
-    if (rb_scan_args(argc, argv, "01", &seed) == 0) {
+    if (argc == 0) {
 	seed = random_seed();
     }
+    else {
+	rb_scan_args(argc, argv, "01", &seed);
+    }
     old = rand_init(seed);
 
     return old;
@@ -372,20 +401,24 @@ limited_big_rand(struct RBignum *limit)
     struct RBignum *val;
     int i, len, boundary;
 
-    len = (limit->len * SIZEOF_BDIGITS + 3) / 4;
+    len = (RBIGNUM_LEN(limit) * SIZEOF_BDIGITS + 3) / 4;
     val = (struct RBignum *)rb_big_clone((VALUE)limit);
-    val->sign = 1;
+    RBIGNUM_SET_SIGN(val, 1);
 #if SIZEOF_BDIGITS == 2
-# define BIG_GET32(big,i) (((BDIGIT *)(big)->digits)[(i)*2] | \
-                           ((i)*2+1 < (big)->len ? (((BDIGIT *)(big)->digits)[(i)*2+1] << 16) \
-                                                 : 0))
-# define BIG_SET32(big,i,d) ((((BDIGIT *)(big)->digits)[(i)*2] = (d) & 0xffff), \
-                             ((i)*2+1 < (big)->len ? (((BDIGIT *)(big)->digits)[(i)*2+1] = (d) >> 16) \
-                                                   : 0))
+# define BIG_GET32(big,i) \
+    (RBIGNUM_DIGITS(big)[(i)*2] | \
+     ((i)*2+1 < RBIGNUM_LEN(big) ? \
+      (RBIGNUM_DIGITS(big)[(i)*2+1] << 16) : \
+      0))
+# define BIG_SET32(big,i,d) \
+    ((RBIGNUM_DIGITS(big)[(i)*2] = (d) & 0xffff), \
+     ((i)*2+1 < RBIGNUM_LEN(big) ? \
+      (RBIGNUM_DIGITS(big)[(i)*2+1] = (d) >> 16) : \
+      0))
 #else
     /* SIZEOF_BDIGITS == 4 */
-# define BIG_GET32(big,i) (((BDIGIT *)(big)->digits)[i])
-# define BIG_SET32(big,i,d) (((BDIGIT *)(big)->digits)[i] = (d))
+# define BIG_GET32(big,i) (RBIGNUM_DIGITS(big)[i])
+# define BIG_SET32(big,i,d) (RBIGNUM_DIGITS(big)[i] = (d))
 #endif
   retry:
     mask = 0;
@@ -421,7 +454,7 @@ limited_big_rand(struct RBignum *limit)
  *  than or equal to zero and less than max1. <code>Kernel::srand</code>
  *  may be used to ensure repeatable sequences of random numbers between
  *  different runs of the program. Ruby currently uses a modified
- *  Mersenne Twister with a period of 219937-1.
+ *  Mersenne Twister with a period of 2**19937-1.
  *     
  *     srand 1234                 #=> 0
  *     [ rand,  rand ]            #=> [0.191519450163469, 0.49766366626136]
@@ -437,32 +470,32 @@ rb_f_rand(int argc, VALUE *argv, VALUE obj)
     long val, max;
 
     rb_scan_args(argc, argv, "01", &vmax);
-    if (first) {
-	rand_init(random_seed());
+    if (!seed_initialized) {
+       rand_init(random_seed());
     }
     switch (TYPE(vmax)) {
       case T_FLOAT:
-	if (RFLOAT(vmax)->value <= LONG_MAX && RFLOAT(vmax)->value >= LONG_MIN) {
-	    max = (long)RFLOAT(vmax)->value;
+	if (RFLOAT_VALUE(vmax) <= LONG_MAX && RFLOAT_VALUE(vmax) >= LONG_MIN) {
+	    max = (long)RFLOAT_VALUE(vmax);
 	    break;
 	}
-        if (RFLOAT(vmax)->value < 0)
-            vmax = rb_dbl2big(-RFLOAT(vmax)->value);
+        if (RFLOAT_VALUE(vmax) < 0)
+            vmax = rb_dbl2big(-RFLOAT_VALUE(vmax));
         else
-            vmax = rb_dbl2big(RFLOAT(vmax)->value);
+            vmax = rb_dbl2big(RFLOAT_VALUE(vmax));
 	/* fall through */
       case T_BIGNUM:
       bignum:
         {
             struct RBignum *limit = (struct RBignum *)vmax;
-            if (!limit->sign) {
+            if (!RBIGNUM_SIGN(limit)) {
                 limit = (struct RBignum *)rb_big_clone(vmax);
-                limit->sign = 1;
+                RBIGNUM_SET_SIGN(limit, 1);
             }
             limit = (struct RBignum *)rb_big_minus((VALUE)limit, INT2FIX(1));
             if (FIXNUM_P((VALUE)limit)) {
                 if (FIX2LONG((VALUE)limit) == -1)
-                    return rb_float_new(genrand_real());
+                    return DOUBLE2NUM(genrand_real());
                 return LONG2NUM(limited_rand(FIX2LONG((VALUE)limit)));
             }
             return limited_big_rand(limit);
@@ -479,16 +512,41 @@ rb_f_rand(int argc, VALUE *argv, VALUE obj)
     }
 
     if (max == 0) {
-	return rb_float_new(genrand_real());
+	return DOUBLE2NUM(genrand_real());
     }
     if (max < 0) max = -max;
     val = limited_rand(max-1);
     return LONG2NUM(val);
 }
 
+static char initial_seed[DEFAULT_SEED_LEN];
+
+void
+Init_RandomSeed(void)
+{
+    fill_random_seed(initial_seed);
+    init_by_array((unsigned long*)initial_seed, DEFAULT_SEED_LEN/sizeof(unsigned long));
+    seed_initialized = 1;
+}
+
+static void
+Init_RandomSeed2(void)
+{
+    saved_seed = make_seed_value(initial_seed);
+    memset(initial_seed, 0, DEFAULT_SEED_LEN);
+}
+
+void
+rb_reset_random_seed(void)
+{
+    seed_initialized = 0;
+    saved_seed = INT2FIX(0);
+}
+
 void
 Init_Random(void)
 {
+    Init_RandomSeed2();
     rb_define_global_function("srand", rb_f_srand, -1);
     rb_define_global_function("rand", rb_f_rand, -1);
     rb_global_variable(&saved_seed);
