@@ -451,7 +451,7 @@ static int
 ss_apply_all_case_fold(OnigCaseFoldType flag ARG_UNUSED,
 		       OnigApplyAllCaseFoldFunc f, void* arg)
 {
-  OnigCodePoint ss[] = { 0x73, 0x73 };
+  static OnigCodePoint ss[] = { 0x73, 0x73 };
 
   return (*f)((OnigCodePoint )0xdf, ss, 2, arg);
 }
@@ -629,8 +629,6 @@ onigenc_single_byte_code_to_mbclen(OnigCodePoint code ARG_UNUSED, OnigEncoding e
 extern int
 onigenc_single_byte_code_to_mbc(OnigCodePoint code, UChar *buf, OnigEncoding enc ARG_UNUSED)
 {
-  if (code > 0xff)
-      rb_raise(rb_eRangeError, "%"PRIdVALUE " out of char range", code);
   *buf = (UChar )(code & 0xff);
   return 1;
 }
@@ -785,7 +783,7 @@ onigenc_mb4_code_to_mbc(OnigEncoding enc, OnigCodePoint code, UChar *buf)
 extern int
 onigenc_minimum_property_name_to_ctype(OnigEncoding enc, UChar* p, UChar* end)
 {
-  static const PosixBracketEntryType PBS[] = {
+  static PosixBracketEntryType PBS[] = {
     { (UChar* )"Alnum",  ONIGENC_CTYPE_ALNUM,  5 },
     { (UChar* )"Alpha",  ONIGENC_CTYPE_ALPHA,  5 },
     { (UChar* )"Blank",  ONIGENC_CTYPE_BLANK,  5 },
@@ -803,7 +801,7 @@ onigenc_minimum_property_name_to_ctype(OnigEncoding enc, UChar* p, UChar* end)
     { (UChar* )NULL, -1, 0 }
   };
 
-  const PosixBracketEntryType *pb;
+  PosixBracketEntryType *pb;
   int len;
 
   len = onigenc_strlen(enc, p, end);
