@@ -2,7 +2,7 @@
   regerror.c -  Oniguruma (regular expression library)
 **********************************************************************/
 /*-
- * Copyright (c) 2002-2006  K.Kosako  <sndgk393 AT ybb DOT ne DOT jp>
+ * Copyright (c) 2002-2007  K.Kosako  <sndgk393 AT ybb DOT ne DOT jp>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,7 @@
 extern UChar*
 onig_error_code_to_format(int code)
 {
-  char *p;
+  const char *p;
 
   if (code >= 0) return (UChar* )0;
 
@@ -85,9 +85,9 @@ onig_error_code_to_format(int code)
   case ONIGERR_END_PATTERN_AT_CONTROL:
     p = "end pattern at control"; break;
   case ONIGERR_META_CODE_SYNTAX:
-    p = "illegal meta-code syntax"; break;
+    p = "invalid meta-code syntax"; break;
   case ONIGERR_CONTROL_CODE_SYNTAX:
-    p = "illegal control-code syntax"; break;
+    p = "invalid control-code syntax"; break;
   case ONIGERR_CHAR_CLASS_VALUE_AT_END_OF_RANGE:
     p = "char-class value at end of range"; break;
   case ONIGERR_CHAR_CLASS_VALUE_AT_START_OF_RANGE:
@@ -142,8 +142,8 @@ onig_error_code_to_format(int code)
     p = "too big wide-char value"; break;
   case ONIGERR_TOO_LONG_WIDE_CHAR_VALUE:
     p = "too long wide-char value"; break;
-  case ONIGERR_INVALID_WIDE_CHAR_VALUE:
-    p = "invalid wide-char value"; break;
+  case ONIGERR_INVALID_CODE_POINT_VALUE:
+    p = "invalid code point value"; break;
   case ONIGERR_EMPTY_GROUP_NAME:
     p = "group name is empty"; break;
   case ONIGERR_INVALID_GROUP_NAME:
@@ -182,6 +182,64 @@ onig_error_code_to_format(int code)
   return (UChar* )p;
 }
 
+static void sprint_byte(char* s, unsigned int v)
+{
+  sprintf(s, "%02x", (v & 0377));
+}
+
+static void sprint_byte_with_x(char* s, unsigned int v)
+{
+  sprintf(s, "\\x%02x", (v & 0377));
+}
+
+static int to_ascii(OnigEncoding enc, UChar *s, UChar *end,
+		    UChar buf[], int buf_size, int *is_over)
+{
+  int len;
+  UChar *p;
+  OnigCodePoint code;
+
+  if (ONIGENC_MBC_MINLEN(enc) > 1) {
+    p = s;
+    len = 0;
+    while (p < end) {
+      code = ONIGENC_MBC_TO_CODE(enc, p, end);
+      if (code >= 0x80) {
+ 	if (code > 0xffff && len + 10 <= buf_size) {
+ 	  sprint_byte_with_x((char*)(&(buf[len])), (unsigned int)(code >> 24));
+ 	  sprint_byte((char*)(&(buf[len+4])),      (unsigned int)(code >> 16));
+ 	  sprint_byte((char*)(&(buf[len+6])),      (unsigned int)(code >>  8));
+ 	  sprint_byte((char*)(&(buf[len+8])),      (unsigned int)code);
+ 	  len += 10;
+ 	}
+ 	else if (len + 6 <= buf_size) {
+ 	  sprint_byte_with_x((char*)(&(buf[len])), (unsigned int)(code >> 8));
+ 	  sprint_byte((char*)(&(buf[len+4])),      (unsigned int)code);
+ 	  len += 6;
+	}
+	else {
+	  break;
+	}
+      }
+      else {
+	buf[len++] = (UChar )code;
+      }
+
+      p += enclen(enc, p, end);
+      if (len >= buf_size) break;
+    }
+
+    *is_over = ((p < end) ? 1 : 0);
+  }
+  else {
+    len = MIN((end - s), buf_size);
+    xmemcpy(buf, s, (size_t )len);
+    *is_over = ((buf_size < (end - s)) ? 1 : 0);
+  }
+
+  return len;
+}
+
 
 /* for ONIG_MAX_ERROR_MESSAGE_LEN */
 #define MAX_ERROR_PAR_LEN   30
@@ -198,7 +256,8 @@ onig_error_code_to_str(s, code, va_alist)
 {
   UChar *p, *q;
   OnigErrorInfo* einfo;
-  int len;
+  int len, is_over;
+  UChar parbuf[MAX_ERROR_PAR_LEN];
   va_list vargs;
 
   va_init_list(vargs, code);
@@ -212,23 +271,20 @@ onig_error_code_to_str(s, code, va_alist)
   case ONIGERR_INVALID_CHAR_IN_GROUP_NAME:
   case ONIGERR_INVALID_CHAR_PROPERTY_NAME:
     einfo = va_arg(vargs, OnigErrorInfo*);
-    len = einfo->par_end - einfo->par;
+    len = to_ascii(einfo->enc, einfo->par, einfo->par_end,
+		   parbuf, MAX_ERROR_PAR_LEN - 3, &is_over);
     q = onig_error_code_to_format(code);
     p = s;
     while (*q != '\0') {
       if (*q == '%') {
 	q++;
 	if (*q == 'n') { /* '%n': name */
-	  if (len > MAX_ERROR_PAR_LEN) {
-	    xmemcpy(p, einfo->par, MAX_ERROR_PAR_LEN - 3);
-	    p += (MAX_ERROR_PAR_LEN - 3);
+	  xmemcpy(p, parbuf, len);
+	  p += len;
+	  if (is_over != 0) {
 	    xmemcpy(p, "...", 3);
 	    p += 3;
 	  }
-	  else {
-	    xmemcpy(p, einfo->par, len);
-	    p += len;
-	  }
 	  q++;
 	}
 	else
@@ -276,8 +332,8 @@ onig_snprintf_with_pattern(buf, bufsize, enc, pat, pat_end, fmt, va_alist)
   UChar bs[6];
   va_list args;
 
-  va_init_list(args, (char* )fmt);
-  n = vsnprintf((char* )buf, bufsize, (char* )fmt, args);
+  va_init_list(args, fmt);
+  n = xvsnprintf((char* )buf, bufsize, (const char* )fmt, args);
   va_end(args);
 
   need = (pat_end - pat) * 4 + 4;
@@ -288,17 +344,17 @@ onig_snprintf_with_pattern(buf, bufsize, enc, pat, pat_end, fmt, va_alist)
 
     p = pat;
     while (p < pat_end) {
-      if (*p == MC_ESC(enc)) {
+      if (*p == '\\') {
 	*s++ = *p++;
-	len = enc_len(enc, p);
+	len = enclen(enc, p, pat_end);
 	while (len-- > 0) *s++ = *p++;
       }
       else if (*p == '/') {
-	*s++ = (unsigned char )MC_ESC(enc);
+	*s++ = (unsigned char )'\\';
 	*s++ = *p++;
       }
-      else if (ONIGENC_IS_MBC_HEAD(enc, p)) {
-        len = enc_len(enc, p);
+      else if (ONIGENC_IS_MBC_HEAD(enc, p, pat_end)) {
+        len = enclen(enc, p, pat_end);
         if (ONIGENC_MBC_MINLEN(enc) == 1) {
           while (len-- > 0) *s++ = *p++;
         }
@@ -306,7 +362,7 @@ onig_snprintf_with_pattern(buf, bufsize, enc, pat, pat_end, fmt, va_alist)
           int blen;
 
           while (len-- > 0) {
-            sprintf((char* )bs, "\\%03o", *p++ & 0377);
+            sprint_byte_with_x((char* )bs, (unsigned int )(*p++));
             blen = onigenc_str_bytelen_null(ONIG_ENCODING_ASCII, bs);
             bp = bs;
             while (blen-- > 0) *s++ = *bp++;
@@ -315,7 +371,7 @@ onig_snprintf_with_pattern(buf, bufsize, enc, pat, pat_end, fmt, va_alist)
       }
       else if (!ONIGENC_IS_CODE_PRINT(enc, *p) &&
 	       !ONIGENC_IS_CODE_SPACE(enc, *p)) {
-	sprintf((char* )bs, "\\%03o", *p++ & 0377);
+	sprint_byte_with_x((char* )bs, (unsigned int )(*p++));
 	len = onigenc_str_bytelen_null(ONIG_ENCODING_ASCII, bs);
         bp = bs;
 	while (len-- > 0) *s++ = *bp++;
