@@ -3,7 +3,6 @@
   ruby.c -
 
   $Author: nobu $
-  $Date: 2007-12-25 23:16:03 +0900 (Tue, 25 Dec 2007) $
   created at: Tue Aug 10 12:47:31 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -58,17 +57,24 @@
 char *getenv();
 #endif
 
-/* TODO: move to VM */
-VALUE ruby_debug = Qfalse;
-VALUE ruby_verbose = Qfalse;
 VALUE rb_parser_get_yydebug(VALUE);
 VALUE rb_parser_set_yydebug(VALUE, VALUE);
 
-char *ruby_inplace_mode = 0;
+const char *ruby_get_inplace_mode(void);
+void ruby_set_inplace_mode(const char *);
+
+#define DISABLE_BIT(bit) (1U << disable_##bit)
+enum disable_flag_bits {
+    disable_gems,
+    disable_rubyopt,
+};
+
+#define DUMP_BIT(bit) (1U << dump_##bit)
+enum dump_flag_bits {
+    dump_insns,
+};
 
 struct cmdline_options {
-    int argc;
-    char **argv;
     int sflag, xflag;
     int do_loop, do_print;
     int do_check, do_line;
@@ -76,16 +82,45 @@ struct cmdline_options {
     int usage;
     int version;
     int copyright;
-    int disable_gems;
+    unsigned int disable;
     int verbose;
     int yydebug;
-    char *script;
+    unsigned int setids;
+    unsigned int dump;
+    const char *script;
+    VALUE script_name;
     VALUE e_script;
-    int enc_index;
+    struct {
+	struct {
+	    VALUE name;
+	    int index;
+	} enc;
+    } src, ext;
+    VALUE req_list;
+};
+
+static void init_ids(struct cmdline_options *);
+
+#define src_encoding_index GET_VM()->src_encoding_index
+
+static struct cmdline_options *
+cmdline_options_init(struct cmdline_options *opt)
+{
+    MEMZERO(opt, *opt, 1);
+    init_ids(opt);
+    opt->src.enc.index = src_encoding_index;
+    return opt;
+}
+
+struct cmdline_arguments {
+    int argc;
+    char **argv;
+    struct cmdline_options *opt;
 };
 
 static NODE *load_file(VALUE, const char *, int, struct cmdline_options *);
-static void forbid_setid(const char *);
+static void forbid_setid(const char *, struct cmdline_options *);
+#define forbid_setid(s) forbid_setid(s, opt)
 
 static struct {
     int argc;
@@ -108,10 +143,10 @@ usage(const char *name)
 	"-Cdirectory     cd to directory, before executing your script",
 	"-d              set debugging flags (set $DEBUG to true)",
 	"-e 'command'    one line of script. Several -e's allowed. Omit [programfile]",
+	"-Eencoding      specifies the character encoding for the program codes",
 	"-Fpattern       split() pattern for autosplit (-a)",
 	"-i[extension]   edit ARGV files in place (make backup if extension supplied)",
 	"-Idirectory     specify $LOAD_PATH directory (may be used more than once)",
-	"-Kkcode         specifies KANJI (Japanese) code-set",
 	"-l              enable line ending processing",
 	"-n              assume 'while gets(); ... end' loop around your script",
 	"-p              assume loop like -n but print line also like sed",
@@ -123,7 +158,6 @@ usage(const char *name)
 	"-w              turn warnings on for your script",
 	"-W[level]       set warning level; 0=silence, 1=medium, 2=verbose (default)",
 	"-x[directory]   strip off text before #!ruby line and perhaps cd to directory",
-	"--disable-gems  disable gem libraries",
 	"--copyright     print the copyright",
 	"--version       print the version",
 	NULL
@@ -135,24 +169,7 @@ usage(const char *name)
 	printf("  %s\n", *p++);
 }
 
-static rb_encoding *
-locale_encoding(void)
-{
-    VALUE codeset = rb_locale_charmap(Qnil);
-    char *name;
-    int idx;
-
-    if (codeset == Qnil)
-        return rb_ascii8bit_encoding();
-
-    name = StringValueCStr(codeset);
-    idx = rb_enc_find_index(name);
-    if (idx < 0)
-        return rb_ascii8bit_encoding();
-    return rb_enc_from_index(idx);
-}
-
-extern VALUE rb_load_path;
+VALUE rb_get_load_path(void);
 
 #ifndef CharNext		/* defined as CharNext[AW] on Windows. */
 #define CharNext(p) ((p) + mblen(p, RUBY_MBCHAR_MAXSIZE))
@@ -200,7 +217,7 @@ rubylib_mangled_path(const char *s, unsigned int l)
 	    notfound = 1;
 	}
     }
-    if (!newp || l < oldl || strncasecmp(oldp, s, oldl) != 0) {
+    if (!newp || l < oldl || STRNCASECMP(oldp, s, oldl) != 0) {
 	return rb_str_new(s, l);
     }
     ret = rb_str_new(0, l + newl - oldl);
@@ -226,6 +243,7 @@ push_include(const char *path, VALUE (*filter)(VALUE))
 {
     const char sep = PATH_SEP_CHAR;
     const char *p, *s;
+    VALUE load_path = GET_VM()->load_path;
 
     p = path;
     while (*p) {
@@ -233,7 +251,7 @@ push_include(const char *path, VALUE (*filter)(VALUE))
 	    p++;
 	if (!*p) break;
 	for (s = p; *s && *s != sep; s = CharNext(s));
-	rb_ary_push(rb_load_path, (*filter)(rubylib_mangled_path(p, s - p)));
+	rb_ary_push(load_path, (*filter)(rubylib_mangled_path(p, s - p)));
 	p = s;
     }
 }
@@ -331,6 +349,7 @@ DllMain(HINSTANCE dll, DWORD reason, LPVOID reserved)
 void
 ruby_init_loadpath(void)
 {
+    VALUE load_path;
 #if defined LOAD_RELATIVE
     char libpath[MAXPATHLEN + 1];
     char *p;
@@ -361,7 +380,7 @@ ruby_init_loadpath(void)
     p = strrchr(libpath, '/');
     if (p) {
 	*p = 0;
-	if (p - libpath > 3 && !strcasecmp(p - 4, "/bin")) {
+	if (p - libpath > 3 && !STRCASECMP(p - 4, "/bin")) {
 	    p -= 4;
 	    *p = 0;
 	}
@@ -377,7 +396,8 @@ ruby_init_loadpath(void)
 #else
 #define RUBY_RELATIVE(path) (path)
 #endif
-#define incpush(path) rb_ary_push(rb_load_path, rubylib_mangled_path2(path))
+#define incpush(path) rb_ary_push(load_path, rubylib_mangled_path2(path))
+    load_path = GET_VM()->load_path;
 
     if (rb_safe_level() == 0) {
 	ruby_incpush(getenv("RUBYLIB"));
@@ -412,48 +432,35 @@ ruby_init_loadpath(void)
     }
 }
 
-struct req_list {
-    char *name;
-    struct req_list *next;
-};
-static struct {
-    struct req_list *last, head;
-} req_list = {&req_list.head,};
 
 static void
-add_modules(const char *mod)
+add_modules(struct cmdline_options *opt, const char *mod)
 {
-    struct req_list *list;
+    VALUE list = opt->req_list;
 
-    list = ALLOC(struct req_list);
-    list->name = ALLOC_N(char, strlen(mod) + 1);
-    strcpy(list->name, mod);
-    list->next = 0;
-    req_list.last->next = list;
-    req_list.last = list;
+    if (!list) {
+	opt->req_list = list = rb_ary_new();
+	RBASIC(list)->klass = 0;
+    }
+    rb_ary_push(list, rb_obj_freeze(rb_str_new2(mod)));
 }
 
 extern void Init_ext(void);
 extern VALUE rb_vm_top_self(void);
 
 static void
-require_libraries(void)
+require_libraries(struct cmdline_options *opt)
 {
-    struct req_list *list = req_list.head.next;
-    struct req_list *tmp;
-    ID require = rb_intern("require");
+    VALUE list = opt->req_list;
+    ID require;
 
     Init_ext();		/* should be called here for some reason :-( */
-    req_list.last = 0;
-    while (list) {
-	VALUE feature = rb_str_new2(list->name);
-	tmp = list->next;
-	free(list->name);
-	free(list);
-	list = tmp;
+    CONST_ID(require, "require");
+    while (list && RARRAY_LEN(list) > 0) {
+	VALUE feature = rb_ary_shift(list);
 	rb_funcall2(rb_vm_top_self(), require, 1, &feature);
     }
-    req_list.head.next = 0;
+    opt->req_list = 0;
 }
 
 static void
@@ -462,9 +469,10 @@ process_sflag(struct cmdline_options *opt)
     if (opt->sflag) {
 	long n;
 	VALUE *args;
+	VALUE argv = rb_argv;
 
-	n = RARRAY_LEN(rb_argv);
-	args = RARRAY_PTR(rb_argv);
+	n = RARRAY_LEN(argv);
+	args = RARRAY_PTR(argv);
 	while (n > 0) {
 	    VALUE v = *args++;
 	    char *s = StringValuePtr(v);
@@ -511,9 +519,9 @@ process_sflag(struct cmdline_options *opt)
 	    }
 	    rb_gv_set(s, v);
 	}
-	n = RARRAY_LEN(rb_argv) - n;
+	n = RARRAY_LEN(argv) - n;
 	while (n--) {
-	    rb_ary_shift(rb_argv);
+	    rb_ary_shift(argv);
 	}
     }
     opt->sflag = 0;
@@ -544,12 +552,60 @@ moreswitches(const char *s, struct cmdline_options *opt)
     return (char *)s;
 }
 
+#define NAME_MATCH_P(name, str, len) \
+    ((len) < sizeof(name) && strncmp((str), name, (len)) == 0)
+
+#define UNSET_WHEN(name, bit, str, len)	\
+    if (NAME_MATCH_P(name, str, len)) { \
+	*(unsigned int *)arg &= ~(bit); \
+	return;				\
+    }
+
+#define SET_WHEN(name, bit, str, len)	\
+    if (NAME_MATCH_P(name, str, len)) { \
+	*(unsigned int *)arg |= (bit);	\
+	return;				\
+    }
+
+static void
+enable_option(const char *str, int len, void *arg)
+{
+#define UNSET_WHEN_DISABLE(bit) UNSET_WHEN(#bit, DISABLE_BIT(bit), str, len)
+    UNSET_WHEN_DISABLE(gems);
+    UNSET_WHEN_DISABLE(rubyopt);
+    if (NAME_MATCH_P("all", str, len)) {
+	*(unsigned int *)arg = 0U;
+	return;
+    }
+    rb_warn("unknown argument for --enable: `%.*s'", len, str);
+}
+
+static void
+disable_option(const char *str, int len, void *arg)
+{
+#define SET_WHEN_DISABLE(bit) SET_WHEN(#bit, DISABLE_BIT(bit), str, len)
+    SET_WHEN_DISABLE(gems);
+    SET_WHEN_DISABLE(rubyopt);
+    if (NAME_MATCH_P("all", str, len)) {
+	*(unsigned int *)arg = ~0U;
+	return;
+    }
+    rb_warn("unknown argument for --disable: `%.*s'", len, str);
+}
+
+static void
+dump_option(const char *str, int len, void *arg)
+{
+#define SET_WHEN_DUMP(bit) SET_WHEN(#bit, DUMP_BIT(bit), str, len)
+    SET_WHEN_DUMP(insns);
+    rb_warn("don't know how to dump `%.*s', (insns)", len, str);
+}
+
 static int
 proc_options(int argc, char **argv, struct cmdline_options *opt)
 {
-    int argc0 = argc;
+    int n, argc0 = argc;
     const char *s;
-    const char *enc_name = 0;
 
     if (argc == 0)
 	return 0;
@@ -671,19 +727,17 @@ proc_options(int argc, char **argv, struct cmdline_options *opt)
 	  case 'r':
 	    forbid_setid("-r");
 	    if (*++s) {
-		add_modules(s);
+		add_modules(opt, s);
 	    }
 	    else if (argv[1]) {
-		add_modules(argv[1]);
+		add_modules(opt, argv[1]);
 		argc--, argv++;
 	    }
 	    break;
 
 	  case 'i':
 	    forbid_setid("-i");
-	    if (ruby_inplace_mode)
-		free(ruby_inplace_mode);
-	    ruby_inplace_mode = strdup(s + 1);
+	    ruby_set_inplace_mode(s + 1);
 	    break;
 
 	  case 'x':
@@ -711,36 +765,34 @@ proc_options(int argc, char **argv, struct cmdline_options *opt)
 
 	  case 'F':
 	    if (*++s) {
-		rb_fs = rb_reg_new(rb_str_new2(s), 0);
+		rb_fs = rb_reg_new(s, strlen(s), 0);
 	    }
 	    break;
 
 	  case 'E':
-	    if (!*++s) {
-		s = argv[1];
-		argc--, argv++;
-	    }
+	    if (!*++s) goto next_encoding;
 	    goto encoding;
 
 	  case 'K':
 	    if (*++s) {
-		rb_encoding *enc = 0;
+		const char *enc_name = 0;
 		switch (*s) {
 		  case 'E': case 'e':
-		    enc = ONIG_ENCODING_EUC_JP;
+		    enc_name = "EUC-JP";
 		    break;
 		  case 'S': case 's':
-		    enc = ONIG_ENCODING_SJIS;
+		    enc_name = "Windows-31J";
 		    break;
 		  case 'U': case 'u':
-		    enc = ONIG_ENCODING_UTF8;
+		    enc_name = "UTF-8";
 		    break;
 		  case 'N': case 'n': case 'A': case 'a':
-		    enc = ONIG_ENCODING_ASCII;
+		    enc_name = "ASCII-8BIT";
 		    break;
 		}
-		if (enc) {
-		    opt->enc_index = rb_enc_find_index(rb_enc_name(enc));
+		if (enc_name) {
+		    opt->src.enc.name = rb_str_new2(enc_name);
+		    opt->ext.enc.name = opt->src.enc.name;
 		}
 		s++;
 	    }
@@ -803,23 +855,30 @@ proc_options(int argc, char **argv, struct cmdline_options *opt)
 		ruby_debug = Qtrue;
                 ruby_verbose = Qtrue;
             }
-            else if (strcmp("disable-gems", s) == 0)
-		opt->disable_gems = 1;
-	    else if (strcmp("encoding", s) == 0) {
-		int idx;
-		if (!--argc || !(s = *++argv)) {
-		  noencoding:
-		    rb_raise(rb_eRuntimeError, "missing argument for --encoding");
+	    else if (strncmp("enable", s, n = 6) == 0 &&
+		     (!s[n] || s[n] == '-' || s[n] == '=')) {
+		if ((s += n + 1)[-1] ? !*s : (!--argc || !(s = *++argv))) {
+		    rb_raise(rb_eRuntimeError, "missing argument for --enable");
+		}
+		ruby_each_words(s, enable_option, &opt->disable);
+	    }
+	    else if (strncmp("disable", s, n = 7) == 0 &&
+		     (!s[n] || s[n] == '-' || s[n] == '=')) {
+		if ((s += n + 1)[-1] ? !*s : (!--argc || !(s = *++argv))) {
+		    rb_raise(rb_eRuntimeError, "missing argument for --disable");
+		}
+		ruby_each_words(s, disable_option, &opt->disable);
+	    }
+	    else if (strncmp("encoding", s, n = 8) == 0 && (!s[n] || s[n] == '=')) {
+		s += n;
+		if (!*s++) {
+		  next_encoding:
+		    if (!--argc || !(s = *++argv)) {
+			rb_raise(rb_eRuntimeError, "missing argument for --encoding");
+		    }
 		}
 	      encoding:
-		if ((idx = rb_enc_find_index(s)) < 0) {
-		    rb_raise(rb_eRuntimeError, "unknown encoding name - %s", s);
-		}
-		opt->enc_index = idx;
-	    }
-	    else if (strncmp("encoding=", s, 9) == 0) {
-		if (!*(s += 9)) goto noencoding;
-		goto encoding;
+		opt->ext.enc.name = rb_str_new2(s);
 	    }
 	    else if (strcmp("version", s) == 0)
 		opt->version = 1;
@@ -829,6 +888,10 @@ proc_options(int argc, char **argv, struct cmdline_options *opt)
 	    }
 	    else if (strcmp("yydebug", s) == 0)
 		opt->yydebug = 1;
+	    else if (strncmp("dump", s, n = 4) == 0 && (!s[n] || s[n] == '=')) {
+		if (!(s += n + 1)[-1] && (!--argc || !(s = *++argv)) && *s != '-') break;
+		ruby_each_words(s, dump_option, &opt->dump);
+	    }
 	    else if (strcmp("help", s) == 0) {
 		usage(origarg.argv[0]);
 		rb_exit(EXIT_SUCCESS);
@@ -870,30 +933,54 @@ proc_options(int argc, char **argv, struct cmdline_options *opt)
 void Init_prelude(void);
 
 static void
-ruby_init_gems(struct cmdline_options *opt)
+ruby_init_gems(int enable)
 {
-    VALUE gem;
-    gem = rb_define_module("Gem");
-    rb_const_set(gem, rb_intern("Enable"), opt->disable_gems ? Qfalse : Qtrue);
+    if (enable) rb_define_module("Gem");
     Init_prelude();
 }
 
+static int
+opt_enc_index(VALUE enc_name)
+{
+    const char *s = RSTRING_PTR(enc_name);
+    int i = rb_enc_find_index(s);
+
+    if (i < 0) {
+	rb_raise(rb_eRuntimeError, "unknown encoding name - %s", s);
+    }
+    else if (rb_enc_dummy_p(rb_enc_from_index(i))) {
+	rb_raise(rb_eRuntimeError, "dummy encoding is not acceptable - %s ", s);
+    }
+    return i;
+}
+
+#define rb_progname (GET_VM()->progname)
+VALUE rb_argv0;
+
 static VALUE
 process_options(VALUE arg)
 {
-    struct cmdline_options *opt = (struct cmdline_options *)arg;
-    int argc = opt->argc;
-    char **argv = opt->argv;
+    struct cmdline_arguments *argp = (struct cmdline_arguments *)arg;
+    struct cmdline_options *opt = argp->opt;
+    int argc = argp->argc;
+    char **argv = argp->argv;
     NODE *tree = 0;
     VALUE parser;
-    rb_encoding *enc;
+    VALUE iseq;
+    rb_encoding *enc, *lenc;
     const char *s;
+    char fbuf[MAXPATHLEN];
     int i = proc_options(argc, argv, opt);
+    int safe;
 
     argc -= i;
     argv += i;
 
-    if (rb_safe_level() == 0 && (s = getenv("RUBYOPT"))) {
+    if (!(opt->disable & DISABLE_BIT(rubyopt)) &&
+	rb_safe_level() == 0 && (s = getenv("RUBYOPT"))) {
+	VALUE src_enc_name = opt->src.enc.name;
+	VALUE ext_enc_name = opt->ext.enc.name;
+
 	while (ISSPACE(*s))
 	    s++;
 	if (*s == 'T' || (*s == '-' && *(s + 1) == 'T')) {
@@ -922,12 +1009,16 @@ process_options(VALUE arg)
 		}
 		if (!*s)
 		    break;
-		if (!strchr("IdvwWrK", *s))
+		if (!strchr("EIdvwWrK", *s))
 		    rb_raise(rb_eRuntimeError,
 			     "invalid switch in RUBYOPT: -%c", *s);
 		s = moreswitches(s, opt);
 	    }
 	}
+	if (src_enc_name)
+	    opt->src.enc.name = src_enc_name;
+	if (ext_enc_name)
+	    opt->ext.enc.name = ext_enc_name;
     }
 
     if (opt->version) {
@@ -940,7 +1031,7 @@ process_options(VALUE arg)
 
     if (rb_safe_level() >= 4) {
 	OBJ_TAINT(rb_argv);
-	OBJ_TAINT(rb_load_path);
+	OBJ_TAINT(GET_VM()->load_path);
     }
 
     if (!opt->e_script) {
@@ -959,35 +1050,64 @@ process_options(VALUE arg)
 
 		opt->script = 0;
 		if (path) {
-		    opt->script = dln_find_file(argv[0], path);
+		    opt->script = dln_find_file_r(argv[0], path, fbuf, sizeof(fbuf));
 		}
 		if (!opt->script) {
-		    opt->script = dln_find_file(argv[0], getenv(PATH_ENV));
+		    opt->script = dln_find_file_r(argv[0], getenv(PATH_ENV), fbuf, sizeof(fbuf));
 		}
 		if (!opt->script)
 		    opt->script = argv[0];
 	    }
-#if defined DOSISH || defined __CYGWIN__
-	    /* assume that we can change argv[n] if never change its length. */
-	    translate_char(opt->script, '\\', '/');
-#endif
 	    argc--;
 	    argv++;
 	}
     }
 
     ruby_script(opt->script);
+#if defined DOSISH || defined __CYGWIN__
+    translate_char(RSTRING_PTR(rb_progname), '\\', '/');
+#endif
+    opt->script_name = rb_str_new4(rb_progname);
+    opt->script = RSTRING_PTR(opt->script_name);
     ruby_set_argv(argc, argv);
     process_sflag(opt);
 
     ruby_init_loadpath();
-    ruby_init_gems(opt);
+    safe = rb_safe_level();
+    rb_set_safe_level_force(0);
+    ruby_init_gems(!(opt->disable & DISABLE_BIT(gems)));
+    lenc = rb_locale_encoding();
+    for (i = 0; i < RARRAY_LEN(rb_argv); i++) {
+	rb_enc_associate(RARRAY_PTR(rb_argv)[i], lenc);
+    }
     parser = rb_parser_new();
     if (opt->yydebug) rb_parser_set_yydebug(parser, Qtrue);
+    if (opt->ext.enc.name != 0) {
+	opt->ext.enc.index = opt_enc_index(opt->ext.enc.name);
+    }
+    if (opt->src.enc.name != 0) {
+	opt->src.enc.index = opt_enc_index(opt->src.enc.name);
+	src_encoding_index = opt->src.enc.index;
+    }
+    if (opt->ext.enc.index >= 0) {
+	enc = rb_enc_from_index(opt->ext.enc.index);
+    }
+    else {
+	enc = lenc;
+    }
+    rb_enc_set_default_external(rb_enc_from_encoding(enc));
+
+    rb_set_safe_level_force(safe);
     if (opt->e_script) {
-	if (opt->enc_index >= 0)
-	    rb_enc_associate_index(opt->e_script, opt->enc_index);
-	require_libraries();
+	rb_encoding *eenc;
+	if (opt->src.enc.index >= 0) {
+	    eenc = rb_enc_from_index(opt->src.enc.index);
+	}
+	else {
+	    eenc = lenc;
+	}
+	rb_enc_associate(opt->e_script, eenc);
+	require_libraries(opt);
 	tree = rb_parser_compile_string(parser, opt->script, opt->e_script, 1);
     }
     else {
@@ -1004,7 +1124,7 @@ process_options(VALUE arg)
 
     if (rb_safe_level() >= 4) {
 	FL_UNSET(rb_argv, FL_TAINT);
-	FL_UNSET(rb_load_path, FL_TAINT);
+	FL_UNSET(GET_VM()->load_path, FL_TAINT);
     }
 
     if (opt->do_check) {
@@ -1012,24 +1132,23 @@ process_options(VALUE arg)
 	return Qtrue;
     }
 
-    if (tree) {
-	if (opt->do_print) {
-	    tree = rb_parser_append_print(parser, tree);
-	}
-	if (opt->do_loop) {
-	    tree = rb_parser_while_loop(parser, tree, opt->do_line, opt->do_split);
-	}
+    if (opt->do_print) {
+	tree = rb_parser_append_print(parser, tree);
+    }
+    if (opt->do_loop) {
+	tree = rb_parser_while_loop(parser, tree, opt->do_line, opt->do_split);
     }
 
-    if (opt->enc_index >= 0) {
-	enc = rb_enc_from_index(opt->enc_index);
-    }
-    else {
-	enc = locale_encoding();
-    }
-    rb_enc_set_default_external(rb_enc_from_encoding(enc));
+    iseq = rb_iseq_new(tree, rb_str_new2("<main>"),
+		       opt->script_name, Qfalse, ISEQ_TYPE_TOP);
 
-    return (VALUE)tree;
+    if (opt->dump & DUMP_BIT(insns)) {
+	rb_io_write(rb_stdout, ruby_iseq_disasm(iseq));
+	rb_io_flush(rb_stdout);
+	return Qtrue;
+    }
+
+    return iseq;
 }
 
 static NODE *
@@ -1039,6 +1158,7 @@ load_file(VALUE parser, const char *fname, int script, struct cmdline_options *o
     VALUE f;
     int line_start = 1;
     NODE *tree = 0;
+    rb_encoding *enc;
 
     if (!fname)
 	rb_load_fail(fname);
@@ -1050,7 +1170,7 @@ load_file(VALUE parser, const char *fname, int script, struct cmdline_options *o
 #if defined DOSISH || defined __CYGWIN__
 	{
 	    const char *ext = strrchr(fname, '.');
-	    if (ext && strcasecmp(ext, ".exe") == 0)
+	    if (ext && STRCASECMP(ext, ".exe") == 0)
 		mode |= O_BINARY;
 	}
 #endif
@@ -1065,6 +1185,11 @@ load_file(VALUE parser, const char *fname, int script, struct cmdline_options *o
 	VALUE c = 1;		/* something not nil */
 	VALUE line;
 	char *p;
+	int no_src_enc = !opt->src.enc.name;
+	int no_ext_enc = !opt->ext.enc.name;
+
+	enc = rb_usascii_encoding();
+	rb_funcall(f, rb_intern("set_encoding"), 1, rb_enc_from_encoding(enc));
 
 	if (opt->xflag) {
 	    forbid_setid("-x");
@@ -1141,14 +1266,31 @@ load_file(VALUE parser, const char *fname, int script, struct cmdline_options *o
 		rb_io_ungetc(f, c);
 	    }
 	    rb_io_ungetc(f, INT2FIX('#'));
+	    if (no_src_enc && opt->src.enc.name) {
+		opt->src.enc.index = opt_enc_index(opt->src.enc.name);
+		src_encoding_index = opt->src.enc.index;
+	    }
+	    if (no_ext_enc && opt->ext.enc.name) {
+		opt->ext.enc.index = opt_enc_index(opt->ext.enc.name);
+	    }
 	}
 	else if (!NIL_P(c)) {
 	    rb_io_ungetc(f, c);
 	}
-	require_libraries();	/* Why here? unnatural */
+	require_libraries(opt);	/* Why here? unnatural */
     }
-    if (opt->enc_index >= 0) rb_enc_associate_index(f, opt->enc_index);
+    if (opt->src.enc.index >= 0) {
+	enc = rb_enc_from_index(opt->src.enc.index);
+    }
+    else if (f == rb_stdin) {
+	enc = rb_locale_encoding();
+    }
+    else {
+	enc = rb_usascii_encoding();
+    }
+    rb_funcall(f, rb_intern("set_encoding"), 1, rb_enc_from_encoding(enc));
     tree = (NODE *)rb_parser_compile_file(parser, fname, f, line_start);
+    rb_funcall(f, rb_intern("set_encoding"), 1, rb_parser_encoding(parser));
     if (script && rb_parser_end_seen_p(parser)) {
 	rb_define_global_const("DATA", f);
     }
@@ -1163,14 +1305,9 @@ rb_load_file(const char *fname)
 {
     struct cmdline_options opt;
 
-    MEMZERO(&opt, opt, 1);
-    return load_file(rb_parser_new(), fname, 0, &opt);
+    return load_file(rb_parser_new(), fname, 0, cmdline_options_init(&opt));
 }
 
-VALUE rb_progname;
-VALUE rb_argv;
-VALUE rb_argv0;
-
 #if !defined(PSTAT_SETCMD) && !defined(HAVE_SETPROCTITLE)
 #if !defined(_WIN32) && !(defined(HAVE_SETENV) && defined(HAVE_UNSETENV))
 #define USE_ENVSPACE_FOR_ARG0
@@ -1186,6 +1323,7 @@ get_arglen(int argc, char **argv)
     char *s = argv[0];
     int i;
 
+    if (!argc) return 0;
     s += strlen(s);
     /* See if all the arguments are contiguous in memory */
     for (i = 1; i < argc; i++) {
@@ -1216,7 +1354,7 @@ get_arglen(int argc, char **argv)
 static void
 set_arg0(VALUE val, ID id)
 {
-    char *s, *t;
+    char *s;
     long i;
 
     if (origarg.argv == 0)
@@ -1247,13 +1385,13 @@ set_arg0(VALUE val, ID id)
     }
 
     memcpy(origarg.argv[0], s, i);
-    t = origarg.argv[0] + i;
-    *t = '\0';
-
-    if (i + 1 < origarg.len) memset(t + 1, ' ', origarg.len - i - 1);
 
     {
 	int j;
+	char *t = origarg.argv[0] + i;
+	*t = '\0';
+
+	if (i + 1 < origarg.len) memset(t + 1, ' ', origarg.len - i - 1);
 	for (j = 1; j < origarg.argc; j++) {
 	    origarg.argv[j] = t;
 	}
@@ -1270,70 +1408,72 @@ ruby_script(const char *name)
     }
 }
 
-static int uid, euid, gid, egid;
-
 static void
-init_ids(void)
+init_ids(struct cmdline_options *opt)
 {
-    uid = (int)getuid();
-    euid = (int)geteuid();
-    gid = (int)getgid();
-    egid = (int)getegid();
+    rb_uid_t uid = getuid();
+    rb_uid_t euid = geteuid();
+    rb_gid_t gid = getgid();
+    rb_gid_t egid = getegid();
+
 #ifdef VMS
     uid |= gid << 16;
     euid |= egid << 16;
 #endif
-    if (uid && (euid != uid || egid != gid)) {
+    if (uid != euid) opt->setids |= 1;
+    if (egid != gid) opt->setids |= 2;
+    if (uid && opt->setids) {
 	rb_set_safe_level(1);
     }
 }
 
+#undef forbid_setid
 static void
-forbid_setid(const char *s)
+forbid_setid(const char *s, struct cmdline_options *opt)
 {
-    if (euid != uid)
+    if (opt->setids & 1)
         rb_raise(rb_eSecurityError, "no %s allowed while running setuid", s);
-    if (egid != gid)
+    if (opt->setids & 2)
         rb_raise(rb_eSecurityError, "no %s allowed while running setgid", s);
     if (rb_safe_level() > 0)
         rb_raise(rb_eSecurityError, "no %s allowed in tainted mode", s);
 }
 
 static void
-verbose_setter(VALUE val, ID id, VALUE *variable)
+verbose_setter(VALUE val, ID id, void *data)
 {
-    ruby_verbose = RTEST(val) ? Qtrue : val;
+    VALUE *variable = data;
+    *variable = RTEST(val) ? Qtrue : val;
 }
 
 static VALUE
-opt_W_getter(VALUE val, ID id)
+opt_W_getter(ID id, void *data)
 {
-    if (ruby_verbose == Qnil)
+    VALUE *variable = data;
+    switch (*variable) {
+      case Qnil:
 	return INT2FIX(0);
-    if (ruby_verbose == Qfalse)
+      case Qfalse:
 	return INT2FIX(1);
-    if (ruby_verbose == Qtrue)
+      case Qtrue:
 	return INT2FIX(2);
+    }
     return Qnil;		/* not reached */
 }
 
 void
 ruby_prog_init(void)
 {
-    init_ids();
-
     rb_define_hooked_variable("$VERBOSE", &ruby_verbose, 0, verbose_setter);
     rb_define_hooked_variable("$-v", &ruby_verbose, 0, verbose_setter);
     rb_define_hooked_variable("$-w", &ruby_verbose, 0, verbose_setter);
-    rb_define_virtual_variable("$-W", opt_W_getter, 0);
+    rb_define_hooked_variable("$-W", &ruby_verbose, opt_W_getter, 0);
     rb_define_variable("$DEBUG", &ruby_debug);
     rb_define_variable("$-d", &ruby_debug);
 
     rb_define_hooked_variable("$0", &rb_progname, 0, set_arg0);
     rb_define_hooked_variable("$PROGRAM_NAME", &rb_progname, 0, set_arg0);
 
-    rb_define_readonly_variable("$*", &rb_argv);
-    rb_argv = rb_ary_new();
     rb_define_global_const("ARGV", rb_argv);
     rb_global_variable(&rb_argv0);
 
@@ -1351,6 +1491,7 @@ void
 ruby_set_argv(int argc, char **argv)
 {
     int i;
+    VALUE av = rb_argv;
 
 #if defined(USE_DLN_A_OUT)
     if (origarg.argv)
@@ -1358,12 +1499,12 @@ ruby_set_argv(int argc, char **argv)
     else
 	dln_argv0 = argv[0];
 #endif
-    rb_ary_clear(rb_argv);
+    rb_ary_clear(av);
     for (i = 0; i < argc; i++) {
 	VALUE arg = rb_tainted_str_new2(argv[i]);
 
 	OBJ_FREEZE(arg);
-	rb_ary_push(rb_argv, arg);
+	rb_ary_push(av, arg);
     }
 }
 
@@ -1385,17 +1526,18 @@ true_value(void)
 void *
 ruby_process_options(int argc, char **argv)
 {
+    struct cmdline_arguments args;
     struct cmdline_options opt;
     NODE *tree;
 
-    MEMZERO(&opt, opt, 1);
     ruby_script(argv[0]);	/* for the time being */
-    rb_argv0 = rb_progname;
-    opt.argc = argc;
-    opt.argv = argv;
-    opt.enc_index = -1;
+    rb_argv0 = rb_str_new4(rb_progname);
+    args.argc = argc;
+    args.argv = argv;
+    args.opt = cmdline_options_init(&opt);
+    opt.ext.enc.index = -1;
     tree = (NODE *)rb_vm_call_cfunc(rb_vm_top_self(),
-				    process_options, (VALUE)&opt,
+				    process_options, (VALUE)&args,
 				    0, rb_progname);
 
     rb_define_readonly_boolean("$-p", opt.do_print);
