@@ -2,8 +2,7 @@
 
   sprintf.c -
 
-  $Author: matz $
-  $Date: 2007-08-25 12:29:39 +0900 (土, 25  8月 2007) $
+  $Author: akr $
   created at: Fri Oct 15 10:39:26 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -19,6 +18,8 @@
 #include <stdarg.h>
 
 #define BIT_DIGITS(N)   (((N)*146)/485 + 1)  /* log2(10) =~ 146/485 */
+#define BITSPERDIG (SIZEOF_BDIGITS*CHAR_BIT)
+#define EXTENDSIGN(n, l) (((~0 << (n)) >> (((n)*(l)) % BITSPERDIG)) & ~(~0 << (n)))
 
 static void fmt_setup(char*,int,int,int,int);
 
@@ -35,7 +36,7 @@ remove_sign_bits(char *str, int base)
 	}
     }
     else if (base == 8) {
-	if (*t == '3') t++;
+	*t |= EXTENDSIGN(3, strlen(t));
 	while (*t == '7') {
 	    t++;
 	}
@@ -45,12 +46,8 @@ remove_sign_bits(char *str, int base)
 	    t++;
 	}
     }
-    if (t > s) {
-	while (*t) *s++ = *t++;
-	*s = '\0';
-    }
 
-    return str;
+    return t;
 }
 
 static char
@@ -147,94 +144,238 @@ sign_bits(int base, const char *p)
  *     sprintf(format_string [, arguments...] )  => string
  *  
  *  Returns the string resulting from applying <i>format_string</i> to
- *  any additional arguments. Within the format string, any characters
- *  other than format sequences are copied to the result. A format
+ *  any additional arguments.  Within the format string, any characters
+ *  other than format sequences are copied to the result. 
+ *
+ *  The syntax of a format sequence is follows.
+ *
+ *    %[flags][width][.precision]type
+ *
+ *  A format
  *  sequence consists of a percent sign, followed by optional flags,
  *  width, and precision indicators, then terminated with a field type
- *  character. The field type controls how the corresponding
+ *  character.  The field type controls how the corresponding
  *  <code>sprintf</code> argument is to be interpreted, while the flags
- *  modify that interpretation. The field type characters are listed
- *  in the table at the end of this section. The flag characters are:
+ *  modify that interpretation.
  *
- *    Flag     | Applies to   | Meaning
- *    ---------+--------------+-----------------------------------------
- *    space    | bdeEfgGiouxX | Leave a space at the start of 
- *             |              | positive numbers.
- *    ---------+--------------+-----------------------------------------
- *    (digit)$ | all          | Specifies the absolute argument number
- *             |              | for this field. Absolute and relative
- *             |              | argument numbers cannot be mixed in a
- *             |              | sprintf string.
- *    ---------+--------------+-----------------------------------------
- *     #       | beEfgGoxX    | Use an alternative format. For the
- *             |              | conversions `o', `x', `X', and `b', 
- *             |              | prefix the result with ``0'', ``0x'', ``0X'',
- *             |              |  and ``0b'', respectively. For `e',
- *             |              | `E', `f', `g', and 'G', force a decimal
- *             |              | point to be added, even if no digits follow.
- *             |              | For `g' and 'G', do not remove trailing zeros.
- *    ---------+--------------+-----------------------------------------
- *    +        | bdeEfgGiouxX | Add a leading plus sign to positive numbers.
- *    ---------+--------------+-----------------------------------------
- *    -        | all          | Left-justify the result of this conversion.
- *    ---------+--------------+-----------------------------------------
- *    0 (zero) | bdeEfgGiouxX | Pad with zeros, not spaces.
- *    ---------+--------------+-----------------------------------------
- *    *        | all          | Use the next argument as the field width. 
- *             |              | If negative, left-justify the result. If the
- *             |              | asterisk is followed by a number and a dollar 
- *             |              | sign, use the indicated argument as the width.
+ *  The field type characters are:
  *
+ *      Field |  Integer Format
+ *      ------+--------------------------------------------------------------
+ *        b   | Convert argument as a binary number.
+ *            | Negative numbers will be displayed as a two's complement
+ *            | prefixed with `..1'.
+ *        B   | Equivalent to `b', but uses an uppercase 0B for prefix
+ *            | in the alternative format by #.
+ *        d   | Convert argument as a decimal number.
+ *        i   | Identical to `d'.
+ *        o   | Convert argument as an octal number.
+ *            | Negative numbers will be displayed as a two's complement
+ *            | prefixed with `..7'.
+ *        u   | Identical to `d'.
+ *        x   | Convert argument as a hexadecimal number.
+ *            | Negative numbers will be displayed as a two's complement
+ *            | prefixed with `..f' (representing an infinite string of
+ *            | leading 'ff's).
+ *        X   | Equivalent to `x', but uses uppercase letters.
+ *
+ *      Field |  Float Format
+ *      ------+--------------------------------------------------------------
+ *        e   | Convert floating point argument into exponential notation 
+ *            | with one digit before the decimal point as [-]d.dddddde[+-]dd.
+ *            | The precision specifies the number of digits after the decimal
+ *            | point (defaulting to six).
+ *        E   | Equivalent to `e', but uses an uppercase E to indicate
+ *            | the exponent.
+ *        f   | Convert floating point argument as [-]ddd.dddddd, 
+ *            | where the precision specifies the number of digits after
+ *            | the decimal point.
+ *        g   | Convert a floating point number using exponential form
+ *            | if the exponent is less than -4 or greater than or
+ *            | equal to the precision, or in dd.dddd form otherwise.
+ *            | The precision specifies the number of significant digits.
+ *        G   | Equivalent to `g', but use an uppercase `E' in exponent form.
+ *
+ *      Field |  Other Format
+ *      ------+--------------------------------------------------------------
+ *        c   | Argument is the numeric code for a single character or
+ *            | a single character string itself.
+ *        p   | The valuing of argument.inspect.
+ *        s   | Argument is a string to be substituted.  If the format
+ *            | sequence contains a precision, at most that many characters
+ *            | will be copied.
+ *        %   | A percent sign itself will be displayed.  No argument taken.
+ *     
+ *  The flags modifies the behavior of the formats.
+ *  The flag characters are:
+ *
+ *    Flag     | Applies to    | Meaning
+ *    ---------+---------------+-----------------------------------------
+ *    space    | bBdiouxX      | Leave a space at the start of 
+ *             | eEfgG         | non-negative numbers.
+ *             | (numeric fmt) | For `o', `x', `X', `b' and `B', use
+ *             |               | a minus sign with absolute value for
+ *             |               | negative values.
+ *    ---------+---------------+-----------------------------------------
+ *    (digit)$ | all           | Specifies the absolute argument number
+ *             |               | for this field.  Absolute and relative
+ *             |               | argument numbers cannot be mixed in a
+ *             |               | sprintf string.
+ *    ---------+---------------+-----------------------------------------
+ *     #       | bBoxX         | Use an alternative format.
+ *             | eEfgG         | For the conversions `o', increase the precision
+ *             |               | until the first digit will be `0' if
+ *             |               | it is not formatted as complements.
+ *             |               | For the conversions `x', `X', `b' and `B'
+ *             |               | on non-zero, prefix the result with ``0x'',
+ *             |               | ``0X'', ``0b'' and ``0B'', respectively.
+ *             |               | For `e', `E', `f', `g', and 'G',
+ *             |               | force a decimal point to be added,
+ *             |               | even if no digits follow.
+ *             |               | For `g' and 'G', do not remove trailing zeros.
+ *    ---------+---------------+-----------------------------------------
+ *    +        | bBdiouxX      | Add a leading plus sign to non-negative
+ *             | eEfgG         | numbers.
+ *             | (numeric fmt) | For `o', `x', `X', `b' and `B', use
+ *             |               | a minus sign with absolute value for
+ *             |               | negative values.
+ *    ---------+---------------+-----------------------------------------
+ *    -        | all           | Left-justify the result of this conversion.
+ *    ---------+---------------+-----------------------------------------
+ *    0 (zero) | bBdiouxX      | Pad with zeros, not spaces.
+ *             | eEfgG         | For `o', `x', `X', `b' and `B', radix-1
+ *             | (numeric fmt) | is used for negative numbers formatted as
+ *             |               | complements.
+ *    ---------+---------------+-----------------------------------------
+ *    *        | all           | Use the next argument as the field width. 
+ *             |               | If negative, left-justify the result. If the
+ *             |               | asterisk is followed by a number and a dollar 
+ *             |               | sign, use the indicated argument as the width.
+ *
+ *  Examples of flags:
+ *
+ *   # `+' and space flag specifies the sign of non-negative numbers.
+ *   sprintf("%d", 123)  #=> "123"
+ *   sprintf("%+d", 123) #=> "+123"
+ *   sprintf("% d", 123) #=> " 123"
+ *
+ *   # `#' flag for `o' increases number of digits to show `0'.
+ *   # `+' and space flag changes format of negative numbers.
+ *   sprintf("%o", 123)   #=> "173"
+ *   sprintf("%#o", 123)  #=> "0173"
+ *   sprintf("%+o", -123) #=> "-173"
+ *   sprintf("%o", -123)  #=> "..7605"
+ *   sprintf("%#o", -123) #=> "..7605"
+ *
+ *   # `#' flag for `x' add a prefix `0x' for non-zero numbers.
+ *   # `+' and space flag disables complements for negative numbers.
+ *   sprintf("%x", 123)   #=> "7b"
+ *   sprintf("%#x", 123)  #=> "0x7b"
+ *   sprintf("%+x", -123) #=> "-7b"
+ *   sprintf("%x", -123)  #=> "..f85"
+ *   sprintf("%#x", -123) #=> "0x..f85"
+ *   sprintf("%#x", 0)    #=> "0"
+ *
+ *   # `#' for `X' uses the prefix `0X'.
+ *   sprintf("%X", 123)  #=> "7B"
+ *   sprintf("%#X", 123) #=> "0X7B"
+ *
+ *   # `#' flag for `b' add a prefix `0b' for non-zero numbers.
+ *   # `+' and space flag disables complements for negative numbers.
+ *   sprintf("%b", 123)   #=> "1111011"
+ *   sprintf("%#b", 123)  #=> "0b1111011"
+ *   sprintf("%+b", -123) #=> "-1111011"
+ *   sprintf("%b", -123)  #=> "..10000101"
+ *   sprintf("%#b", -123) #=> "0b..10000101"
+ *   sprintf("%#b", 0)    #=> "0"
+ *
+ *   # `#' for `B' uses the prefix `0B'.
+ *   sprintf("%B", 123)  #=> "1111011"
+ *   sprintf("%#B", 123) #=> "0B1111011"
+ *
+ *   # `#' for `e' forces to show the decimal point.
+ *   sprintf("%.0e", 1)  #=> "1e+00"
+ *   sprintf("%#.0e", 1) #=> "1.e+00"
+ *
+ *   # `#' for `f' forces to show the decimal point.
+ *   sprintf("%.0f", 1234)  #=> "1234"
+ *   sprintf("%#.0f", 1234) #=> "1234."
+ *
+ *   # `#' for `g' forces to show the decimal point.
+ *   # It also disables stripping lowest zeros.
+ *   sprintf("%g", 123.4)   #=> "123.4"
+ *   sprintf("%#g", 123.4)  #=> "123.400"
+ *   sprintf("%g", 123456)  #=> "123456"
+ *   sprintf("%#g", 123456) #=> "123456."
  *     
  *  The field width is an optional integer, followed optionally by a
- *  period and a precision. The width specifies the minimum number of
- *  characters that will be written to the result for this field. For
+ *  period and a precision.  The width specifies the minimum number of
+ *  characters that will be written to the result for this field.
+ *
+ *  Examples of width:
+ *
+ *   # padding is done by spaces,       width=20
+ *   # 0 or radix-1.             <------------------>
+ *   sprintf("%20d", 123)   #=> "                 123"
+ *   sprintf("%+20d", 123)  #=> "                +123"
+ *   sprintf("%020d", 123)  #=> "00000000000000000123"
+ *   sprintf("%+020d", 123) #=> "+0000000000000000123"
+ *   sprintf("% 020d", 123) #=> " 0000000000000000123"
+ *   sprintf("%-20d", 123)  #=> "123                 "
+ *   sprintf("%-+20d", 123) #=> "+123                "
+ *   sprintf("%- 20d", 123) #=> " 123                "
+ *   sprintf("%020x", -123) #=> "..ffffffffffffffff85"
+ *
+ *  For
  *  numeric fields, the precision controls the number of decimal places
- *  displayed. For string fields, the precision determines the maximum
- *  number of characters to be copied from the string. (Thus, the format
+ *  displayed.  For string fields, the precision determines the maximum
+ *  number of characters to be copied from the string.  (Thus, the format
  *  sequence <code>%10.10s</code> will always contribute exactly ten
  *  characters to the result.)
  *
- *  The field types are:
+ *  Examples of precisions:
+ *
+ *   # precision for `d', 'o', 'x' and 'b' is
+ *   # minimum number of digits               <------>
+ *   sprintf("%20.8d", 123)  #=> "            00000123"
+ *   sprintf("%20.8o", 123)  #=> "            00000173"
+ *   sprintf("%20.8x", 123)  #=> "            0000007b"
+ *   sprintf("%20.8b", 123)  #=> "            01111011"
+ *   sprintf("%20.8d", -123) #=> "           -00000123"
+ *   sprintf("%20.8o", -123) #=> "            ..777605"
+ *   sprintf("%20.8x", -123) #=> "            ..ffff85"
+ *   sprintf("%20.8b", -11)  #=> "            ..110101"
+ *
+ *   # "0x" and "0b" for `#x' and `#b' is not counted for
+ *   # precision but "0" for `#o' is counted.  <------>
+ *   sprintf("%#20.8d", 123)  #=> "            00000123"
+ *   sprintf("%#20.8o", 123)  #=> "            00000173"
+ *   sprintf("%#20.8x", 123)  #=> "          0x0000007b"
+ *   sprintf("%#20.8b", 123)  #=> "          0b01111011"
+ *   sprintf("%#20.8d", -123) #=> "           -00000123"
+ *   sprintf("%#20.8o", -123) #=> "            ..777605"
+ *   sprintf("%#20.8x", -123) #=> "          0x..ffff85"
+ *   sprintf("%#20.8b", -11)  #=> "          0b..110101"
+ *
+ *   # precision for `e' is number of
+ *   # digits after the decimal point           <------>
+ *   sprintf("%20.8e", 1234.56789) #=> "      1.23456789e+03"
+ *                                    
+ *   # precision for `f' is number of
+ *   # digits after the decimal point               <------>
+ *   sprintf("%20.8f", 1234.56789) #=> "       1234.56789000"
+ *
+ *   # precision for `g' is number of
+ *   # significant digits                          <------->
+ *   sprintf("%20.8g", 1234.56789) #=> "           1234.5679"
+ *
+ *   #                                         <------->
+ *   sprintf("%20.8g", 123456789)  #=> "       1.2345679e+08"
+ *
+ *   # precision for `s' is
+ *   # maximum number of characters                    <------>
+ *   sprintf("%20.8s", "string test") #=> "            string t"
  *
- *      Field |  Conversion
- *      ------+--------------------------------------------------------------
- *        b   | Convert argument as a binary number.
- *        c   | Argument is the numeric code for a single character.
- *        d   | Convert argument as a decimal number.
- *        E   | Equivalent to `e', but uses an uppercase E to indicate
- *            | the exponent.
- *        e   | Convert floating point argument into exponential notation 
- *            | with one digit before the decimal point. The precision
- *            | determines the number of fractional digits (defaulting to six).
- *        f   | Convert floating point argument as [-]ddd.ddd, 
- *            |  where the precision determines the number of digits after
- *            | the decimal point.
- *        G   | Equivalent to `g', but use an uppercase `E' in exponent form.
- *        g   | Convert a floating point number using exponential form
- *            | if the exponent is less than -4 or greater than or
- *            | equal to the precision, or in d.dddd form otherwise.
- *        i   | Identical to `d'.
- *        o   | Convert argument as an octal number.
- *        p   | The valuing of argument.inspect.
- *        s   | Argument is a string to be substituted. If the format
- *            | sequence contains a precision, at most that many characters
- *            | will be copied.
- *        u   | Treat argument as an unsigned decimal number. Negative integers
- *            | are displayed as a 32 bit two's complement plus one for the
- *            | underlying architecture; that is, 2 ** 32 + n.  However, since
- *            | Ruby has no inherent limit on bits used to represent the
- *            | integer, this value is preceded by two dots (..) in order to
- *            | indicate a infinite number of leading sign bits.
- *        X   | Convert argument as a hexadecimal number using uppercase
- *            | letters. Negative numbers will be displayed with two
- *            | leading periods (representing an infinite string of
- *            | leading 'FF's.
- *        x   | Convert argument as a hexadecimal number.
- *            | Negative numbers will be displayed with two
- *            | leading periods (representing an infinite string of
- *            | leading 'ff's.
- *     
  *  Examples:
  *
  *     sprintf("%d %04x", 123, 123)               #=> "123 007b"
@@ -242,7 +383,7 @@ sign_bits(int base, const char *p)
  *     sprintf("%1$*2$s %2$d %1$s", "hello", 8)   #=> "   hello 8 hello"
  *     sprintf("%1$*2$s %2$d", "hello", -8)       #=> "hello    -8"
  *     sprintf("%+g:% g:%-g", 1.23, 1.23, 1.23)   #=> "+1.23: 1.23:1.23"
- *     sprintf("%u", -123)                        #=> "..4294967173"
+ *     sprintf("%u", -123)                        #=> "-123"
  */
 
 VALUE
@@ -294,7 +435,9 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
     blen = 0;
     bsiz = 120;
     result = rb_str_buf_new(bsiz);
+    rb_enc_copy(result, fmt);
     buf = RSTRING_PTR(result);
+    memset(buf, 0, bsiz);
 
     for (; p < end; p++) {
 	const char *t;
@@ -402,7 +545,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 	    p--;
 	  case '%':
 	    if (flags != FNONE) {
-		rb_raise(rb_eArgError, "illegal format character - %%");
+		rb_raise(rb_eArgError, "invalid format character - %%");
 	    }
 	    PUSH("%", 1);
 	    break;
@@ -457,24 +600,20 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		str = rb_obj_as_string(arg);
 		if (OBJ_TAINTED(str)) tainted = 1;
 		len = RSTRING_LEN(str);
-		enc = rb_enc_check(fmt, str);
+		enc = rb_enc_check(result, str);
 		if (flags&(FPREC|FWIDTH)) {
 		    slen = rb_enc_strlen(RSTRING_PTR(str),RSTRING_END(str),enc);
 		    if (slen < 0) {
 			rb_raise(rb_eArgError, "invalid mbstring sequence");
 		    }
-		}
-		if (flags&FPREC) {
-		    if (prec < slen) {
+		    if ((flags&FPREC) && (prec < slen)) {
 			char *p = rb_enc_nth(RSTRING_PTR(str), RSTRING_END(str),
 					     prec, enc);
 			slen = prec;
 			len = p - RSTRING_PTR(str);
 		    }
-		}
-		/* need to adjust multi-byte string pos */
-		if (flags&FWIDTH) {
-		    if (width > slen) {
+		    /* need to adjust multi-byte string pos */
+		    if ((flags&FWIDTH) && (width > slen)) {
 			width -= slen;
 			if (!(flags&FMINUS)) {
 			    CHECK(width);
@@ -491,10 +630,12 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 				buf[blen++] = ' ';
 			    }
 			}
+			rb_enc_associate(result, enc);
 			break;
 		    }
 		}
 		PUSH(RSTRING_PTR(str), len);
+		rb_enc_associate(result, enc);
 	    }
 	    break;
 
@@ -507,10 +648,11 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 	  case 'B':
 	  case 'u':
 	    {
+		volatile VALUE tmp1;
 		volatile VALUE val = GETARG();
-		char fbuf[32], nbuf[64], *s, *t;
+		char fbuf[32], nbuf[64], *s;
 		const char *prefix = 0;
-		int sign = 0;
+		int sign = 0, dots = 0;
 		char sc = 0;
 		long v = 0;
 		int base, bignum = 0;
@@ -542,19 +684,16 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		      case 'B':
 			prefix = "0B"; break;
 		    }
-		    if (prefix) {
-			width -= strlen(prefix);
-		    }
 		}
 
 	      bin_retry:
 		switch (TYPE(val)) {
 		  case T_FLOAT:
-		    if (FIXABLE((long)RFLOAT(val)->value)) {
-			val = LONG2FIX((long)RFLOAT(val)->value);
+		    if (FIXABLE(RFLOAT_VALUE(val))) {
+			val = LONG2FIX((long)RFLOAT_VALUE(val));
 			goto bin_retry;
 		    }
-		    val = rb_dbl2big(RFLOAT(val)->value);
+		    val = rb_dbl2big(RFLOAT_VALUE(val));
 		    if (FIXNUM_P(val)) goto bin_retry;
 		    bignum = 1;
 		    break;
@@ -611,19 +750,19 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 			}
 			sprintf(fbuf, "%%l%c", c);
 			sprintf(nbuf, fbuf, v);
+			s = nbuf;
 		    }
 		    else {
 			s = nbuf;
 			if (v < 0) {
-			    strcpy(s, "..");
-			    s += 2;
+			    dots = 1;
 			}
 			sprintf(fbuf, "%%l%c", *p == 'X' ? 'x' : *p);
-			sprintf(s, fbuf, v);
+			sprintf(++s, fbuf, v);
 			if (v < 0) {
 			    char d = 0;
 
-			    remove_sign_bits(s, base);
+			    s = remove_sign_bits(s, base);
 			    switch (base) {
 			      case 16:
 				d = 'f'; break;
@@ -631,12 +770,10 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 				d = '7'; break;
 			    }
 			    if (d && *s != d) {
-				memmove(s+1, s, strlen(s)+1);
-				*s = d;
+				*--s = d;
 			    }
 			}
 		    }
-		    s = nbuf;
 		}
 		else {
 		    if (sign) {
@@ -657,55 +794,72 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 			}
 		    }
 		    else {
-                        volatile VALUE tmp1;
-			if (!RBIGNUM(val)->sign) {
+			if (!RBIGNUM_SIGN(val)) {
 			    val = rb_big_clone(val);
 			    rb_big_2comp(val);
 			}
-			tmp1 = tmp = rb_big2str0(val, base, RBIGNUM(val)->sign);
+			tmp1 = tmp = rb_big2str0(val, base, RBIGNUM_SIGN(val));
 			s = RSTRING_PTR(tmp);
 			if (*s == '-') {
+			    dots = 1;
 			    if (base == 10) {
 				rb_warning("negative number for %%u specifier");
 			    }
-			    remove_sign_bits(++s, base);
-			    tmp = rb_str_new(0, 3+strlen(s));
-			    t = RSTRING_PTR(tmp);
-			    if (!(flags&(FPREC|FZERO))) {
-				strcpy(t, "..");
-				t += 2;
-			    }
+			    s = remove_sign_bits(++s, base);
 			    switch (base) {
 			      case 16:
-				if (s[0] != 'f') strcpy(t++, "f"); break;
+				if (s[0] != 'f') *--s = 'f'; break;
 			      case 8:
-				if (s[0] != '7') strcpy(t++, "7"); break;
+				if (s[0] != '7') *--s = '7'; break;
 			      case 2:
-				if (s[0] != '1') strcpy(t++, "1"); break;
+				if (s[0] != '1') *--s = '1'; break;
 			    }
-			    strcpy(t, s);
-			    s  = RSTRING_PTR(tmp);
 			}
 		    }
 		}
 
 		pos = -1;
 		len = strlen(s);
+		if (dots) {
+		    prec -= 2;
+		    width -= 2;
+		}
 
 		if (*p == 'X') {
 		    char *pp = s;
 		    int c;
-		    while (c = (int)*pp) {
+		    while ((c = (int)(unsigned char)*pp) != 0) {
 			*pp = rb_enc_toupper(c, enc);
 			pp++;
 		    }
 		}
-		if ((flags&(FZERO|FPREC)) == FZERO) {
+		if (prefix && !prefix[1]) { /* octal */
+		    if (dots) {
+			prefix = 0;
+		    }
+		    else if (len == 1 && *s == '0') {
+			len = 0;
+			if (flags & FPREC) prec--;
+		    }
+		    else if ((flags & FPREC) && (prec > len)) {
+			prefix = 0;
+		    }
+		}
+		else if (len == 1 && *s == '0') {
+		    prefix = 0;
+		}
+		if (prefix) {
+		    width -= strlen(prefix);
+		}
+		if ((flags & (FZERO|FMINUS|FPREC)) == FZERO) {
 		    prec = width;
 		    width = 0;
 		}
 		else {
-		    if (prec < len) prec = len;
+		    if (prec < len) {
+			if (!prefix && prec == 0 && len == 1 && *s == '0') len = 0;
+			prec = len;
+		    }
 		    width -= prec;
 		}
 		if (!(flags&FMINUS)) {
@@ -720,16 +874,17 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		    PUSH(prefix, plen);
 		}
 		CHECK(prec - len);
+		if (dots) PUSH("..", 2);
 		if (!bignum && v < 0) {
 		    char c = sign_bits(base, p);
 		    while (len < prec--) {
 			buf[blen++] = c;
 		    }
 		}
-		else {
+		else if ((flags & (FMINUS|FPREC)) != FMINUS) {
 		    char c;
 
-		    if (!sign && bignum && !RBIGNUM(val)->sign)
+		    if (!sign && bignum && !RBIGNUM_SIGN(val))
 			c = sign_bits(base, p);
 		    else
 			c = '0';
@@ -756,11 +911,11 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		int i, need = 6;
 		char fbuf[32];
 
-		fval = RFLOAT(rb_Float(val))->value;
+		fval = RFLOAT_VALUE(rb_Float(val));
 		if (isnan(fval) || isinf(fval)) {
 		    const char *expr;
 
-		    if  (isnan(fval)) {
+		    if (isnan(fval)) {
 			expr = "NaN";
 		    }
 		    else {
@@ -783,29 +938,13 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 			    blen++;
 			strncpy(&buf[blen], expr, strlen(expr));
 		    }
-		    else if (flags & FZERO) {
-			if (!isnan(fval) && fval < 0.0) {
-			    buf[blen++] = '-';
-			    need--;
-			}
-			else if (flags & FPLUS) {
-			    buf[blen++] = '+';
-			    need--;
-			}
-			else if (flags & FSPACE) {
-			    blen++;
-			    need--;
-			}
-			while (need-- - strlen(expr) > 0) {
-			    buf[blen++] = '0';
-			}
-			strncpy(&buf[blen], expr, strlen(expr));
-		    }
 		    else {
 			if (!isnan(fval) && fval < 0.0)
 			    buf[blen + need - strlen(expr) - 1] = '-';
 			else if (flags & FPLUS)
 			    buf[blen + need - strlen(expr) - 1] = '+';
+			else if ((flags & FSPACE) && need > width)
+			    blen++;
 			strncpy(&buf[blen + need - strlen(expr)], expr,
 				strlen(expr));
 		    }
@@ -836,7 +975,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
     }
 
   sprint_exit:
-    /* XXX - We cannot validiate the number of arguments if (digit)$ style used.
+    /* XXX - We cannot validate the number of arguments if (digit)$ style used.
      */
     if (posarg >= 0 && nextarg < argc) {
 	const char *mesg = "too many arguments for format string";
@@ -920,7 +1059,7 @@ ruby__sfvwrite(register rb_printf_buffer *fp, register struct __suio *uio)
 }
 
 VALUE
-rb_vsprintf(const char *fmt, va_list ap)
+rb_enc_vsprintf(rb_encoding *enc, const char *fmt, va_list ap)
 {
     rb_printf_buffer f;
     VALUE result;
@@ -929,6 +1068,7 @@ rb_vsprintf(const char *fmt, va_list ap)
     f._bf._size = 0;
     f._w = 120;
     result = rb_str_buf_new(f._w);
+    if (enc) rb_enc_associate(result, enc);
     f._bf._base = (unsigned char *)result;
     f._p = (unsigned char *)RSTRING_PTR(result);
     RBASIC(result)->klass = 0;
@@ -940,6 +1080,25 @@ rb_vsprintf(const char *fmt, va_list ap)
     return result;
 }
 
+VALUE
+rb_enc_sprintf(rb_encoding *enc, const char *format, ...)
+{
+    VALUE result;
+    va_list ap;
+
+    va_start(ap, format);
+    result = rb_enc_vsprintf(enc, format, ap);
+    va_end(ap);
+
+    return result;
+}
+
+VALUE
+rb_vsprintf(const char *fmt, va_list ap)
+{
+    return rb_enc_vsprintf(NULL, fmt, ap);
+}
+
 VALUE
 rb_sprintf(const char *format, ...)
 {
