@@ -1,5 +1,3 @@
-
-__END__
 require 'test/unit'
 require 'drb/drb'
 require 'drb/extservm'
@@ -24,7 +22,7 @@ class DRbService
   %w(ut_drb.rb ut_array.rb ut_port.rb ut_large.rb ut_safe1.rb ut_eval.rb).each do |nm|
     add_service_command(nm)
   end
-  @server = @@server = DRb::DRbServer.new(nil, @@manager, {})
+  @server = @@server = DRb::DRbServer.new('druby://localhost:0', @@manager, {})
   @@manager.uri = @@server.uri
   def self.manager
     @@manager
@@ -181,7 +179,7 @@ module DRbCore
   def test_07_public_private_protected_missing
     assert_nothing_raised() {
       begin
-	@there.method_missing(:eval)
+	@there.method_missing(:eval, 'nil')
       rescue NoMethodError
 	assert_match(/^private method \`eval\'/, $!.message)
       end
@@ -210,7 +208,7 @@ module DRbCore
 	assert_match(/^undefined method \`undefined_method_test\'/, $!.message)
       end
     }
-    assert_raises(SecurityError) do
+    assert_raises(DRb::DRbConnError) do
       @there.method_missing(:__send__, :to_s)
     end
     assert_equal(true, @there.missing)
@@ -307,18 +305,19 @@ module DRbAry
     assert_equal([1, 2, 'III', 'III', 4, 'five', 6], ary)
   end
 
-  def test_04_retry
-    retried = false
-    ary = []
-    @there.each do |x|
-      ary.push x
-      if x == 4 && !retried
-	retried = true
-	retry
-      end
-    end
-    assert_equal([1, 2, 'III', 4, 1, 2, 'III', 4, 'five', 6], ary)
-  end
+  # retry in block is not supported on ruby 1.9
+  #def test_04_retry
+  #  retried = false
+  #  ary = []
+  #  @there.each do |x|
+  #    ary.push x
+  #    if x == 4 && !retried
+  #	retried = true
+  #	retry
+  #    end
+  #  end
+  #  assert_equal([1, 2, 'III', 4, 1, 2, 'III', 4, 'five', 6], ary)
+  #end
 
   def test_05_break
     ary = []
