@@ -1,4 +1,4 @@
-# $Id: fileasserts.rb,v 1.4 2005/01/26 15:03:37 aamine Exp $
+# $Id: fileasserts.rb 13992 2007-11-21 07:40:36Z akr $
 
 module Test
   module Unit
@@ -17,7 +17,7 @@ module Test
         b = File.stat(to)
         assert_equal a.mode, b.mode, "mode #{a.mode} != #{b.mode}"
         #assert_equal a.atime, b.atime
-        assert_equal a.mtime, b.mtime, "mtime #{a.mtime} != #{b.mtime}"
+        assert_equal_timestamp a.mtime, b.mtime, "mtime #{a.mtime} != #{b.mtime}"
         assert_equal a.uid, b.uid, "uid #{a.uid} != #{b.uid}"
         assert_equal a.gid, b.gid, "gid #{a.gid} != #{b.gid}"
       end
@@ -62,6 +62,39 @@ module Test
         }
       end
 
+      def assert_equal_time(expected, actual, message=nil)
+        _wrap_assertion {
+	  expected_str = expected.to_s
+	  actual_str = actual.to_s
+	  if expected_str == actual_str
+	    expected_str << " (nsec=#{expected.nsec})"
+	    actual_str << " (nsec=#{actual.nsec})"
+	  end
+	  full_message = build_message(message, <<EOT)
+<#{expected_str}> expected but was
+<#{actual_str}>.
+EOT
+	  assert_block(full_message) { expected == actual }
+        }
+      end
+
+      def assert_equal_timestamp(expected, actual, message=nil)
+        _wrap_assertion {
+	  expected_str = expected.to_s
+	  actual_str = actual.to_s
+	  if expected_str == actual_str
+	    expected_str << " (nsec=#{expected.nsec})"
+	    actual_str << " (nsec=#{actual.nsec})"
+	  end
+	  full_message = build_message(message, <<EOT)
+<#{expected_str}> expected but was
+<#{actual_str}>.
+EOT
+          # subsecond timestamp is not portable. 
+	  assert_block(full_message) { expected.tv_sec == actual.tv_sec }
+        }
+      end
+
     end
   end
 end
