@@ -1,8 +1,7 @@
 #!/usr/bin/env ruby
-# -*- coding: utf-8 -*-
 
 require 'test/unit'
-require File.join(File.dirname(__FILE__), 'setup_variant')
+require 'json'
 
 class TC_JSONFixtures < Test::Unit::TestCase
   def setup
@@ -14,20 +13,15 @@ class TC_JSONFixtures < Test::Unit::TestCase
 
   def test_passing
     for name, source in @passed
-      begin
-        assert JSON.parse(source),
-          "Did not pass for fixture '#{name}': #{source.inspect}"
-      rescue => e
-        warn "\nCaught #{e.class}(#{e}) for fixture '#{name}': #{source.inspect}\n#{e.backtrace * "\n"}"
-        raise e
-      end
+      assert JSON.parse(source),
+        "Did not pass for fixture '#{name}'"
     end
   end
 
   def test_failing
     for name, source in @failed
       assert_raises(JSON::ParserError, JSON::NestingError,
-        "Did not fail for fixture '#{name}': #{source.inspect}") do
+        "Did not fail for fixture '#{name}'") do
         JSON.parse(source)
       end
     end
