@@ -5,6 +5,7 @@ require 'pathname'
 
 require 'fileutils'
 require 'tmpdir'
+require 'enumerator'
 
 class TestPathname < Test::Unit::TestCase
   def self.define_assertion(name, &block)
@@ -115,7 +116,7 @@ class TestPathname < Test::Unit::TestCase
 
   # has_trailing_separator?(path) -> bool
   def has_trailing_separator?(path)
-    Pathname.allocate.funcall(:has_trailing_separator?, path)
+    Pathname.allocate.__send__(:has_trailing_separator?, path)
   end
 
   defassert(:has_trailing_separator?, false, "/")
@@ -124,11 +125,11 @@ class TestPathname < Test::Unit::TestCase
   defassert(:has_trailing_separator?, true, "a/")
 
   def add_trailing_separator(path)
-    Pathname.allocate.funcall(:add_trailing_separator, path)
+    Pathname.allocate.__send__(:add_trailing_separator, path)
   end
 
   def del_trailing_separator(path)
-    Pathname.allocate.funcall(:del_trailing_separator, path)
+    Pathname.allocate.__send__(:del_trailing_separator, path)
   end
 
   defassert(:del_trailing_separator, "/", "/")
@@ -166,7 +167,10 @@ class TestPathname < Test::Unit::TestCase
 
   if DOSISH
     defassert(:del_trailing_separator, "a", "a\\")
-    defassert(:del_trailing_separator, "\225\\", "\225\\\\") # SJIS
+    require 'Win32API'
+    if Win32API.new('kernel32', 'GetACP', nil, 'L').call == 932
+      defassert(:del_trailing_separator, "\225\\", "\225\\\\") # SJIS
+    end
   end
 
   def plus(path1, path2) # -> path
@@ -276,16 +280,12 @@ class TestPathname < Test::Unit::TestCase
       return
     rescue TypeError
     end
-    dir = "#{Dir.tmpdir}/tst-pathname-#$$"
-    Dir.mkdir(dir)
-    begin
+    Dir.mktmpdir('rubytest-pathname') {|dir|
       File.symlink("not-exist-target", "#{dir}/not-exist")
       assert_raise(Errno::ENOENT) { realpath("#{dir}/not-exist") }
       File.symlink("loop", "#{dir}/loop")
       assert_raise(Errno::ELOOP) { realpath("#{dir}/loop") }
-    ensure
-      FileUtils.rmtree(dir)
-    end
+    }
   end
 
   def descend(path)
@@ -313,6 +313,10 @@ class TestPathname < Test::Unit::TestCase
     assert_equal(p1, p2)
   end
 
+  def test_initialize_nul
+    assert_raise(ArgumentError) { Pathname.new("a\0") }
+  end
+
   class AnotherStringLike # :nodoc:
     def initialize(s) @s = s end
     def to_str() @s end
@@ -374,6 +378,20 @@ class TestPathname < Test::Unit::TestCase
     assert_equal(nil, "a" <=> Pathname.new("a"))
   end
 
+  def pathsub(path, pat, repl) Pathname.new(path).sub(pat, repl).to_s end
+  defassert(:pathsub, "a.o", "a.c", /\.c\z/, ".o")
+
+  def pathsubext(path, repl) Pathname.new(path).sub_ext(repl).to_s end
+  defassert(:pathsubext, 'a.o', 'a.c', '.o')
+  defassert(:pathsubext, 'a.o', 'a.c++', '.o')
+  defassert(:pathsubext, 'a.png', 'a.gif', '.png')
+  defassert(:pathsubext, 'ruby.tar.bz2', 'ruby.tar.gz', '.bz2')
+  defassert(:pathsubext, 'd/a.o', 'd/a.c', '.o')
+  defassert(:pathsubext, 'foo', 'foo.exe', '')
+  defassert(:pathsubext, 'lex.yy.o', 'lex.yy.c', '.o')
+  defassert(:pathsubext, 'fooaa.o', 'fooaa', '.o')
+  defassert(:pathsubext, 'd.e/aa.o', 'd.e/aa', '.o')
+
   def root?(path)
     Pathname.new(path).root?
   end
@@ -464,4 +482,36 @@ class TestPathname < Test::Unit::TestCase
     Pathname.new("/usr/bin/ruby").each_filename {|f| result << f }
     assert_equal(%w[usr bin ruby], result)
   end
+
+  def test_kernel_pathname
+    assert_equal(Pathname.new("a"), Pathname("a"))
+  end
+
+  def test_file_basename
+    assert_equal("bar", File.basename(Pathname.new("foo/bar")))
+  end
+
+  def test_file_dirname
+    assert_equal("foo", File.dirname(Pathname.new("foo/bar")))
+  end
+
+  def test_file_split
+    assert_equal(["foo", "bar"], File.split(Pathname.new("foo/bar")))
+  end
+
+  def test_file_extname
+    assert_equal(".baz", File.extname(Pathname.new("bar.baz")))
+  end
+
+  def test_file_fnmatch
+    assert(File.fnmatch("*.*", Pathname.new("bar.baz")))
+  end
+
+  def test_file_join
+    assert_equal("foo/bar", File.join(Pathname.new("foo"), Pathname.new("bar")))
+    lambda {
+      $SAFE = 1
+      assert_equal("foo/bar", File.join(Pathname.new("foo"), Pathname.new("bar").taint))
+    }.call
+  end
 end
