@@ -6,17 +6,17 @@ require_relative 'envutil'
 
 class TestArgf < Test::Unit::TestCase
   def setup
-    @t1 = Tempfile.new("argf-foo")
+    @t1 = Tempfile.new("foo")
     @t1.binmode
     @t1.puts "1"
     @t1.puts "2"
     @t1.close
-    @t2 = Tempfile.new("argf-bar")
+    @t2 = Tempfile.new("bar")
     @t2.binmode
     @t2.puts "3"
     @t2.puts "4"
     @t2.close
-    @t3 = Tempfile.new("argf-baz")
+    @t3 = Tempfile.new("baz")
     @t3.binmode
     @t3.puts "5"
     @t3.puts "6"
@@ -28,12 +28,11 @@ class TestArgf < Test::Unit::TestCase
     @tmps.each {|t|
       bak = t.path + ".bak"
       File.unlink bak if File.file? bak
-      t.close(true)
     }
   end
 
   def make_tempfile
-    t = Tempfile.new("argf-qux")
+    t = Tempfile.new("foo")
     t.puts "foo"
     t.puts "bar"
     t.puts "baz"
@@ -55,21 +54,8 @@ class TestArgf < Test::Unit::TestCase
     /cygwin|mswin|mingw|bccwin/ =~ RUBY_PLATFORM
   end
 
-  def assert_src_expected(line, src, args = nil)
-    args ||= [@t1.path, @t2.path, @t3.path]
-    expected = src.split(/^/)
-    ruby('-e', src, *args) do |f|
-      expected.each_with_index do |e, i|
-        /#=> *(.*)/ =~ e or next
-        a = f.gets
-        assert_not_nil(a, "[ruby-dev:34445]: remained")
-        assert_equal($1, a.chomp, "[ruby-dev:34445]: line #{line+i}")
-      end
-    end
-  end
-
   def test_argf
-    assert_src_expected(__LINE__+1, <<-'SRC')
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       a = ARGF
       b = a.dup
       p [a.gets.chomp, a.lineno, b.gets.chomp, b.lineno] #=> ["1", 1, "1", 1]
@@ -86,10 +72,23 @@ class TestArgf < Test::Unit::TestCase
       p [a.gets.chomp, a.lineno, b.gets.chomp, b.lineno] #=> ["5", 5, "5", 8]
       p [a.gets.chomp, a.lineno, b.gets.chomp, b.lineno] #=> ["6", 6, "6", 9]
     SRC
+      a = f.read.split("\n")
+      assert_equal('["1", 1, "1", 1]', a.shift)
+      assert_equal('["2", 2, "2", 2]', a.shift)
+      assert_equal('["1", 1, "1", 3]', a.shift)
+      assert_equal('["2", 2, "2", 4]', a.shift)
+      assert_equal('["3", 3, "3", 5]', a.shift)
+      assert_equal('["4", 4, "4", 6]', a.shift)
+      assert_equal('["5", 5, "5", 7]', a.shift)
+      assert_equal('["5", 5, "5", 8]', a.shift)
+      assert_equal('["6", 6, "6", 9]', a.shift)
+
+      # is this test OK? [ruby-dev:34445]
+    end
   end
 
   def test_lineno
-    assert_src_expected(__LINE__+1, <<-'SRC')
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       a = ARGF
       a.gets; p $.  #=> 1
       a.gets; p $.  #=> 2
@@ -106,10 +105,12 @@ class TestArgf < Test::Unit::TestCase
       a.gets; p $.  #=> 2001
       a.gets; p $.  #=> 2001
     SRC
+      assert_equal("1,2,3,3,3,4,4,3,1000,1001,1002,2001,2001", f.read.chomp.gsub("\n", ","))
+    end
   end
 
   def test_lineno2
-    assert_src_expected(__LINE__+1, <<-'SRC')
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       a = ARGF.dup
       a.gets; p $.  #=> 1
       a.gets; p $.  #=> 2
@@ -122,98 +123,100 @@ class TestArgf < Test::Unit::TestCase
       a.gets; p $.  #=> 2
       a.gets; p $.  #=> 2
       $. = 2000
-      a.gets; p $.  #=> 2000
+      a.gets; p $.  #=> 2001
       a.gets; p $.  #=> 2000
     SRC
-  end
-
-  def test_lineno3
-    assert_in_out_err(["-", @t1.path, @t2.path], <<-INPUT, %w"1 1 1 2 2 2 3 3 1 4 4 2", [], "[ruby-core:25205]")
-      ARGF.each do |line|
-        puts [$., ARGF.lineno, ARGF.file.lineno]
-      end
-    INPUT
+      assert_equal("1,2,1,1,1,2,1,1,2,2,2000,2000", f.read.chomp.gsub("\n", ","))
+    end
   end
 
   def test_inplace
-    assert_in_out_err(["-", @t1.path, @t2.path, @t3.path], <<-INPUT, [], [])
-      ARGF.inplace_mode = '.bak'
-      while line = ARGF.gets
-        puts line.chomp + '.new'
-      end
-    INPUT
-    assert_equal("1.new\n2.new\n", File.read(@t1.path))
-    assert_equal("3.new\n4.new\n", File.read(@t2.path))
-    assert_equal("5.new\n6.new\n", File.read(@t3.path))
-    assert_equal("1\n2\n", File.read(@t1.path + ".bak"))
-    assert_equal("3\n4\n", File.read(@t2.path + ".bak"))
-    assert_equal("5\n6\n", File.read(@t3.path + ".bak"))
+    EnvUtil.rubyexec("-", @t1.path, @t2.path, @t3.path) do |w, r, e|
+      w.puts "ARGF.inplace_mode = '.bak'"
+      w.puts "while line = ARGF.gets"
+      w.puts "  puts line.chomp + '.new'"
+      w.puts "end"
+      w.close
+      assert_equal("", e.read)
+      assert_equal("", r.read)
+      assert_equal("1.new\n2.new\n", File.read(@t1.path))
+      assert_equal("3.new\n4.new\n", File.read(@t2.path))
+      assert_equal("5.new\n6.new\n", File.read(@t3.path))
+      assert_equal("1\n2\n", File.read(@t1.path + ".bak"))
+      assert_equal("3\n4\n", File.read(@t2.path + ".bak"))
+      assert_equal("5\n6\n", File.read(@t3.path + ".bak"))
+    end
   end
 
   def test_inplace2
-    assert_in_out_err(["-", @t1.path, @t2.path, @t3.path], <<-INPUT, [], [])
-      ARGF.inplace_mode = '.bak'
-      puts ARGF.gets.chomp + '.new'
-      puts ARGF.gets.chomp + '.new'
-      p ARGF.inplace_mode
-      ARGF.inplace_mode = nil
-      puts ARGF.gets.chomp + '.new'
-      puts ARGF.gets.chomp + '.new'
-      p ARGF.inplace_mode
-      ARGF.inplace_mode = '.bak'
-      puts ARGF.gets.chomp + '.new'
-      p ARGF.inplace_mode
-      ARGF.inplace_mode = nil
-      puts ARGF.gets.chomp + '.new'
-    INPUT
-    assert_equal("1.new\n2.new\n\".bak\"\n3.new\n4.new\nnil\n", File.read(@t1.path))
-    assert_equal("3\n4\n", File.read(@t2.path))
-    assert_equal("5.new\n\".bak\"\n6.new\n", File.read(@t3.path))
-    assert_equal("1\n2\n", File.read(@t1.path + ".bak"))
-    assert_equal(false, File.file?(@t2.path + ".bak"))
-    assert_equal("5\n6\n", File.read(@t3.path + ".bak"))
+    EnvUtil.rubyexec("-", @t1.path, @t2.path, @t3.path) do |w, r, e|
+      w.puts "ARGF.inplace_mode = '.bak'"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "p ARGF.inplace_mode"
+      w.puts "ARGF.inplace_mode = nil"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "p ARGF.inplace_mode"
+      w.puts "ARGF.inplace_mode = '.bak'"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "p ARGF.inplace_mode"
+      w.puts "ARGF.inplace_mode = nil"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.close
+      assert_equal("", e.read)
+      assert_equal("", r.read)
+      assert_equal("1.new\n2.new\n\".bak\"\n3.new\n4.new\nnil\n", File.read(@t1.path))
+      assert_equal("3\n4\n", File.read(@t2.path))
+      assert_equal("5.new\n\".bak\"\n6.new\n", File.read(@t3.path))
+      assert_equal("1\n2\n", File.read(@t1.path + ".bak"))
+      assert_equal(false, File.file?(@t2.path + ".bak"))
+      assert_equal("5\n6\n", File.read(@t3.path + ".bak"))
+    end
   end
 
   def test_inplace3
-    assert_in_out_err(["-i.bak", "-", @t1.path, @t2.path, @t3.path], <<-INPUT, [], [])
-      puts ARGF.gets.chomp + '.new'
-      puts ARGF.gets.chomp + '.new'
-      p $-i
-      $-i = nil
-      puts ARGF.gets.chomp + '.new'
-      puts ARGF.gets.chomp + '.new'
-      p $-i
-      $-i = '.bak'
-      puts ARGF.gets.chomp + '.new'
-      p $-i
-      $-i = nil
-      puts ARGF.gets.chomp + '.new'
-    INPUT
-    assert_equal("1.new\n2.new\n\".bak\"\n3.new\n4.new\nnil\n", File.read(@t1.path))
-    assert_equal("3\n4\n", File.read(@t2.path))
-    assert_equal("5.new\n\".bak\"\n6.new\n", File.read(@t3.path))
-    assert_equal("1\n2\n", File.read(@t1.path + ".bak"))
-    assert_equal(false, File.file?(@t2.path + ".bak"))
-    assert_equal("5\n6\n", File.read(@t3.path + ".bak"))
+    EnvUtil.rubyexec("-i.bak", "-", @t1.path, @t2.path, @t3.path) do |w, r, e|
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "p $-i"
+      w.puts "$-i = nil"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "p $-i"
+      w.puts "$-i = '.bak'"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.puts "p $-i"
+      w.puts "$-i = nil"
+      w.puts "puts ARGF.gets.chomp + '.new'"
+      w.close
+      assert_equal("", e.read)
+      assert_equal("", r.read)
+      assert_equal("1.new\n2.new\n\".bak\"\n3.new\n4.new\nnil\n", File.read(@t1.path))
+      assert_equal("3\n4\n", File.read(@t2.path))
+      assert_equal("5.new\n\".bak\"\n6.new\n", File.read(@t3.path))
+      assert_equal("1\n2\n", File.read(@t1.path + ".bak"))
+      assert_equal(false, File.file?(@t2.path + ".bak"))
+      assert_equal("5\n6\n", File.read(@t3.path + ".bak"))
+    end
   end
 
   def test_inplace_rename_impossible
     t = make_tempfile
 
-    assert_in_out_err(["-", t.path], <<-INPUT) do |r, e|
-      ARGF.inplace_mode = '/\\\\'
-      while line = ARGF.gets
-        puts line.chomp + '.new'
-      end
-    INPUT
+    EnvUtil.rubyexec("-", t.path) do |w, r, e|
+      w.puts "ARGF.inplace_mode = '/\\\\'"
+      w.puts "while line = ARGF.gets"
+      w.puts "  puts line.chomp + '.new'"
+      w.puts "end"
+      w.close
       if no_safe_rename
-        assert_equal([], e)
-        assert_equal([], r)
+        assert_equal("", e.read)
+        assert_equal("", r.read)
         assert_equal("foo.new\nbar.new\nbaz.new\n", File.read(t.path))
-        File.unlink(t.path + ".~~~") rescue nil
       else
-        assert_match(/Can't rename .* to .*: .*. skipping file/, e.first) #'
-        assert_equal([], r)
+        assert_match(/Can't rename .* to .*: .*. skipping file/, e.read) #'
+        assert_equal("", r.read)
         assert_equal("foo\nbar\nbaz\n", File.read(t.path))
       end
     end
@@ -222,17 +225,17 @@ class TestArgf < Test::Unit::TestCase
   def test_inplace_no_backup
     t = make_tempfile
 
-    assert_in_out_err(["-", t.path], <<-INPUT) do |r, e|
-      ARGF.inplace_mode = ''
-      while line = ARGF.gets
-        puts line.chomp + '.new'
-      end
-    INPUT
+    EnvUtil.rubyexec("-", t.path) do |w, r, e|
+      w.puts "ARGF.inplace_mode = ''"
+      w.puts "while line = ARGF.gets"
+      w.puts "  puts line.chomp + '.new'"
+      w.puts "end"
+      w.close
       if no_safe_rename
-        assert_match(/Can't do inplace edit without backup/, e.join) #'
+        assert_match(/Can't do inplace edit without backup/, e.read) #'
       else
-        assert_equal([], e)
-        assert_equal([], r)
+        assert_equal("", e.read)
+        assert_equal("", r.read)
         assert_equal("foo.new\nbar.new\nbaz.new\n", File.read(t.path))
       end
     end
@@ -241,37 +244,46 @@ class TestArgf < Test::Unit::TestCase
   def test_inplace_dup
     t = make_tempfile
 
-    assert_in_out_err(["-", t.path], <<-INPUT, [], [])
-      ARGF.inplace_mode = '.bak'
-      f = ARGF.dup
-      while line = f.gets
-        puts line.chomp + '.new'
-      end
-    INPUT
-    assert_equal("foo.new\nbar.new\nbaz.new\n", File.read(t.path))
+    EnvUtil.rubyexec("-", t.path) do |w, r, e|
+      w.puts "ARGF.inplace_mode = '.bak'"
+      w.puts "f = ARGF.dup"
+      w.puts "while line = f.gets"
+      w.puts "  puts line.chomp + '.new'"
+      w.puts "end"
+      w.close
+      assert_equal("", e.read)
+      assert_equal("", r.read)
+      assert_equal("foo.new\nbar.new\nbaz.new\n", File.read(t.path))
+    end
   end
 
   def test_inplace_stdin
     t = make_tempfile
 
-    assert_in_out_err(["-", "-"], <<-INPUT, [], /Can't do inplace edit for stdio; skipping/)
-      ARGF.inplace_mode = '.bak'
-      f = ARGF.dup
-      while line = f.gets
-        puts line.chomp + '.new'
-      end
-    INPUT
+    EnvUtil.rubyexec("-", "-") do |w, r, e|
+      w.puts "ARGF.inplace_mode = '.bak'"
+      w.puts "f = ARGF.dup"
+      w.puts "while line = f.gets"
+      w.puts "  puts line.chomp + '.new'"
+      w.puts "end"
+      w.close
+      assert_match("Can't do inplace edit for stdio; skipping", e.read)
+      assert_equal("", r.read)
+    end
   end
 
   def test_inplace_stdin2
     t = make_tempfile
 
-    assert_in_out_err(["-"], <<-INPUT, [], /Can't do inplace edit for stdio/)
-      ARGF.inplace_mode = '.bak'
-      while line = ARGF.gets
-        puts line.chomp + '.new'
-      end
-    INPUT
+    EnvUtil.rubyexec("-") do |w, r, e|
+      w.puts "ARGF.inplace_mode = '.bak'"
+      w.puts "while line = ARGF.gets"
+      w.puts "  puts line.chomp + '.new'"
+      w.puts "end"
+      w.close
+      assert_match("Can't do inplace edit for stdio", e.read)
+      assert_equal("", r.read)
+    end
   end
 
   def test_encoding
@@ -309,54 +321,73 @@ class TestArgf < Test::Unit::TestCase
   end
 
   def test_seek
-    assert_src_expected(__LINE__+1, <<-'SRC')
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       ARGF.seek(4)
-      p ARGF.gets #=> "3\n"
+      p ARGF.gets #=> "3"
       ARGF.seek(0, IO::SEEK_END)
-      p ARGF.gets #=> "5\n"
+      p ARGF.gets #=> "5"
       ARGF.seek(4)
       p ARGF.gets #=> nil
       begin
         ARGF.seek(0)
       rescue
-        puts "end" #=> end
+        puts "end"
       end
     SRC
+      a = f.read.split("\n")
+      assert_equal('"3\n"', a.shift)
+      assert_equal('"5\n"', a.shift)
+      assert_equal('nil', a.shift)
+      assert_equal('end', a.shift)
+    end
   end
 
   def test_set_pos
-    assert_src_expected(__LINE__+1, <<-'SRC')
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       ARGF.pos = 4
-      p ARGF.gets #=> "3\n"
+      p ARGF.gets #=> "3"
       ARGF.pos = 4
-      p ARGF.gets #=> "5\n"
+      p ARGF.gets #=> "5"
       ARGF.pos = 4
       p ARGF.gets #=> nil
       begin
         ARGF.pos = 4
       rescue
-        puts "end" #=> end
+        puts "end"
       end
     SRC
+      a = f.read.split("\n")
+      assert_equal('"3\n"', a.shift)
+      assert_equal('"5\n"', a.shift)
+      assert_equal('nil', a.shift)
+      assert_equal('end', a.shift)
+    end
   end
 
   def test_rewind
-    assert_src_expected(__LINE__+1, <<-'SRC')
+    ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       ARGF.pos = 4
       ARGF.rewind
-      p ARGF.gets #=> "1\n"
+      p ARGF.gets #=> "1"
       ARGF.pos = 4
-      p ARGF.gets #=> "3\n"
+      p ARGF.gets #=> "3"
       ARGF.pos = 4
-      p ARGF.gets #=> "5\n"
+      p ARGF.gets #=> "5"
       ARGF.pos = 4
       p ARGF.gets #=> nil
       begin
         ARGF.rewind
       rescue
-        puts "end" #=> end
+        puts "end"
       end
     SRC
+      a = f.read.split("\n")
+      assert_equal('"1\n"', a.shift)
+      assert_equal('"3\n"', a.shift)
+      assert_equal('"5\n"', a.shift)
+      assert_equal('nil', a.shift)
+      assert_equal('end', a.shift)
+    end
   end
 
   def test_fileno
@@ -419,22 +450,10 @@ class TestArgf < Test::Unit::TestCase
       end
     SRC
       a = f.read.split("\n")
-      (%w(false) + (%w(false true) * 3) + %w(end)).each do |x|
+      ((%w(true false) * 4).take(7) + %w(end)).each do |x|
         assert_equal(x, a.shift)
       end
     end
-
-    t1 = Tempfile.new("argf-foo")
-    t1.binmode
-    t1.puts "foo"
-    t1.close
-    t2 = Tempfile.new("argf-bar")
-    t2.binmode
-    t2.puts "bar"
-    t2.close
-    ruby('-e', 'STDERR.reopen(STDOUT); ARGF.gets; ARGF.skip; p ARGF.eof?', t1.path, t2.path) do |f|
-      assert_equal(%w(false), f.read.split(/\n/))
-    end
   end
 
   def test_read
@@ -488,11 +507,9 @@ class TestArgf < Test::Unit::TestCase
           t = ""; ARGF.readpartial(1, t); s << t
         end
       rescue EOFError
-        $stdout.binmode
         puts s
       end
     SRC
-      f.binmode
       f.puts("foo")
       f.puts("bar")
       f.puts("baz")
@@ -565,11 +582,6 @@ class TestArgf < Test::Unit::TestCase
     end
   end
 
-  def test_each_line_paragraph
-    assert_in_out_err(['-e', 'ARGF.each_line("") {|para| p para}'], "a\n\nb\n",
-                      ["\"a\\n\\n\"", "\"b\\n\""], [])
-  end
-
   def test_each_byte
     ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
       s = []
@@ -648,31 +660,24 @@ class TestArgf < Test::Unit::TestCase
   end
 
   def test_binmode
-    bug5268 = '[ruby-core:39234]'
-    open(@t3.path, "wb") {|f| f.write "5\r\n6\r\n"}
     ruby('-e', "ARGF.binmode; STDOUT.binmode; puts ARGF.read", @t1.path, @t2.path, @t3.path) do |f|
       f.binmode
-      assert_equal("1\n2\n3\n4\n5\r\n6\r\n", f.read, bug5268)
+      assert_equal("1\n2\n3\n4\n5\n6\n", f.read)
     end
   end
 
-  def test_textmode
-    bug5268 = '[ruby-core:39234]'
-    open(@t3.path, "wb") {|f| f.write "5\r\n6\r\n"}
-    ruby('-e', "STDOUT.binmode; puts ARGF.read", @t1.path, @t2.path, @t3.path) do |f|
-      f.binmode
-      assert_equal("1\n2\n3\n4\n5\n6\n", f.read, bug5268)
-    end
-  end unless IO::BINARY.zero?
-
   def test_skip
     ruby('-e', <<-SRC, @t1.path, @t2.path, @t3.path) do |f|
-      ARGF.skip
+      begin
+        ARGF.skip
+      rescue
+        puts "cannot skip" # ???
+      end
       puts ARGF.gets
       ARGF.skip
       puts ARGF.read
     SRC
-      assert_equal("1\n3\n4\n5\n6\n", f.read)
+      assert_equal("cannot skip\n1\n3\n4\n5\n6\n", f.read)
     end
   end
 
@@ -706,46 +711,4 @@ class TestArgf < Test::Unit::TestCase
       assert_equal([@t1.path, @t2.path, @t3.path].inspect, f.gets.chomp)
     end
   end
-
-  def test_readlines_limit_0
-    bug4024 = '[ruby-dev:42538]'
-    t = make_tempfile
-    argf = ARGF.class.new(t.path)
-    begin
-      assert_raise(ArgumentError, bug4024) do
-        argf.readlines(0)
-      end
-    ensure
-      argf.close
-    end
-  end
-
-  def test_each_line_limit_0
-    bug4024 = '[ruby-dev:42538]'
-    t = make_tempfile
-    argf = ARGF.class.new(t.path)
-    begin
-      assert_raise(ArgumentError, bug4024) do
-        argf.each_line(0).next
-      end
-    ensure
-      argf.close
-    end
-  end
-
-  def test_unreadable
-    bug4274 = '[ruby-core:34446]'
-    paths = (1..2).map do
-      t = Tempfile.new("bug4274-")
-      path = t.path
-      t.close!
-      path
-    end
-    argf = ARGF.class.new(*paths)
-    paths.each do |path|
-      e = assert_raise(Errno::ENOENT) {argf.gets}
-      assert_match(/- #{Regexp.quote(path)}\z/, e.message)
-    end
-    assert_nil(argf.gets, bug4274)
-  end
 end
