@@ -1,6 +1,16 @@
 require 'test/unit'
 
 class TestArray < Test::Unit::TestCase
+  def setup
+    @verbose = $VERBOSE
+    $VERBOSE = nil
+    @cls = Array
+  end
+
+  def teardown
+    $VERBOSE = @verbose
+  end
+
   def test_0_literal
     assert_equal([1, 2, 3, 4], [1, 2] + [3, 4])
     assert_equal([1, 2, 1, 2], [1, 2] * 2)
@@ -64,7 +74,7 @@ class TestArray < Test::Unit::TestCase
   end
 
   def test_split_0
-    x = "The Boassert of Mormon"
+    x = "The Book of Mormon"
     assert_equal(x.reverse, x.split(//).reverse!.join)
     assert_equal(x.reverse, x.reverse!)
     assert_equal("g:n:i:r:t:s: :e:t:y:b: :1", "1 byte string".split(//).reverse.join(":"))
@@ -176,10 +186,6 @@ class TestArray < Test::Unit::TestCase
 
   # From rubicon
 
-  def setup
-    @cls = Array
-  end
-
   def test_00_new
     a = @cls.new()
     assert_instance_of(@cls, a)
@@ -531,6 +537,15 @@ class TestArray < Test::Unit::TestCase
     assert_equal([1, 2, 3, 1, 2, 3], a)
   end
 
+  def test_count
+    a = @cls[1, 2, 3, 1, 2]
+    assert_equal(5, a.count)
+    assert_equal(2, a.count(1))
+    assert_equal(3, a.count {|x| x % 2 == 1 })
+    assert_equal(2, a.count(1) {|x| x % 2 == 1 })
+    assert_raise(ArgumentError) { a.count(0, 1) }
+  end
+
   def test_delete
     a = @cls[*('cab'..'cat').to_a]
     assert_equal('cap', a.delete('cap'))
@@ -712,6 +727,20 @@ class TestArray < Test::Unit::TestCase
                  @cls[@cls[@cls[@cls[],@cls[]],@cls[@cls[]],@cls[]],@cls[@cls[@cls[]]]].flatten)
   end
 
+  def test_flatten_with_callcc
+    respond_to?(:callcc, true) or require 'continuation'
+    o = Object.new
+    def o.to_ary() callcc {|k| @cont = k; [1,2,3]} end
+    begin
+      assert_equal([10, 20, 1, 2, 3, 30, 1, 2, 3, 40], [10, 20, o, 30, o, 40].flatten)
+    rescue => e
+    else
+      o.instance_eval {@cont}.call
+    end
+    assert_instance_of(RuntimeError, e, '[ruby-dev:34798]')
+    assert_match(/reentered/, e.message, '[ruby-dev:34798]')
+  end
+
   def test_hash
     a1 = @cls[ 'cat', 'dog' ]
     a2 = @cls[ 'cat', 'dog' ]
@@ -799,14 +828,6 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[], a)
   end
 
-  def test_nitems
-    assert_equal(0, @cls[].nitems)
-    assert_equal(1, @cls[1].nitems)
-    assert_equal(1, @cls[1, nil].nitems)
-    assert_equal(1, @cls[nil, 1].nitems)
-    assert_equal(3, @cls[1, nil, nil, 2, nil, 3, nil].nitems)
-  end
-
   def test_pack
     a = @cls[*%w( cat wombat x yy)]
     assert_equal("catwomx  yy ", a.pack("A3A3A3A3"))
@@ -1119,6 +1140,35 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[], @cls[].sort!)
   end
 
+  def test_sort_with_callcc
+    respond_to?(:callcc, true) or require 'continuation'
+    n = 1000
+    cont = nil
+    ary = (1..100).to_a
+    begin
+      ary.sort! {|a,b|
+        callcc {|k| cont = k} unless cont
+        assert_equal(100, ary.size, '[ruby-core:16679]')
+        a <=> b
+      }
+    rescue => e
+    end
+    n -= 1
+    cont.call if 0 < n
+    assert_instance_of(RuntimeError, e, '[ruby-core:16679]')
+    assert_match(/reentered/, e.message, '[ruby-core:16679]')
+  end
+
+  def test_sort_with_replace
+    xary = (1..100).to_a
+    100.times do
+      ary = (1..100).to_a
+      ary.sort! {|a,b| ary.replace(xary); a <=> b}
+      GC.start
+      assert_equal(xary, ary, '[ruby-dev:34732]')
+    end
+  end
+
   def test_to_a
     a = @cls[ 1, 2, 3 ]
     a_id = a.__id__
@@ -1229,4 +1279,295 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[].permutation(0).to_a, @cls[[]])
 
   end
+
+  def test_take
+    assert_equal([1,2,3], [1,2,3,4,5,0].take(3))
+    assert_raise(ArgumentError, '[ruby-dev:34123]') { [1,2].take(-1) }
+    assert_equal([1,2], [1,2].take(1000000000), '[ruby-dev:34123]')
+  end
+
+  def test_take_while
+    assert_equal([1,2], [1,2,3,4,5,0].take_while {|i| i < 3 })
+  end
+
+  def test_drop
+    assert_equal([4,5,0], [1,2,3,4,5,0].drop(3))
+    assert_raise(ArgumentError, '[ruby-dev:34123]') { [1,2].drop(-1) }
+    assert_equal([], [1,2].drop(1000000000), '[ruby-dev:34123]')
+  end
+
+  def test_drop_while
+    assert_equal([3,4,5,0], [1,2,3,4,5,0].drop_while {|i| i < 3 })
+  end
+
+  def test_modify_check
+    a = []
+    a.freeze
+    assert_raise(RuntimeError) { a.shift }
+    a = [1, 2]
+    assert_raise(SecurityError) do
+      Thread.new do
+        $SAFE = 4
+       a.shift
+      end.value
+    end
+  end
+
+  LONGP = [127, 63, 31, 15, 7].map {|x| 2**x-1 }.find do |x|
+    begin
+      [].first(x)
+    rescue ArgumentError
+      true
+    rescue RangeError
+      false
+    end
+  end
+
+  def test_ary_new
+    assert_raise(ArgumentError) { [].to_enum.first(-1) }
+    assert_raise(ArgumentError) { [].to_enum.first(LONGP) }
+  end
+
+  def test_try_convert
+    assert_equal([1], Array.try_convert([1]))
+    assert_equal(nil, Array.try_convert("1"))
+  end
+
+  def test_initialize
+    assert_nothing_raised { [].instance_eval { initialize } }
+    assert_nothing_raised { Array.new { } }
+    assert_equal([1, 2, 3], Array.new([1, 2, 3]))
+    assert_raise(ArgumentError) { Array.new(-1, 1) }
+    assert_raise(ArgumentError) { Array.new(LONGP, 1) }
+    assert_equal([1, 1, 1], Array.new(3, 1))
+    assert_equal([1, 1, 1], Array.new(3) { 1 })
+    assert_equal([1, 1, 1], Array.new(3, 1) { 1 })
+  end
+
+  def test_aset
+    assert_raise(IndexError) { [0][-2] = 1 }
+    assert_raise(ArgumentError) { [0][LONGP] = 2 }
+    assert_raise(ArgumentError) { [0][(LONGP + 1) / 2 - 1] = 2 }
+    a = [0]
+    a[2] = 4
+    assert_equal([0, nil, 4], a)
+    assert_raise(ArgumentError) { [0][0, 0, 0] = 0 }
+  end
+
+  def test_first2
+    assert_equal([0], [0].first(2))
+    assert_raise(ArgumentError) { [0].first(-1) }
+  end
+
+  def test_shift2
+    assert_equal(0, ([0] * 16).shift)
+    # check
+    a = [0, 1, 2]
+    a[3] = 3
+    a.shift(2)
+    assert_equal([2, 3], a)
+  end
+
+  def test_unshift2
+    Struct.new(:a, :b, :c)
+  end
+
+  def test_aref
+    assert_raise(ArgumentError) { [][0, 0, 0] }
+  end
+
+  def test_fetch
+    assert_equal(1, [].fetch(0, 0) { 1 })
+    assert_equal(1, [0, 1].fetch(-1))
+    assert_raise(IndexError) { [0, 1].fetch(2) }
+    assert_raise(IndexError) { [0, 1].fetch(-3) }
+    assert_equal(2, [0, 1].fetch(2, 2))
+  end
+
+  def test_index2
+    a = [0, 1, 2]
+    assert_equal(a, a.index.to_a)
+    assert_equal(1, a.index {|x| x == 1 })
+  end
+
+  def test_rindex2
+    a = [0, 1, 2]
+    assert_equal([2, 1, 0], a.rindex.to_a)
+    assert_equal(1, a.rindex {|x| x == 1 })
+
+    a = [0, 1]
+    e = a.rindex
+    assert_equal(1, e.next)
+    a.clear
+    assert_raise(StopIteration) { e.next }
+
+    o = Object.new
+    class << o; self; end.class_eval do
+      define_method(:==) {|x| a.clear; false }
+    end
+    a = [nil, o]
+    assert_equal(nil, a.rindex(0))
+  end
+
+  def test_ary_to_ary
+    o = Object.new
+    def o.to_ary; [1, 2, 3]; end
+    a, b, c = o
+    assert_equal([1, 2, 3], [a, b, c])
+  end
+
+  def test_splice
+    a = [0]
+    assert_raise(IndexError) { a[-2, 0] = nil }
+  end
+
+  def test_insert
+    a = [0]
+    assert_equal([0], a.insert(1))
+    assert_equal([0, 1], a.insert(1, 1))
+    assert_raise(ArgumentError) { a.insert }
+    assert_equal([0, 1, 2], a.insert(-1, 2))
+    assert_equal([0, 1, 3, 2], a.insert(-2, 3))
+  end
+
+  def test_join2
+    a = []
+    a << a
+    assert_equal("[...]", a.join)
+  end
+
+  def test_to_a2
+    klass = Class.new(Array)
+    a = klass.new.to_a
+    assert_equal([], a)
+    assert_equal(Array, a.class)
+  end
+
+  def test_values_at2
+    a = [0, 1, 2, 3, 4, 5]
+    assert_equal([1, 2, 3], a.values_at(1..3))
+    assert_equal([], a.values_at(7..8))
+    assert_equal([nil], a.values_at(2**31-1))
+  end
+
+  def test_select
+    assert_equal([0, 2], [0, 1, 2, 3].select {|x| x % 2 == 0 })
+  end
+
+  def test_delete2
+    a = [0] * 1024 + [1] + [0] * 1024
+    a.delete(0)
+    assert_equal([1], a)
+  end
+
+  def test_reject
+    assert_equal([1, 3], [0, 1, 2, 3].reject {|x| x % 2 == 0 })
+  end
+
+  def test_zip
+    assert_equal([[1, :a, "a"], [2, :b, "b"], [3, nil, "c"]],
+      [1, 2, 3].zip([:a, :b], ["a", "b", "c", "d"]))
+    a = []
+    [1, 2, 3].zip([:a, :b], ["a", "b", "c", "d"]) {|x| a << x }
+    assert_equal([[1, :a, "a"], [2, :b, "b"], [3, nil, "c"]], a)
+  end
+
+  def test_transpose
+    assert_equal([[1, :a], [2, :b], [3, :c]],
+      [[1, 2, 3], [:a, :b, :c]].transpose)
+    assert_raise(IndexError) { [[1, 2, 3], [:a, :b]].transpose }
+  end
+
+  def test_clear2
+    assert_equal([], ([0] * 1024).clear)
+  end
+
+  def test_fill2
+    assert_raise(ArgumentError) { [].fill(0, 1, LONGP) }
+  end
+
+  def test_times
+    assert_raise(ArgumentError) { [0, 0, 0, 0] * ((LONGP + 1) / 4) }
+  end
+
+  def test_equal
+    o = Object.new
+    def o.to_ary; end
+    def o.==(x); :foo; end
+    assert(:foo, [0, 1, 2] == o)
+    assert([0, 1, 2] != [0, 1, 3])
+  end
+
+  def test_hash2
+    a = []
+    a << a
+    b = []
+    b << b
+    assert_equal(a.hash, b.hash)
+  end
+
+  def test_flatten2
+    a = []
+    a << a
+    assert_raise(ArgumentError) { a.flatten }
+  end
+
+  def test_shuffle
+    100.times do
+      assert_equal([0, 1, 2], [2, 1, 0].shuffle.sort)
+    end
+  end
+
+  def test_choice
+    100.times do
+      assert([0, 1, 2].include?([2, 1, 0].choice))
+    end
+  end
+
+  def test_cycle
+    a = []
+    [0, 1, 2].cycle do |i|
+      a << i
+      break if a.size == 10
+    end
+    assert_equal([0, 1, 2, 0, 1, 2, 0, 1, 2, 0], a)
+
+    a = [0, 1, 2]
+    assert_nil(a.cycle { a.clear })
+
+    a = []
+    [0, 1, 2].cycle(3) {|i| a << i }
+    assert_equal([0, 1, 2, 0, 1, 2, 0, 1, 2], a)
+  end
+
+  def test_reverse_each2
+    a = [0, 1, 2, 3, 4, 5]
+    r = []
+    a.reverse_each do |x|
+      r << x
+      a.pop
+      a.pop
+    end
+    assert_equal([5, 3, 1], r)
+  end
+
+  def test_combination2
+    assert_raise(RangeError) do
+      (0..100).to_a.combination(50) {}
+    end
+  end
+
+  def test_product2
+    a = (0..100).to_a
+    assert_raise(RangeError) do
+      a.product(a, a, a, a, a, a, a, a, a, a) {}
+    end
+  end
+
+  class Array2 < Array
+  end
+
+  def test_array_subclass
+    assert_equal(Array2, Array2[1,2,3].uniq.class, "[ruby-dev:34581]")
+  end
 end
