@@ -1,6 +1,6 @@
 require 'test/unit'
 require 'tempfile'
-require 'envutil'
+require_relative 'envutil'
 
 class TestBeginEndBlock < Test::Unit::TestCase
   DIR = File.dirname(File.expand_path(__FILE__))
@@ -12,9 +12,8 @@ class TestBeginEndBlock < Test::Unit::TestCase
   def test_beginendblock
     ruby = EnvUtil.rubybin
     target = File.join(DIR, 'beginmainend.rb')
-    io = IO.popen("#{q(ruby)} #{q(target)}")
-    assert_equal(%w(b1 b2-1 b2 main b3-1 b3 b4 e1 e4 e3 e2 e4-2 e4-1 e1-1 e4-1-1), io.read.split)
-    io.close
+    result = IO.popen([ruby, target]){|io|io.read}
+    assert_equal(%w(b1 b2-1 b2 main b3-1 b3 b4 e1 e4 e3 e2 e4-2 e4-1 e1-1 e4-1-1), result.split)
   end
 
   def test_begininmethod
@@ -38,19 +37,50 @@ errout = ARGV.shift
 STDERR.reopen(File.open(errout, "w"))
 STDERR.sync = true
 Dir.chdir(#{q(DIR)})
-cmd = "\\"#{ruby}\\" \\"endblockwarn.rb\\""
-system(cmd)
+system("#{ruby}", "endblockwarn_rb")
 EOF
     launcher.close
     launcherpath = launcher.path
     errout.close
     erroutpath = errout.path
-    system("#{q(ruby)} #{q(launcherpath)} #{q(erroutpath)}")
+    system(ruby, launcherpath, erroutpath)
     expected = <<EOW
-endblockwarn.rb:2: warning: END in method; use at_exit
+endblockwarn_rb:2: warning: END in method; use at_exit
 (eval):2: warning: END in method; use at_exit
 EOW
     assert_equal(expected, File.read(erroutpath))
     # expecting Tempfile to unlink launcher and errout file.
   end
+
+  def test_raise_in_at_exit
+    ruby = EnvUtil.rubybin
+    out = IO.popen([ruby, '-e', 'STDERR.reopen(STDOUT)',
+                     '-e', 'at_exit{raise %[SomethingBad]}',
+                     '-e', 'raise %[SomethingElse]']) {|f|
+      f.read
+    }
+    assert_match /SomethingBad/, out, "[ruby-core:9675]"
+    assert_match /SomethingElse/, out, "[ruby-core:9675]"
+  end
+
+  def test_should_propagate_exit_code
+    ruby = EnvUtil.rubybin
+    assert_equal false, system(ruby, '-e', 'at_exit{exit 2}')
+    assert_equal 2, $?.exitstatus
+    assert_nil $?.termsig
+  end
+
+  def test_should_propagate_signaled
+    ruby = EnvUtil.rubybin
+    out = IO.popen(
+      [ruby,
+       '-e', 'STDERR.reopen(STDOUT)',
+       '-e', 'at_exit{Process.kill(:INT, $$); loop{}}']) {|f|
+      f.read
+    }
+    assert_match /Interrupt$/, out
+    Process.kill(0, 0) rescue return # check if signal works
+    assert_nil $?.exitstatus
+    assert_equal Signal.list["INT"], $?.termsig
+  end
 end
