@@ -1,6 +1,11 @@
 require 'test/unit'
+require_relative 'envutil'
 
 class TestException < Test::Unit::TestCase
+  def ruby(*r, &b)
+    EnvUtil.rubyexec(*r, &b)
+  end
+
   def test_exception
     begin
       raise "this must be handled"
@@ -185,25 +190,60 @@ class TestException < Test::Unit::TestCase
     end
   end
 
-  def test_to_s_taintness_propagation
-    for exc in [Exception, NameError]
-      m = "abcdefg"
-      e = exc.new(m)
-      e.taint
-      s = e.to_s
-      assert_equal(false, m.tainted?,
-                   "#{exc}#to_s should not propagate taintness")
-      assert_equal(false, s.tainted?,
-                   "#{exc}#to_s should not propagate taintness")
+  def test_raise_with_wrong_number_of_arguments
+    assert_raise(TypeError) { raise nil }
+    assert_raise(TypeError) { raise 1, 1 }
+    assert_raise(ArgumentError) { raise 1, 1, 1, 1 }
+  end
+
+  def test_errat
+    ruby do |w, r, e|
+      w.puts "p $@"
+      w.close
+      assert_equal("nil", r.read.chomp)
+      assert_equal("", e.read.chomp)
     end
-    
-    o = Object.new
-    def o.to_str
-      "foo"
+
+    ruby do |w, r, e|
+      w.puts "$@ = 1"
+      w.close
+      assert_equal("", r.read.chomp)
+      assert_match(/\$! not set \(ArgumentError\)$/, e.read.chomp)
+    end
+
+    ruby do |w, r, e|
+      w.puts "begin"
+      w.puts "  raise"
+      w.puts "rescue"
+      w.puts "  $@ = 1"
+      w.puts "end"
+      w.close
+      assert_equal("", r.read.chomp)
+      assert_match(/backtrace must be Array of String \(TypeError\)$/, e.read.chomp)
+    end
+
+    ruby do |w, r, e|
+      w.puts "begin"
+      w.puts "  raise"
+      w.puts "rescue"
+      w.puts "  $@ = 'foo'"
+      w.puts "  raise"
+      w.puts "end"
+      w.close
+      assert_equal("", r.read.chomp)
+      assert_match(/^foo: unhandled exception$/, e.read.chomp)
+    end
+
+    ruby do |w, r, e|
+      w.puts "begin"
+      w.puts "  raise"
+      w.puts "rescue"
+      w.puts "  $@ = %w(foo bar baz)"
+      w.puts "  raise"
+      w.puts "end"
+      w.close
+      assert_equal("", r.read.chomp)
+      assert_match(/^foo: unhandled exception\s+from bar\s+from baz$/, e.read.chomp)
     end
-    o.taint
-    e = NameError.new(o)
-    s = e.to_s
-    assert_equal(true, s.tainted?)
   end
 end
