@@ -105,8 +105,9 @@ class TestM17NComb < Test::Unit::TestCase
     u("\xc2"), u("\x80"),
 
     # for transitivity test
-    u("\xe0\xa0\xa1"), e("\xe0\xa0\xa1"), s("\xe0\xa0\xa1"), # [ruby-dev:32693]
-    e("\xa1\xa1"), a("\xa1\xa1"), s("\xa1\xa1"), # [ruby-dev:36484]
+    u("\xe0\xa0\xa1"),
+    e("\xe0\xa0\xa1"),
+    s("\xe0\xa0\xa1"),
 
     #"aa".force_encoding("utf-16be"),
     #"aaaa".force_encoding("utf-32be"),
@@ -115,7 +116,6 @@ class TestM17NComb < Test::Unit::TestCase
 
   def combination(*args, &b)
     AllPairs.each(*args, &b)
-    #AllPairs.exhaustive_each(*args, &b)
   end
 
   def encdump(str)
@@ -261,7 +261,7 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_plus
     combination(STRINGS, STRINGS) {|s1, s2|
       if s1.encoding != s2.encoding && !s1.ascii_only? && !s2.ascii_only?
-        assert_raise(Encoding::CompatibilityError) { s1 + s2 }
+        assert_raise(ArgumentError) { s1 + s2 }
       else
         t = enccall(s1, :+, s2)
         assert(t.valid_encoding?) if s1.valid_encoding? && s2.valid_encoding?
@@ -286,7 +286,7 @@ class TestM17NComb < Test::Unit::TestCase
       assert_strenc(a(s), s.encoding, "%s".force_encoding(s.encoding) % s)
       if !s.empty? # xxx
         t = enccall(a("%s"), :%, s)
-        assert_strenc(a(s), (a('')+s).encoding, t)
+        assert_strenc(a(s), s.encoding, t)
       end
     }
   end
@@ -318,9 +318,10 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_eq
     combination(STRINGS, STRINGS) {|s1, s2|
       desc_eq = "#{encdump s1} == #{encdump s2}"
-      if a(s1) == a(s2) and
-          (s1.ascii_only? && s2.ascii_only? or
-           s1.encoding == s2.encoding) then
+      if s1.ascii_only? && s2.ascii_only? && a(s1) == a(s2)
+        assert(s1 == s2, desc_eq)
+        assert(s1.eql?(s2), desc_eq)
+      elsif s1.encoding == s2.encoding && a(s1) == a(s2)
         assert(s1 == s2, desc_eq)
         assert(!(s1 != s2))
         assert_equal(0, s1 <=> s2)
@@ -343,7 +344,7 @@ class TestM17NComb < Test::Unit::TestCase
         assert_equal(a(s), a(s1) + a(s2))
         assert_str_enc_propagation(s, s1, s2)
       else
-        assert_raise(Encoding::CompatibilityError) { s << s2 }
+        assert_raise(ArgumentError) { s << s2 }
       end
     }
   end
@@ -395,7 +396,7 @@ class TestM17NComb < Test::Unit::TestCase
           end
         end
       else
-        assert_raise(Encoding::CompatibilityError) { s1[s2] }
+        assert_raise(ArgumentError) { s1[s2] }
       end
     }
   end
@@ -480,7 +481,7 @@ class TestM17NComb < Test::Unit::TestCase
             end
           end
         else
-          assert_raise(Encoding::CompatibilityError) { t[i] = s2 }
+          assert_raise(ArgumentError) { t[i] = s2 }
         end
       }
     }
@@ -512,7 +513,7 @@ class TestM17NComb < Test::Unit::TestCase
           end
         end
       else
-        assert_raise(Encoding::CompatibilityError) { t[i,len] = s2 }
+        assert_raise(ArgumentError) { t[i,len] = s2 }
       end
     }
   end
@@ -525,7 +526,7 @@ class TestM17NComb < Test::Unit::TestCase
         !s2.ascii_only? ? s2.encoding : nil,
         !s3.ascii_only? ? s3.encoding : nil].uniq.compact
       if 1 < encs.length
-        assert_raise(Encoding::CompatibilityError, IndexError) { t[s2] = s3 }
+        assert_raise(ArgumentError, IndexError) { t[s2] = s3 }
       else
         if encs.empty?
           encs = [
@@ -564,7 +565,7 @@ class TestM17NComb < Test::Unit::TestCase
           end
         end
       else
-        assert_raise(Encoding::CompatibilityError, RangeError,
+        assert_raise(ArgumentError, RangeError,
                      "t=#{encdump(s1)};t[#{first}..#{last}]=#{encdump(s2)}") {
           t[first..last] = s2
         }
@@ -591,7 +592,7 @@ class TestM17NComb < Test::Unit::TestCase
           end
         end
       else
-        assert_raise(Encoding::CompatibilityError, RangeError,
+        assert_raise(ArgumentError, RangeError,
                      "t=#{encdump(s1)};t[#{first}...#{last}]=#{encdump(s2)}") {
           t[first...last] = s2
         }
@@ -633,9 +634,13 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_casecmp
     combination(STRINGS, STRINGS) {|s1, s2|
       #puts "#{encdump(s1)}.casecmp(#{encdump(s2)})"
-      next unless s1.valid_encoding? && s2.valid_encoding? && Encoding.compatible?(s1, s2)
-      r = s1.casecmp(s2)
-      assert_equal(s1.upcase <=> s2.upcase, r)
+      begin
+        r = s1.casecmp(s2)
+      rescue ArgumentError
+        assert(!s1.valid_encoding? || !s2.valid_encoding?)
+        next
+      end
+      #assert_equal(s1.upcase <=> s2.upcase, r)
     }
   end
 
@@ -650,7 +655,7 @@ class TestM17NComb < Test::Unit::TestCase
         next
       end
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.center(width, s2) }
+        assert_raise(ArgumentError) { s1.center(width, s2) }
         next
       end
       t = enccall(s1, :center, width, s2)
@@ -671,7 +676,7 @@ class TestM17NComb < Test::Unit::TestCase
         next
       end
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.ljust(width, s2) }
+        assert_raise(ArgumentError) { s1.ljust(width, s2) }
         next
       end
       t = enccall(s1, :ljust, width, s2)
@@ -692,7 +697,7 @@ class TestM17NComb < Test::Unit::TestCase
         next
       end
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.rjust(width, s2) }
+        assert_raise(ArgumentError) { s1.rjust(width, s2) }
         next
       end
       t = enccall(s1, :rjust, width, s2)
@@ -705,8 +710,8 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_chomp
     combination(STRINGS, STRINGS) {|s1, s2|
       if !s1.ascii_only? && !s2.ascii_only? && !Encoding.compatible?(s1,s2)
-        if s1.bytesize > s2.bytesize
-          assert_raise(Encoding::CompatibilityError) { s1.chomp(s2) }
+        if s1.bytesize > s2.bytesize 
+          assert_raise(ArgumentError) { s1.chomp(s2) }
         end
         next
       end
@@ -723,6 +728,15 @@ class TestM17NComb < Test::Unit::TestCase
     STRINGS.each {|s|
       s = s.dup
       desc = "#{encdump s}.chop"
+      if !s.valid_encoding?
+        #assert_raise(ArgumentError, desc) { s.chop }
+        begin
+          s.chop
+        rescue ArgumentError
+          e = $!
+        end
+        next if e
+      end
       t = nil
       assert_nothing_raised(desc) { t = s.chop }
       assert(t.valid_encoding?) if s.valid_encoding?
@@ -763,11 +777,11 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_count
     combination(STRINGS, STRINGS) {|s1, s2|
       if !s1.valid_encoding? || !s2.valid_encoding?
-        assert_raise(ArgumentError, Encoding::CompatibilityError) { s1.count(s2) }
+        assert_raise(ArgumentError) { s1.count(s2) }
         next
       end
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.count(s2) }
+        assert_raise(ArgumentError) { s1.count(s2) }
         next
       end
       n = enccall(s1, :count, s2)
@@ -783,7 +797,7 @@ class TestM17NComb < Test::Unit::TestCase
         next
       end
       t = str.crypt(salt)
-      assert_equal(a(str).crypt(a(salt)), t, "#{encdump(str)}.crypt(#{encdump(salt)})")
+      assert_equal(a(str).crypt(a(salt)), t)
       assert_encoding('ASCII-8BIT', t.encoding)
     }
   end
@@ -795,11 +809,11 @@ class TestM17NComb < Test::Unit::TestCase
         next
       end
       if !s1.valid_encoding? || !s2.valid_encoding?
-        assert_raise(ArgumentError, Encoding::CompatibilityError) { s1.delete(s2) }
+        assert_raise(ArgumentError) { s1.delete(s2) }
         next
       end
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.delete(s2) }
+        assert_raise(ArgumentError) { s1.delete(s2) }
         next
       end
       t = enccall(s1, :delete, s2)
@@ -841,11 +855,11 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_each_line
     combination(STRINGS, STRINGS) {|s1, s2|
       if !s1.valid_encoding? || !s2.valid_encoding?
-        assert_raise(ArgumentError, Encoding::CompatibilityError) { s1.each_line(s2) {} }
+        assert_raise(ArgumentError) { s1.each_line(s2) {} }
         next
       end
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.each_line(s2) {} }
+        assert_raise(ArgumentError) { s1.each_line(s2) {} }
         next
       end
       lines = []
@@ -894,9 +908,9 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_include?
     combination(STRINGS, STRINGS) {|s1, s2|
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.include?(s2) }
-        assert_raise(Encoding::CompatibilityError) { s1.index(s2) }
-        assert_raise(Encoding::CompatibilityError) { s1.rindex(s2) }
+        assert_raise(ArgumentError) { s1.include?(s2) }
+        assert_raise(ArgumentError) { s1.index(s2) }
+        assert_raise(ArgumentError) { s1.rindex(s2) }
         next
       end
       t = enccall(s1, :include?, s2)
@@ -927,7 +941,7 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_index
     combination(STRINGS, STRINGS, -2..2) {|s1, s2, pos|
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.index(s2) }
+        assert_raise(ArgumentError) { s1.index(s2) }
         next
       end
       t = enccall(s1, :index, s2, pos)
@@ -960,7 +974,7 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_rindex
     combination(STRINGS, STRINGS, -2..2) {|s1, s2, pos|
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.rindex(s2) }
+        assert_raise(ArgumentError) { s1.rindex(s2) }
         next
       end
       t = enccall(s1, :rindex, s2, pos)
@@ -1009,11 +1023,11 @@ class TestM17NComb < Test::Unit::TestCase
       t2 = s1.dup
       begin
         t1[nth, 0] = s2
-      rescue Encoding::CompatibilityError, IndexError => e1
+      rescue ArgumentError, IndexError => e1
       end
       begin
         t2.insert(nth, s2)
-      rescue Encoding::CompatibilityError, IndexError => e2
+      rescue ArgumentError, IndexError => e2
       end
       assert_equal(t1, t2, "t=#{encdump s1}; t.insert(#{nth},#{encdump s2}); t")
       assert_equal(e1.class, e2.class, "begin #{encdump s1}.insert(#{nth},#{encdump s2}); rescue ArgumentError, IndexError => e; e end")
@@ -1027,7 +1041,7 @@ class TestM17NComb < Test::Unit::TestCase
         t1.insert(nth, s2)
         slen = s2.length
         assert_equal(t1[nth-slen+1,slen], s2, "t=#{encdump s1}; t.insert(#{nth},#{encdump s2}); t")
-      rescue Encoding::CompatibilityError, IndexError
+      rescue ArgumentError, IndexError => e
       end
     }
   end
@@ -1036,12 +1050,10 @@ class TestM17NComb < Test::Unit::TestCase
     STRINGS.each {|s|
       if /\0/ =~ a(s)
         assert_raise(ArgumentError) { s.intern }
-      elsif s.valid_encoding?
+      else
         sym = s.intern
         assert_equal(s, sym.to_s, "#{encdump s}.intern.to_s")
         assert_equal(sym, s.to_sym)
-      else
-        assert_raise(EncodingError) { s.intern }
       end
     }
   end
@@ -1055,7 +1067,7 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_oct
     STRINGS.each {|s|
       t = s.oct
-      t2 = a(s)[/\A[0-9a-fA-FxX]*/].oct
+      t2 = a(s)[/\A[0-9a-fA-FxXbB]*/].oct
       assert_equal(t2, t)
     }
   end
@@ -1088,11 +1100,7 @@ class TestM17NComb < Test::Unit::TestCase
         next
       end
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        if s1.valid_encoding?
-          assert_raise(Encoding::CompatibilityError) { s1.scan(s2) }
-        else
-          assert_match(/invalid byte sequence/, assert_raise(ArgumentError) { s1.scan(s2) }.message)
-        end
+        assert_raise(ArgumentError) { s1.scan(s2) }
         next
       end
       if !s1.valid_encoding?
@@ -1114,8 +1122,6 @@ class TestM17NComb < Test::Unit::TestCase
 
   def test_str_slice!
     each_slice_call {|s, *args|
-      desc_slice = "#{encdump s}.slice#{encdumpargs args}"
-      desc_slice_bang = "#{encdump s}.slice!#{encdumpargs args}"
       t = s.dup
       begin
         r = t.slice!(*args)
@@ -1123,14 +1129,14 @@ class TestM17NComb < Test::Unit::TestCase
         e = $!
       end
       if e
-        assert_raise(e.class, desc_slice) { s.slice(*args) }
+        assert_raise(e.class, "#{encdump s}.slice#{encdumpargs args}") { s.slice(*args) }
         next
       end
       if !r
-        assert_nil(s.slice(*args), desc_slice)
+        assert_nil(s.slice(*args))
         next
       end
-      assert_equal(s.slice(*args), r, desc_slice_bang)
+      assert_equal(s.slice(*args), r)
       assert_equal(s.bytesize, r.bytesize + t.bytesize)
       if args.length == 1 && String === args[0]
         assert_equal(args[0].encoding, r.encoding,
@@ -1150,11 +1156,11 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_split
     combination(STRINGS, STRINGS) {|s1, s2|
       if !s2.valid_encoding?
-        assert_raise(ArgumentError, RegexpError) { s1.split(s2) }
+        assert_raise(RegexpError) { s1.split(s2) }
         next
       end
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(ArgumentError, Encoding::CompatibilityError) { s1.split(s2) }
+        assert_raise(ArgumentError) { s1.split(s2) }
         next
       end
       if !s1.valid_encoding?
@@ -1178,11 +1184,11 @@ class TestM17NComb < Test::Unit::TestCase
   def test_str_squeeze
     combination(STRINGS, STRINGS) {|s1, s2|
       if !s1.valid_encoding? || !s2.valid_encoding?
-        assert_raise(ArgumentError, Encoding::CompatibilityError, "#{encdump s1}.squeeze(#{encdump s2})") { s1.squeeze(s2) }
+        assert_raise(ArgumentError, "#{encdump s1}.squeeze(#{encdump s2})") { s1.squeeze(s2) }
         next
       end
       if !s1.ascii_only? && !s2.ascii_only? && s1.encoding != s2.encoding
-        assert_raise(Encoding::CompatibilityError) { s1.squeeze(s2) }
+        assert_raise(ArgumentError) { s1.squeeze(s2) }
         next
       end
       t = enccall(s1, :squeeze, s2)
@@ -1272,7 +1278,7 @@ class TestM17NComb < Test::Unit::TestCase
         next
       end
       if !str_enc_compatible?(s1, s2, s3)
-        assert_raise(Encoding::CompatibilityError, desc) { s1.tr(s2, s3) }
+        assert_raise(ArgumentError, desc) { s1.tr(s2, s3) }
         next
       end
       if !s1.valid_encoding?
@@ -1301,11 +1307,11 @@ class TestM17NComb < Test::Unit::TestCase
         next
       end
       if !s1.valid_encoding?
-        assert_raise(ArgumentError, Encoding::CompatibilityError, desc) { s1.tr_s(s2, s3) }
+        assert_raise(ArgumentError, desc) { s1.tr_s(s2, s3) }
         next
       end
       if !str_enc_compatible?(s1, s2, s3)
-        assert_raise(Encoding::CompatibilityError, desc) { s1.tr(s2, s3) }
+        assert_raise(ArgumentError, desc) { s1.tr(s2, s3) }
         next
       end
       if s2.empty?
@@ -1343,7 +1349,7 @@ class TestM17NComb < Test::Unit::TestCase
     STRINGS.each {|s0|
       next if s0.empty?
       s = s0.dup
-      n = 300
+      n = 1000
       h = {}
       n.times {|i|
         if h[s]
@@ -1387,39 +1393,35 @@ class TestM17NComb < Test::Unit::TestCase
       [
         [
           "#{encdump s1}.sub(Regexp.new(#{encdump s2}), #{encdump s3})",
-          lambda { s1.sub(r2, s3) },
-          false
+          lambda { s1.sub(r2, s3) }
         ],
         [
           "#{encdump s1}.sub(Regexp.new(#{encdump s2}), #{encdump s3})",
-          lambda { s1.sub(r2) { s3 } },
-          false
+          lambda { s1.sub(r2) { s3 } }
         ],
         [
           "#{encdump s1}.gsub(Regexp.new(#{encdump s2}), #{encdump s3})",
-          lambda { s1.gsub(r2, s3) },
-          true
+          lambda { s1.gsub(r2, s3) }
         ],
         [
           "#{encdump s1}.gsub(Regexp.new(#{encdump s2}), #{encdump s3})",
-          lambda { s1.gsub(r2) { s3 } },
-          true
+          lambda { s1.gsub(r2) { s3 } }
         ]
-      ].each {|desc, doit, g|
+      ].each {|desc, doit|
         if !s1.valid_encoding?
           assert_raise(ArgumentError, desc) { doit.call }
           next
         end
         if !str_enc_compatible?(s1, s2)
-          assert_raise(Encoding::CompatibilityError, desc) { doit.call }
+          assert_raise(ArgumentError, desc) { doit.call }
           next
         end
         if !enccall(s1, :include?, s2)
           assert_equal(s1, doit.call)
           next
         end
-        if !str_enc_compatible?(g ? s1.gsub(r2, '') : s1.sub(r2, ''), s3)
-          assert_raise(Encoding::CompatibilityError, desc) { doit.call }
+        if !str_enc_compatible?(s1.gsub(r2, ''), s3)
+          assert_raise(ArgumentError, desc) { doit.call }
           next
         end
         t = nil
@@ -1445,39 +1447,35 @@ class TestM17NComb < Test::Unit::TestCase
       [
         [
           "t=#{encdump s1}.dup;t.sub!(Regexp.new(#{encdump s2}), #{encdump s3})",
-          lambda { t=s1.dup; [t, t.sub!(r2, s3)] },
-          false
+          lambda { t=s1.dup; [t, t.sub!(r2, s3)] }
         ],
         [
           "t=#{encdump s1}.dup;t.sub!(Regexp.new(#{encdump s2}), #{encdump s3})",
-          lambda { t=s1.dup; [t, t.sub!(r2) { s3 }] },
-          false
+          lambda { t=s1.dup; [t, t.sub!(r2) { s3 }] }
         ],
         [
           "t=#{encdump s1}.dup;t.gsub!(Regexp.new(#{encdump s2}), #{encdump s3})",
-          lambda { t=s1.dup; [t, t.gsub!(r2, s3)] },
-          true
+          lambda { t=s1.dup; [t, t.gsub!(r2, s3)] }
         ],
         [
           "t=#{encdump s1}.dup;t.gsub!(Regexp.new(#{encdump s2}), #{encdump s3})",
-          lambda { t=s1.dup; [t, t.gsub!(r2) { s3 }] },
-          true
+          lambda { t=s1.dup; [t, t.gsub!(r2) { s3 }] }
         ]
-      ].each {|desc, doit, g|
+      ].each {|desc, doit|
         if !s1.valid_encoding?
           assert_raise(ArgumentError, desc) { doit.call }
           next
         end
         if !str_enc_compatible?(s1, s2)
-          assert_raise(Encoding::CompatibilityError, desc) { doit.call }
+          assert_raise(ArgumentError, desc) { doit.call }
           next
         end
         if !enccall(s1, :include?, s2)
           assert_equal([s1, nil], doit.call)
           next
         end
-        if !str_enc_compatible?(g ? s1.gsub(r2, '') : s1.sub(r2, ''), s3)
-          assert_raise(Encoding::CompatibilityError, desc) { doit.call }
+        if !str_enc_compatible?(s1.gsub(r2, ''), s3)
+          assert_raise(ArgumentError, desc) { doit.call }
           next
         end
         t = ret = nil
@@ -1538,7 +1536,7 @@ class TestM17NComb < Test::Unit::TestCase
     combination(STRINGS, STRINGS) {|s1, s2|
       desc = "#{encdump s1}.end_with?(#{encdump s2})"
       if !str_enc_compatible?(s1, s2)
-        assert_raise(Encoding::CompatibilityError, desc) { s1.end_with?(s2) }
+        assert_raise(ArgumentError, desc) { s1.end_with?(s2) }
         next
       end
       if s1.length < s2.length
@@ -1557,7 +1555,7 @@ class TestM17NComb < Test::Unit::TestCase
     combination(STRINGS, STRINGS) {|s1, s2|
       desc = "#{encdump s1}.start_with?(#{encdump s2})"
       if !str_enc_compatible?(s1, s2)
-        assert_raise(Encoding::CompatibilityError, desc) { s1.start_with?(s2) }
+        assert_raise(ArgumentError, desc) { s1.start_with?(s2) }
         next
       end
       s1 = s1.dup.force_encoding("ASCII-8BIT")
@@ -1592,7 +1590,7 @@ class TestM17NComb < Test::Unit::TestCase
     combination(STRINGS, STRINGS) {|s1, s2|
       desc = "#{encdump s1}.partition(#{encdump s2})"
       if !str_enc_compatible?(s1, s2)
-        assert_raise(Encoding::CompatibilityError, desc) { s1.partition(s2) }
+        assert_raise(ArgumentError, desc) { s1.partition(s2) }
         next
       end
       i = enccall(s1, :index, s2)
@@ -1608,7 +1606,7 @@ class TestM17NComb < Test::Unit::TestCase
     combination(STRINGS, STRINGS) {|s1, s2|
       desc = "#{encdump s1}.rpartition(#{encdump s2})"
       if !str_enc_compatible?(s1, s2)
-        assert_raise(Encoding::CompatibilityError, desc) { s1.rpartition(s2) }
+        assert_raise(ArgumentError, desc) { s1.rpartition(s2) }
         next
       end
       i = enccall(s1, :rindex, s2)
