@@ -5,10 +5,6 @@ require_relative 'envutil'
 $m0 = Module.nesting
 
 class TestModule < Test::Unit::TestCase
-  def _wrap_assertion
-    yield
-  end
-
   def assert_method_defined?(klass, mid, message="")
     message = build_message(message, "#{klass}\##{mid} expected to be defined.")
     _wrap_assertion do
@@ -34,6 +30,10 @@ class TestModule < Test::Unit::TestCase
     $VERBOSE = @verbose
   end
 
+  def ruby(*r, &b)
+    EnvUtil.rubyexec(*r, &b)
+  end
+
   def test_LT_0
     assert_equal true, String < Object
     assert_equal false, Object < String
@@ -55,7 +55,7 @@ class TestModule < Test::Unit::TestCase
   end
 
   def test_CMP_0
-    assert_equal(-1, (String <=> Object))
+    assert_equal -1, (String <=> Object)
     assert_equal 1, (Object <=> String)
     assert_nil(Array <=> String)
   end
@@ -72,17 +72,6 @@ class TestModule < Test::Unit::TestCase
     list.reject {|c| c.to_s.start_with?("JSON") }
   end
 
-  def remove_rake_mixins(list)
-    list.reject {|c|
-      name = c.name
-      name.start_with?("Rake") or name.start_with?("FileUtils")
-    }
-  end
-
-  def remove_minitest_mixins(list)
-    list.reject {|c| c.to_s.start_with?("MiniTest") }
-  end
-
   module Mixin
     MIXIN = 1
     def mixin
@@ -111,19 +100,16 @@ class TestModule < Test::Unit::TestCase
     def AClass.cm3
       "cm3"
     end
-
+    
     private_class_method :cm1, "cm3"
 
     def aClass
-      :aClass
     end
 
     def aClass1
-      :aClass1
     end
 
     def aClass2
-      :aClass2
     end
 
     private :aClass1
@@ -132,23 +118,15 @@ class TestModule < Test::Unit::TestCase
 
   class BClass < AClass
     def bClass1
-      :bClass1
     end
 
     private
 
     def bClass2
-      :bClass2
     end
 
     protected
     def bClass3
-      :bClass3
-    end
-  end
-
-  class CClass < BClass
-    def self.cClass
     end
   end
 
@@ -221,34 +199,23 @@ class TestModule < Test::Unit::TestCase
     assert_equal([Mixin],            Mixin.ancestors)
 
     assert_equal([Object, Kernel, BasicObject],
-                 remove_minitest_mixins(remove_rake_mixins(remove_json_mixins(remove_pp_mixins(Object.ancestors)))))
+                 remove_json_mixins(remove_pp_mixins(Object.ancestors)))
     assert_equal([String, Comparable, Object, Kernel, BasicObject],
-                 remove_minitest_mixins(remove_rake_mixins(remove_json_mixins(remove_pp_mixins(String.ancestors)))))
+                 remove_json_mixins(remove_pp_mixins(String.ancestors)))
   end
 
-  CLASS_EVAL = 2
-  @@class_eval = 'b'
-
   def test_class_eval
     Other.class_eval("CLASS_EVAL = 1")
     assert_equal(1, Other::CLASS_EVAL)
     assert(Other.constants.include?(:CLASS_EVAL))
-    assert_equal(2, Other.class_eval { CLASS_EVAL })
+  end
 
-    Other.class_eval("@@class_eval = 'a'")
-    assert_equal('a', Other.class_variable_get(:@@class_eval))
-    assert_equal('b', Other.class_eval { @@class_eval })
+  def test_class_variable_set
+    # TODO
+  end
 
-    Other.class_eval do
-      module_function
-
-      def class_eval_test
-        "foo"
-      end
-    end
-    assert_equal("foo", Other.class_eval_test)
-
-    assert_equal([Other], Other.class_eval { |*args| args })
+  def test_class_variable_get
+    # TODO
   end
 
   def test_const_defined?
@@ -281,9 +248,9 @@ class TestModule < Test::Unit::TestCase
     assert_equal([], Mixin.included_modules)
     assert_equal([Mixin], User.included_modules)
     assert_equal([Kernel],
-                 remove_minitest_mixins(remove_rake_mixins(remove_json_mixins(remove_pp_mixins(Object.included_modules)))))
+                 remove_json_mixins(remove_pp_mixins(Object.included_modules)))
     assert_equal([Comparable, Kernel],
-                 remove_minitest_mixins(remove_rake_mixins(remove_json_mixins(remove_pp_mixins(String.included_modules)))))
+                 remove_json_mixins(remove_pp_mixins(String.included_modules)))
   end
 
   def test_instance_methods
@@ -291,9 +258,6 @@ class TestModule < Test::Unit::TestCase
     assert_equal([:user, :mixin].sort, User.instance_methods(true).sort)
     assert_equal([:mixin], Mixin.instance_methods)
     assert_equal([:mixin], Mixin.instance_methods(true))
-    assert_equal([:cClass], (class << CClass; self; end).instance_methods(false))
-    assert_equal([], (class << BClass; self; end).instance_methods(false))
-    assert_equal([:cm2], (class << AClass; self; end).instance_methods(false))
     # Ruby 1.8 feature change:
     # #instance_methods includes protected methods.
     #assert_equal([:aClass], AClass.instance_methods(false))
@@ -393,26 +357,6 @@ class TestModule < Test::Unit::TestCase
     Object.module_eval "WALTER = 99"
     c2 = Module.constants
     assert_equal([:WALTER], c2 - c1)
-
-    assert_equal([], Module.constants(true))
-    assert_equal([], Module.constants(false))
-
-    src = <<-INPUT
-      ary = Module.constants
-      module M
-        WALTER = 99
-      end
-      class Module
-        include M
-      end
-      p Module.constants - ary, Module.constants(true), Module.constants(false)
-    INPUT
-    assert_in_out_err([], src, %w([:M] [:WALTER] []), [])
-
-    klass = Class.new do
-      const_set(:X, 123)
-    end
-    assert_equal(false, klass.class_eval { Module.constants }.include?(:X))
   end
 
   module M1
@@ -461,7 +405,7 @@ class TestModule < Test::Unit::TestCase
     assert_equal(false, o.respond_to?(:bar=))
   end
 
-  def test_const_get_evaled
+  def test_const_get2
     c1 = Class.new
     c2 = Class.new(c1)
 
@@ -491,101 +435,39 @@ class TestModule < Test::Unit::TestCase
     assert_raise(NameError) { c1.const_get(:foo) }
   end
 
-  def test_const_set_invalid_name
+  def test_const_set2
     c1 = Class.new
     assert_raise(NameError) { c1.const_set(:foo, :foo) }
   end
 
-  def test_const_get_invalid_name
+  def test_const_get3
     c1 = Class.new
     assert_raise(NameError) { c1.const_defined?(:foo) }
   end
 
-  def test_const_get_no_inherited
-    bug3422 = '[ruby-core:30719]'
-    assert_in_out_err([], <<-INPUT, %w[1 NameError A], [], bug3422)
-    BasicObject::A = 1
-    puts [true, false].map {|inh|
-      begin
-        Object.const_get(:A, inh)
-      rescue NameError => e
-        [e.class, e.name]
-      end
-    }
-    INPUT
-  end
-
-  def test_const_get_inherited
-    bug3423 = '[ruby-core:30720]'
-    assert_in_out_err([], <<-INPUT, %w[NameError A NameError A], [], bug3423)
-    module Foo; A = 1; end
-    class Object; include Foo; end
-    class Bar; include Foo; end
-
-    puts [Object, Bar].map {|klass|
-      begin
-        klass.const_get(:A, false)
-      rescue NameError => e
-        [e.class, e.name]
-      end
-    }
-    INPUT
-  end
-
-  def test_const_in_module
-    bug3423 = '[ruby-core:37698]'
-    assert_in_out_err([], <<-INPUT, %w[ok], [], bug3423)
-    module LangModuleSpecInObject
-      module LangModuleTop
-      end
-    end
-    include LangModuleSpecInObject
-    module LangModuleTop
-    end
-    puts "ok" if LangModuleSpecInObject::LangModuleTop == LangModuleTop
-    INPUT
-
-    bug5264 = '[ruby-core:39227]'
-    assert_in_out_err([], <<-'INPUT', [], [], bug5264)
-    class A
-      class X; end
-    end
-    class B < A
-      module X; end
-    end
-    INPUT
-  end
-
-  def test_class_variable_get
+  def test_class_variable_get2
     c = Class.new
-    c.class_eval('@@foo = :foo')
+    c.class_eval { @@foo = :foo }
     assert_equal(:foo, c.class_variable_get(:@@foo))
     assert_raise(NameError) { c.class_variable_get(:@@bar) } # c.f. instance_variable_get
     assert_raise(NameError) { c.class_variable_get(:foo) }
   end
 
-  def test_class_variable_set
+  def test_class_variable_set2
     c = Class.new
     c.class_variable_set(:@@foo, :foo)
-    assert_equal(:foo, c.class_eval('@@foo'))
+    assert_equal(:foo, c.class_eval { @@foo })
     assert_raise(NameError) { c.class_variable_set(:foo, 1) }
   end
 
   def test_class_variable_defined
     c = Class.new
-    c.class_eval('@@foo = :foo')
+    c.class_eval { @@foo = :foo }
     assert_equal(true, c.class_variable_defined?(:@@foo))
     assert_equal(false, c.class_variable_defined?(:@@bar))
     assert_raise(NameError) { c.class_variable_defined?(:foo) }
   end
 
-  def test_remove_class_variable
-    c = Class.new
-    c.class_eval('@@foo = :foo')
-    c.class_eval { remove_class_variable(:@@foo) }
-    assert_equal(false, c.class_variable_defined?(:@@foo))
-  end
-
   def test_export_method
     m = Module.new
     assert_raise(NameError) do
@@ -594,21 +476,24 @@ class TestModule < Test::Unit::TestCase
   end
 
   def test_attr
-    assert_in_out_err([], <<-INPUT, %w(:ok nil), /warning: private attribute\?$/)
-      $VERBOSE = true
-      c = Class.new
-      c.instance_eval do
-        private
-        attr_reader :foo
-      end
-      o = c.new
-      o.foo rescue p(:ok)
-      p(o.instance_eval { foo })
-    INPUT
+    ruby do |w, r, e|
+      w.puts "$VERBOSE = true"
+      w.puts "c = Class.new"
+      w.puts "c.instance_eval do"
+      w.puts "  private"
+      w.puts "  attr_reader :foo"
+      w.puts "end"
+      w.puts "o = c.new"
+      w.puts "o.foo rescue p(:ok)"
+      w.puts "p(o.instance_eval { foo })"
+      w.close
+      assert_equal(":ok\nnil", r.read.chomp)
+      assert_match(/warning: private attribute\?$/, e.read.chomp)
+    end
 
     c = Class.new
     assert_raise(NameError) do
-      c.instance_eval { attr_reader :"." }
+      c.instance_eval { attr_reader :"$" }
     end
   end
 
@@ -635,11 +520,14 @@ class TestModule < Test::Unit::TestCase
       class << o; self; end.instance_eval { undef_method(:foo) }
     end
 
-    %w(object_id __send__ initialize).each do |n|
-      assert_in_out_err([], <<-INPUT, [], /warning: undefining `#{n}' may cause serious problems$/)
-        $VERBOSE = false
-        Class.new.instance_eval { undef_method(:#{n}) }
-      INPUT
+    %w(object_id __send__ initialize).each do |m|
+      ruby do |w, r, e|
+        w.puts "$VERBOSE = false"
+        w.puts "Class.new.instance_eval { undef_method(:#{m}) }"
+        w.close
+        assert_equal("", r.read.chomp)
+        assert_match(/warning: undefining `#{m}' may cause serious problem$/, e.read.chomp)
+      end
     end
   end
 
@@ -649,24 +537,26 @@ class TestModule < Test::Unit::TestCase
       m.class_eval { alias foo bar }
     end
 
-    assert_in_out_err([], <<-INPUT, %w(2), /discarding old foo$/)
-      $VERBOSE = true
-      c = Class.new
-      c.class_eval do
-        def foo; 1; end
-        def bar; 2; end
-      end
-      c.class_eval { alias foo bar }
-      p c.new.foo
-    INPUT
+    ruby do |w, r, e|
+      w.puts "$VERBOSE = true"
+      w.puts "c = Class.new"
+      w.puts "c.class_eval do"
+      w.puts "  def foo; 1; end"
+      w.puts "  def bar; 2; end"
+      w.puts "end"
+      w.puts "c.class_eval { alias foo bar }"
+      w.puts "p c.new.foo"
+      w.close
+      assert_equal("2", r.read.chomp)
+      assert_match(/warning: discarding old foo$/, e.read.chomp)
+    end
   end
 
   def test_mod_constants
-    m = Module.new
-    m.const_set(:Foo, :foo)
-    assert_equal([:Foo], m.constants(true))
-    assert_equal([:Foo], m.constants(false))
-    m.instance_eval { remove_const(:Foo) }
+    Module.const_set(:Foo, :foo)
+    assert_equal([:Foo], Module.constants(true))
+    assert_equal([:Foo], Module.constants(false))
+    Module.instance_eval { remove_const(:Foo) }
   end
 
   def test_frozen_class
@@ -728,14 +618,17 @@ class TestModule < Test::Unit::TestCase
   end
 
   def test_top_public_private
-    assert_in_out_err([], <<-INPUT, %w([:foo] [:bar]), [])
-      private
-      def foo; :foo; end
-      public
-      def bar; :bar; end
-      p self.private_methods.grep(/^foo$|^bar$/)
-      p self.methods.grep(/^foo$|^bar$/)
-    INPUT
+    ruby do |w, r, e|
+      w.puts "private"
+      w.puts "def foo; :foo; end"
+      w.puts "public"
+      w.puts "def bar; :bar; end"
+      w.puts "p self.private_methods.grep(/^foo$|^bar$/)"
+      w.puts "p self.methods.grep(/^foo$|^bar$/)"
+      w.close
+      assert_equal("[:foo]\n[:bar]", r.read.chomp)
+      assert_equal("", e.read.chomp)
+    end
   end
 
   def test_append_features
@@ -822,310 +715,4 @@ class TestModule < Test::Unit::TestCase
     assert_equal(true, c2.include?(m))
     assert_equal(false, m.include?(m))
   end
-
-  def test_include_under_safe4
-    m = Module.new
-    c1 = Class.new
-    assert_raise(SecurityError) do
-      lambda {
-        $SAFE = 4
-        c1.instance_eval { include(m) }
-      }.call
-    end
-    assert_nothing_raised do
-      lambda {
-        $SAFE = 4
-        c2 = Class.new
-        c2.instance_eval { include(m) }
-      }.call
-    end
-  end
-
-  def test_send
-    a = AClass.new
-    assert_equal(:aClass, a.__send__(:aClass))
-    assert_equal(:aClass1, a.__send__(:aClass1))
-    assert_equal(:aClass2, a.__send__(:aClass2))
-    b = BClass.new
-    assert_equal(:aClass, b.__send__(:aClass))
-    assert_equal(:aClass1, b.__send__(:aClass1))
-    assert_equal(:aClass2, b.__send__(:aClass2))
-    assert_equal(:bClass1, b.__send__(:bClass1))
-    assert_equal(:bClass2, b.__send__(:bClass2))
-    assert_equal(:bClass3, b.__send__(:bClass3))
-  end
-
-
-  def test_nonascii_name
-    c = eval("class ::C\u{df}; self; end")
-    assert_equal("C\u{df}", c.name, '[ruby-core:24600]')
-    c = eval("class C\u{df}; self; end")
-    assert_equal("TestModule::C\u{df}", c.name, '[ruby-core:24600]')
-  end
-
-  def test_method_added
-    memo = []
-    mod = Module.new do
-      mod = self
-      (class << self ; self ; end).class_eval do
-        define_method :method_added do |sym|
-          memo << sym
-          memo << mod.instance_methods(false)
-          memo << (mod.instance_method(sym) rescue nil)
-        end
-      end
-      def f
-      end
-      alias g f
-      attr_reader :a
-      attr_writer :a
-    end
-    assert_equal :f, memo.shift
-    assert_equal [:f], memo.shift, '[ruby-core:25536]'
-    assert_equal mod.instance_method(:f), memo.shift
-    assert_equal :g, memo.shift
-    assert_equal [:f, :g], memo.shift
-    assert_equal mod.instance_method(:f), memo.shift
-    assert_equal :a, memo.shift
-    assert_equal [:f, :g, :a], memo.shift
-    assert_equal mod.instance_method(:a), memo.shift
-    assert_equal :a=, memo.shift
-    assert_equal [:f, :g, :a, :a=], memo.shift
-    assert_equal mod.instance_method(:a=), memo.shift
-  end
-
-  def test_method_redefinition
-    feature2155 = '[ruby-dev:39400]'
-
-    line = __LINE__+4
-    stderr = EnvUtil.verbose_warning do
-      Module.new do
-        def foo; end
-        def foo; end
-      end
-    end
-    assert_match(/:#{line}: warning: method redefined; discarding old foo/, stderr)
-    assert_match(/:#{line-1}: warning: previous definition of foo/, stderr, feature2155)
-
-    stderr = EnvUtil.verbose_warning do
-      Module.new do
-        def foo; end
-        alias bar foo
-        def foo; end
-      end
-    end
-    assert_equal("", stderr)
-
-    stderr = EnvUtil.verbose_warning do
-      Module.new do
-        def foo; end
-        alias bar foo
-        alias bar foo
-      end
-    end
-    assert_equal("", stderr)
-
-    line = __LINE__+4
-    stderr = EnvUtil.verbose_warning do
-      Module.new do
-        define_method(:foo) do end
-        def foo; end
-      end
-    end
-    assert_match(/:#{line}: warning: method redefined; discarding old foo/, stderr)
-    assert_match(/:#{line-1}: warning: previous definition of foo/, stderr, feature2155)
-
-    stderr = EnvUtil.verbose_warning do
-      Module.new do
-        define_method(:foo) do end
-        alias bar foo
-        alias barf oo
-      end
-    end
-    assert_equal("", stderr)
-
-    stderr = EnvUtil.verbose_warning do
-      Module.new do
-        module_function
-        def foo; end
-        module_function :foo
-      end
-    end
-    assert_equal("", stderr, '[ruby-dev:39397]')
-
-    stderr = EnvUtil.verbose_warning do
-      Module.new do
-        def foo; end
-        undef foo
-      end
-    end
-    assert_equal("", stderr)
-  end
-
-  def test_protected_singleton_method
-    klass = Class.new
-    x = klass.new
-    class << x
-      protected
-
-      def foo
-      end
-    end
-    assert_raise(NoMethodError) do
-      x.foo
-    end
-    klass.send(:define_method, :bar) do
-      x.foo
-    end
-    assert_nothing_raised do
-      x.bar
-    end
-    y = klass.new
-    assert_raise(NoMethodError) do
-      y.bar
-    end
-  end
-
-  def test_uninitialized_toplevel_constant
-    bug3123 = '[ruby-dev:40951]'
-    e = assert_raise(NameError) {eval("Bug3123", TOPLEVEL_BINDING)}
-    assert_not_match(/Object::/, e.message, bug3123)
-  end
-
-  def test_attr_inherited_visibility
-    bug3406 = '[ruby-core:30638]'
-    c = Class.new do
-      class << self
-        private
-        def attr_accessor(*); super; end
-      end
-      attr_accessor :x
-    end.new
-    assert_nothing_raised(bug3406) {c.x = 1}
-    assert_equal(1, c.x, bug3406)
-  end
-
-  def test_private_constant
-    c = Class.new
-    c.const_set(:FOO, "foo")
-    assert_equal("foo", c::FOO)
-    c.private_constant(:FOO)
-    assert_raise(NameError) { c::FOO }
-    assert_equal("foo", c.class_eval("FOO"))
-    assert_equal("foo", c.const_get("FOO"))
-    $VERBOSE, verbose = nil, $VERBOSE
-    c.const_set(:FOO, "foo")
-    $VERBOSE = verbose
-    assert_raise(NameError) { c::FOO }
-  end
-
-  class PrivateClass
-  end
-  private_constant :PrivateClass
-
-  def test_define_module_under_private_constant
-    assert_raise(NameError) do
-      eval %q{class TestModule::PrivateClass; end}
-    end
-    assert_raise(NameError) do
-      eval %q{module TestModule::PrivateClass::TestModule; end}
-    end
-    eval %q{class PrivateClass; end}
-    eval %q{module PrivateClass::TestModule; end}
-    assert_instance_of(Module, PrivateClass::TestModule)
-    PrivateClass.class_eval { remove_const(:TestModule) }
-  end
-
-  def test_public_constant
-    c = Class.new
-    c.const_set(:FOO, "foo")
-    assert_equal("foo", c::FOO)
-    c.private_constant(:FOO)
-    assert_raise(NameError) { c::FOO }
-    assert_equal("foo", c.class_eval("FOO"))
-    c.public_constant(:FOO)
-    assert_equal("foo", c::FOO)
-  end
-
-  def test_constants_with_private_constant
-    assert(!(::TestModule).constants.include?(:PrivateClass))
-  end
-
-  def test_toplevel_private_constant
-    src = <<-INPUT
-      class Object
-        private_constant :Object
-      end
-      p Object
-      begin
-        p ::Object
-      rescue
-        p :ok
-      end
-    INPUT
-    assert_in_out_err([], src, %w(Object :ok), [])
-  end
-
-  def test_constant_lookup_in_method_defined_by_class_eval
-    src = <<-INPUT
-      class A
-        B = 42
-      end
-
-      A.class_eval do
-        def self.f
-          B
-        end
-
-        def f
-          B
-        end
-      end
-
-      begin
-        A.f
-      rescue NameError
-        puts "A.f"
-      end
-      begin
-        A.new.f
-      rescue NameError
-        puts "A.new.f"
-      end
-    INPUT
-    assert_in_out_err([], src, %w(A.f A.new.f), [])
-  end
-
-  def test_constant_lookup_in_toplevel_class_eval
-    src = <<-INPUT
-      module X
-        A = 123
-      end
-      begin
-        X.class_eval { A }
-      rescue NameError => e
-        puts e
-      end
-    INPUT
-    assert_in_out_err([], src, ["uninitialized constant A"], [])
-  end
-
-  def test_constant_lookup_in_module_in_class_eval
-    src = <<-INPUT
-      class A
-        B = 42
-      end
-
-      A.class_eval do
-        module C
-          begin
-            B
-          rescue NameError
-            puts "NameError"
-          end
-        end
-      end
-    INPUT
-    assert_in_out_err([], src, ["NameError"], [])
-  end
 end
