@@ -20,7 +20,7 @@ class TestParse < Test::Unit::TestCase
     END
     assert_equal(42, x)
   end
-
+  
   def test_alias_backref
     assert_raise(SyntaxError) do
       eval <<-END
@@ -105,7 +105,7 @@ class TestParse < Test::Unit::TestCase
     a = nil
     assert_nothing_raised do
       t.instance_eval <<-END
-        a = bar "foo" do "bar" end
+        a = bar "foo" { "bar" }
       END
     end
     assert_equal("foobar", a)
@@ -113,7 +113,7 @@ class TestParse < Test::Unit::TestCase
     a = nil
     assert_nothing_raised do
       eval <<-END
-        a = t::bar "foo" do "bar" end
+        a = t::bar "foo" { "bar" }
       END
     end
     assert_equal("foobar", a)
@@ -122,10 +122,45 @@ class TestParse < Test::Unit::TestCase
       @baz = r + (block_given? ? [yield] : [])
     end
 
+    assert_nothing_raised do
+      t.instance_eval "baz ()"
+    end
+    assert_equal([], t.instance_eval { @baz })
+
     assert_nothing_raised do
       t.instance_eval "baz (1), 2"
     end
     assert_equal([1, 2], t.instance_eval { @baz })
+
+    assert_nothing_raised do
+      t.instance_eval "baz (3, 4)"
+    end
+    assert_equal([3, 4], t.instance_eval { @baz })
+
+    assert_nothing_raised do
+      t.instance_eval "baz (5, &proc{6})"
+    end
+    assert_equal([5, 6], t.instance_eval { @baz })
+
+    assert_nothing_raised do
+      t.instance_eval "baz (7 => 8)"
+    end
+    assert_equal([{ 7 => 8 }], t.instance_eval { @baz })
+
+    assert_nothing_raised do
+      t.instance_eval "baz (9, 10 => 11)"
+    end
+    assert_equal([9, { 10 => 11 }], t.instance_eval { @baz })
+
+    assert_nothing_raised do
+      t.instance_eval "baz (12, 13, 14 => 15)"
+    end
+    assert_equal([12, 13, { 14 => 15 }], t.instance_eval { @baz })
+
+    assert_nothing_raised do
+      t.instance_eval "baz (&proc {16})"
+    end
+    assert_equal([16], t.instance_eval { @baz })
   end
 
   def test_mlhs_node
@@ -193,7 +228,7 @@ class TestParse < Test::Unit::TestCase
         class foo; end
       END
     end
-
+    
     assert_raise(SyntaxError) do
       eval <<-END
         def foo
@@ -253,6 +288,19 @@ class TestParse < Test::Unit::TestCase
     assert_equal(-4.0, a)
   end
 
+  def test_open_args
+    o = Object.new
+    def o.foo(*r); r.inject(42, :+); end
+
+    a = nil
+    assert_nothing_raised do
+      o.instance_eval <<-END
+        a = foo ()
+      END
+    end
+    assert_equal(42, a)
+  end
+
   def test_block_variable
     o = Object.new
     def o.foo(*r); yield(*r); end
@@ -260,7 +308,7 @@ class TestParse < Test::Unit::TestCase
     a = nil
     assert_nothing_raised do
       eval <<-END
-        o.foo 1 do|; a| a = 42 end
+        o.foo 1 {|; a| a = 42 }
       END
     end
     assert_nil(a)
@@ -350,7 +398,7 @@ class TestParse < Test::Unit::TestCase
   def test_words
     assert_equal([], %W( ))
   end
-
+  
   def test_dstr
     @@foo = 1
     assert_equal("foo 1 bar", "foo #@@foo bar")
@@ -491,16 +539,14 @@ class TestParse < Test::Unit::TestCase
   end
 
   def test_symbol
-    bug = '[ruby-dev:41447]'
-    sym = "foo\0bar".to_sym
-    assert_nothing_raised(SyntaxError, bug) do
-      assert_equal(sym, eval(":'foo\0bar'"))
+    assert_raise(SyntaxError) do
+      eval ":'foo\0bar'"
     end
-    assert_nothing_raised(SyntaxError, bug) do
-      assert_equal(sym, eval(':"foo\u0000bar"'))
+    assert_raise(SyntaxError) do
+      eval ':"foo\u0000bar"'
     end
-    assert_nothing_raised(SyntaxError, bug) do
-      assert_equal(sym, eval(':"foo\u{0}bar"'))
+    assert_raise(SyntaxError) do
+      eval ':"foo\u{0}bar"'
     end
     assert_raise(SyntaxError) do
       eval ':"foo\u{}bar"'
@@ -519,8 +565,8 @@ class TestParse < Test::Unit::TestCase
     x = nil
 
     assert_raise(SyntaxError) do
-      eval %Q(
-<\<FOO
+      eval %q(
+<<FOO
       )
     end
 
@@ -533,8 +579,8 @@ FOO
     end
 
     assert_raise(SyntaxError) do
-      eval %Q(
-<\<\"
+      eval %q(
+<<"
       )
     end
 
@@ -560,7 +606,7 @@ FOO
     end
 
     assert_nothing_raised do
-      eval "x = <<""FOO\r\n1\r\nFOO"
+      eval "x = <<FOO\r\n1\r\nFOO"
     end
     assert_equal("1\n", x)
   end
