@@ -4,24 +4,15 @@ class RationalSub < Rational; end
 
 class Rational_Test < Test::Unit::TestCase
 
-  def setup
-    @complex = defined?(Complex)
-    if @complex
-      @keiju = Complex.instance_variables.include?(:@RCS_ID)
-    end
-    seps = [File::SEPARATOR, File::ALT_SEPARATOR].compact.map{|x| Regexp.escape(x)}.join("|")
-    @unify = $".grep(/(?:^|#{seps})mathn(?:\.(?:rb|so))?/).size != 0
-  end
-
   def test_ratsub
-    c = RationalSub.__send__(:convert, 1)
-
-    assert_kind_of(Numeric, c)
-
-    if @unify
+    c = RationalSub.__send__(:new, 1)
+    cc = RationalSub.__send__(:convert, 1)
+    if defined?(RationalSub::Unify)
       assert_instance_of(Fixnum, c)
+      assert_instance_of(Fixnum, cc)
     else
       assert_instance_of(RationalSub, c)
+      assert_instance_of(RationalSub, cc)
 
       c2 = c + 1
       assert_instance_of(RationalSub, c2)
@@ -36,25 +27,6 @@ class Rational_Test < Test::Unit::TestCase
       assert_equal(c, c5)
       assert_instance_of(RationalSub, c5)
     end
-
-    c1 = Rational(1)
-    assert_equal(c1.hash, c.hash, '[ruby-dev:38850]')
-    assert_equal([true, true], [c.eql?(c1), c1.eql?(c)])
-  end
-
-  def test_eql_p
-    c = Rational(0)
-    c2 = Rational(0)
-    c3 = Rational(1)
-
-    assert_equal(true, c.eql?(c2))
-    assert_equal(false, c.eql?(c3))
-
-    if @unify
-      assert_equal(true, c.eql?(0))
-    else
-      assert_equal(false, c.eql?(0))
-    end
   end
 
   def test_hash
@@ -76,71 +48,140 @@ class Rational_Test < Test::Unit::TestCase
   def test_freeze
     c = Rational(1)
     c.freeze
-    unless @unify
+    unless defined?(Rational::Unify)
       assert_equal(true, c.frozen?)
     end
     assert_instance_of(String, c.to_s)
   end
 
+  def test_new_bang # no unify & no reduce
+    assert_instance_of(Rational, Rational.__send__(:new!, 2,1))
+    assert_equal([2,1], Rational.__send__(:new!, 2,1).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([2,4],  Rational.__send__(:new!, 2,4).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([-2,4], Rational.__send__(:new!, -2,4).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([-2,4], Rational.__send__(:new!, 2,-4).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([2,4], Rational.__send__(:new!, -2,-4).
+		 instance_eval{[numerator, denominator]})
+
+    # to_i
+    assert_equal([2,1], Rational.__send__(:new!, Rational(2)).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([2,3], Rational.__send__(:new!, Rational(2), Rational(3)).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([2,3], Rational.__send__(:new!, 2, Rational(3)).
+		 instance_eval{[numerator, denominator]})
+
+    assert_equal([1,1], Rational.__send__(:new!, 1.1).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([-1,1], Rational.__send__(:new!, -1.1).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([1,1], Rational.__send__(:new!, '1').
+		 instance_eval{[numerator, denominator]})
+    assert_equal([0,1], Rational.__send__(:new!, nil).
+		 instance_eval{[numerator, denominator]})
+  end
+
+=begin
+  def test_reduce
+    if defined?(Rational::Unify)
+      assert_instance_of(Fixnum, Rational.__send__(:reduce, 2,1))
+    else
+      assert_instance_of(Rational, Rational.__send__(:reduce, 2,1))
+      assert_instance_of(Rational, Rational.__send__(:reduce, 2,1))
+    end
+    assert_equal([2,1], Rational.__send__(:reduce, 2,1).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([1,2], Rational.__send__(:reduce, 2,4).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([-1,2], Rational.__send__(:reduce, -2,4).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([-1,2], Rational.__send__(:reduce, 2,-4).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([1,2], Rational.__send__(:reduce, -2,-4).
+		 instance_eval{[numerator, denominator]})
+
+    assert_raise(ArgumentError){Rational.__send__(:reduce, Rational(1,2),2)}
+    assert_raise(ArgumentError){Rational.__send__(:reduce, 2,Rational(1,2))}
+    assert_raise(ArgumentError){Rational.
+      __send__(:reduce, Rational(1,2),Rational(1,2))}
+
+    assert_raise(ArgumentError){Rational.__send__(:reduce, 1.1)}
+    assert_raise(ArgumentError){Rational.__send__(:reduce, -1.1)}
+    assert_raise(ArgumentError){Rational.__send__(:reduce, '1')}
+    assert_raise(ArgumentError){Rational.__send__(:reduce, nil)}
+  end
+=end
+
+  def test_new
+    if defined?(Rational::Unify)
+      assert_instance_of(Fixnum, Rational.__send__(:new, 2,1))
+    else
+      assert_instance_of(Rational, Rational.__send__(:new, 2,1))
+      assert_equal([2,1], Rational.__send__(:new, 2,1).
+		   instance_eval{[numerator, denominator]})
+    end
+    assert_equal([1,2], Rational.__send__(:new, 2,4).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([-1,2], Rational.__send__(:new, -2,4).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([-1,2], Rational.__send__(:new, 2,-4).
+		 instance_eval{[numerator, denominator]})
+    assert_equal([1,2], Rational.__send__(:new, -2,-4).
+		 instance_eval{[numerator, denominator]})
+
+    assert_raise(ArgumentError){Rational.__send__(:new, Rational(1,2),2)}
+    assert_raise(ArgumentError){Rational.__send__(:new, 2,Rational(1,2))}
+    assert_raise(ArgumentError){Rational.__send__(:new, Rational(1,2),Rational(1,2))}
+
+    assert_raise(ArgumentError){Rational.__send__(:new, 1.1)}
+    assert_raise(ArgumentError){Rational.__send__(:new, -1.1)}
+    assert_raise(ArgumentError){Rational.__send__(:new, '1')}
+    assert_raise(ArgumentError){Rational.__send__(:new, nil)}
+=begin
+    assert_raise(ArgumentError){Rational.__send__(:new, Rational(1))}
+    if defined?(Complex)
+      assert_raise(ArgumentError){Rational.__send__(:new, Complex(1))}
+    end
+=end
+  end
+
   def test_conv
     c = Rational(0,1)
-    assert_equal(Rational(0,1), c)
+    assert_equal(Rational.__send__(:new, 0,1), c)
 
     c = Rational(2**32, 2**32)
-    assert_equal(Rational(2**32,2**32), c)
+    assert_equal(Rational.__send__(:new, 2**32,2**32), c)
     assert_equal([1,1], [c.numerator,c.denominator])
 
     c = Rational(-2**32, 2**32)
-    assert_equal(Rational(-2**32,2**32), c)
+    assert_equal(Rational.__send__(:new, -2**32,2**32), c)
     assert_equal([-1,1], [c.numerator,c.denominator])
 
     c = Rational(2**32, -2**32)
-    assert_equal(Rational(2**32,-2**32), c)
+    assert_equal(Rational.__send__(:new, 2**32,-2**32), c)
     assert_equal([-1,1], [c.numerator,c.denominator])
 
     c = Rational(-2**32, -2**32)
-    assert_equal(Rational(-2**32,-2**32), c)
+    assert_equal(Rational.__send__(:new, -2**32,-2**32), c)
     assert_equal([1,1], [c.numerator,c.denominator])
 
     c = Rational(Rational(1,2),2)
-    assert_equal(Rational(1,4), c)
+    assert_equal(Rational.__send__(:new, 1,4), c)
 
     c = Rational(2,Rational(1,2))
-    assert_equal(Rational(4), c)
+    assert_equal(Rational.__send__(:new, 4), c)
 
     c = Rational(Rational(1,2),Rational(1,2))
-    assert_equal(Rational(1), c)
+    assert_equal(Rational.__send__(:new, 1), c)
 
-    if @complex && !@keiju
-      c = Rational(Complex(1,2),2)
-      assert_equal(Complex(Rational(1,2),1), c)
-
-      c = Rational(2,Complex(1,2))
-      assert_equal(Complex(Rational(2,5),Rational(-4,5)), c)
-
-      c = Rational(Complex(1,2),Complex(1,2))
-      assert_equal(Rational(1), c)
-    end
-
-    assert_equal(Rational(3),Rational(3))
-    assert_equal(Rational(1),Rational(3,3))
-    assert_equal(3.3.to_r,Rational(3.3))
-    assert_equal(1,Rational(3.3,3.3))
-    assert_equal(Rational(3),Rational('3'))
-    assert_equal(Rational(1),Rational('3.0','3.0'))
-    assert_equal(Rational(1),Rational('3/3','3/3'))
-    assert_raise(TypeError){Rational(nil)}
-    assert_raise(ArgumentError){Rational('')}
-    assert_raise(TypeError){Rational(Object.new)}
-    assert_raise(ArgumentError){Rational()}
-    assert_raise(ArgumentError){Rational(1,2,3)}
-
-    if (0.0/0).nan?
-      assert_raise(FloatDomainError){Rational(0.0/0)}
-    end
-    if (1.0/0).infinite?
-      assert_raise(FloatDomainError){Rational(1.0/0)}
-    end
+    assert_equal(Rational.__send__(:new, 1),Rational(1))
+    assert_equal(1.1.to_r,Rational(1.1))
+    assert_equal(Rational.__send__(:new, 1),Rational('1'))
+    assert_raise(ArgumentError){Rational(nil)}
   end
 
   def test_attr
@@ -154,22 +195,22 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal(4, c.numerator)
     assert_equal(5, c.denominator)
 
-    c = Rational(4)
+    c = Rational.__send__(:new, 4)
 
     assert_equal(4, c.numerator)
     assert_equal(1, c.denominator)
 
-    c = Rational(4,5)
+    c = Rational.__send__(:new, 4,5)
 
     assert_equal(4, c.numerator)
     assert_equal(5, c.denominator)
 
-    c = Rational(4)
+    c = Rational.__send__(:new!, 4)
 
     assert_equal(4, c.numerator)
     assert_equal(1, c.denominator)
 
-    c = Rational(4,5)
+    c = Rational.__send__(:new!, 4,5)
 
     assert_equal(4, c.numerator)
     assert_equal(5, c.denominator)
@@ -178,7 +219,8 @@ class Rational_Test < Test::Unit::TestCase
   def test_attr2
     c = Rational(1)
 
-    if @unify
+    if defined?(Rational::Unify)
+      assert_equal(true, c.scalar?)
 =begin
       assert_equal(true, c.finite?)
       assert_equal(false, c.infinite?)
@@ -186,14 +228,13 @@ class Rational_Test < Test::Unit::TestCase
       assert_equal(true, c.integer?)
       assert_equal(false, c.float?)
       assert_equal(true, c.rational?)
-=end
       assert_equal(true, c.real?)
-=begin
       assert_equal(false, c.complex?)
       assert_equal(true, c.exact?)
       assert_equal(false, c.inexact?)
 =end
     else
+      assert_equal(true, c.scalar?)
 =begin
       assert_equal(true, c.finite?)
       assert_equal(false, c.infinite?)
@@ -201,9 +242,7 @@ class Rational_Test < Test::Unit::TestCase
       assert_equal(false, c.integer?)
       assert_equal(false, c.float?)
       assert_equal(true, c.rational?)
-=end
       assert_equal(true, c.real?)
-=begin
       assert_equal(false, c.complex?)
       assert_equal(true, c.exact?)
       assert_equal(false, c.inexact?)
@@ -294,12 +333,6 @@ class Rational_Test < Test::Unit::TestCase
 
     assert_equal(Rational(1,4), c / 2)
     assert_equal(0.25, c / 2.0)
-
-    assert_raise(ZeroDivisionError){Rational(1, 3) / 0}
-    assert_raise(ZeroDivisionError){Rational(1, 3) / Rational(0)}
-
-    assert_equal(0, Rational(1, 3) / Float::INFINITY)
-    assert((Rational(1, 3) / 0.0).infinite?, '[ruby-core:31626]')
   end
 
   def assert_eql(exp, act, *args)
@@ -341,7 +374,7 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal(-2, (-c).div(c2))
     assert_equal(1, (-c).div(-c2))
 
-    unless @unify
+    unless defined?(Rational::Unify)
       c = Rational(11)
       c2 = Rational(3)
 
@@ -352,41 +385,6 @@ class Rational_Test < Test::Unit::TestCase
     end
   end
 
-  def test_modulo
-    c = Rational(1,2)
-    c2 = Rational(2,3)
-
-    assert_eql(Rational(1,2), c.modulo(c2))
-    assert_eql(Rational(1,2), c.modulo(2))
-    assert_eql(0.5, c.modulo(2.0))
-
-    c = Rational(301,100)
-    c2 = Rational(7,5)
-
-    assert_equal(Rational(21,100), c.modulo(c2))
-    assert_equal(Rational(-119,100), c.modulo(-c2))
-    assert_equal(Rational(119,100), (-c).modulo(c2))
-    assert_equal(Rational(-21,100), (-c).modulo(-c2))
-
-    c = Rational(301,100)
-    c2 = Rational(2)
-
-    assert_equal(Rational(101,100), c.modulo(c2))
-    assert_equal(Rational(-99,100), c.modulo(-c2))
-    assert_equal(Rational(99,100), (-c).modulo(c2))
-    assert_equal(Rational(-101,100), (-c).modulo(-c2))
-
-    unless @unify
-      c = Rational(11)
-      c2 = Rational(3)
-
-      assert_equal(2, c.modulo(c2))
-      assert_equal(-1, c.modulo(-c2))
-      assert_equal(1, (-c).modulo(c2))
-      assert_equal(-2, (-c).modulo(-c2))
-    end
-  end
-
   def test_divmod
     c = Rational(1,2)
     c2 = Rational(2,3)
@@ -411,7 +409,7 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal([-2, Rational(99,100)], (-c).divmod(c2))
     assert_equal([1, Rational(-101,100)], (-c).divmod(-c2))
 
-    unless @unify
+    unless defined?(Rational::Unify)
       c = Rational(11)
       c2 = Rational(3)
 
@@ -447,7 +445,7 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal(-1, (-c).quot(c2))
     assert_equal(1, (-c).quot(-c2))
 
-    unless @unify
+    unless defined?(Rational::Unify)
       c = Rational(11)
       c2 = Rational(3)
 
@@ -457,44 +455,7 @@ class Rational_Test < Test::Unit::TestCase
       assert_equal(3, (-c).quot(-c2))
     end
   end
-=end
 
-  def test_remainder
-    c = Rational(1,2)
-    c2 = Rational(2,3)
-
-    assert_eql(Rational(1,2), c.remainder(c2))
-    assert_eql(Rational(1,2), c.remainder(2))
-    assert_eql(0.5, c.remainder(2.0))
-
-    c = Rational(301,100)
-    c2 = Rational(7,5)
-
-    assert_equal(Rational(21,100), c.remainder(c2))
-    assert_equal(Rational(21,100), c.remainder(-c2))
-    assert_equal(Rational(-21,100), (-c).remainder(c2))
-    assert_equal(Rational(-21,100), (-c).remainder(-c2))
-
-    c = Rational(301,100)
-    c2 = Rational(2)
-
-    assert_equal(Rational(101,100), c.remainder(c2))
-    assert_equal(Rational(101,100), c.remainder(-c2))
-    assert_equal(Rational(-101,100), (-c).remainder(c2))
-    assert_equal(Rational(-101,100), (-c).remainder(-c2))
-
-    unless @unify
-      c = Rational(11)
-      c2 = Rational(3)
-
-      assert_equal(2, c.remainder(c2))
-      assert_equal(2, c.remainder(-c2))
-      assert_equal(-2, (-c).remainder(c2))
-      assert_equal(-2, (-c).remainder(-c2))
-    end
-  end
-
-=begin
   def test_quotrem
     c = Rational(1,2)
     c2 = Rational(2,3)
@@ -519,7 +480,7 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal([-1, Rational(-101,100)], (-c).quotrem(c2))
     assert_equal([1, Rational(-101,100)], (-c).quotrem(-c2))
 
-    unless @unify
+    unless defined?(Rational::Unify)
       c = Rational(11)
       c2 = Rational(3)
 
@@ -538,7 +499,7 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal(Rational(3,4), c.quo(c2))
 
     assert_equal(Rational(1,4), c.quo(2))
-    assert_equal(0.25, c.quo(2.0))
+    assert_equal(Rational(0.25), c.quo(2.0))
   end
 
   def test_fdiv
@@ -549,8 +510,6 @@ class Rational_Test < Test::Unit::TestCase
 
     assert_equal(0.25, c.fdiv(2))
     assert_equal(0.25, c.fdiv(2.0))
-    assert_equal(0, c.fdiv(Float::INFINITY))
-    assert(c.fdiv(0).infinite?, '[ruby-core:31626]')
   end
 
   def test_expt
@@ -578,7 +537,7 @@ class Rational_Test < Test::Unit::TestCase
     # p ** p
     x = 2 ** Rational(2)
     assert_equal(Rational(4), x)
-    unless @unify
+    unless defined?(Rational::Unify)
       assert_instance_of(Rational, x)
     end
     assert_equal(4, x.numerator)
@@ -586,7 +545,7 @@ class Rational_Test < Test::Unit::TestCase
 
     x = Rational(2) ** 2
     assert_equal(Rational(4), x)
-    unless @unify
+    unless defined?(Rational::Unify)
       assert_instance_of(Rational, x)
     end
     assert_equal(4, x.numerator)
@@ -594,7 +553,7 @@ class Rational_Test < Test::Unit::TestCase
 
     x = Rational(2) ** Rational(2)
     assert_equal(Rational(4), x)
-    unless @unify
+    unless defined?(Rational::Unify)
       assert_instance_of(Rational, x)
     end
     assert_equal(4, x.numerator)
@@ -603,7 +562,7 @@ class Rational_Test < Test::Unit::TestCase
     # -p ** p
     x = (-2) ** Rational(2)
     assert_equal(Rational(4), x)
-    unless @unify
+    unless defined?(Rational::Unify)
       assert_instance_of(Rational, x)
     end
     assert_equal(4, x.numerator)
@@ -611,7 +570,7 @@ class Rational_Test < Test::Unit::TestCase
 
     x = Rational(-2) ** 2
     assert_equal(Rational(4), x)
-    unless @unify
+    unless defined?(Rational::Unify)
       assert_instance_of(Rational, x)
     end
     assert_equal(4, x.numerator)
@@ -619,7 +578,7 @@ class Rational_Test < Test::Unit::TestCase
 
     x = Rational(-2) ** Rational(2)
     assert_equal(Rational(4), x)
-    unless @unify
+    unless defined?(Rational::Unify)
       assert_instance_of(Rational, x)
     end
     assert_equal(4, x.numerator)
@@ -662,10 +621,6 @@ class Rational_Test < Test::Unit::TestCase
     assert_instance_of(Rational, x)
     assert_equal(1, x.numerator)
     assert_equal(4, x.denominator)
-
-    unless @unify # maybe bug mathn
-      assert_raise(ZeroDivisionError){0 ** -1}
-    end
   end
 
   def test_cmp
@@ -702,53 +657,31 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal(true, Rational(0) <= Rational(0))
     assert_equal(true, Rational(0) >= Rational(0))
     assert_equal(false, Rational(0) > Rational(0))
-
-    assert_equal(nil, Rational(0) <=> nil)
-    assert_equal(nil, Rational(0) <=> 'foo')
   end
 
-  def test_eqeq
+  def test_equal
     assert(Rational(1,1) == Rational(1))
+    assert(Rational(1,1) == Rational.__send__(:new, 1))
+    assert(Rational(1,1) == Rational.__send__(:new, 1,1))
+    assert(Rational(1,1) == Rational.__send__(:new!, 1))
+    assert(Rational(1,1) == Rational.__send__(:new!, 1,1))
+
     assert(Rational(-1,1) == Rational(-1))
+    assert(Rational(-1,1) == Rational.__send__(:new, -1))
+    assert(Rational(-1,1) == Rational.__send__(:new, -1,1))
+    assert(Rational(-1,1) == Rational.__send__(:new!, -1))
+    assert(Rational(-1,1) == Rational.__send__(:new!, -1,1))
 
     assert_equal(false, Rational(2,1) == Rational(1))
     assert_equal(true, Rational(2,1) != Rational(1))
     assert_equal(false, Rational(1) == nil)
     assert_equal(false, Rational(1) == '')
-  end
 
-  def test_coerce
-    assert_equal([Rational(2),Rational(1)], Rational(1).coerce(2))
-    assert_equal([Rational(2.2),Rational(1)], Rational(1).coerce(2.2))
-    assert_equal([Rational(2),Rational(1)], Rational(1).coerce(Rational(2)))
-
-    assert_nothing_raised(TypeError, '[Bug #5020] [ruby-devl:44088]') do
-      Rational(1,2).coerce(Complex(1,1))
-    end
-  end
-
-  class ObjectX
-    def + (x) Rational(1) end
-    alias - +
-    alias * +
-    alias / +
-    alias quo +
-    alias div +
-    alias % +
-    alias remainder +
-    alias ** +
-    def coerce(x) [x, Rational(1)] end
-  end
-
-  def test_coerce2
-    x = ObjectX.new
-    %w(+ - * / quo div % remainder **).each do |op|
-      assert_kind_of(Numeric, Rational(1).__send__(op, x))
-    end
+    assert_equal(false, Rational(1,2**100) == 1)
   end
 
   def test_unify
-    if @unify
+    if defined?(Rational::Unify)
       assert_instance_of(Fixnum, Rational(1,2) + Rational(1,2))
       assert_instance_of(Fixnum, Rational(1,2) - Rational(1,2))
       assert_instance_of(Fixnum, Rational(1,2) * 2)
@@ -759,46 +692,14 @@ class Rational_Test < Test::Unit::TestCase
     end
   end
 
-  def test_math
-    assert_equal(Rational(1,2), Rational(1,2).abs)
-    assert_equal(Rational(1,2), Rational(-1,2).abs)
-    if @complex && !@keiju
-      assert_equal(Rational(1,2), Rational(1,2).magnitude)
-      assert_equal(Rational(1,2), Rational(-1,2).magnitude)
-    end
-
-    assert_equal(1, Rational(1,2).numerator)
-    assert_equal(2, Rational(1,2).denominator)
-  end
-
-  def test_trunc
-    [[Rational(13, 5),  [ 2,  3,  2,  3]], #  2.6
-     [Rational(5, 2),   [ 2,  3,  2,  3]], #  2.5
-     [Rational(12, 5),  [ 2,  3,  2,  2]], #  2.4
-     [Rational(-12,5),  [-3, -2, -2, -2]], # -2.4
-     [Rational(-5, 2),  [-3, -2, -2, -3]], # -2.5
-     [Rational(-13, 5), [-3, -2, -2, -3]], # -2.6
-    ].each do |i, a|
-      assert_equal(a[0], i.floor)
-      assert_equal(a[1], i.ceil)
-      assert_equal(a[2], i.truncate)
-      assert_equal(a[3], i.round)
-    end
-  end
-
   def test_to_s
     c = Rational(1,2)
 
     assert_instance_of(String, c.to_s)
     assert_equal('1/2', c.to_s)
 
-    if @unify
-      assert_equal('0', Rational(0,2).to_s)
-      assert_equal('0', Rational(0,-2).to_s)
-    else
-      assert_equal('0/1', Rational(0,2).to_s)
-      assert_equal('0/1', Rational(0,-2).to_s)
-    end
+    assert_equal('0', Rational(0,2).to_s)
+    assert_equal('0', Rational(0,-2).to_s)
     assert_equal('1/2', Rational(1,2).to_s)
     assert_equal('-1/2', Rational(-1,2).to_s)
     assert_equal('1/2', Rational(-1,-2).to_s)
@@ -810,61 +711,43 @@ class Rational_Test < Test::Unit::TestCase
     c = Rational(1,2)
 
     assert_instance_of(String, c.inspect)
-    assert_equal('(1/2)', c.inspect)
+    assert_equal('Rational(1, 2)', c.inspect)
   end
 
   def test_marshal
     c = Rational(1,2)
-    c.instance_eval{@ivar = 9}
 
     s = Marshal.dump(c)
     c2 = Marshal.load(s)
     assert_equal(c, c2)
-    assert_equal(9, c2.instance_variable_get(:@ivar))
     assert_instance_of(Rational, c2)
-
-    assert_raise(ZeroDivisionError){
-      Marshal.load("\x04\bU:\rRational[\ai\x06i\x05")
-    }
-
-    bug3656 = '[ruby-core:31622]'
-    assert_raise(TypeError, bug3656) {
-      Rational(1,2).marshal_load(0)
-    }
   end
 
   def test_parse
+    assert_equal(Rational(0), ''.to_r)
+    assert_equal(Rational(0), ' '.to_r)
     assert_equal(Rational(5), '5'.to_r)
     assert_equal(Rational(-5), '-5'.to_r)
     assert_equal(Rational(5,3), '5/3'.to_r)
     assert_equal(Rational(-5,3), '-5/3'.to_r)
-#    assert_equal(Rational(5,-3), '5/-3'.to_r)
-#    assert_equal(Rational(-5,-3), '-5/-3'.to_r)
+    assert_equal(Rational(5,-3), '5/-3'.to_r)
+    assert_equal(Rational(-5,-3), '-5/-3'.to_r)
 
     assert_equal(Rational(5), '5.0'.to_r)
     assert_equal(Rational(-5), '-5.0'.to_r)
     assert_equal(Rational(5,3), '5.0/3'.to_r)
     assert_equal(Rational(-5,3), '-5.0/3'.to_r)
-#    assert_equal(Rational(5,-3), '5.0/-3'.to_r)
-#    assert_equal(Rational(-5,-3), '-5.0/-3'.to_r)
+    assert_equal(Rational(5,-3), '5.0/-3'.to_r)
+    assert_equal(Rational(-5,-3), '-5.0/-3'.to_r)
 
     assert_equal(Rational(5), '5e0'.to_r)
     assert_equal(Rational(-5), '-5e0'.to_r)
     assert_equal(Rational(5,3), '5e0/3'.to_r)
     assert_equal(Rational(-5,3), '-5e0/3'.to_r)
-#    assert_equal(Rational(5,-3), '5e0/-3'.to_r)
-#    assert_equal(Rational(-5,-3), '-5e0/-3'.to_r)
+    assert_equal(Rational(5,-3), '5e0/-3'.to_r)
+    assert_equal(Rational(-5,-3), '-5e0/-3'.to_r)
 
-    assert_equal(Rational(5e1), '5e1'.to_r)
-    assert_equal(Rational(-5e2), '-5e2'.to_r)
-    assert_equal(Rational(5e3,3), '5e003/3'.to_r)
-    assert_equal(Rational(-5e4,3), '-5e004/3'.to_r)
-#    assert_equal(Rational(5e1,-3), '5e1/-3'.to_r)
-#    assert_equal(Rational(-5e2,-3), '-5e2/-3'.to_r)
-
-    assert_equal(Rational(33,100), '.33'.to_r)
     assert_equal(Rational(33,100), '0.33'.to_r)
-    assert_equal(Rational(-33,100), '-.33'.to_r)
     assert_equal(Rational(-33,100), '-0.33'.to_r)
     assert_equal(Rational(-33,100), '-0.3_3'.to_r)
 
@@ -882,49 +765,27 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal(Rational(-5), Rational('-5'))
     assert_equal(Rational(5,3), Rational('5/3'))
     assert_equal(Rational(-5,3), Rational('-5/3'))
-#    assert_equal(Rational(5,-3), Rational('5/-3'))
-#    assert_equal(Rational(-5,-3), Rational('-5/-3'))
+    assert_equal(Rational(5,-3), Rational('5/-3'))
+    assert_equal(Rational(-5,-3), Rational('-5/-3'))
 
-    assert_equal(Rational(5), Rational('5.0'))
-    assert_equal(Rational(-5), Rational('-5.0'))
-    assert_equal(Rational(5,3), Rational('5.0/3'))
-    assert_equal(Rational(-5,3), Rational('-5.0/3'))
-#    assert_equal(Rational(5,-3), Rational('5.0/-3'))
-#    assert_equal(Rational(-5,-3), Rational('-5.0/-3'))
-
-    assert_equal(Rational(5), Rational('5e0'))
-    assert_equal(Rational(-5), Rational('-5e0'))
-    assert_equal(Rational(5,3), Rational('5e0/3'))
-    assert_equal(Rational(-5,3), Rational('-5e0/3'))
-#    assert_equal(Rational(5,-3), Rational('5e0/-3'))
-#    assert_equal(Rational(-5,-3), Rational('-5e0/-3'))
-
-    assert_equal(Rational(5e1), Rational('5e1'))
-    assert_equal(Rational(-5e2), Rational('-5e2'))
-    assert_equal(Rational(5e3,3), Rational('5e003/3'))
-    assert_equal(Rational(-5e4,3), Rational('-5e004/3'))
-#    assert_equal(Rational(5e1,-3), Rational('5e1/-3'))
-#    assert_equal(Rational(-5e2,-3), Rational('-5e2/-3'))
-
-    assert_equal(Rational(33,100), Rational('.33'))
     assert_equal(Rational(33,100), Rational('0.33'))
-    assert_equal(Rational(-33,100), Rational('-.33'))
     assert_equal(Rational(-33,100), Rational('-0.33'))
     assert_equal(Rational(-33,100), Rational('-0.3_3'))
 
     assert_equal(Rational(1,2), Rational('5e-1'))
-    assert_equal(Rational(50), Rational('5e+1'))
-    assert_equal(Rational(1,2), Rational('5.0e-1'))
-    assert_equal(Rational(50), Rational('5.0e+1'))
     assert_equal(Rational(50), Rational('5e1'))
     assert_equal(Rational(50), Rational('5E1'))
     assert_equal(Rational(500), Rational('5e2'))
     assert_equal(Rational(5000), Rational('5e3'))
     assert_equal(Rational(500000000000), Rational('5e1_1'))
 
-    assert_equal(Rational(0), ''.to_r)
-    assert_equal(Rational(0), ' '.to_r)
-    assert_equal(Rational(5), "\f\n\r\t\v5\0".to_r)
+    assert_equal(Rational(5.0), Rational('5.0'))
+    assert_equal(Rational(-5.0), Rational('-5.0'))
+    assert_equal(Rational(5.0,3), Rational('5.0/3'))
+    assert_equal(Rational(-5.0,3), Rational('-5.0/3'))
+    assert_equal(Rational(5.0,-3), Rational('5.0/-3'))
+    assert_equal(Rational(-5.0,-3), Rational('-5.0/-3'))
+
     assert_equal(Rational(0), '_'.to_r)
     assert_equal(Rational(0), '_5'.to_r)
     assert_equal(Rational(5), '5_'.to_r)
@@ -935,7 +796,6 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal(Rational(5,3), '5/3x'.to_r)
     assert_raise(ArgumentError){ Rational('')}
     assert_raise(ArgumentError){ Rational('_')}
-    assert_raise(ArgumentError){ Rational("\f\n\r\t\v5\0")}
     assert_raise(ArgumentError){ Rational('_5')}
     assert_raise(ArgumentError){ Rational('5_')}
     assert_raise(ArgumentError){ Rational('5x')}
@@ -951,10 +811,6 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal(Rational(9,1), Rational(1,9).reciprocal)
     assert_equal(Rational(-1,9), Rational(-9,1).reciprocal)
     assert_equal(Rational(-9,1), Rational(-1,9).reciprocal)
-    assert_equal(Rational(1,9), Rational(9,1).inverse)
-    assert_equal(Rational(9,1), Rational(1,9).inverse)
-    assert_equal(Rational(-1,9), Rational(-9,1).inverse)
-    assert_equal(Rational(-9,1), Rational(-1,9).inverse)
   end
 =end
 
@@ -969,8 +825,8 @@ class Rational_Test < Test::Unit::TestCase
   end
 
   def test_to_c
-    if @complex && !@keiju
-      if @unify
+    if defined?(Complex) && !Complex.instance_variable_get('@RCS_ID')
+      if defined?(Rational::Unify)
 	assert_equal(Rational(3,2), Rational(3,2).to_c)
 	assert_equal(Rational(3,2), Complex(Rational(3,2)))
       else
@@ -982,112 +838,81 @@ class Rational_Test < Test::Unit::TestCase
 
   def test_to_r
     c = nil.to_r
-    assert_equal([0,1], [c.numerator, c.denominator])
+    assert_equal([0,1] , [c.numerator, c.denominator])
 
     c = 0.to_r
-    assert_equal([0,1], [c.numerator, c.denominator])
+    assert_equal([0,1] , [c.numerator, c.denominator])
 
     c = 1.to_r
-    assert_equal([1,1], [c.numerator, c.denominator])
+    assert_equal([1,1] , [c.numerator, c.denominator])
 
     c = 1.1.to_r
     assert_equal([2476979795053773, 2251799813685248],
 		 [c.numerator, c.denominator])
 
     c = Rational(1,2).to_r
-    assert_equal([1,2], [c.numerator, c.denominator])
+    assert_equal([1,2] , [c.numerator, c.denominator])
 
-    if @complex
-      if @keiju
+    if defined?(Complex)
+      if Complex.instance_variable_get('@RCS_ID')
 	assert_raise(NoMethodError){Complex(1,2).to_r}
       else
 	assert_raise(RangeError){Complex(1,2).to_r}
       end
     end
-
-    if (0.0/0).nan?
-      assert_raise(FloatDomainError){(0.0/0).to_r}
-    end
-    if (1.0/0).infinite?
-      assert_raise(FloatDomainError){(1.0/0).to_r}
-    end
   end
 
-  def test_rationalize
-    c = nil.rationalize
-    assert_equal([0,1], [c.numerator, c.denominator])
+  def test_prec
+    assert_equal(true, Rational < Precision)
 
-    c = 0.rationalize
-    assert_equal([0,1], [c.numerator, c.denominator])
+    c = Rational(3,2)
 
-    c = 1.rationalize
-    assert_equal([1,1], [c.numerator, c.denominator])
-
-    c = 1.1.rationalize
-    assert_equal([11, 10], [c.numerator, c.denominator])
-
-    c = Rational(1,2).rationalize
-    assert_equal([1,2], [c.numerator, c.denominator])
-
-    assert_equal(nil.rationalize(Rational(1,10)), Rational(0))
-    assert_equal(0.rationalize(Rational(1,10)), Rational(0))
-    assert_equal(10.rationalize(Rational(1,10)), Rational(10))
-
-    r = 0.3333
-    assert_equal(r.rationalize, Rational(3333, 10000))
-    assert_equal(r.rationalize(Rational(1,10)), Rational(1,3))
-    assert_equal(r.rationalize(Rational(-1,10)), Rational(1,3))
-
-    r = Rational(5404319552844595,18014398509481984)
-    assert_equal(r.rationalize, r)
-    assert_equal(r.rationalize(Rational(1,10)), Rational(1,3))
-    assert_equal(r.rationalize(Rational(-1,10)), Rational(1,3))
-
-    r = -0.3333
-    assert_equal(r.rationalize, Rational(-3333, 10000))
-    assert_equal(r.rationalize(Rational(1,10)), Rational(-1,3))
-    assert_equal(r.rationalize(Rational(-1,10)), Rational(-1,3))
-
-    r = Rational(-5404319552844595,18014398509481984)
-    assert_equal(r.rationalize, r)
-    assert_equal(r.rationalize(Rational(1,10)), Rational(-1,3))
-    assert_equal(r.rationalize(Rational(-1,10)), Rational(-1,3))
-
-    if @complex
-      if @keiju
-      else
-	assert_raise(RangeError){Complex(1,2).rationalize}
-      end
-    end
-
-    if (0.0/0).nan?
-      assert_raise(FloatDomainError){(0.0/0).rationalize}
-    end
-    if (1.0/0).infinite?
-      assert_raise(FloatDomainError){(1.0/0).rationalize}
-    end
-  end
-
-  def test_gcdlcm
-    assert_equal(7, 91.gcd(-49))
-    assert_equal(5, 5.gcd(0))
-    assert_equal(5, 0.gcd(5))
-    assert_equal(70, 14.lcm(35))
-    assert_equal(0, 5.lcm(0))
-    assert_equal(0, 0.lcm(5))
-    assert_equal([5,0], 0.gcdlcm(5))
-    assert_equal([5,0], 5.gcdlcm(0))
-
-    assert_equal(1, 1073741827.gcd(1073741789))
-    assert_equal(1152921470247108503, 1073741827.lcm(1073741789))
-
-    assert_equal(1, 1073741789.gcd(1073741827))
-    assert_equal(1152921470247108503, 1073741789.lcm(1073741827))
+    assert_eql(1, c.prec(Integer))
+    assert_eql(1.5, c.prec(Float))
+    assert_eql(c, c.prec(Rational))
   end
 
   def test_supp
-    assert_equal(true, 1.real?)
-    assert_equal(true, 1.1.real?)
+    assert_equal(true, 1.scalar?)
+    assert_equal(true, 1.1.scalar?)
+
+    if defined?(Complex)
+      assert_equal(1, 1.real)
+      assert_equal(0, 1.image)
+      assert_equal(0, 1.imag)
+
+      assert_equal(1.1, 1.1.real)
+      assert_equal(0, 1.1.image)
+      assert_equal(0, 1.1.imag)
+
+      assert_equal(0, 1.arg)
+      assert_equal(0, 1.angle)
+
+      assert_equal(0, 1.0.arg)
+      assert_equal(0, 1.0.angle)
+
+      assert_equal(Math::PI, -1.arg)
+      assert_equal(Math::PI, -1.angle)
+
+      assert_equal(Math::PI, -1.0.arg)
+      assert_equal(Math::PI, -1.0.angle)
+
+      assert_equal([1,0], 1.polar)
+      assert_equal([1, Math::PI], -1.polar)
+
+      assert_equal([1.0,0], 1.0.polar)
+      assert_equal([1.0, Math::PI], -1.0.polar)
+
+      assert_equal(1, 1.conjugate)
+      assert_equal(-1, -1.conjugate)
+      assert_equal(1, 1.conj)
+      assert_equal(-1, -1.conj)
+
+      assert_equal(1.1, 1.1.conjugate)
+      assert_equal(-1.1, -1.1.conjugate)
+      assert_equal(1.1, 1.1.conj)
+      assert_equal(-1.1, -1.1.conj)
+    end
 
     assert_equal(1, 1.numerator)
     assert_equal(9, 9.numerator)
@@ -1101,17 +926,15 @@ class Rational_Test < Test::Unit::TestCase
 
 =begin
     assert_equal(Rational(1,9), 9.reciprocal)
-    assert_in_delta(0.1111, 9.0.reciprocal, 0.001)
+    assert_equal(Rational(1,9), 9.0.reciprocal)
     assert_equal(Rational(1,9), 9.inverse)
-    assert_in_delta(0.1111, 9.0.inverse, 0.001)
+    assert_equal(Rational(1,9), 9.0.inverse)
 =end
 
     assert_equal(Rational(1,2), 1.quo(2))
     assert_equal(Rational(5000000000), 10000000000.quo(2))
-    assert_equal(0.5, 1.0.quo(2))
+    assert_equal(Rational(1,2), 1.0.quo(2))
     assert_equal(Rational(1,4), Rational(1,2).quo(2))
-    assert_equal(0, Rational(1,2).quo(Float::INFINITY))
-    assert(Rational(1,2).quo(0.0).infinite?, '[ruby-core:31626]')
 
     assert_equal(0.5, 1.fdiv(2))
     assert_equal(5000000000.0, 10000000000.fdiv(2))
@@ -1119,13 +942,110 @@ class Rational_Test < Test::Unit::TestCase
     assert_equal(0.25, Rational(1,2).fdiv(2))
   end
 
-  def test_ruby19
-    assert_raise(NoMethodError){ Rational.new(1) }
-    assert_raise(NoMethodError){ Rational.new!(1) }
+  def test_zero_div
+    assert_raise(ZeroDivisionError) { Rational(1, 0) }
+    assert_raise(ZeroDivisionError) { Rational(1, 1) / 0 }
+  end
+
+  def test_gcd
+    assert_equal(0, Rational(0, 2**100))
+  end
+
+  def test_unify2
+    f = defined?(Rational::Unify)
+    Rational.const_set(:Unify, true) unless f
+
+    assert_same(42, Rational(84, 2))
+    assert_same(1, Rational(1, 2) + Rational(1, 2))
+
+    Rational.instance_eval { remove_const(:Unify) } unless f
+  end
+
+  def test_coerce
+    r = Rational(7, 3)
+    assert_equal(Rational(42, 1), r.coerce(42).first)
+    assert_raise(TypeError) { r.coerce(Object.new) }
+
+    o = Object.new
+    def o.coerce(x); [x.numerator, x.denominator]; end
+    assert_equal(10, r + o)
+    assert_equal(4, r - o)
+    assert_equal(21, r * o)
+    assert_equal(2, r / o)
+    assert_equal(343, r ** o)
+    assert_equal(1, r <=> o)
+
+    b = 2**100
+    def b.<=>(x); 0; end rescue nil
+    assert_equal(1, r ** b)
+    b = 2**100
+    def b.**(x); -1; end rescue nil
+    assert_equal(-1, Rational(1, b)**3)
+  end
+
+  def test_modulo_remainder
+    assert_equal(Rational(1, 2), Rational(5, 2).modulo(1))
+    assert_equal(Rational(1, 2), Rational(5, 2).modulo(2))
+    assert_equal(Rational(5, 2), Rational(5, 2).modulo(3))
+    assert_equal(Rational(5, 6), Rational(5, 2).modulo(Rational(5, 3)))
+    assert_equal(Rational(1, 2), Rational(-5, 2).modulo(1))
+    assert_equal(Rational(-1, 2), Rational(5, 2).modulo(-1))
+    assert_equal(Rational(-1, 2), Rational(-5, 2).modulo(-1))
+
+    assert_equal(Rational(1, 2), Rational(5, 2).remainder(1))
+    assert_equal(Rational(1, 2), Rational(5, 2).remainder(2))
+    assert_equal(Rational(5, 2), Rational(5, 2).remainder(3))
+    assert_equal(Rational(5, 6), Rational(5, 2).remainder(Rational(5, 3)))
+    assert_equal(Rational(-1, 2), Rational(-5, 2).remainder(1))
+    assert_equal(Rational(1, 2), Rational(5, 2).remainder(-1))
+    assert_equal(Rational(-1, 2), Rational(-5, 2).remainder(-1))
+  end
+
+  def test_abs
+    assert_equal(Rational(1, 2), Rational(1, 2).abs)
+    assert_equal(Rational(1, 2), Rational(-1, 2).abs)
+  end
+
+  def test_floor_ceil_truncate_round
+    assert_equal( 2, Rational( 5, 2).floor)
+    assert_equal(-3, Rational(-5, 2).floor)
+    assert_equal( 3, Rational( 5, 2).ceil)
+    assert_equal(-2, Rational(-5, 2).ceil)
+    assert_equal( 2, Rational( 5, 2).truncate)
+    assert_equal(-2, Rational(-5, 2).truncate)
+    assert_equal( 3, Rational( 5, 2).round)
+    assert_equal(-3, Rational(-5, 2).round)
+    assert_equal( 1, Rational( 4, 3).round)
+    assert_equal(-1, Rational(-4, 3).round)
+    assert_equal( 2, Rational( 5, 3).round)
+    assert_equal(-2, Rational(-5, 3).round)
+  end
+
+  def test_convert
+    assert_equal(Rational(1, 2), Rational(Complex(1, 0), 2))
+    assert_raise(RangeError) { Rational(Complex(1, 1), 1) }
+    assert_equal(Rational(1, 2), Rational(1, Complex(2, 0)))
+    assert_raise(RangeError) { Rational(1, Complex(2, 1)) }
+    assert_equal(Rational(1, 2), Rational(0.25, 0.5))
+    assert_equal(Rational(1, 2), Rational('1', '2'))
+  end
+
+  def test_add2
+    assert_equal(Rational(2**100, 3), Rational(0, 1) + Rational(2**100, 3))
+    assert_equal(Rational(2, 3**100), Rational(0, 1) + Rational(2, 3**100))
+  end
+
+  def test_div2
+    assert_raise(ZeroDivisionError) { Rational(1, 1) / Rational(0, 1) }
+  end
+
+  def test_to_f2
+    assert_equal(1, Rational(2**5000,3).to_f.infinite?)
+    assert_equal(0, Rational(1, 2**5000).to_f)
   end
 
   def test_fixed_bug
-    if @unify
+    if defined?(Rational::Unify)
       assert_instance_of(Fixnum, Rational(1,2) ** 0) # mathn's bug
     end
 
