@@ -1,6 +1,6 @@
 require 'test/unit'
-require 'envutil'
 require 'tmpdir'
+require_relative 'envutil'
 
 class TestSystem < Test::Unit::TestCase
   def valid_syntax?(code, fname)
