@@ -54,4 +54,25 @@ class TestVariable < Test::Unit::TestCase
     assert_equal("Zeus", atlas.ruler3)
     assert_equal("Cronus", atlas.ruler4)
   end
+
+  def test_local_variables
+    lvar = 1
+    assert_instance_of(Symbol, local_variables[0], "[ruby-dev:34008]")
+  end
+
+  def test_local_variables2
+    x = 1
+    proc do |y|
+      assert_equal([:x, :y], local_variables.sort)
+    end.call
+  end
+
+  def test_local_variables3
+    x = 1
+    proc do |y|
+      1.times do |z|
+        assert_equal([:x, :y, :z], local_variables.sort)
+      end
+    end.call
+  end
 end
