@@ -1,8 +1,22 @@
-require 'rubygems/test_case'
+require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
+require 'test/unit'
 require 'rubygems/platform'
 require 'rbconfig'
 
-class TestGemPlatform < Gem::TestCase
+class TestGemPlatform < RubyGemTestCase
+
+  def test_self_const_missing
+    consts = [:DARWIN, :LINUX_586, :MSWIN32, :PPC_DARWIN, :WIN32, :X86_LINUX]
+
+    consts.each do |const|
+      e = assert_raise NameError do
+        Gem::Platform.const_missing const
+      end
+
+      assert_equal "#{const} has been removed, use CURRENT instead",
+                   e.message
+    end
+  end
 
   def test_self_local
     util_set_arch 'i686-darwin8.10.1'
@@ -30,9 +44,6 @@ class TestGemPlatform < Gem::TestCase
       'hppa2.0w-hpux11.31'     => ['hppa2.0w',  'hpux',      '11'],
       'java'                   => [nil,         'java',      nil],
       'jruby'                  => [nil,         'java',      nil],
-      'universal-dotnet'       => ['universal', 'dotnet',    nil],
-      'universal-dotnet2.0'    => ['universal', 'dotnet',  '2.0'],
-      'universal-dotnet4.0'    => ['universal', 'dotnet',  '4.0'],
       'powerpc-aix5.3.0.0'     => ['powerpc',   'aix',       '5'],
       'powerpc-darwin7'        => ['powerpc',   'darwin',    '7'],
       'powerpc-darwin8'        => ['powerpc',   'darwin',    '8'],
@@ -94,17 +105,11 @@ class TestGemPlatform < Gem::TestCase
     platform = Gem::Platform.new 'i386-mswin32-80'
 
     assert_equal expected, platform.to_a, 'i386-mswin32-80'
-
-    expected = ['x86', 'solaris', '2.10']
-
-    platform = Gem::Platform.new 'i386-solaris-2.10'
-
-    assert_equal expected, platform.to_a, 'i386-solaris-2.10'
   end
 
   def test_initialize_mswin32_vc6
-    orig_RUBY_SO_NAME = RbConfig::CONFIG['RUBY_SO_NAME']
-    RbConfig::CONFIG['RUBY_SO_NAME'] = 'msvcrt-ruby18'
+    orig_RUBY_SO_NAME = Config::CONFIG['RUBY_SO_NAME']
+    Config::CONFIG['RUBY_SO_NAME'] = 'msvcrt-ruby18'
 
     expected = ['x86', 'mswin32', nil]
 
@@ -112,11 +117,12 @@ class TestGemPlatform < Gem::TestCase
 
     assert_equal expected, platform.to_a, 'i386-mswin32 VC6'
   ensure
-    RbConfig::CONFIG['RUBY_SO_NAME'] = orig_RUBY_SO_NAME
+    Config::CONFIG['RUBY_SO_NAME'] = orig_RUBY_SO_NAME
   end
 
   def test_initialize_platform
     platform = Gem::Platform.new 'cpu-my_platform1'
+    expected = Gem::Platform.new platform
 
     assert_equal 'cpu', platform.cpu
     assert_equal 'my_platform', platform.os
@@ -135,12 +141,6 @@ class TestGemPlatform < Gem::TestCase
     assert_equal '1', platform.version
   end
 
-  def test_empty
-    platform = Gem::Platform.new 'cpu-other_platform1'
-    assert_respond_to platform, :empty?
-    assert_equal false, Gem::Deprecate.skip_during { platform.empty? }
-  end
-
   def test_to_s
     if win_platform? then
       assert_equal 'x86-mswin32-60', Gem::Platform.local.to_s
@@ -154,8 +154,8 @@ class TestGemPlatform < Gem::TestCase
     other = Gem::Platform.new %w[cpu other_platform 1]
 
     assert_equal my, my
-    refute_equal my, other
-    refute_equal other, my
+    assert_not_equal my, other
+    assert_not_equal other, my
   end
 
   def test_equals3
@@ -163,8 +163,8 @@ class TestGemPlatform < Gem::TestCase
     other = Gem::Platform.new %w[cpu other_platform 1]
 
     assert(my === my)
-    refute(other === my)
-    refute(my === other)
+    assert !(other === my)
+    assert !(my === other)
   end
 
   def test_equals3_cpu
@@ -175,10 +175,10 @@ class TestGemPlatform < Gem::TestCase
     util_set_arch 'powerpc-darwin8'
     assert((ppc_darwin8 === Gem::Platform.local), 'powerpc =~ universal')
     assert((uni_darwin8 === Gem::Platform.local), 'powerpc =~ universal')
-    refute((x86_darwin8 === Gem::Platform.local), 'powerpc =~ universal')
+    assert !(x86_darwin8 === Gem::Platform.local), 'powerpc =~ universal'
 
     util_set_arch 'i686-darwin8'
-    refute((ppc_darwin8 === Gem::Platform.local), 'powerpc =~ universal')
+    assert !(ppc_darwin8 === Gem::Platform.local), 'powerpc =~ universal'
     assert((uni_darwin8 === Gem::Platform.local), 'x86 =~ universal')
     assert((x86_darwin8 === Gem::Platform.local), 'powerpc =~ universal')
 
@@ -199,8 +199,8 @@ class TestGemPlatform < Gem::TestCase
     assert((x86_darwin  === Gem::Platform.local), 'x86_darwin === x86_darwin8')
     assert((x86_darwin8 === Gem::Platform.local), 'x86_darwin8 === x86_darwin8')
 
-    refute((x86_darwin7 === Gem::Platform.local), 'x86_darwin7 === x86_darwin8')
-    refute((x86_darwin9 === Gem::Platform.local), 'x86_darwin9 === x86_darwin8')
+    assert !(x86_darwin7 === Gem::Platform.local), 'x86_darwin7 === x86_darwin8'
+    assert !(x86_darwin9 === Gem::Platform.local), 'x86_darwin9 === x86_darwin8'
   end
 
   def test_equals_tilde
@@ -212,10 +212,10 @@ class TestGemPlatform < Gem::TestCase
     # oddballs
     assert_match 'i386-mswin32-mq5.3', Gem::Platform.local
     assert_match 'i386-mswin32-mq6',   Gem::Platform.local
-    refute_match 'win32-1.8.2-VC7',    Gem::Platform.local
-    refute_match 'win32-1.8.4-VC6',    Gem::Platform.local
-    refute_match 'win32-source',       Gem::Platform.local
-    refute_match 'windows',            Gem::Platform.local
+    deny_match   'win32-1.8.2-VC7',    Gem::Platform.local
+    deny_match   'win32-1.8.4-VC6',    Gem::Platform.local
+    deny_match   'win32-source',       Gem::Platform.local
+    deny_match   'windows',            Gem::Platform.local
 
     util_set_arch 'i686-linux'
     assert_match 'i486-linux', Gem::Platform.local
@@ -230,22 +230,6 @@ class TestGemPlatform < Gem::TestCase
     assert_match 'java',  Gem::Platform.local
     assert_match 'jruby', Gem::Platform.local
 
-    util_set_arch 'universal-dotnet2.0'
-    assert_match 'universal-dotnet',     Gem::Platform.local
-    assert_match 'universal-dotnet-2.0', Gem::Platform.local
-    refute_match 'universal-dotnet-4.0', Gem::Platform.local
-    assert_match 'dotnet',               Gem::Platform.local
-    assert_match 'dotnet-2.0',           Gem::Platform.local
-    refute_match 'dotnet-4.0',           Gem::Platform.local
-
-    util_set_arch 'universal-dotnet4.0'
-    assert_match 'universal-dotnet',      Gem::Platform.local
-    refute_match 'universal-dotnet-2.0',  Gem::Platform.local
-    assert_match 'universal-dotnet-4.0',  Gem::Platform.local
-    assert_match 'dotnet',                Gem::Platform.local
-    refute_match 'dotnet-2.0',            Gem::Platform.local
-    assert_match 'dotnet-4.0',            Gem::Platform.local
-
     util_set_arch 'powerpc-darwin'
     assert_match 'powerpc-darwin', Gem::Platform.local
 
@@ -259,5 +243,17 @@ class TestGemPlatform < Gem::TestCase
     assert_match 'sparc-solaris2.8-mq5.3', Gem::Platform.local
   end
 
+  def assert_match(pattern, platform, message = '')
+    full_message = build_message message, "<?> expected to be =~\n<?>.",
+                                 platform, pattern
+    assert_block(full_message) { platform =~ pattern }
+  end
+
+  def deny_match(pattern, platform, message = '')
+    full_message = build_message message, "<?> expected to be !~\n<?>.",
+                                 platform, pattern
+    assert_block(full_message) { platform !~ pattern }
+  end
+
 end
 
