@@ -1,8 +1,15 @@
-require_relative 'gemutilities'
-require 'ostruct'
+#!/usr/bin/env ruby
+#--
+# Copyright 2006 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
+# All rights reserved.
+# See LICENSE.txt for permissions.
+#++
+
+require 'test/unit'
+require File.join(File.expand_path(File.dirname(__FILE__)), 'gemutilities')
 require 'webrick'
+require 'zlib'
 require 'rubygems/remote_fetcher'
-require 'rubygems/format'
 
 # = Testing Proxy Settings
 #
@@ -99,29 +106,19 @@ gems:
     @a1, @a1_gem = util_gem 'a', '1' do |s| s.executables << 'a_bin' end
 
     Gem::RemoteFetcher.fetcher = nil
-
-    @fetcher = Gem::RemoteFetcher.fetcher
-  end
-
-  def teardown
-    super
-    Gem.configuration[:http_proxy] = nil
   end
 
   def test_self_fetcher
     fetcher = Gem::RemoteFetcher.fetcher
-    refute_nil fetcher
+    assert_not_nil fetcher
     assert_kind_of Gem::RemoteFetcher, fetcher
   end
 
   def test_self_fetcher_with_proxy
     proxy_uri = 'http://proxy.example.com'
     Gem.configuration[:http_proxy] = proxy_uri
-    Gem::RemoteFetcher.fetcher = nil
-
     fetcher = Gem::RemoteFetcher.fetcher
-
-    refute_nil fetcher
+    assert_not_nil fetcher
     assert_kind_of Gem::RemoteFetcher, fetcher
     assert_equal proxy_uri, fetcher.instance_variable_get(:@proxy_uri).to_s
   end
@@ -129,11 +126,8 @@ gems:
   def test_self_fetcher_with_proxy_URI
     proxy_uri = URI.parse 'http://proxy.example.com'
     Gem.configuration[:http_proxy] = proxy_uri
-    Gem::RemoteFetcher.fetcher = nil
-
     fetcher = Gem::RemoteFetcher.fetcher
-    refute_nil fetcher
-
+    assert_not_nil fetcher
     assert_kind_of Gem::RemoteFetcher, fetcher
     assert_equal proxy_uri, fetcher.instance_variable_get(:@proxy_uri)
   end
@@ -141,31 +135,33 @@ gems:
   def test_fetch_size_bad_uri
     fetcher = Gem::RemoteFetcher.new nil
 
-    e = assert_raises ArgumentError do
+    e = assert_raise ArgumentError do
       fetcher.fetch_size 'gems.example.com/yaml'
     end
 
-    assert_equal 'uri scheme is invalid', e.message
+    assert_equal 'uri is not an HTTP URI', e.message
   end
 
   def test_fetch_size_socket_error
     fetcher = Gem::RemoteFetcher.new nil
     def fetcher.connection_for(uri)
-      raise SocketError, "tarded"
+      raise SocketError
     end
 
     uri = 'http://gems.example.com/yaml'
-    e = assert_raises Gem::RemoteFetcher::FetchError do
+    e = assert_raise Gem::RemoteFetcher::FetchError do
       fetcher.fetch_size uri
     end
 
-    assert_equal "SocketError: tarded (#{uri})", e.message
+    assert_equal "SocketError (SocketError)\n\tfetching size (#{uri})",
+                 e.message
   end
 
   def test_no_proxy
     use_ui @ui do
-      assert_data_from_server @fetcher.fetch_path(@server_uri)
-      assert_equal SERVER_DATA.size, @fetcher.fetch_size(@server_uri)
+      fetcher = Gem::RemoteFetcher.new nil
+      assert_data_from_server fetcher.fetch_path(@server_uri)
+      assert_equal SERVER_DATA.size, fetcher.fetch_size(@server_uri)
     end
   end
 
@@ -202,7 +198,7 @@ gems:
 
     fetcher = util_fuck_with_fetcher a1_data
 
-    a1_cache_gem = File.join(@gemhome, 'cache', @a1.file_name)
+    a1_cache_gem = File.join(@gemhome, 'cache', "#{@a1.full_name}.gem")
     assert_equal a1_cache_gem, fetcher.download(@a1, 'http://gems.example.com')
     assert_equal("http://gems.example.com/gems/a-1.gem",
                  fetcher.instance_variable_get(:@test_arg).to_s)
@@ -214,35 +210,20 @@ gems:
 
     inst = Gem::RemoteFetcher.fetcher
 
-    assert_equal File.join(@gemhome, 'cache', @a1.file_name),
+    assert_equal File.join(@gemhome, 'cache', "#{@a1.full_name}.gem"),
                  inst.download(@a1, 'http://gems.example.com')
   end
 
   def test_download_local
     FileUtils.mv @a1_gem, @tempdir
-    local_path = File.join @tempdir, @a1.file_name
+    local_path = File.join @tempdir, "#{@a1.full_name}.gem"
     inst = nil
 
     Dir.chdir @tempdir do
       inst = Gem::RemoteFetcher.fetcher
     end
 
-    assert_equal File.join(@gemhome, 'cache', @a1.file_name),
-                 inst.download(@a1, local_path)
-  end
-
-  def test_download_local_space
-    space_path = File.join @tempdir, 'space path'
-    FileUtils.mkdir space_path
-    FileUtils.mv @a1_gem, space_path
-    local_path = File.join space_path, @a1.file_name
-    inst = nil
-
-    Dir.chdir @tempdir do
-      inst = Gem::RemoteFetcher.fetcher
-    end
-
-    assert_equal File.join(@gemhome, 'cache', @a1.file_name),
+    assert_equal File.join(@gemhome, 'cache', "#{@a1.full_name}.gem"),
                  inst.download(@a1, local_path)
   end
 
@@ -256,8 +237,7 @@ gems:
 
     install_dir = File.join @tempdir, 'more_gems'
 
-    a1_cache_gem = File.join install_dir, 'cache', @a1.file_name
-    FileUtils.mkdir_p(File.dirname(a1_cache_gem))
+    a1_cache_gem = File.join install_dir, 'cache', "#{@a1.full_name}.gem"
     actual = fetcher.download(@a1, 'http://gems.example.com', install_dir)
 
     assert_equal a1_cache_gem, actual
@@ -267,10 +247,10 @@ gems:
     assert File.exist?(a1_cache_gem)
   end
 
-  unless win_platform? # File.chmod doesn't work
+  unless win_platform? then # File.chmod doesn't work
     def test_download_local_read_only
       FileUtils.mv @a1_gem, @tempdir
-      local_path = File.join @tempdir, @a1.file_name
+      local_path = File.join @tempdir, "#{@a1.full_name}.gem"
       inst = nil
       File.chmod 0555, File.join(@gemhome, 'cache')
 
@@ -278,23 +258,11 @@ gems:
         inst = Gem::RemoteFetcher.fetcher
       end
 
-      assert_equal File.join(@tempdir, @a1.file_name),
+      assert_equal File.join(@tempdir, "#{@a1.full_name}.gem"),
         inst.download(@a1, local_path)
     ensure
       File.chmod 0755, File.join(@gemhome, 'cache')
     end
-
-    def test_download_read_only
-      File.chmod 0555, File.join(@gemhome, 'cache')
-      File.chmod 0555, File.join(@gemhome)
-
-      fetcher = util_fuck_with_fetcher File.read(@a1_gem)
-      fetcher.download(@a1, 'http://gems.example.com')
-      assert File.exist?(File.join(Gem.user_dir, 'cache', @a1.file_name))
-    ensure
-      File.chmod 0755, File.join(@gemhome)
-      File.chmod 0755, File.join(@gemhome, 'cache')
-    end
   end
 
   def test_download_platform_legacy
@@ -312,7 +280,7 @@ gems:
 
     fetcher = util_fuck_with_fetcher e1_data, :blow_chunks
 
-    e1_cache_gem = File.join(@gemhome, 'cache', e1.file_name)
+    e1_cache_gem = File.join(@gemhome, 'cache', "#{e1.full_name}.gem")
 
     assert_equal e1_cache_gem, fetcher.download(e1, 'http://gems.example.com')
 
@@ -321,27 +289,10 @@ gems:
     assert File.exist?(e1_cache_gem)
   end
 
-  def test_download_same_file
-    FileUtils.mv @a1_gem, @tempdir
-    local_path = File.join @tempdir, @a1.file_name
-    inst = nil
-
-    Dir.chdir @tempdir do
-      inst = Gem::RemoteFetcher.fetcher
-    end
-
-    cache_path = File.join @gemhome, 'cache', @a1.file_name
-    FileUtils.mv local_path, cache_path
-
-    gem = Gem::Format.from_file_by_path cache_path
-
-    assert_equal cache_path, inst.download(gem.spec, cache_path)
-  end
-
   def test_download_unsupported
     inst = Gem::RemoteFetcher.fetcher
 
-    e = assert_raises Gem::InstallError do
+    e = assert_raise Gem::InstallError do
       inst.download @a1, 'ftp://gems.rubyforge.org'
     end
 
@@ -412,34 +363,12 @@ gems:
     end
   end
 
-  def test_fetch_path_gzip
-    fetcher = Gem::RemoteFetcher.new nil
-
-    def fetcher.open_uri_or_path(uri, mtime, head = nil)
-      Gem.gzip 'foo'
-    end
-
-    assert_equal 'foo', fetcher.fetch_path(@uri + 'foo.gz')
-  end
-
-  def test_fetch_path_gzip_unmodified
-    fetcher = Gem::RemoteFetcher.new nil
-
-    def fetcher.open_uri_or_path(uri, mtime, head = nil)
-      nil
-    end
-
-    assert_equal nil, fetcher.fetch_path(@uri + 'foo.gz', Time.at(0))
-  end
-
   def test_fetch_path_io_error
     fetcher = Gem::RemoteFetcher.new nil
 
-    def fetcher.open_uri_or_path(uri, mtime, head = nil)
-      raise EOFError
-    end
+    def fetcher.open_uri_or_path(uri) raise EOFError; end
 
-    e = assert_raises Gem::RemoteFetcher::FetchError do
+    e = assert_raise Gem::RemoteFetcher::FetchError do
       fetcher.fetch_path 'uri'
     end
 
@@ -450,11 +379,9 @@ gems:
   def test_fetch_path_socket_error
     fetcher = Gem::RemoteFetcher.new nil
 
-    def fetcher.open_uri_or_path(uri, mtime, head = nil)
-      raise SocketError
-    end
+    def fetcher.open_uri_or_path(uri) raise SocketError; end
 
-    e = assert_raises Gem::RemoteFetcher::FetchError do
+    e = assert_raise Gem::RemoteFetcher::FetchError do
       fetcher.fetch_path 'uri'
     end
 
@@ -465,11 +392,11 @@ gems:
   def test_fetch_path_system_call_error
     fetcher = Gem::RemoteFetcher.new nil
 
-    def fetcher.open_uri_or_path(uri, mtime = nil, head = nil)
+    def fetcher.open_uri_or_path(uri);
       raise Errno::ECONNREFUSED, 'connect(2)'
     end
 
-    e = assert_raises Gem::RemoteFetcher::FetchError do
+    e = assert_raise Gem::RemoteFetcher::FetchError do
       fetcher.fetch_path 'uri'
     end
 
@@ -478,23 +405,6 @@ gems:
     assert_equal 'uri', e.uri
   end
 
-  def test_fetch_path_unmodified
-    fetcher = Gem::RemoteFetcher.new nil
-
-    def fetcher.open_uri_or_path(uri, mtime, head = nil)
-      nil
-    end
-
-    assert_equal nil, fetcher.fetch_path(URI.parse(@gem_repo), Time.at(0))
-  end
-
-  def test_get_proxy_from_env_auto_normalizes
-    fetcher = Gem::RemoteFetcher.new(nil)
-    ENV['HTTP_PROXY'] = 'fakeurl:12345'
-
-    assert_equal('http://fakeurl:12345', fetcher.get_proxy_from_env.to_s)
-  end
-
   def test_get_proxy_from_env_empty
     orig_env_HTTP_PROXY = ENV['HTTP_PROXY']
     orig_env_http_proxy = ENV['http_proxy']
@@ -552,7 +462,7 @@ gems:
     def conn.request(req)
       unless defined? @requested then
         @requested = true
-        res = Net::HTTPMovedPermanently.new nil, 301, nil
+        res = Net::HTTPRedirection.new nil, 301, nil
         res.add_field 'Location', 'http://gems.example.com/real_path'
         res
       else
@@ -565,9 +475,9 @@ gems:
     conn = { 'gems.example.com:80' => conn }
     fetcher.instance_variable_set :@connections, conn
 
-    data = fetcher.open_uri_or_path 'http://gems.example.com/redirect'
-
-    assert_equal 'real_path', data
+    fetcher.send :open_uri_or_path, 'http://gems.example.com/redirect' do |io|
+      assert_equal 'real_path', io.read
+    end
   end
 
   def test_open_uri_or_path_limited_redirects
@@ -576,7 +486,7 @@ gems:
     conn = Object.new
     def conn.started?() true end
     def conn.request(req)
-      res = Net::HTTPMovedPermanently.new nil, 301, nil
+      res = Net::HTTPRedirection.new nil, 301, nil
       res.add_field 'Location', 'http://gems.example.com/redirect'
       res
     end
@@ -584,44 +494,30 @@ gems:
     conn = { 'gems.example.com:80' => conn }
     fetcher.instance_variable_set :@connections, conn
 
-    e = assert_raises Gem::RemoteFetcher::FetchError do
-      fetcher.open_uri_or_path 'http://gems.example.com/redirect'
+    e = assert_raise Gem::RemoteFetcher::FetchError do
+      fetcher.send :open_uri_or_path, 'http://gems.example.com/redirect'
     end
 
     assert_equal 'too many redirects (http://gems.example.com/redirect)',
                  e.message
   end
 
-  def test_request
-    uri = URI.parse "#{@gem_repo}/specs.#{Gem.marshal_version}"
-    util_stub_connection_for :body => :junk, :code => 200
-
-    response = @fetcher.request uri, Net::HTTP::Get
-
-    assert_equal 200, response.code
-    assert_equal :junk, response.body
+  def test_zip
+    use_ui @ui do
+      self.class.enable_zip = true
+      fetcher = Gem::RemoteFetcher.new nil
+      assert_equal SERVER_DATA.size, fetcher.fetch_size(@server_uri), "probably not from proxy"
+      zip_data = fetcher.fetch_path(@server_z_uri)
+      assert zip_data.size < SERVER_DATA.size, "Zipped data should be smaller"
+    end
   end
 
-  def test_request_head
-    uri = URI.parse "#{@gem_repo}/specs.#{Gem.marshal_version}"
-    util_stub_connection_for :body => '', :code => 200
-    response = @fetcher.request uri, Net::HTTP::Head
-
-    assert_equal 200, response.code
-    assert_equal '', response.body
-  end
-
-  def test_request_unmodifed
-    uri = URI.parse "#{@gem_repo}/specs.#{Gem.marshal_version}"
-    conn = util_stub_connection_for :body => '', :code => 304
-
-    t = Time.now
-    response = @fetcher.request uri, Net::HTTP::Head, t
-
-    assert_equal 304, response.code
-    assert_equal '', response.body
-
-    assert_equal t.rfc2822, conn.payload['if-modified-since']
+  def test_no_zip
+    use_ui @ui do
+      self.class.enable_zip = false
+      fetcher = Gem::RemoteFetcher.new nil
+      assert_error { fetcher.fetch_path(@server_z_uri) }
+    end
   end
 
   def test_yaml_error_on_size
@@ -632,17 +528,7 @@ gems:
     end
   end
 
-  def util_stub_connection_for hash
-    def @fetcher.connection= conn
-      @conn = conn
-    end
-
-    def @fetcher.connection_for uri
-      @conn
-    end
-
-    @fetcher.connection = Conn.new OpenStruct.new(hash)
-  end
+  private
 
   def assert_error(exception_class=Exception)
     got_exception = false
@@ -662,20 +548,6 @@ gems:
     assert_block("Data is not from proxy") { data =~ /0\.4\.2/ }
   end
 
-  class Conn
-    attr_accessor :payload
-
-    def initialize(response)
-      @response = response
-      self.payload = nil
-    end
-
-    def request(req)
-      self.payload = req
-      @response
-    end
-  end
-
   class NilLog < WEBrick::Log
     def log(level, data) #Do nothing
     end
