@@ -1,20 +1,13 @@
 require "webrick"
 require File.join(File.dirname(__FILE__), "utils.rb")
 require "test/unit"
-begin
-  loadpath = $:.dup
-  $:.replace($: | [File.expand_path("../ruby", File.dirname(__FILE__))])
-  require 'envutil'
-ensure
-  $:.replace(loadpath)
-end
 
 class TestWEBrickCGI < Test::Unit::TestCase
   def test_cgi
     accepted = started = stopped = 0
     requested0 = requested1 = 0
     config = {
-      :CGIInterpreter => EnvUtil.rubybin,
+      :CGIInterpreter => TestWEBrick::RubyBin,
       :DocumentRoot => File.dirname(__FILE__),
       :DirectoryIndex => ["webrick.cgi"],
       :RequestHandler => Proc.new{|req, res|
@@ -54,7 +47,7 @@ class TestWEBrickCGI < Test::Unit::TestCase
         assert_equal("a=1, a=2, b=x", res.body)}
       req = Net::HTTP::Get.new("/")
       http.request(req){|res|
-        ary = res.body.to_a
+        ary = res.body.lines.to_a
         assert_match(%r{/$}, ary[0])
         assert_match(%r{/webrick.cgi$}, ary[1])
       }
