@@ -30,6 +30,27 @@ if defined?(WIN32OLE_TYPE)
       }
       ole_type = WIN32OLE_TYPE.new("Microsoft Shell Controls And Automation", "Shell")
       assert_instance_of(WIN32OLE_TYPE, ole_type)
+
+      ole_type2 = WIN32OLE_TYPE.new("{13709620-C279-11CE-A49E-444553540000}", "Shell")
+      assert_instance_of(WIN32OLE_TYPE, ole_type)
+      assert_equal(ole_type.name, ole_type2.name)
+      assert_equal(ole_type.ole_type, ole_type2.ole_type)
+      assert_equal(ole_type.guid, ole_type2.guid)
+      assert_equal(ole_type.progid, ole_type2.progid)
+      assert_equal(ole_type.visible?, ole_type2.visible?)
+      assert_equal(ole_type.to_s, ole_type2.to_s)
+      assert_equal(ole_type.major_version, ole_type2.major_version)
+      assert_equal(ole_type.minor_version, ole_type2.minor_version)
+      assert_equal(ole_type.typekind, ole_type2.typekind)
+      assert_equal(ole_type.helpstring, ole_type2.helpstring)
+      assert_equal(ole_type.src_type, ole_type2.src_type)
+      assert_equal(ole_type.helpfile, ole_type2.helpfile)
+      assert_equal(ole_type.helpcontext, ole_type2.helpcontext)
+      assert_equal(ole_type.variables.size, ole_type2.variables.size)
+      assert_equal(ole_type.ole_methods[0].name, ole_type2.ole_methods[0].name)
+      assert_equal(ole_type.ole_typelib.name, ole_type2.ole_typelib.name)
+      assert_equal(ole_type.implemented_ole_types.size, ole_type2.implemented_ole_types.size)
+      assert_equal(ole_type.inspect, ole_type2.inspect)
     end
 
     def setup
@@ -131,12 +152,26 @@ if defined?(WIN32OLE_TYPE)
       ole_types = @ole_type.implemented_ole_types
       assert_instance_of(Array, ole_types)
       assert(ole_types.size > 0)
-      assert_equal("IShellDispatch", ole_types[0].name)
+      assert_match(/^IShellDispatch5{0,1}$/, ole_types[0].name)
     end
 
     def test_inspect
       assert_equal("#<WIN32OLE_TYPE:Shell>", @ole_type.inspect)
     end
+    # WIN32OLE_TYPE.typelibs will be obsoleted.
+    def test_s_typelibs
+      tlibs = WIN32OLE_TYPE.typelibs.sort
+      tlibs2 = WIN32OLE_TYPELIB.typelibs.collect{|t|t.name}.sort
+      assert_equal(tlibs2, tlibs)
+    end
 
+    # WIN32OLE_TYPE.ole_classes will be obsoleted.
+    def test_s_ole_classes
+      ots1 = WIN32OLE_TYPE.ole_classes("Microsoft Shell Controls And Automation")
+      ots2 = WIN32OLE_TYPELIB.new("Microsoft Shell Controls And Automation").ole_types
+      otns1 = ots1.collect{|t| t.name}.sort
+      otns2 = ots2.collect{|t| t.name}.sort
+      assert_equal(otns2, otns1)
+    end
   end
 end
