@@ -2,7 +2,6 @@ begin
   require 'win32ole'
 rescue LoadError
 end
-
 require "test/unit"
 
 if defined?(WIN32OLE_TYPELIB)
@@ -74,17 +73,27 @@ if defined?(WIN32OLE_TYPELIB)
       assert_match(/shell32\.dll$/i, tlib.path)
     end
 
+    def test_visible?
+      tlib = WIN32OLE_TYPELIB.new("Microsoft Shell Controls And Automation")
+      assert(tlib.visible?)
+    end
+
+    def test_library_name
+      tlib = WIN32OLE_TYPELIB.new("Microsoft Shell Controls And Automation")
+      assert_equal("Shell32", tlib.library_name)
+    end
+
     def test_to_s
       tlib = WIN32OLE_TYPELIB.new("Microsoft Shell Controls And Automation")
       assert_equal("Microsoft Shell Controls And Automation", tlib.to_s)
     end
 
-    def test_ole_classes
+    def test_ole_types
       tlib = WIN32OLE_TYPELIB.new("Microsoft Shell Controls And Automation")
-      ole_classes = tlib.ole_classes
-      assert_instance_of(Array, ole_classes)
-      assert(ole_classes.size > 0)
-      assert_instance_of(WIN32OLE_TYPE, ole_classes[0])
+      ole_types = tlib.ole_types
+      assert_instance_of(Array, ole_types)
+      assert(ole_types.size > 0)
+      assert_instance_of(WIN32OLE_TYPE, ole_types[0])
     end
 
     def test_inspect
