@@ -4,7 +4,6 @@
   thread_win32.c -
 
   $Author: ko1 $
-  $Date: 2007-12-25 13:35:17 +0900 (Tue, 25 Dec 2007) $
 
   Copyright (C) 2004-2007 Koichi Sasada
 
@@ -74,7 +73,7 @@ w32_error(void)
 		  GetLastError(),
 		  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
 		  (LPTSTR) & lpMsgBuf, 0, NULL);
-    rb_bug("%s", lpMsgBuf);
+    rb_bug("%s", (char*)lpMsgBuf);
 }
 
 static void
@@ -104,7 +103,7 @@ w32_wait_events(HANDLE *events, int count, DWORD timeout, rb_thread_t *th)
 		 events, count, timeout, th);
     if (th && (intr = th->native_thread_data.interrupt_event)) {
 	w32_reset_event(intr);
-	if (th->interrupt_flag) {
+	if (RUBY_VM_INTERRUPTED(th)) {
 	    w32_set_event(intr);
 	}
 
@@ -117,7 +116,7 @@ w32_wait_events(HANDLE *events, int count, DWORD timeout, rb_thread_t *th)
 
     thread_debug("  WaitForMultipleObjects start (count: %d)\n", count);
     ret = WaitForMultipleObjects(count, targets, FALSE, timeout);
-    thread_debug("  WaitForMultipleObjects end (ret: %d)\n", ret);
+    thread_debug("  WaitForMultipleObjects end (ret: %lu)\n", ret);
 
     if (ret == WAIT_OBJECT_0 + count - 1 && th) {
 	errno = EINTR;
@@ -205,9 +204,11 @@ rb_w32_Sleep(unsigned long msec)
 }
 
 static void
-native_sleep(rb_thread_t *th, struct timeval *tv)
+native_sleep(rb_thread_t *th, struct timeval *tv, int deadlockable)
 {
+    int prev_status = th->status;
     DWORD msec;
+
     if (tv) {
 	msec = tv->tv_sec * 1000 + tv->tv_usec / 1000;
     }
@@ -215,29 +216,37 @@ native_sleep(rb_thread_t *th, struct timeval *tv)
 	msec = INFINITE;
     }
 
+    if (!tv && deadlockable) {
+	th->status = THREAD_STOPPED_FOREVER;
+	th->vm->sleeper++;
+	rb_check_deadlock(th->vm);
+    }
+    else {
+	th->status = THREAD_STOPPED;
+    }
+
     GVL_UNLOCK_BEGIN();
     {
 	DWORD ret;
-	int status = th->status;
 
-	th->status = THREAD_STOPPED;
-	th->unblock_function = ubf_handle;
-	th->unblock_function_arg = th;
+	th->unblock.func = ubf_handle;
+	th->unblock.arg = th;
 
 	if (RUBY_VM_INTERRUPTED(th)) {
 	    /* interrupted.  return immediate */
 	}
 	else {
-	    thread_debug("native_sleep start (%d)\n", (int)msec);
+	    thread_debug("native_sleep start (%lu)\n", msec);
 	    ret = w32_wait_events(0, 0, msec, th);
-	    thread_debug("native_sleep done (%d)\n", ret);
+	    thread_debug("native_sleep done (%lu)\n", ret);
 	}
 
-	th->unblock_function = 0;
-	th->unblock_function_arg = 0;
-	th->status = status;
+	th->unblock.func = 0;
+	th->unblock.arg = 0;
     }
     GVL_UNLOCK_END();
+    th->status = prev_status;
+    if (!tv && deadlockable) th->vm->sleeper--;
     RUBY_VM_CHECK_INTS();
 }
 
@@ -394,7 +403,7 @@ native_cond_wait(rb_thread_cond_t *cond, rb_thread_lock_t *mutex)
     {
 	r = WaitForSingleObject(entry.event, INFINITE);
 	if (r != WAIT_OBJECT_0) {
-	    rb_bug("native_cond_wait: WaitForSingleObject returns %d", r);
+	    rb_bug("native_cond_wait: WaitForSingleObject returns %lu", r);
 	}
     }
     native_mutex_lock(mutex);
@@ -415,10 +424,37 @@ native_cond_destroy(rb_thread_cond_t *cond)
     /* */
 }
 
+void
+ruby_init_stack(VALUE *addr)
+{
+}
+
+#define CHECK_ERR(expr) \
+    {if (!(expr)) {rb_bug("err: %lu - %s", GetLastError(), #expr);}}
+
+static void
+native_thread_init_stack(rb_thread_t *th)
+{
+    MEMORY_BASIC_INFORMATION mi;
+    char *base, *end;
+    DWORD size, space;
+
+    CHECK_ERR(VirtualQuery(&mi, &mi, sizeof(mi)));
+    base = mi.AllocationBase;
+    end = mi.BaseAddress;
+    end += mi.RegionSize;
+    size = end - base;
+    space = size / 5;
+    if (space > 1024*1024) space = 1024*1024;
+    th->machine_stack_start = (VALUE *)end - 1;
+    th->machine_stack_maxsize = size - space;
+}
+
 static void
 native_thread_destroy(rb_thread_t *th)
 {
     HANDLE intr = th->native_thread_data.interrupt_event;
+    native_mutex_destroy(&th->interrupt_lock);
     thread_debug("close handle - intr: %p, thid: %p\n", intr, th->thread_id);
     th->native_thread_data.interrupt_event = 0;
     w32_close_handle(intr);
@@ -431,6 +467,7 @@ thread_start_func_1(void *th_ptr)
     VALUE stack_start;
     volatile HANDLE thread_id = th->thread_id;
 
+    native_thread_init_stack(th);
     th->native_thread_data.interrupt_event = CreateEvent(0, TRUE, FALSE, 0);
 
     /* run */
@@ -443,16 +480,12 @@ thread_start_func_1(void *th_ptr)
     return 0;
 }
 
-extern size_t rb_gc_stack_maxsize;
-
 static int
 native_thread_create(rb_thread_t *th)
 {
     size_t stack_size = 4 * 1024; /* 4KB */
     th->thread_id = w32_create_thread(stack_size, thread_start_func_1, th);
 
-    th->machine_stack_maxsize = rb_gc_stack_maxsize; /* not tested. */
-
     if ((th->thread_id) == 0) {
 	st_delete_wrap(th->vm->living_threads, th->self);
 	rb_raise(rb_eThreadError, "can't create Thread (%d)", errno);
