@@ -8,7 +8,7 @@ $:.unshift(File.expand_path("../..", __FILE__))
 
 preludes = ARGV.dup
 outfile = preludes.pop
-init_name = outfile[/\w+(?=_prelude.c\z)/] || 'prelude'
+init_name = outfile[/\w+(?=_prelude.c\b)/] || 'prelude'
 
 C_ESC = {
   "\\" => "\\\\",
@@ -16,8 +16,8 @@ C_ESC = {
   "\n" => '\n',
 }
 
-0x00.upto(0x1f) {|ch| C_ESC[[ch].pack("C")] ||= "\\x%02x" % ch }
-0x7f.upto(0xff) {|ch| C_ESC[[ch].pack("C")] = "\\x%02x" % ch }
+0x00.upto(0x1f) {|ch| C_ESC[[ch].pack("C")] ||= "\\%03o" % ch }
+0x7f.upto(0xff) {|ch| C_ESC[[ch].pack("C")] = "\\%03o" % ch }
 C_ESC_PAT = Regexp.union(*C_ESC.keys)
 
 def c_esc(str)
