@@ -66,13 +66,14 @@ module RubyVM
         ret = "int inc = 0;\n"
 
         @opes.each_with_index{|(t, v), i|
-          if t == 'rb_num_t'
-            ret << "        unsigned long #{v} = FIX2INT(opes[#{i}]);\n"
+          if t == 'rb_num_t' && ((re = /\b#{v}\b/n) =~ @sp_inc ||
+                                 @defopes.any?{|t, val| re =~ val})
+            ret << "        #{t} #{v} = FIX2INT(opes[#{i}]);\n"
           end
         }
         @defopes.each_with_index{|((t, var), val), i|
-          if t == 'rb_num_t' && val != '*'
-            ret << "        unsigned long #{var} = #{val};\n"
+          if t == 'rb_num_t' && val != '*' && /\b#{var}\b/ =~ @sp_inc
+            ret << "        #{t} #{var} = #{val};\n"
           end
         }
 
@@ -707,7 +708,10 @@ module RubyVM
           break
         end
 
-        ops << "  #{type} #{var} = (#{type})GET_OPERAND(#{i+1});"
+        re = /\b#{var}\b/n
+        if re =~ insn.body or re =~ insn.sp_inc or insn.rets.any?{|t, v| re =~ v}
+          ops << "  #{type} #{var} = (#{type})GET_OPERAND(#{i+1});"
+        end
         n   += 1
       }
       @opn = n
@@ -1139,7 +1143,7 @@ module RubyVM
 
           uni_insn, uni_insns = *unif
           uni_insns = uni_insns[1..-1]
-          unif_insns_each << "static int UNIFIED_#{insn.name}_#{i}[] = {" +
+          unif_insns_each << "static const int UNIFIED_#{insn.name}_#{i}[] = {" +
                              "  BIN(#{uni_insn.name}), #{uni_insns.size + 2}, \n  " +
                              uni_insns.map{|e| "BIN(#{e.name})"}.join(", ") + "};\n"
           }
@@ -1147,14 +1151,14 @@ module RubyVM
           
         end
         if size > 0
-          unif_insns << "static int *UNIFIED_#{insn.name}[] = {(int *)#{size+1}, \n"
+          unif_insns << "static const int *const UNIFIED_#{insn.name}[] = {(int *)#{size+1}, \n"
           unif_insns << (0...size).map{|e| "  UNIFIED_#{insn.name}_#{e}"}.join(",\n") + "};\n"
           unif_insns_data << "  UNIFIED_#{insn.name}"
         else
           unif_insns_data << "  0"
         end
       }
-      unif_insns_data = "static int **unified_insns_data[] = {\n" +
+      unif_insns_data = "static const int *const *const unified_insns_data[] = {\n" +
                         unif_insns_data.join(",\n") + "};\n"
       ERB.new(vpath.read('template/optunifs.inc.tmpl')).result(binding)
     end
