@@ -2,8 +2,7 @@
 
   variable.c -
 
-  $Author: matz $
-  $Date: 2007-08-25 12:29:39 +0900 (土, 25  8月 2007) $
+  $Author: nobu $
   created at: Tue Apr 19 23:55:15 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -27,9 +26,9 @@ Init_var_tables(void)
 {
     rb_global_tbl = st_init_numtable();
     rb_class_tbl = st_init_numtable();
-    autoload = rb_intern("__autoload__");
-    classpath = rb_intern("__classpath__");
-    tmp_classpath = rb_intern("__tmp_classpath__");
+    CONST_ID(autoload, "__autoload__");
+    CONST_ID(classpath, "__classpath__");
+    CONST_ID(tmp_classpath, "__tmp_classpath__");
 }
 
 struct fc_result {
@@ -45,18 +44,18 @@ fc_path(struct fc_result *fc, ID name)
 {
     VALUE path, tmp;
 
-    path = rb_str_new2(rb_id2name(name));
+    path = rb_str_dup(rb_id2str(name));
     while (fc) {
 	if (fc->track == rb_cObject) break;
-	if (ROBJECT(fc->track)->iv_tbl &&
-	    st_lookup(ROBJECT(fc->track)->iv_tbl, classpath, &tmp)) {
+	if (RCLASS_IV_TBL(fc->track) &&
+	    st_lookup(RCLASS_IV_TBL(fc->track), classpath, &tmp)) {
 	    tmp = rb_str_dup(tmp);
 	    rb_str_cat2(tmp, "::");
 	    rb_str_append(tmp, path);
 	    path = tmp;
 	    break;
 	}
-	tmp = rb_str_new2(rb_id2name(fc->name));
+	tmp = rb_str_dup(rb_id2str(fc->name));
 	rb_str_cat2(tmp, "::");
 	rb_str_append(tmp, path);
 	path = tmp;
@@ -78,7 +77,7 @@ fc_i(ID key, VALUE value, struct fc_result *res)
     switch (TYPE(value)) {
       case T_MODULE:
       case T_CLASS:
-	if (!RCLASS(value)->iv_tbl) return ST_CONTINUE;
+	if (!RCLASS_IV_TBL(value)) return ST_CONTINUE;
 	else {
 	    struct fc_result arg;
 	    struct fc_result *list;
@@ -94,7 +93,7 @@ fc_i(ID key, VALUE value, struct fc_result *res)
 	    arg.klass = res->klass;
 	    arg.track = value;
 	    arg.prev = res;
-	    st_foreach(RCLASS(value)->iv_tbl, fc_i, (st_data_t)&arg);
+	    st_foreach(RCLASS_IV_TBL(value), fc_i, (st_data_t)&arg);
 	    if (arg.path) {
 		res->path = arg.path;
 		return ST_STOP;
@@ -118,18 +117,18 @@ find_class_path(VALUE klass)
     arg.klass = klass;
     arg.track = rb_cObject;
     arg.prev = 0;
-    if (RCLASS(rb_cObject)->iv_tbl) {
-	st_foreach_safe(RCLASS(rb_cObject)->iv_tbl, fc_i, (st_data_t)&arg);
+    if (RCLASS_IV_TBL(rb_cObject)) {
+	st_foreach_safe(RCLASS_IV_TBL(rb_cObject), fc_i, (st_data_t)&arg);
     }
     if (arg.path == 0) {
 	st_foreach_safe(rb_class_tbl, fc_i, (st_data_t)&arg);
     }
     if (arg.path) {
-	if (!ROBJECT(klass)->iv_tbl) {
-	    ROBJECT(klass)->iv_tbl = st_init_numtable();
+	if (!RCLASS_IV_TBL(klass)) {
+	    RCLASS_IV_TBL(klass) = st_init_numtable();
 	}
-	st_insert(ROBJECT(klass)->iv_tbl, classpath, arg.path);
-	st_delete(RCLASS(klass)->iv_tbl, &tmp_classpath, 0);
+	st_insert(RCLASS_IV_TBL(klass), classpath, arg.path);
+	st_delete(RCLASS_IV_TBL(klass), &tmp_classpath, 0);
 	return arg.path;
     }
     return Qnil;
@@ -141,17 +140,19 @@ classname(VALUE klass)
     VALUE path = Qnil;
 
     if (!klass) klass = rb_cObject;
-    if (ROBJECT(klass)->iv_tbl) {
-	if (!st_lookup(ROBJECT(klass)->iv_tbl, classpath, &path)) {
-	    ID classid = rb_intern("__classid__");
+    if (RCLASS_IV_TBL(klass)) {
+	if (!st_lookup(RCLASS_IV_TBL(klass), classpath, &path)) {
+	    ID classid;
 
-	    if (!st_lookup(ROBJECT(klass)->iv_tbl, classid, &path)) {
+	    CONST_ID(classid, "__classid__");
+
+	    if (!st_lookup(RCLASS_IV_TBL(klass), classid, &path)) {
 		return find_class_path(klass);
 	    }
-	    path = rb_str_new2(rb_id2name(SYM2ID(path)));
+	    path = rb_str_dup(rb_id2str(SYM2ID(path)));
 	    OBJ_FREEZE(path);
-	    st_insert(ROBJECT(klass)->iv_tbl, classpath, path);
-	    st_delete(RCLASS(klass)->iv_tbl, (st_data_t*)&classid, 0);
+	    st_insert(RCLASS_IV_TBL(klass), classpath, path);
+	    st_delete(RCLASS_IV_TBL(klass), (st_data_t*)&classid, 0);
 	}
 	if (TYPE(path) != T_STRING) {
 	    rb_bug("class path is not set properly");
@@ -183,7 +184,7 @@ rb_class_path(VALUE klass)
     VALUE path = classname(klass);
 
     if (!NIL_P(path)) return path;
-    if (RCLASS(klass)->iv_tbl && st_lookup(RCLASS(klass)->iv_tbl,
+    if (RCLASS_IV_TBL(klass) && st_lookup(RCLASS_IV_TBL(klass),
 					   tmp_classpath, &path)) {
 	return path;
     }
@@ -271,13 +272,13 @@ rb_class_name(VALUE klass)
     return rb_class_path(rb_class_real(klass));
 }
 
-char *
+const char *
 rb_class2name(VALUE klass)
 {
     return RSTRING_PTR(rb_class_name(klass));
 }
 
-char *
+const char *
 rb_obj_classname(VALUE obj)
 {
     return rb_class2name(CLASS_OF(obj));
@@ -456,13 +457,20 @@ rb_define_hooked_variable(
     void  (*setter)(ANYARGS))
 {
     struct global_variable *gvar;
-    ID id = global_id(name);
+    ID id;
+    VALUE tmp;
+    
+    if (var)
+        tmp = *var;
 
+    id = global_id(name);
     gvar = rb_global_entry(id)->var;
     gvar->data = (void*)var;
     gvar->getter = getter?getter:var_getter;
     gvar->setter = setter?setter:var_setter;
     gvar->marker = var_marker;
+
+    RB_GC_GUARD(tmp);
 }
 
 void
@@ -558,7 +566,7 @@ remove_trace(struct global_variable *var)
 	next = trace->next;
 	if (next->removed) {
 	    trace->next = next->next;
-	    free(next);
+	    xfree(next);
 	}
 	else {
 	    trace = next;
@@ -708,7 +716,7 @@ gvar_i(ID key, struct global_entry *entry, VALUE ary)
  *  
  *  Returns an array of the names of global variables.
  *     
- *     global_variables.grep /std/   #=> ["$stderr", "$stdout", "$stdin"]
+ *     global_variables.grep /std/   #=> [:$stdin, :$stdout, :$stderr]
  */
 
 VALUE
@@ -716,14 +724,12 @@ rb_f_global_variables(void)
 {
     VALUE ary = rb_ary_new();
     char buf[4];
-    const char *s = "&`'+123456789";
+    const char *s = "123456789";
 
     st_foreach_safe(rb_global_tbl, gvar_i, ary);
-    if (!NIL_P(rb_backref_get())) {
-	while (*s) {
-	    sprintf(buf, "$%c", *s++);
-	    rb_ary_push(ary, rb_str_new2(buf));
-	}
+    while (*s) {
+	sprintf(buf, "$%c", *s++);
+	rb_ary_push(ary, ID2SYM(rb_intern(buf)));
     }
     return ary;
 }
@@ -739,7 +745,7 @@ rb_alias_variable(ID name1, ID name2)
 
     entry2 = rb_global_entry(name2);
     if (!st_lookup(rb_global_tbl, name1, &data1)) {
-	entry1 = ALLOC(struct global_entry);
+ 	entry1 = ALLOC(struct global_entry);
 	entry1->id = name1;
 	st_add_direct(rb_global_tbl, name1, (st_data_t)entry1);
     }
@@ -753,10 +759,10 @@ rb_alias_variable(ID name1, ID name2)
 	    struct trace_var *trace = var->trace;
 	    while (trace) {
 		struct trace_var *next = trace->next;
-		free(trace);
+		xfree(trace);
 		trace = next;
 	    }
-	    free(var);
+	    xfree(var);
 	}
     }
     else {
@@ -806,12 +812,12 @@ generic_ivar_set(VALUE obj, ID id, VALUE val)
     st_data_t data;
 
     if (rb_special_const_p(obj)) {
+	if (rb_obj_frozen_p(obj)) rb_error_frozen("object");
 	special_generic_ivar = 1;
     }
     if (!generic_iv_tbl) {
 	generic_iv_tbl = st_init_numtable();
     }
-
     if (!st_lookup(generic_iv_tbl, obj, &data)) {
 	FL_SET(obj, FL_EXIVAR);
 	tbl = st_init_numtable();
@@ -907,10 +913,20 @@ rb_copy_generic_ivar(VALUE clone, VALUE obj)
     st_data_t data;
 
     if (!generic_iv_tbl) return;
-    if (!FL_TEST(obj, FL_EXIVAR)) return;
+    if (!FL_TEST(obj, FL_EXIVAR)) {
+clear:
+        if (FL_TEST(clone, FL_EXIVAR)) {
+            rb_free_generic_ivar(clone);
+            FL_UNSET(clone, FL_EXIVAR);
+        }
+        return;
+    }
     if (st_lookup(generic_iv_tbl, obj, &data)) {
 	st_table *tbl = (st_table *)data;
 
+        if (tbl->num_entries == 0)
+            goto clear;
+
 	if (st_lookup(generic_iv_tbl, clone, &data)) {
 	    st_free_table((st_table *)data);
 	    st_insert(generic_iv_tbl, clone, (st_data_t)st_copy(tbl));
@@ -925,13 +941,26 @@ rb_copy_generic_ivar(VALUE clone, VALUE obj)
 static VALUE
 ivar_get(VALUE obj, ID id, int warn)
 {
-    VALUE val;
+    VALUE val, *ptr;
+    struct st_table *iv_index_tbl;
+    long len;
+    st_data_t index;
 
     switch (TYPE(obj)) {
       case T_OBJECT:
+        len = ROBJECT_NUMIV(obj);
+        ptr = ROBJECT_IVPTR(obj);
+        iv_index_tbl = ROBJECT_IV_INDEX_TBL(obj);
+        if (!iv_index_tbl) break; 
+        if (!st_lookup(iv_index_tbl, id, &index)) break;
+        if (len <= index) break;
+        val = ptr[index];
+        if (val != Qundef)
+            return val;
+	break;
       case T_CLASS:
       case T_MODULE:
-	if (ROBJECT(obj)->iv_tbl && st_lookup(ROBJECT(obj)->iv_tbl, id, &val))
+	if (RCLASS_IV_TBL(obj) && st_lookup(RCLASS_IV_TBL(obj), id, &val))
 	    return val;
 	break;
       default:
@@ -960,16 +989,70 @@ rb_attr_get(VALUE obj, ID id)
 VALUE
 rb_ivar_set(VALUE obj, ID id, VALUE val)
 {
+    struct st_table *iv_index_tbl;
+    st_data_t index;
+    long i, len;
+    int ivar_extended;
+
     if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify instance variable");
     if (OBJ_FROZEN(obj)) rb_error_frozen("object");
     switch (TYPE(obj)) {
       case T_OBJECT:
+        iv_index_tbl = ROBJECT_IV_INDEX_TBL(obj);
+        if (!iv_index_tbl) {
+            VALUE klass = rb_obj_class(obj);
+            iv_index_tbl = RCLASS_IV_INDEX_TBL(klass);
+            if (!iv_index_tbl) {
+                iv_index_tbl = RCLASS_IV_INDEX_TBL(klass) = st_init_numtable();
+            }
+        }
+        ivar_extended = 0;
+        if (!st_lookup(iv_index_tbl, id, &index)) {
+            index = iv_index_tbl->num_entries;
+            st_add_direct(iv_index_tbl, id, index);
+            ivar_extended = 1;
+        }
+        len = ROBJECT_NUMIV(obj);
+        if (len <= index) {
+            VALUE *ptr = ROBJECT_IVPTR(obj);
+            if (index < ROBJECT_EMBED_LEN_MAX) {
+                RBASIC(obj)->flags |= ROBJECT_EMBED;
+                ptr = ROBJECT(obj)->as.ary;
+                for (i = 0; i < ROBJECT_EMBED_LEN_MAX; i++) {
+                    ptr[i] = Qundef;
+                }
+            }
+            else {
+                VALUE *newptr;
+                long newsize = (index+1) + (index+1)/4; /* (index+1)*1.25 */
+                if (!ivar_extended &&
+                    iv_index_tbl->num_entries < newsize) {
+                    newsize = iv_index_tbl->num_entries;
+                }
+                if (RBASIC(obj)->flags & ROBJECT_EMBED) {
+                    newptr = ALLOC_N(VALUE, newsize);
+                    MEMCPY(newptr, ptr, VALUE, len);
+                    RBASIC(obj)->flags &= ~ROBJECT_EMBED;
+                    ROBJECT(obj)->as.heap.ivptr = newptr;
+                }
+                else {
+                    REALLOC_N(ROBJECT(obj)->as.heap.ivptr, VALUE, newsize);
+                    newptr = ROBJECT(obj)->as.heap.ivptr;
+                }
+                for (; len < newsize; len++)
+                    newptr[len] = Qundef;
+                ROBJECT(obj)->as.heap.numiv = newsize;
+                ROBJECT(obj)->as.heap.iv_index_tbl = iv_index_tbl;
+            }
+        }
+        ROBJECT_IVPTR(obj)[index] = val;
+	break;
       case T_CLASS:
       case T_MODULE:
-	if (!ROBJECT(obj)->iv_tbl) ROBJECT(obj)->iv_tbl = st_init_numtable();
-	st_insert(ROBJECT(obj)->iv_tbl, id, val);
-	break;
+	if (!RCLASS_IV_TBL(obj)) RCLASS_IV_TBL(obj) = st_init_numtable();
+	st_insert(RCLASS_IV_TBL(obj), id, val);
+        break;
       default:
 	generic_ivar_set(obj, id, val);
 	break;
@@ -980,11 +1063,22 @@ rb_ivar_set(VALUE obj, ID id, VALUE val)
 VALUE
 rb_ivar_defined(VALUE obj, ID id)
 {
+    VALUE val;
+    struct st_table *iv_index_tbl;
+    st_data_t index;
     switch (TYPE(obj)) {
       case T_OBJECT:
+        iv_index_tbl = ROBJECT_IV_INDEX_TBL(obj);
+        if (!iv_index_tbl) break;
+        if (!st_lookup(iv_index_tbl, id, &index)) break;
+        if (ROBJECT_NUMIV(obj) <= index) break;
+        val = ROBJECT_IVPTR(obj)[index];
+        if (val != Qundef)
+            return Qtrue;
+	break;
       case T_CLASS:
       case T_MODULE:
-	if (ROBJECT(obj)->iv_tbl && st_lookup(ROBJECT(obj)->iv_tbl, id, 0))
+	if (RCLASS_IV_TBL(obj) && st_lookup(RCLASS_IV_TBL(obj), id, 0))
 	    return Qtrue;
 	break;
       default:
@@ -995,8 +1089,68 @@ rb_ivar_defined(VALUE obj, ID id)
     return Qfalse;
 }
 
+struct obj_ivar_tag {
+    VALUE obj;
+    int (*func)(ID key, VALUE val, st_data_t arg);
+    st_data_t arg;
+};
+
 static int
-ivar_i(ID key, struct global_entry *entry, VALUE ary)
+obj_ivar_i(ID key, VALUE index, struct obj_ivar_tag *data)
+{
+    if (index < ROBJECT_NUMIV(data->obj)) {
+        VALUE val = ROBJECT_IVPTR(data->obj)[index];
+        if (val != Qundef) {
+            return (data->func)(key, val, data->arg);
+        }
+    }
+    return ST_CONTINUE;
+}
+
+static void
+obj_ivar_each(VALUE obj, int (*func)(ANYARGS), st_data_t arg)
+{
+    st_table *tbl;
+    struct obj_ivar_tag data;
+
+    tbl = ROBJECT_IV_INDEX_TBL(obj);
+    if (!tbl)
+        return;
+
+    data.obj = obj;
+    data.func = (int (*)(ID key, VALUE val, st_data_t arg))func;
+    data.arg = arg;
+
+    st_foreach_safe(tbl, obj_ivar_i, (st_data_t)&data);
+}
+
+void rb_ivar_foreach(VALUE obj, int (*func)(ANYARGS), st_data_t arg)
+{
+    switch (TYPE(obj)) {
+      case T_OBJECT:
+        obj_ivar_each(obj, func, arg);
+	break;
+      case T_CLASS:
+      case T_MODULE:
+	if (RCLASS_IV_TBL(obj)) {
+	    st_foreach_safe(RCLASS_IV_TBL(obj), func, arg);
+	}
+	break;
+      default:
+	if (!generic_iv_tbl) break;
+	if (FL_TEST(obj, FL_EXIVAR) || rb_special_const_p(obj)) {
+	    st_data_t tbl;
+
+	    if (st_lookup(generic_iv_tbl, obj, &tbl)) {
+		st_foreach_safe((st_table *)tbl, func, arg);
+	    }
+	}
+	break;
+    }
+}
+
+static int
+ivar_i(ID key, VALUE val, VALUE ary)
 {
     if (rb_is_instance_id(key)) {
 	rb_ary_push(ary, ID2SYM(key));
@@ -1018,7 +1172,7 @@ ivar_i(ID key, struct global_entry *entry, VALUE ary)
  *         @iv = 3
  *       end
  *     end
- *     Fred.new.instance_variables   #=> ["@iv"]
+ *     Fred.new.instance_variables   #=> [:@iv]
  */
 
 VALUE
@@ -1027,25 +1181,7 @@ rb_obj_instance_variables(VALUE obj)
     VALUE ary;
 
     ary = rb_ary_new();
-    switch (TYPE(obj)) {
-      case T_OBJECT:
-      case T_CLASS:
-      case T_MODULE:
-	if (ROBJECT(obj)->iv_tbl) {
-	    st_foreach_safe(ROBJECT(obj)->iv_tbl, ivar_i, ary);
-	}
-	break;
-      default:
-	if (!generic_iv_tbl) break;
-	if (FL_TEST(obj, FL_EXIVAR) || rb_special_const_p(obj)) {
-	    st_data_t tbl;
-
-	    if (st_lookup(generic_iv_tbl, obj, &tbl)) {
-		st_foreach_safe((st_table *)tbl, ivar_i, ary);
-	    }
-	}
-	break;
-    }
+    rb_ivar_foreach(obj, ivar_i, ary);
     return ary;
 }
 
@@ -1076,6 +1212,8 @@ rb_obj_remove_instance_variable(VALUE obj, VALUE name)
 {
     VALUE val = Qnil;
     ID id = rb_to_id(name);
+    struct st_table *iv_index_tbl;
+    st_data_t index;
 
     if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify instance variable");
@@ -1086,9 +1224,19 @@ rb_obj_remove_instance_variable(VALUE obj, VALUE name)
 
     switch (TYPE(obj)) {
       case T_OBJECT:
+        iv_index_tbl = ROBJECT_IV_INDEX_TBL(obj);
+        if (!iv_index_tbl) break;
+        if (!st_lookup(iv_index_tbl, id, &index)) break;
+        if (ROBJECT_NUMIV(obj) <= index) break;
+        val = ROBJECT_IVPTR(obj)[index];
+        if (val != Qundef) {
+            ROBJECT_IVPTR(obj)[index] = Qundef;
+            return val;
+        }
+	break;
       case T_CLASS:
       case T_MODULE:
-	if (ROBJECT(obj)->iv_tbl && st_delete(ROBJECT(obj)->iv_tbl, (st_data_t*)&id, &val)) {
+	if (RCLASS_IV_TBL(obj) && st_delete(RCLASS_IV_TBL(obj), (st_data_t*)&id, &val)) {
 	    return val;
 	}
 	break;
@@ -1166,7 +1314,8 @@ check_autoload_table(VALUE av)
     Check_Type(av, T_DATA);
     if (RDATA(av)->dmark != (RUBY_DATA_FUNC)rb_mark_tbl ||
 	RDATA(av)->dfree != (RUBY_DATA_FUNC)st_free_table) {
-	rb_raise(rb_eTypeError, "wrong autoload table: %s", RSTRING_PTR(rb_inspect(av)));
+	VALUE desc = rb_inspect(av);
+	rb_raise(rb_eTypeError, "wrong autoload table: %s", RSTRING_PTR(desc));
     }
     return (struct st_table *)DATA_PTR(av);
 }
@@ -1184,11 +1333,11 @@ rb_autoload(VALUE mod, ID id, const char *file)
 	rb_raise(rb_eArgError, "empty file name");
     }
 
-    if ((tbl = RCLASS(mod)->iv_tbl) && st_lookup(tbl, id, &av) && av != Qundef)
+    if ((tbl = RCLASS_IV_TBL(mod)) && st_lookup(tbl, id, &av) && av != Qundef)
 	return;
 
     rb_const_set(mod, id, Qundef);
-    tbl = RCLASS(mod)->iv_tbl;
+    tbl = RCLASS_IV_TBL(mod);
     if (st_lookup(tbl, autoload, &av)) {
 	tbl = check_autoload_table(av);
     }
@@ -1209,8 +1358,8 @@ autoload_delete(VALUE mod, ID id)
     VALUE val;
     st_data_t load = 0;
 
-    st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, 0);
-    if (st_lookup(RCLASS(mod)->iv_tbl, autoload, &val)) {
+    st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, 0);
+    if (st_lookup(RCLASS_IV_TBL(mod), autoload, &val)) {
 	struct st_table *tbl = check_autoload_table(val);
 
 	st_delete(tbl, (st_data_t*)&id, &load);
@@ -1219,7 +1368,7 @@ autoload_delete(VALUE mod, ID id)
 	    DATA_PTR(val) = 0;
 	    st_free_table(tbl);
 	    id = autoload;
-	    if (st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val)) {
+	    if (st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, &val)) {
 		rb_gc_force_recycle(val);
 	    }
 	}
@@ -1234,7 +1383,7 @@ rb_autoload_load(VALUE klass, ID id)
     VALUE file;
     NODE *load = autoload_delete(klass, id);
 
-    if (!load || !(file = load->nd_lit) || rb_provided(RSTRING_PTR(file))) {
+    if (!load || !(file = load->nd_lit)) {
 	return Qfalse;
     }
     return rb_require_safe(file, load->nd_nth);
@@ -1247,13 +1396,13 @@ autoload_file(VALUE mod, ID id)
     struct st_table *tbl;
     st_data_t load;
 
-    if (!st_lookup(RCLASS(mod)->iv_tbl, autoload, &val) ||
+    if (!st_lookup(RCLASS_IV_TBL(mod), autoload, &val) ||
 	!(tbl = check_autoload_table(val)) || !st_lookup(tbl, id, &load)) {
 	return Qnil;
     }
     file = ((NODE *)load)->nd_lit;
     Check_Type(file, T_STRING);
-    if (!RSTRING_PTR(file)) {
+    if (!RSTRING_PTR(file) || !*RSTRING_PTR(file)) {
 	rb_raise(rb_eArgError, "empty file name");
     }
     if (!rb_provided(RSTRING_PTR(file))) {
@@ -1266,7 +1415,7 @@ autoload_file(VALUE mod, ID id)
 	DATA_PTR(val) = 0;
 	st_free_table(tbl);
 	id = autoload;
-	if (st_delete(RCLASS(mod)->iv_tbl, (st_data_t*)&id, &val)) {
+	if (st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, &val)) {
 	    rb_gc_force_recycle(val);
 	}
     }
@@ -1276,7 +1425,7 @@ autoload_file(VALUE mod, ID id)
 VALUE
 rb_autoload_p(VALUE mod, ID id)
 {
-    struct st_table *tbl = RCLASS(mod)->iv_tbl;
+    struct st_table *tbl = RCLASS_IV_TBL(mod);
     VALUE val;
 
     if (!tbl || !st_lookup(tbl, id, &val) || val != Qundef) {
@@ -1293,8 +1442,8 @@ rb_const_get_0(VALUE klass, ID id, int exclude, int recurse)
 
     tmp = klass;
   retry:
-    while (tmp && !NIL_P(tmp)) {
-	while (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,&value)) {
+    while (RTEST(tmp)) {
+	while (RCLASS_IV_TBL(tmp) && st_lookup(RCLASS_IV_TBL(tmp),id,&value)) {
 	    if (value == Qundef) {
 		if (!RTEST(rb_autoload_load(tmp, id))) break;
 		continue;
@@ -1306,7 +1455,7 @@ rb_const_get_0(VALUE klass, ID id, int exclude, int recurse)
 	    return value;
 	}
 	if (!recurse && klass != rb_cObject) break;
-	tmp = RCLASS(tmp)->super;
+	tmp = RCLASS_SUPER(tmp);
     }
     if (!exclude && !mod_retry && BUILTIN_TYPE(klass) == T_MODULE) {
 	mod_retry = 1;
@@ -1359,7 +1508,7 @@ rb_mod_remove_const(VALUE mod, VALUE name)
 	rb_raise(rb_eSecurityError, "Insecure: can't remove constant");
     if (OBJ_FROZEN(mod)) rb_error_frozen("class/module");
 
-    if (RCLASS(mod)->iv_tbl && st_delete(ROBJECT(mod)->iv_tbl, (st_data_t*)&id, &val)) {
+    if (RCLASS_IV_TBL(mod) && st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, &val)) {
 	if (val == Qundef) {
 	    autoload_delete(mod, id);
 	    val = Qnil;
@@ -1393,8 +1542,8 @@ rb_mod_const_at(VALUE mod, void *data)
     if (!tbl) {
 	tbl = st_init_numtable();
     }
-    if (RCLASS(mod)->iv_tbl) {
-	st_foreach_safe(RCLASS(mod)->iv_tbl, sv_i, (st_data_t)tbl);
+    if (RCLASS_IV_TBL(mod)) {
+	st_foreach_safe(RCLASS_IV_TBL(mod), sv_i, (st_data_t)tbl);
     }
     return tbl;
 }
@@ -1405,7 +1554,7 @@ rb_mod_const_of(VALUE mod, void *data)
     VALUE tmp = mod;
     for (;;) {
 	data = rb_mod_const_at(tmp, data);
-	tmp = RCLASS(tmp)->super;
+	tmp = RCLASS_SUPER(tmp);
 	if (!tmp) break;
 	if (tmp == rb_cObject && mod != rb_cObject) break;
     }
@@ -1442,8 +1591,8 @@ rb_const_list(void *data)
  *  modules (example at start of section), unless the <i>all</i>
  *  parameter is set to <code>false</code>.
  *
- *    IO.constants.include?("SYNC")         => true
- *    IO.constants(false).include?("SYNC")  => false
+ *    IO.constants.include?(:SYNC)         => true
+ *    IO.constants(false).include?(:SYNC)  => false
  *
  *  Also see <code>Module::const_defined?</code>.
  */
@@ -1478,13 +1627,13 @@ rb_const_defined_0(VALUE klass, ID id, int exclude, int recurse)
     tmp = klass;
   retry:
     while (tmp) {
-	if (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl, id, &value)) {
+	if (RCLASS_IV_TBL(tmp) && st_lookup(RCLASS_IV_TBL(tmp), id, &value)) {
 	    if (value == Qundef && NIL_P(autoload_file(klass, id)))
 		return Qfalse;
 	    return Qtrue;
 	}
 	if (!recurse && klass != rb_cObject) break;
-	tmp = RCLASS(tmp)->super;
+	tmp = RCLASS_SUPER(tmp);
     }
     if (!exclude && !mod_retry && BUILTIN_TYPE(klass) == T_MODULE) {
 	mod_retry = 1;
@@ -1527,13 +1676,13 @@ mod_av_set(VALUE klass, ID id, VALUE val, int isconst)
 	    rb_error_frozen("class");
 	}
     }
-    if (!RCLASS(klass)->iv_tbl) {
-	RCLASS(klass)->iv_tbl = st_init_numtable();
+    if (!RCLASS_IV_TBL(klass)) {
+	RCLASS_IV_TBL(klass) = st_init_numtable();
     }
     else if (isconst) {
 	VALUE value = Qfalse;
 
-	if (st_lookup(RCLASS(klass)->iv_tbl, id, &value)) {
+	if (st_lookup(RCLASS_IV_TBL(klass), id, &value)) {
 	    if (value == Qundef)
 	      autoload_delete(klass, id);
 	    else
@@ -1544,7 +1693,7 @@ mod_av_set(VALUE klass, ID id, VALUE val, int isconst)
     if(isconst){
 	rb_vm_change_state();
     }
-    st_insert(RCLASS(klass)->iv_tbl, id, val);
+    st_insert(RCLASS_IV_TBL(klass), id, val);
 }
 
 void
@@ -1577,15 +1726,17 @@ rb_define_global_const(const char *name, VALUE val)
     rb_define_const(rb_cObject, name, val);
 }
 
-void
-rb_cvar_set(VALUE klass, ID id, VALUE val)
+static VALUE
+original_module(VALUE c)
 {
-    mod_av_set(klass, id, val, Qfalse);
+    if (TYPE(c) == T_ICLASS)
+	return RBASIC(c)->klass;
+    return c;
 }
 
 #define CVAR_LOOKUP(v,r) do {\
-    if (RCLASS(klass)->iv_tbl && st_lookup(RCLASS(klass)->iv_tbl,id,(v))) {\
-	return (r);\
+    if (RCLASS_IV_TBL(klass) && st_lookup(RCLASS_IV_TBL(klass),id,(v))) {\
+	r;\
     }\
     if (FL_TEST(klass, FL_SINGLETON) ) {\
 	VALUE obj = rb_iv_get(klass, "__attached__");\
@@ -1595,38 +1746,79 @@ rb_cvar_set(VALUE klass, ID id, VALUE val)
 	    klass = obj;\
 	    break;\
 	  default:\
-	    klass = RCLASS(klass)->super;\
+	    klass = RCLASS_SUPER(klass);\
 	    break;\
 	}\
     }\
     else {\
-	klass = RCLASS(klass)->super;\
+	klass = RCLASS_SUPER(klass);\
     }\
     while (klass) {\
-	if (RCLASS(klass)->iv_tbl && st_lookup(RCLASS(klass)->iv_tbl,id,(v))) {\
-	    return (r);\
+	if (RCLASS_IV_TBL(klass) && st_lookup(RCLASS_IV_TBL(klass),id,(v))) {\
+	    r;\
 	}\
-	klass = RCLASS(klass)->super;\
+	klass = RCLASS_SUPER(klass);\
     }\
 } while(0)
 
+void
+rb_cvar_set(VALUE klass, ID id, VALUE val)
+{
+    VALUE tmp, front = 0, target = 0;
+
+    tmp = klass;
+    CVAR_LOOKUP(0, {if (!front) front = klass; target = klass;});
+    if (target) {
+	if (front && target != front) {
+	    ID did = id;
+
+	    if (RTEST(ruby_verbose)) {
+		rb_warning("class variable %s of %s is overtaken by %s",
+			   rb_id2name(id), rb_class2name(original_module(front)),
+			   rb_class2name(original_module(target)));
+	    }
+	    if (BUILTIN_TYPE(front) == T_CLASS) {
+		st_delete(RCLASS_IV_TBL(front),&did,0);
+	    }
+	}
+    }
+    else {
+	target = tmp;
+    }
+    mod_av_set(target, id, val, Qfalse);
+}
+
 VALUE
 rb_cvar_get(VALUE klass, ID id)
 {
-    VALUE value, tmp;
+    VALUE value, tmp, front = 0, target = 0;
 
     tmp = klass;
-    CVAR_LOOKUP(&value, value);
-    rb_name_error(id,"uninitialized class variable %s in %s",
-		  rb_id2name(id), rb_class2name(tmp));
-    return Qnil;		/* not reached */
+    CVAR_LOOKUP(&value, {if (!front) front = klass; target = klass;});
+    if (!target) {
+	rb_name_error(id,"uninitialized class variable %s in %s",
+		      rb_id2name(id), rb_class2name(tmp));
+    }
+    if (front && target != front) {
+	ID did = id;
+
+	if (RTEST(ruby_verbose)) {
+	    rb_warning("class variable %s of %s is overtaken by %s",
+		       rb_id2name(id), rb_class2name(original_module(front)),
+		       rb_class2name(original_module(target)));
+	}
+	if (BUILTIN_TYPE(front) == T_CLASS) {
+	    st_delete(RCLASS_IV_TBL(front),&did,0);
+	}
+    }
+    return value;
 }
 
 VALUE
 rb_cvar_defined(VALUE klass, ID id)
 {
     if (!klass) return Qfalse;
-    CVAR_LOOKUP(0,Qtrue);
+    CVAR_LOOKUP(0,return Qtrue);
     return Qfalse;
 }
 
@@ -1685,8 +1877,8 @@ cv_i(ID key, VALUE value, VALUE ary)
  *     class Two < One
  *       @@var2 = 2
  *     end
- *     One.class_variables   #=> ["@@var1"]
- *     Two.class_variables   #=> ["@@var2"]
+ *     One.class_variables   #=> [:@@var1]
+ *     Two.class_variables   #=> [:@@var2]
  */
 
 VALUE
@@ -1694,8 +1886,8 @@ rb_mod_class_variables(VALUE obj)
 {
     VALUE ary = rb_ary_new();
 
-    if (RCLASS(obj)->iv_tbl) {
-	st_foreach_safe(RCLASS(obj)->iv_tbl, cv_i, ary);
+    if (RCLASS_IV_TBL(obj)) {
+	st_foreach_safe(RCLASS_IV_TBL(obj), cv_i, ary);
     }
     return ary;
 }
@@ -1733,7 +1925,7 @@ rb_mod_remove_cvar(VALUE mod, VALUE name)
 	rb_raise(rb_eSecurityError, "Insecure: can't remove class variable");
     if (OBJ_FROZEN(mod)) rb_error_frozen("class/module");
 
-    if (RCLASS(mod)->iv_tbl && st_delete(ROBJECT(mod)->iv_tbl, (st_data_t*)&id, &val)) {
+    if (RCLASS_IV_TBL(mod) && st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, &val)) {
 	return val;
     }
     if (rb_cvar_defined(mod, id)) {
