@@ -2,8 +2,7 @@
 
   vm.c -
 
-  $Author: nobu $
-  $Date: 2007-08-20 23:17:16 +0900 (月, 20  8月 2007) $
+  $Author: ko1 $
 
   Copyright (C) 2004-2007 Koichi Sasada
 
@@ -12,10 +11,12 @@
 #include "ruby/ruby.h"
 #include "ruby/node.h"
 #include "ruby/st.h"
+#include "ruby/encoding.h"
 #include "gc.h"
 
 #include "insnhelper.h"
-#include "insnhelper.ci"
+#include "vm_insnhelper.c"
+#include "vm_eval.c"
 
 #define BUFSIZE 0x100
 #define PROCDEBUG 0
@@ -32,8 +33,6 @@ void vm_analysis_operand(int insn, int n, VALUE op);
 void vm_analysis_register(int reg, int isset);
 void vm_analysis_insn(int insn);
 
-static NODE *lfp_set_special_cref(VALUE *lfp, NODE * cref);
-
 #if OPT_STACK_CACHING
 static VALUE finish_insn_seq[1] = { BIN(finish_SC_ax_ax) };
 #elif OPT_CALL_THREADED_CODE
@@ -51,17 +50,17 @@ rb_vm_change_state(void)
 /* control stack frame */
 
 static inline VALUE
-rb_vm_set_finish_env(rb_thread_t *th)
+rb_vm_set_finish_env(rb_thread_t * th)
 {
-    vm_push_frame(th, 0, FRAME_MAGIC_FINISH,
+    vm_push_frame(th, 0, VM_FRAME_MAGIC_FINISH,
 		  Qnil, th->cfp->lfp[0], 0,
 		  th->cfp->sp, 0, 1);
     th->cfp->pc = (VALUE *)&finish_insn_seq[0];
     return Qtrue;
 }
 
-void
-rb_vm_set_top_stack(rb_thread_t *th, VALUE iseqval)
+static void
+vm_set_top_stack(rb_thread_t * th, VALUE iseqval)
 {
     rb_iseq_t *iseq;
     GetISeqPtr(iseqval, iseq);
@@ -73,34 +72,71 @@ rb_vm_set_top_stack(rb_thread_t *th, VALUE iseqval)
     /* for return */
     rb_vm_set_finish_env(th);
 
-    vm_push_frame(th, iseq, FRAME_MAGIC_TOP,
+    vm_push_frame(th, iseq, VM_FRAME_MAGIC_TOP,
 		  th->top_self, 0, iseq->iseq_encoded,
 		  th->cfp->sp, 0, iseq->local_size);
 }
 
-void
-rb_vm_set_eval_stack(rb_thread_t *th, VALUE iseqval)
+static void
+vm_set_eval_stack(rb_thread_t * th, VALUE iseqval, const NODE *cref)
 {
     rb_iseq_t *iseq;
-    rb_block_t *block = th->base_block;
+    rb_block_t * const block = th->base_block;
     GetISeqPtr(iseqval, iseq);
 
     /* for return */
     rb_vm_set_finish_env(th);
-    vm_push_frame(th, iseq, FRAME_MAGIC_EVAL, block->self,
+    vm_push_frame(th, iseq, VM_FRAME_MAGIC_EVAL, block->self,
 		  GC_GUARDED_PTR(block->dfp), iseq->iseq_encoded,
 		  th->cfp->sp, block->lfp, iseq->local_size);
+
+    if (cref) {
+	th->cfp->dfp[-1] = (VALUE)cref;
+    }
+}
+
+static rb_control_frame_t *
+vm_get_ruby_level_next_cfp(rb_thread_t *th, rb_control_frame_t *cfp)
+{
+    while (!RUBY_VM_CONTROL_FRAME_STACK_OVERFLOW_P(th, cfp)) {
+	if (RUBY_VM_NORMAL_ISEQ_P(cfp->iseq)) {
+	    return cfp;
+	}
+	cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp);
+    }
+    return 0;
+}
+
+rb_control_frame_t *
+vm_get_ruby_level_caller_cfp(rb_thread_t *th, rb_control_frame_t *cfp)
+{
+    if (RUBY_VM_NORMAL_ISEQ_P(cfp->iseq)) {
+	return cfp;
+    }
+
+    cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp);
+
+    while (!RUBY_VM_CONTROL_FRAME_STACK_OVERFLOW_P(th, cfp)) {
+	if (RUBY_VM_NORMAL_ISEQ_P(cfp->iseq)) {
+	    return cfp;
+	}
+
+	if ((cfp->flag & VM_FRAME_FLAG_PASSED) == 0) {
+	    break;
+	}
+	cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp);
+    }
+    return 0;
 }
 
 /* Env */
 
 static void
-env_free(void *ptr)
+env_free(void * const ptr)
 {
-    rb_env_t *env;
     RUBY_FREE_ENTER("env");
     if (ptr) {
-	env = ptr;
+	const rb_env_t * const env = ptr;
 	RUBY_FREE_UNLESS_NULL(env->env);
 	ruby_xfree(ptr);
     }
@@ -108,12 +144,12 @@ env_free(void *ptr)
 }
 
 static void
-env_mark(void *ptr)
+env_mark(void * const ptr)
 {
-    rb_env_t *env;
     RUBY_MARK_ENTER("env");
     if (ptr) {
-	env = ptr;
+	const rb_env_t * const env = ptr;
+
 	if (env->env) {
 	    /* TODO: should mark more restricted range */
 	    RUBY_GC_INFO("env->env\n");
@@ -122,6 +158,7 @@ env_mark(void *ptr)
 
 	RUBY_GC_INFO("env->prev_envval\n");
 	RUBY_MARK_UNLESS_NULL(env->prev_envval);
+	RUBY_MARK_UNLESS_NULL(env->block.self);
 	RUBY_MARK_UNLESS_NULL(env->block.proc);
 
 	if (env->block.iseq) {
@@ -151,7 +188,7 @@ env_alloc(void)
 static VALUE check_env_value(VALUE envval);
 
 static int
-check_env(rb_env_t *env)
+check_env(rb_env_t * const env)
 {
     printf("---\n");
     printf("envptr: %p\n", &env->block.dfp[0]);
@@ -185,8 +222,8 @@ check_env_value(VALUE envval)
 }
 
 static VALUE
-vm_make_env_each(rb_thread_t *th, rb_control_frame_t *cfp,
-		 VALUE *envptr, VALUE *endptr)
+vm_make_env_each(rb_thread_t * const th, rb_control_frame_t * const cfp,
+		 VALUE *envptr, VALUE * const endptr)
 {
     VALUE envval, penvval = 0;
     rb_env_t *env;
@@ -237,7 +274,7 @@ vm_make_env_each(rb_thread_t *th, rb_control_frame_t *cfp,
     for (i = 0; i <= local_size; i++) {
 	env->env[i] = envptr[-local_size + i];
 #if 0
-	dp(env->env[i]);
+	fprintf(stderr, "%2d ", &envptr[-local_size + i] - th->stack); dp(env->env[i]);
 	if (RUBY_VM_NORMAL_ISEQ_P(cfp->iseq)) {
 	    /* clear value stack for GC */
 	    envptr[-local_size + i] = 0;
@@ -262,12 +299,6 @@ vm_make_env_each(rb_thread_t *th, rb_control_frame_t *cfp,
     env->block.dfp = cfp->dfp;
     env->block.iseq = cfp->iseq;
 
-    if (VMDEBUG &&
-	(!(cfp->lfp[-1] == Qnil ||
-	  BUILTIN_TYPE(cfp->lfp[-1]) == T_VALUES))) {
-	rb_bug("illegal svar");
-    }
-
     if (!RUBY_VM_NORMAL_ISEQ_P(cfp->iseq)) {
 	/* TODO */
 	env->block.iseq = 0;
@@ -276,27 +307,26 @@ vm_make_env_each(rb_thread_t *th, rb_control_frame_t *cfp,
 }
 
 static int
-collect_local_variables_in_env(rb_env_t *env, VALUE ary)
+collect_local_variables_in_env(rb_env_t * const env, const VALUE ary)
 {
     int i;
-    if (env->block.lfp == env->block.dfp) {
-	return 0;
-    }
     for (i = 0; i < env->block.iseq->local_table_size; i++) {
 	ID lid = env->block.iseq->local_table[i];
 	if (lid) {
-	    rb_ary_push(ary, rb_str_new2(rb_id2name(lid)));
+	    rb_ary_push(ary, ID2SYM(lid));
 	}
     }
     if (env->prev_envval) {
-	GetEnvPtr(env->prev_envval, env);
-	collect_local_variables_in_env(env, ary);
+	rb_env_t *prevenv;
+	GetEnvPtr(env->prev_envval, prevenv);
+	collect_local_variables_in_env(prevenv, ary);
     }
     return 0;
 }
 
 int
-vm_collect_local_variables_in_heap(rb_thread_t *th, VALUE *dfp, VALUE ary)
+vm_collect_local_variables_in_heap(rb_thread_t * const th,
+				   VALUE * const dfp, const VALUE ary)
 {
     if (ENV_IN_HEAP_P(th, dfp)) {
 	rb_env_t *env;
@@ -310,10 +340,15 @@ vm_collect_local_variables_in_heap(rb_thread_t *th, VALUE *dfp, VALUE ary)
 }
 
 VALUE
-vm_make_env_object(rb_thread_t *th, rb_control_frame_t *cfp)
+vm_make_env_object(rb_thread_t * th, rb_control_frame_t *cfp)
 {
     VALUE envval;
 
+    if (VM_FRAME_TYPE(cfp) == VM_FRAME_MAGIC_FINISH) {
+	/* for method_missing */
+	cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp);
+    }
+
     envval = vm_make_env_each(th, cfp, cfp->dfp, cfp->lfp);
 
     if (PROCDEBUG) {
@@ -324,10 +359,10 @@ vm_make_env_object(rb_thread_t *th, rb_control_frame_t *cfp)
 }
 
 void
-vm_stack_to_heap(rb_thread_t *th)
+vm_stack_to_heap(rb_thread_t * const th)
 {
     rb_control_frame_t *cfp = th->cfp;
-    while ((cfp = vm_get_ruby_level_cfp(th, cfp)) != 0) {
+    while ((cfp = vm_get_ruby_level_next_cfp(th, cfp)) != 0) {
 	vm_make_env_object(th, cfp);
 	cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp);
     }
@@ -355,7 +390,7 @@ vm_make_proc_from_block(rb_thread_t *th, rb_control_frame_t *cfp,
 
 VALUE
 vm_make_proc(rb_thread_t *th,
-	     rb_control_frame_t *cfp, rb_block_t *block)
+	     rb_control_frame_t *cfp, const rb_block_t *block)
 {
     VALUE procval, envval, blockprocval = 0;
     rb_proc_t *proc;
@@ -386,7 +421,6 @@ vm_make_proc(rb_thread_t *th,
     proc->block.proc = procval;
     proc->envval = envval;
     proc->safe_level = th->safe_level;
-    proc->special_cref_stack = lfp_get_special_cref(block->lfp);
 
     if (VMDEBUG) {
 	if (th->stack < block->dfp && block->dfp < th->stack + th->stack_size) {
@@ -400,225 +434,86 @@ vm_make_proc(rb_thread_t *th,
     return procval;
 }
 
-/* C -> Ruby: method */
-
-VALUE
-vm_call0(rb_thread_t *th, VALUE klass, VALUE recv,
-	 VALUE id, ID oid, int argc, const VALUE *argv,
-	 NODE * body, int nosuper)
-{
-    VALUE val;
-    rb_block_t *blockptr = 0;
-
-    if (0) printf("id: %s, nd: %s, argc: %d, passed: %p\n",
-		  rb_id2name(id), ruby_node_name(nd_type(body)),
-		  argc, th->passed_block);
-
-    if (th->passed_block) {
-	blockptr = th->passed_block;
-	th->passed_block = 0;
-    }
-    switch (nd_type(body)) {
-      case RUBY_VM_METHOD_NODE:{
-	rb_control_frame_t *reg_cfp;
-	VALUE iseqval = (VALUE)body->nd_body;
-	int i;
-
-	rb_vm_set_finish_env(th);
-	reg_cfp = th->cfp;
-
-	CHECK_STACK_OVERFLOW(reg_cfp, argc + 1);
-
-	*reg_cfp->sp++ = recv;
-	for (i = 0; i < argc; i++) {
-	    *reg_cfp->sp++ = argv[i];
-	}
-
-	vm_setup_method(th, reg_cfp, argc, blockptr, 0, iseqval, recv, klass);
-	val = vm_eval_body(th);
-	break;
-      }
-      case NODE_CFUNC: {
-	EXEC_EVENT_HOOK(th, RUBY_EVENT_C_CALL, recv, id, klass);
-	{
-	    rb_control_frame_t *reg_cfp = th->cfp;
-	    rb_control_frame_t *cfp =
-		vm_push_frame(th, 0, FRAME_MAGIC_CFUNC,
-			      recv, (VALUE)blockptr, 0, reg_cfp->sp, 0, 1);
-
-	    cfp->method_id = id;
-	    cfp->method_klass = klass;
-
-	    val = call_cfunc(body->nd_cfnc, recv, body->nd_argc, argc, argv);
-
-	    if (reg_cfp != th->cfp + 1) {
-		SDR2(reg_cfp);
-		SDR2(th->cfp-5);
-		rb_bug("cfp consistency error - call0");
-		th->cfp = reg_cfp;
-	    }
-	    vm_pop_frame(th);
-	}
-	EXEC_EVENT_HOOK(th, RUBY_EVENT_C_RETURN, recv, id, klass);
-	break;
-      }
-      case NODE_ATTRSET:{
-	if (argc != 1) {
-	    rb_raise(rb_eArgError, "wrong number of arguments (%d for 1)",
-		     argc);
-	}
-	val = rb_ivar_set(recv, body->nd_vid, argv[0]);
-	break;
-      }
-      case NODE_IVAR: {
-	if (argc != 0) {
-	    rb_raise(rb_eArgError, "wrong number of arguments (%d for 0)",
-		     argc);
-	}
-	val = rb_attr_get(recv, body->nd_vid);
-	break;
-      }
-      case NODE_BMETHOD:{
-	val = vm_call_bmethod(th, id, body->nd_cval,
-			      recv, klass, argc, (VALUE *)argv);
-	break;
-      }
-      default:
-	rb_bug("unsupported: vm_call0(%s)", ruby_node_name(nd_type(body)));
-    }
-    RUBY_VM_CHECK_INTS();
-    return val;
-}
-
-static VALUE
-vm_call_super(rb_thread_t *th, int argc, const VALUE *argv)
-{
-    VALUE recv = th->cfp->self;
-    VALUE klass;
-    ID id;
-    NODE *body;
-    int nosuper = 0;
-    rb_control_frame_t *cfp = th->cfp;
-
-    if (!cfp->iseq) {
-	klass = cfp->method_klass;
-	klass = RCLASS(klass)->super;
-
-	if (klass == 0) {
-	    klass = vm_search_normal_super_klass(cfp->method_klass, recv);
-	}
-
-	id = cfp->method_id;
-    }
-    else {
-	rb_bug("vm_call_super: should not be reached");
-    }
-
-    body = rb_method_node(klass, id);	/* this returns NODE_METHOD */
-
-    if (body) {
-	body = body->nd_body;
-    }
-    else {
-	dp(recv);
-	dp(klass);
-	dpi(id);
-	rb_bug("vm_call_super: not found");
-    }
-
-    return vm_call0(th, klass, recv, id, id, argc, argv, body, nosuper);
-}
-
-VALUE
-rb_call_super(int argc, const VALUE *argv)
-{
-    return vm_call_super(GET_THREAD(), argc, argv);
-}
-
 /* C -> Ruby: block */
 
-static VALUE
-invoke_block(rb_thread_t *th, rb_block_t *block, VALUE self, int argc, VALUE *argv)
+static inline VALUE
+invoke_block_from_c(rb_thread_t *th, const rb_block_t *block,
+		    VALUE self, int argc, const VALUE *argv,
+		    const rb_block_t *blockptr, const NODE *cref)
 {
-    VALUE val;
     if (BUILTIN_TYPE(block->iseq) != T_NODE) {
-	rb_iseq_t *iseq = block->iseq;
-	int i, opt_pc;
+	const rb_iseq_t *iseq = block->iseq;
+	const rb_control_frame_t *cfp = th->cfp;
+	int i, opt_pc, arg_size = iseq->arg_size;
 	int type = block_proc_is_lambda(block->proc) ?
-	  FRAME_MAGIC_LAMBDA : FRAME_MAGIC_BLOCK;
+	  VM_FRAME_MAGIC_LAMBDA : VM_FRAME_MAGIC_BLOCK;
 
 	rb_vm_set_finish_env(th);
 
-	CHECK_STACK_OVERFLOW(th->cfp, argc);
-	CHECK_STACK_OVERFLOW(th->cfp, iseq->stack_max);
+	CHECK_STACK_OVERFLOW(cfp, argc + iseq->stack_max);
 
 	for (i=0; i<argc; i++) {
-	    th->cfp->sp[i] = argv[i];
+	    cfp->sp[i] = argv[i];
 	}
 
-	if (iseq->arg_block == -1) {
-	    opt_pc = vm_yield_setup_args(th, iseq, argc, th->cfp->sp, 0,
-					 type == FRAME_MAGIC_LAMBDA);
-	}
-	else {
-	    rb_block_t *blockptr = 0;
-	    if (rb_block_given_p()) {
-		rb_proc_t *proc;
-		VALUE procval;
-		procval = rb_block_proc();
-		GetProcPtr(procval, proc);
-		blockptr = &proc->block;
-	    }
-	    opt_pc = vm_yield_setup_args(th, iseq, argc, th->cfp->sp,
-					 blockptr, type == FRAME_MAGIC_LAMBDA);
-	}
-	argc = iseq->arg_size;
-	th->cfp->sp += argc;
+	opt_pc = vm_yield_setup_args(th, iseq, argc, cfp->sp, blockptr,
+				     type == VM_FRAME_MAGIC_LAMBDA);
 
 	vm_push_frame(th, iseq, type,
 		      self, GC_GUARDED_PTR(block->dfp),
-		      iseq->iseq_encoded + opt_pc, th->cfp->sp, block->lfp,
-		      iseq->local_size - argc);
-	val = vm_eval_body(th);
+		      iseq->iseq_encoded + opt_pc, cfp->sp + arg_size, block->lfp,
+		      iseq->local_size - arg_size);
+
+	if (cref) {
+	    th->cfp->dfp[-1] = (VALUE)cref;
+	}
+
+	return vm_eval_body(th);
     }
     else {
-	if (((NODE*)block->iseq)->u3.state == 1) {
-	    VALUE args = rb_ary_new4(argc, argv);
-	    argc = 1;
-	    argv = &args;
-	}
-	val = vm_yield_with_cfunc(th, block, block->self, argc, argv);
+	return vm_yield_with_cfunc(th, block, self, argc, argv, blockptr);
     }
-    return val;
 }
 
-VALUE
-vm_yield(rb_thread_t *th, int argc, VALUE *argv)
+static inline const rb_block_t *
+check_block(rb_thread_t *th)
 {
-    rb_block_t *block = GC_GUARDED_PTR_REF(th->cfp->lfp[0]);
+    const rb_block_t *blockptr = GC_GUARDED_PTR_REF(th->cfp->lfp[0]);
 
-    if (block == 0) {
+    if (blockptr == 0) {
 	vm_localjump_error("no block given", Qnil, 0);
     }
 
-    return invoke_block(th, block, block->self, argc, argv);
+    return blockptr;
+}
+
+static inline VALUE
+vm_yield_with_cref(rb_thread_t *th, int argc, const VALUE *argv, const NODE *cref)
+{
+    const rb_block_t *blockptr = check_block(th);
+    return invoke_block_from_c(th, blockptr, blockptr->self, argc, argv, 0, cref);
+}
+
+static inline VALUE
+vm_yield(rb_thread_t *th, int argc, const VALUE *argv)
+{
+    const rb_block_t *blockptr = check_block(th);
+    return invoke_block_from_c(th, blockptr, blockptr->self, argc, argv, 0, 0);
 }
 
 VALUE
-vm_invoke_proc(rb_thread_t *th, rb_proc_t *proc,
-	       VALUE self, int argc, VALUE *argv)
+vm_invoke_proc(rb_thread_t *th, rb_proc_t *proc, VALUE self,
+	       int argc, const VALUE *argv, rb_block_t * blockptr)
 {
     VALUE val = Qundef;
     int state;
     volatile int stored_safe = th->safe_level;
-    volatile NODE *stored_special_cref_stack =
-      lfp_set_special_cref(proc->block.lfp, proc->special_cref_stack);
     rb_control_frame_t * volatile cfp = th->cfp;
 
     TH_PUSH_TAG(th);
     if ((state = EXEC_TAG()) == 0) {
 	th->safe_level = proc->safe_level;
-	val = invoke_block(th, &proc->block, self, argc, argv);
+	val = invoke_block_from_c(th, &proc->block, self, argc, argv, blockptr, 0);
     }
     TH_POP_TAG();
 
@@ -626,15 +521,13 @@ vm_invoke_proc(rb_thread_t *th, rb_proc_t *proc,
 	th->safe_level = stored_safe;
     }
 
-    lfp_set_special_cref(proc->block.lfp, (NODE*)stored_special_cref_stack);
-
     if (state) {
 	if (state == TAG_RETURN && proc->is_lambda) {
 	    VALUE err = th->errinfo;
 	    VALUE *escape_dfp = GET_THROWOBJ_CATCH_POINT(err);
-	    VALUE *cdfp = proc->block.dfp;
 
-	    if (escape_dfp == cdfp) {
+	    if (escape_dfp == cfp->dfp) {
+		printf("ok\n");
 		state = 0;
 		th->errinfo = Qnil;
 		th->cfp = cfp;
@@ -651,22 +544,30 @@ vm_invoke_proc(rb_thread_t *th, rb_proc_t *proc,
 
 /* special variable */
 
-VALUE
-vm_cfp_svar_get(rb_thread_t *th, rb_control_frame_t *cfp, VALUE key)
+static rb_control_frame_t *
+vm_normal_frame(rb_thread_t *th, rb_control_frame_t *cfp)
 {
     while (cfp->pc == 0) {
-	cfp++;
+	cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp);
+	if (RUBY_VM_CONTROL_FRAME_STACK_OVERFLOW_P(th, cfp)) {
+	    return 0;
+	}
     }
-    return lfp_svar_get(th, cfp->lfp, key);
+    return cfp;
 }
 
-void
-vm_cfp_svar_set(rb_thread_t *th, rb_control_frame_t *cfp, VALUE key, VALUE val)
+static VALUE
+vm_cfp_svar_get(rb_thread_t *th, rb_control_frame_t *cfp, VALUE key)
 {
-    while (cfp->pc == 0) {
-	cfp++;
-    }
-    lfp_svar_set(th, cfp->lfp, key, val);
+    cfp = vm_normal_frame(th, cfp);
+    return lfp_svar_get(th, cfp ? cfp->lfp : 0, key);
+}
+
+static void
+vm_cfp_svar_set(rb_thread_t *th, rb_control_frame_t *cfp, VALUE key, const VALUE val)
+{
+    cfp = vm_normal_frame(th, cfp);
+    lfp_svar_set(th, cfp ? cfp->lfp : 0, key, val);
 }
 
 static VALUE
@@ -710,10 +611,10 @@ rb_lastline_set(VALUE val)
 /* backtrace */
 
 int
-vm_get_sourceline(rb_control_frame_t *cfp)
+vm_get_sourceline(const rb_control_frame_t *cfp)
 {
     int line_no = 0;
-    rb_iseq_t *iseq = cfp->iseq;
+    const rb_iseq_t *iseq = cfp->iseq;
 
     if (RUBY_VM_NORMAL_ISEQ_P(iseq)) {
 	int i;
@@ -733,9 +634,8 @@ vm_get_sourceline(rb_control_frame_t *cfp)
 
 static VALUE
 vm_backtrace_each(rb_thread_t *th,
-		  rb_control_frame_t *limit_cfp,
-		  rb_control_frame_t *cfp,
-		  char *file, int line_no, VALUE ary)
+		  const rb_control_frame_t *limit_cfp, const rb_control_frame_t *cfp,
+		  const char * file, int line_no, VALUE ary)
 {
     VALUE str;
 
@@ -763,12 +663,12 @@ vm_backtrace_each(rb_thread_t *th,
     return rb_ary_reverse(ary);
 }
 
-VALUE
+static inline VALUE
 vm_backtrace(rb_thread_t *th, int lev)
 {
     VALUE ary;
-    rb_control_frame_t *cfp = th->cfp;
-    rb_control_frame_t *top_of_cfp = (void *)(th->stack + th->stack_size);
+    const rb_control_frame_t *cfp = th->cfp;
+    const rb_control_frame_t *top_of_cfp = (void *)(th->stack + th->stack_size);
     top_of_cfp -= 2;
 
     if (lev < 0) {
@@ -790,48 +690,40 @@ vm_backtrace(rb_thread_t *th, int lev)
     return ary;
 }
 
-/* cref */
-
-static void
-check_svar(void)
+const char *
+rb_sourcefile(void)
 {
     rb_thread_t *th = GET_THREAD();
-    rb_control_frame_t *cfp = th->cfp;
-    while ((void *)(cfp + 1) < (void *)(th->stack + th->stack_size)) {
-	/* printf("cfp: %p\n", cfp->type); */
-	if (cfp->lfp && cfp->lfp[-1] != Qnil &&
-	    TYPE(cfp->lfp[-1]) != T_VALUES) {
-	    /* dp(cfp->lfp[-1]); */
-	    rb_bug("!!!illegal svar!!!");
-	}
-	cfp++;
+    rb_control_frame_t *cfp = vm_get_ruby_level_next_cfp(th, th->cfp);
+
+    if (cfp) {
+	return RSTRING_PTR(cfp->iseq->filename);
+    }
+    else {
+	return 0;
     }
 }
 
-static NODE *
-lfp_set_special_cref(VALUE *lfp, NODE * cref)
+int
+rb_sourceline(void)
 {
-    struct RValues *values = (void *) lfp[-1];
-    NODE *old_cref;
+    rb_thread_t *th = GET_THREAD();
+    rb_control_frame_t *cfp = vm_get_ruby_level_next_cfp(th, th->cfp);
 
-    if (VMDEBUG) {
-	check_svar();
-    }
-
-    if (cref == 0 && ((VALUE)values == Qnil || values->basic.klass == 0)) {
-	old_cref = 0;
+    if (cfp) {
+	return vm_get_sourceline(cfp);
     }
     else {
-	old_cref = (NODE *)lfp_svar_get(GET_THREAD(), lfp, 2);
-	lfp_svar_set(GET_THREAD(), lfp, 2, (VALUE)cref);
+	return 0;
     }
-    return old_cref;
 }
 
 NODE *
-vm_set_special_cref(rb_thread_t *th, VALUE *lfp, NODE * cref_stack)
+vm_cref(void)
 {
-    return lfp_set_special_cref(lfp, cref_stack);
+    rb_thread_t *th = GET_THREAD();
+    rb_control_frame_t *cfp = vm_get_ruby_level_next_cfp(th, th->cfp);
+    return vm_get_cref(cfp->iseq, cfp->lfp, cfp->dfp);
 }
 
 #if 0
@@ -846,29 +738,25 @@ debug_cref(NODE *cref)
 }
 #endif
 
-NODE *
-vm_get_cref(rb_thread_t *th, rb_iseq_t *iseq, rb_control_frame_t *cfp)
-{
-    return get_cref(iseq, cfp->lfp);
-}
-
-NODE *
+static NODE *
 vm_cref_push(rb_thread_t *th, VALUE klass, int noex)
 {
+    rb_control_frame_t *cfp = vm_get_ruby_level_caller_cfp(th, th->cfp);
     NODE *cref = NEW_BLOCK(klass);
-    rb_control_frame_t *cfp = vm_get_ruby_level_cfp(th, th->cfp);
-
     cref->nd_file = 0;
-    cref->nd_next = get_cref(cfp->iseq, cfp->lfp);
     cref->nd_visi = noex;
+
+    if (cfp) {
+	cref->nd_next = vm_get_cref(cfp->iseq, cfp->lfp, cfp->dfp);
+    }
+
     return cref;
 }
 
-VALUE
-vm_get_cbase(rb_thread_t *th)
+static inline VALUE
+vm_get_cbase(const rb_iseq_t *iseq, const VALUE *lfp, const VALUE *dfp)
 {
-    rb_control_frame_t *cfp = vm_get_ruby_level_cfp(th, th->cfp);
-    NODE *cref = get_cref(cfp->iseq, cfp->lfp);
+    NODE *cref = vm_get_cref(iseq, lfp, dfp);
     VALUE klass = Qundef;
 
     while (cref) {
@@ -877,9 +765,19 @@ vm_get_cbase(rb_thread_t *th)
 	}
 	cref = cref->nd_next;
     }
+
     return klass;
 }
 
+VALUE
+rb_vm_cbase(void)
+{
+    rb_thread_t *th = GET_THREAD();
+    rb_control_frame_t *cfp = vm_get_ruby_level_next_cfp(th, th->cfp);
+
+    return vm_get_cbase(cfp->iseq, cfp->lfp, cfp->dfp);
+}
+
 /* jump */
 
 static VALUE
@@ -891,22 +789,22 @@ make_localjump_error(const char *mesg, VALUE value, int reason)
 
     switch (reason) {
       case TAG_BREAK:
-	id = rb_intern("break");
+	CONST_ID(id, "break");
 	break;
       case TAG_REDO:
-	id = rb_intern("redo");
+	CONST_ID(id, "redo");
 	break;
       case TAG_RETRY:
-	id = rb_intern("retry");
+	CONST_ID(id, "retry");
 	break;
       case TAG_NEXT:
-	id = rb_intern("next");
+	CONST_ID(id, "next");
 	break;
       case TAG_RETURN:
-	id = rb_intern("return");
+	CONST_ID(id, "return");
 	break;
       default:
-	id = rb_intern("noreason");
+	CONST_ID(id, "noreason");
 	break;
     }
     rb_iv_set(exc, "@exit_value", value);
@@ -926,8 +824,9 @@ vm_make_jump_tag_but_local_jump(int state, VALUE val)
 {
     VALUE result = Qnil;
 
-    if (val == Qundef)
+    if (val == Qundef) {
 	val = GET_THREAD()->tag->retval;
+    }
     switch (state) {
       case 0:
 	break;
@@ -976,7 +875,7 @@ vm_iter_break(rb_thread_t *th)
 }
 
 void
-rb_iter_break()
+rb_iter_break(void)
 {
     vm_iter_break(GET_THREAD());
 }
@@ -986,8 +885,8 @@ rb_iter_break()
 VALUE ruby_vm_redefined_flag = 0;
 static st_table *vm_opt_method_table = 0;
 
-void
-rb_vm_check_redefinition_opt_method(NODE *node)
+static void
+rb_vm_check_redefinition_opt_method(const NODE *node)
 {
     VALUE bop;
 
@@ -1000,7 +899,7 @@ static void
 add_opt_method(VALUE klass, ID mid, VALUE bop)
 {
     NODE *node;
-    if (st_lookup(RCLASS(klass)->m_tbl, mid, (void *)&node) &&
+    if (st_lookup(RCLASS_M_TBL(klass), mid, (void *)&node) &&
 	nd_type(node->nd_body->nd_body) == NODE_CFUNC) {
 	st_insert(vm_opt_method_table, (st_data_t)node, (st_data_t)bop);
     }
@@ -1040,12 +939,12 @@ vm_init_redefined_flag(void)
 
 /* evaluator body */
 
-#include "vm_evalbody.ci"
+#include "vm_evalbody.c"
 
 /*                  finish
   VMe (h1)          finish
     VM              finish F1 F2
-      func          finish F1 F2 C1
+      cfunc         finish F1 F2 C1
         rb_funcall  finish F1 F2 C1
           VMe       finish F1 F2 C1
             VM      finish F1 F2 C1 F3
@@ -1054,16 +953,16 @@ vm_init_redefined_flag(void)
   C1      : pushed by send insn (CFUNC)
 
   struct CONTROL_FRAME {
-    VALUE *pc;                  // cfp[0]
-    VALUE *sp;                  // cfp[1]
-    VALUE *bp;                  // cfp[2]
-    rb_iseq_t *iseq;            // cfp[3]
-    VALUE flag;                 // cfp[4]
-    VALUE self;                 // cfp[5]
-    VALUE *lfp;                 // cfp[6]
-    VALUE *dfp;                 // cfp[7]
-    rb_iseq_t * block_iseq;     // cfp[8]
-    VALUE proc;                 // cfp[9] always 0
+    VALUE *pc;                  // cfp[0], program counter
+    VALUE *sp;                  // cfp[1], stack pointer
+    VALUE *bp;                  // cfp[2], base pointer
+    rb_iseq_t *iseq;            // cfp[3], iseq
+    VALUE flag;                 // cfp[4], magic
+    VALUE self;                 // cfp[5], self
+    VALUE *lfp;                 // cfp[6], local frame pointer
+    VALUE *dfp;                 // cfp[7], dynamic frame pointer
+    rb_iseq_t * block_iseq;     // cfp[8], block iseq
+    VALUE proc;                 // cfp[9], always 0
   };
 
   struct BLOCK {
@@ -1071,15 +970,11 @@ vm_init_redefined_flag(void)
     VALUE *lfp;
     VALUE *dfp;
     rb_iseq_t *block_iseq;
-  };
-
-  struct PROC {
-    VALUE  proc_sig = 0;
-    struct BLOCK;
+    VALUE proc;
   };
 
   struct METHOD_CONTROL_FRAME {
-    struct CONTROL_FRAME;
+    rb_control_frame_t frame;
   };
 
   struct METHOD_FRAME {
@@ -1089,12 +984,13 @@ vm_init_redefined_flag(void)
     VALUE param0;
     ...
     VALUE paramN;
+    VALUE cref;
     VALUE special;                         // lfp [1]
     struct block_object *block_ptr | 0x01; // lfp [0]
   };
 
   struct BLOCK_CONTROL_FRAME {
-    struct STACK_FRAME;
+    rb_control_frame_t frame;
   };
 
   struct BLOCK_FRAME {
@@ -1104,17 +1000,19 @@ vm_init_redefined_flag(void)
     VALUE param0;
     ...
     VALUE paramN;
+    VALUE cref;
     VALUE *(prev_ptr | 0x01); // DFP[0]
   };
 
   struct CLASS_CONTROL_FRAME {
-    struct STACK_FRAME;
+    rb_control_frame_t frame;
   };
 
   struct CLASS_FRAME {
     VALUE param0;
     ...
     VALUE paramN;
+    VALUE cref;
     VALUE prev_dfp; // for frame jump
   };
 
@@ -1141,22 +1039,19 @@ vm_init_redefined_flag(void)
     VALUE *dfp;                      // lfp
     rb_iseq_t * block_iseq; // 0
   };
-
-  struct C_METHDO_FRAME{
-    VALUE block_ptr;
-    VALUE special;
-  };
  */
 
 
-VALUE
+static VALUE
 vm_eval_body(rb_thread_t *th)
 {
     int state;
     VALUE result, err;
     VALUE initial = 0;
+    VALUE *escape_dfp = NULL;
 
     TH_PUSH_TAG(th);
+    _tag.retval = Qnil;
     if ((state = EXEC_TAG()) == 0) {
       vm_loop_start:
 	result = vm_eval(th, initial);
@@ -1172,7 +1067,6 @@ vm_eval_body(rb_thread_t *th)
 	unsigned long epc, cont_pc, cont_sp;
 	VALUE catch_iseqval;
 	rb_control_frame_t *cfp;
-	VALUE *escape_dfp = NULL;
 	VALUE type;
 
 	err = th->errinfo;
@@ -1277,9 +1171,7 @@ vm_eval_body(rb_thread_t *th)
 			cfp->pc = cfp->iseq->iseq_encoded + entry->cont;
 			cfp->sp = cfp->bp + entry->sp;
 
-			if (!(state == TAG_REDO) &&
-			    !(state == TAG_NEXT && !escape_dfp) &&
-			    !(state == TAG_BREAK && !escape_dfp)) {
+			if (state != TAG_REDO) {
 #if OPT_STACK_CACHING
 			    initial = (GET_THROWOBJ_VAL(err));
 #else
@@ -1294,12 +1186,10 @@ vm_eval_body(rb_thread_t *th)
 	}
 	else if (state == TAG_REDO) {
 	    type = CATCH_TYPE_REDO;
-	    escape_dfp = GET_THROWOBJ_CATCH_POINT(err);
 	    goto search_restart_point;
 	}
 	else if (state == TAG_NEXT) {
 	    type = CATCH_TYPE_NEXT;
-	    escape_dfp = GET_THROWOBJ_CATCH_POINT(err);
 	    goto search_restart_point;
 	}
 	else {
@@ -1328,9 +1218,9 @@ vm_eval_body(rb_thread_t *th)
 
 	    /* push block frame */
 	    cfp->sp[0] = err;
-	    vm_push_frame(th, catch_iseq, FRAME_MAGIC_BLOCK,
+	    vm_push_frame(th, catch_iseq, VM_FRAME_MAGIC_BLOCK,
 			  cfp->self, (VALUE)cfp->dfp, catch_iseq->iseq_encoded,
-			  cfp->sp + 1, cfp->lfp, catch_iseq->local_size - 1);
+			  cfp->sp + 1 /* push value */, cfp->lfp, catch_iseq->local_size - 1);
 
 	    state = 0;
 	    th->errinfo = Qnil;
@@ -1363,7 +1253,7 @@ rb_iseq_eval(VALUE iseqval)
     VALUE val;
     volatile VALUE tmp;
 
-    rb_vm_set_top_stack(th, iseqval);
+    vm_set_top_stack(th, iseqval);
 
     if (!rb_const_defined(rb_cObject, rb_intern("TOPLEVEL_BINDING"))) {
 	rb_define_global_const("TOPLEVEL_BINDING", rb_binding_new());
@@ -1374,31 +1264,45 @@ rb_iseq_eval(VALUE iseqval)
 }
 
 int
-rb_thread_method_id_and_klass(rb_thread_t *th, ID *idp, VALUE *klassp)
+rb_thread_method_id_and_class(rb_thread_t *th,
+			      ID *idp, VALUE *klassp)
 {
     rb_control_frame_t *cfp = th->cfp;
-
-    if (cfp->iseq) {
-	if (cfp->pc != 0) {
-	    rb_iseq_t *iseq = cfp->iseq->local_iseq;
-	    if (idp) *idp = rb_intern(RSTRING_PTR(iseq->name));
+    rb_iseq_t *iseq = cfp->iseq;
+    if (!iseq) {
+	if (idp) *idp = cfp->method_id;
+	if (klassp) *klassp = cfp->method_class;
+	return 1;
+    }
+    while (iseq) {
+	if (RUBY_VM_IFUNC_P(iseq)) {
+	    if (idp) CONST_ID(*idp, "<ifunc>");
+	    if (klassp) *klassp = 0;
+	    return 1;
+	}
+	if (iseq->defined_method_id) {
+	    if (idp) *idp = iseq->defined_method_id;
 	    if (klassp) *klassp = iseq->klass;
 	    return 1;
 	}
+	if (iseq->local_iseq == iseq) {
+	    break;
+	}
+	iseq = iseq->parent_iseq;
     }
-    else {
-	if (idp) *idp = cfp->method_id;
-	if (klassp) *klassp = cfp->method_klass;
-	return 1;
-    }
-    *idp = *klassp = 0;
     return 0;
 }
 
-VALUE
-rb_thread_current_status(rb_thread_t *th)
+int
+rb_frame_method_id_and_class(ID *idp, VALUE *klassp)
 {
-    rb_control_frame_t *cfp = th->cfp;
+    return rb_thread_method_id_and_class(GET_THREAD(), idp, klassp);
+}
+
+VALUE
+rb_thread_current_status(const rb_thread_t *th)
+{
+    const rb_control_frame_t *cfp = th->cfp;
     VALUE str = Qnil;
 
     if (cfp->iseq != 0) {
@@ -1412,7 +1316,7 @@ rb_thread_current_status(rb_thread_t *th)
     }
     else if (cfp->method_id) {
 	str = rb_sprintf("`%s#%s' (cfunc)",
-			 RSTRING_PTR(rb_class_name(cfp->method_klass)),
+			 RSTRING_PTR(rb_class_name(cfp->method_class)),
 			 rb_id2name(cfp->method_id));
     }
 
@@ -1420,15 +1324,17 @@ rb_thread_current_status(rb_thread_t *th)
 }
 
 VALUE
-rb_vm_call_cfunc(VALUE recv, VALUE (*func)(VALUE), VALUE arg, rb_block_t *blockptr, VALUE filename)
+rb_vm_call_cfunc(VALUE recv, VALUE (*func)(VALUE), VALUE arg,
+		 const rb_block_t *blockptr, VALUE filename)
 {
     rb_thread_t *th = GET_THREAD();
-    rb_control_frame_t *reg_cfp = th->cfp;
-    volatile VALUE iseq = rb_iseq_new(0, filename, filename, 0, ISEQ_TYPE_TOP);
+    const rb_control_frame_t *reg_cfp = th->cfp;
+    volatile VALUE iseqval = rb_iseq_new(0, filename, filename, 0, ISEQ_TYPE_TOP);
     VALUE val;
 
-    vm_push_frame(th, DATA_PTR(iseq), FRAME_MAGIC_TOP,
+    vm_push_frame(th, DATA_PTR(iseqval), VM_FRAME_MAGIC_TOP,
 		  recv, (VALUE)blockptr, 0, reg_cfp->sp, 0, 1);
+
     val = (*func)(arg);
 
     vm_pop_frame(th);
@@ -1484,8 +1390,10 @@ rb_vm_mark(void *ptr)
 	RUBY_MARK_UNLESS_NULL(vm->thgroup_default);
 	RUBY_MARK_UNLESS_NULL(vm->mark_object_ary);
 	RUBY_MARK_UNLESS_NULL(vm->last_status);
+	RUBY_MARK_UNLESS_NULL(vm->load_path);
 	RUBY_MARK_UNLESS_NULL(vm->loaded_features);
 	RUBY_MARK_UNLESS_NULL(vm->top_self);
+	rb_gc_mark_locations(vm->special_exceptions, vm->special_exceptions + ruby_special_error_count - 1);
 
 	if (vm->loading_table) {
 	    rb_mark_tbl(vm->loading_table);
@@ -1501,10 +1409,55 @@ static void
 vm_init2(rb_vm_t *vm)
 {
     MEMZERO(vm, rb_vm_t, 1);
+    vm->src_encoding_index = -1;
 }
 
 /* Thread */
 
+#define USE_THREAD_DATA_RECYCLE 1
+
+#if USE_THREAD_DATA_RECYCLE
+#define RECYCLE_MAX 64
+VALUE *thread_recycle_stack_slot[RECYCLE_MAX];
+int thread_recycle_stack_count = 0;
+
+static VALUE *
+thread_recycle_stack(int size)
+{
+    if (thread_recycle_stack_count) {
+	return thread_recycle_stack_slot[--thread_recycle_stack_count];
+    }
+    else {
+	return ALLOC_N(VALUE, size);
+    }
+}
+
+#else
+#define thread_recycle_stack(size) ALLOC_N(VALUE, (size))
+#endif
+
+void
+rb_thread_recycle_stack_release(VALUE *stack)
+{
+#if USE_THREAD_DATA_RECYCLE
+    if (thread_recycle_stack_count < RECYCLE_MAX) {
+	thread_recycle_stack_slot[thread_recycle_stack_count++] = stack;
+	return;
+    }
+#endif
+    ruby_xfree(stack);
+}
+
+#ifdef USE_THREAD_RECYCLE
+static rb_thread_t *
+thread_recycle_struct(void)
+{
+    void *p = ALLOC_N(rb_thread_t, 1);
+    memset(p, 0, sizeof(rb_thread_t));
+    return p;
+}
+#endif
+
 static void
 thread_free(void *ptr)
 {
@@ -1513,11 +1466,18 @@ thread_free(void *ptr)
 
     if (ptr) {
 	th = ptr;
-
+	
 	if (!th->root_fiber) {
 	    RUBY_FREE_UNLESS_NULL(th->stack);
 	}
 
+	if (th->locking_mutex != Qfalse) {
+	    rb_bug("thread_free: locking_mutex must be NULL (%p:%ld)", th, th->locking_mutex);
+	}
+	if (th->keeping_mutexes != Qfalse) {
+	    rb_bug("thread_free: keeping_mutexes must be NULL (%p:%ld)", th, th->locking_mutex);
+	}
+
 	if (th->local_storage) {
 	    st_free_table(th->local_storage);
 	}
@@ -1555,14 +1515,15 @@ rb_thread_mark(void *ptr)
 	th = ptr;
 	if (th->stack) {
 	    VALUE *p = th->stack;
-	    VALUE *sp = th->cfp->sp + th->mark_stack_len;
+	    VALUE *sp = th->cfp->sp;
 	    rb_control_frame_t *cfp = th->cfp;
-	    rb_control_frame_t *limit_cfp =
-	      (void *)(th->stack + th->stack_size);
+	    rb_control_frame_t *limit_cfp = (void *)(th->stack + th->stack_size);
 
 	    while (p < sp) {
 		rb_gc_mark(*p++);
 	    }
+	    rb_gc_mark_locations(p, p + th->mark_stack_len);
+
 	    while (cfp != limit_cfp) {
 		rb_gc_mark(cfp->proc);
 		cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp);
@@ -1582,6 +1543,9 @@ rb_thread_mark(void *ptr)
 	RUBY_MARK_UNLESS_NULL(th->top_wrapper);
 	RUBY_MARK_UNLESS_NULL(th->fiber);
 	RUBY_MARK_UNLESS_NULL(th->root_fiber);
+	RUBY_MARK_UNLESS_NULL(th->stat_insn_usage);
+
+	RUBY_MARK_UNLESS_NULL(th->locking_mutex);
 
 	rb_mark_tbl(th->local_storage);
 
@@ -1595,7 +1559,6 @@ rb_thread_mark(void *ptr)
 	mark_event_hooks(th->event_hooks);
     }
 
-    RUBY_MARK_UNLESS_NULL(th->stat_insn_usage);
     RUBY_MARK_LEAVE("thread");
 }
 
@@ -1603,35 +1566,29 @@ static VALUE
 thread_alloc(VALUE klass)
 {
     VALUE volatile obj;
+#ifdef USE_THREAD_RECYCLE
+    rb_thread_t *th = thread_recycle_struct();
+    obj = Data_Wrap_Struct(klass, rb_thread_mark, thread_free, th);
+#else
     rb_thread_t *th;
-    obj = Data_Make_Struct(klass, rb_thread_t,
-			   rb_thread_mark, thread_free, th);
+    obj = Data_Make_Struct(klass, rb_thread_t, rb_thread_mark, thread_free, th);
+#endif
     return obj;
 }
 
 static void
-th_init2(rb_thread_t *th)
+th_init2(rb_thread_t *th, VALUE self)
 {
-    MEMZERO(th, rb_thread_t, 1);
+    th->self = self;
 
     /* allocate thread stack */
     th->stack_size = RUBY_VM_THREAD_STACK_SIZE;
-    th->stack = ALLOC_N(VALUE, th->stack_size);
+    th->stack = thread_recycle_stack(th->stack_size);
 
     th->cfp = (void *)(th->stack + th->stack_size);
-    th->cfp--;
 
-    th->cfp->pc = 0;
-    th->cfp->sp = th->stack + 1;
-    th->cfp->bp = 0;
-    th->cfp->lfp = th->stack;
-    *th->cfp->lfp = 0;
-    th->cfp->dfp = th->stack;
-    th->cfp->self = Qnil;
-    th->cfp->flag = 0;
-    th->cfp->iseq = 0;
-    th->cfp->proc = 0;
-    th->cfp->block_iseq = 0;
+    vm_push_frame(th, 0, VM_FRAME_MAGIC_TOP, Qnil, 0, 0,
+		  th->stack, 0, 1);
 
     th->status = THREAD_RUNNABLE;
     th->errinfo = Qnil;
@@ -1642,9 +1599,9 @@ th_init2(rb_thread_t *th)
 }
 
 static void
-th_init(rb_thread_t *th)
+th_init(rb_thread_t *th, VALUE self)
 {
-    th_init2(th);
+    th_init2(th, self);
 }
 
 static VALUE
@@ -1654,8 +1611,7 @@ ruby_thread_init(VALUE self)
     rb_vm_t *vm = GET_THREAD()->vm;
     GetThreadPtr(self, th);
 
-    th_init(th);
-    th->self = self;
+    th_init(th, self);
     th->vm = vm;
 
     th->top_wrapper = 0;
@@ -1673,10 +1629,13 @@ rb_thread_alloc(VALUE klass)
 
 VALUE insns_name_array(void);
 extern VALUE *rb_gc_stack_start;
+extern size_t rb_gc_stack_maxsize;
 #ifdef __ia64
 extern VALUE *rb_gc_register_stack_start;
 #endif
 
+/* debug functions */
+
 static VALUE
 sdr(void)
 {
@@ -1703,7 +1662,7 @@ nsdr(void)
     for (i=0; i<n; i++) {
 	rb_ary_push(ary, rb_str_new2(syms[i]));
     }
-    free(syms);
+    free(syms); /* OK */
 #endif
     return ary;
 }
@@ -1724,12 +1683,11 @@ Init_VM(void)
     /* ::Thread */
     rb_cThread = rb_define_class("Thread", rb_cObject);
     rb_undef_alloc_func(rb_cThread);
-    rb_define_method(rb_cThread, "initialize", ruby_thread_init, 0);
 
-    /* ::VM::USAGE_ANALISYS_* */
-    rb_define_const(rb_cVM, "USAGE_ANALISYS_INSN", rb_hash_new());
-    rb_define_const(rb_cVM, "USAGE_ANALISYS_REGS", rb_hash_new());
-    rb_define_const(rb_cVM, "USAGE_ANALISYS_INSN_BIGRAM", rb_hash_new());
+    /* ::VM::USAGE_ANALYSIS_* */
+    rb_define_const(rb_cVM, "USAGE_ANALYSIS_INSN", rb_hash_new());
+    rb_define_const(rb_cVM, "USAGE_ANALYSIS_REGS", rb_hash_new());
+    rb_define_const(rb_cVM, "USAGE_ANALYSIS_INSN_BIGRAM", rb_hash_new());
     rb_define_const(rb_cVM, "OPTS", opts = rb_ary_new());
 
 #if   OPT_DIRECT_THREADED_CODE
@@ -1764,22 +1722,28 @@ Init_VM(void)
     rb_define_const(rb_cVM, "INSTRUCTION_NAMES", insns_name_array());
 
     /* debug functions ::VM::SDR(), ::VM::NSDR() */
+#if VMDEBUG
     rb_define_singleton_method(rb_cVM, "SDR", sdr, 0);
     rb_define_singleton_method(rb_cVM, "NSDR", nsdr, 0);
+#else
+    (void)sdr;
+    (void)nsdr;
+#endif
 
     /* VM bootstrap: phase 2 */
     {
 	rb_vm_t *vm = ruby_current_vm;
 	rb_thread_t *th = GET_THREAD();
+	VALUE filename = rb_str_new2("<dummy toplevel>");
+	volatile VALUE iseqval = rb_iseq_new(0, filename, filename, 0, ISEQ_TYPE_TOP);
         volatile VALUE th_self;
+	rb_iseq_t *iseq;
 
 	/* create vm object */
 	vm->self = Data_Wrap_Struct(rb_cVM, rb_vm_mark, vm_free, vm);
 
 	/* create main thread */
-	th_self = th->self = Data_Wrap_Struct(rb_cThread, rb_thread_mark,
-					      thread_free, th);
-
+	th_self = th->self = Data_Wrap_Struct(rb_cThread, rb_thread_mark, thread_free, th);
 	vm->main_thread = th;
 	vm->running_thread = th;
 	th->vm = vm;
@@ -1789,27 +1753,39 @@ Init_VM(void)
 
 	vm->living_threads = st_init_numtable();
 	st_insert(vm->living_threads, th_self, (st_data_t) th->thread_id);
+
+	rb_register_mark_object(iseqval);
+	GetISeqPtr(iseqval, iseq);
+	th->cfp->iseq = iseq;
+	th->cfp->pc = iseq->iseq_encoded;
     }
     vm_init_redefined_flag();
 }
 
+#if defined(ENABLE_VM_OBJSPACE) && ENABLE_VM_OBJSPACE
+struct rb_objspace *rb_objspace_alloc(void);
+#endif
+void ruby_thread_init_stack(rb_thread_t *th);
+
 void
 Init_BareVM(void)
 {
     /* VM bootstrap: phase 1 */
-    rb_vm_t *vm = ALLOC(rb_vm_t);
-    rb_thread_t *th = ALLOC(rb_thread_t);
+    rb_vm_t * vm = malloc(sizeof(*vm));
+    rb_thread_t * th = malloc(sizeof(*th));
+    MEMZERO(th, rb_thread_t, 1);
+
+    rb_thread_set_current_raw(th);
 
     vm_init2(vm);
+#if defined(ENABLE_VM_OBJSPACE) && ENABLE_VM_OBJSPACE
+    vm->objspace = rb_objspace_alloc();
+#endif
     ruby_current_vm = vm;
 
-    th_init2(th);
+    th_init2(th, 0);
     th->vm = vm;
-    th->machine_stack_start = rb_gc_stack_start;
-#ifdef __ia64
-    th->machine_register_stack_start = rb_gc_register_stack_start;
-#endif
-    rb_thread_set_current_raw(th);
+    ruby_thread_init_stack(th);
 }
 
 /* top self */
@@ -1821,16 +1797,40 @@ main_to_s(VALUE obj)
 }
 
 VALUE
-rb_vm_top_self()
+rb_vm_top_self(void)
 {
     return GET_VM()->top_self;
 }
 
 void
-Init_top_self()
+Init_top_self(void)
 {
     rb_vm_t *vm = GET_VM();
 
     vm->top_self = rb_obj_alloc(rb_cObject);
     rb_define_singleton_method(rb_vm_top_self(), "to_s", main_to_s, 0);
 }
+
+VALUE *
+ruby_vm_verbose_ptr(rb_vm_t *vm)
+{
+    return &vm->verbose;
+}
+
+VALUE *
+ruby_vm_debug_ptr(rb_vm_t *vm)
+{
+    return &vm->debug;
+}
+
+VALUE *
+rb_ruby_verbose_ptr(void)
+{
+    return ruby_vm_verbose_ptr(GET_VM());
+}
+
+VALUE *
+rb_ruby_debug_ptr(void)
+{
+    return ruby_vm_debug_ptr(GET_VM());
+}
