@@ -4,15 +4,14 @@
   vm_opts.h - VM optimize option
 
   $Author: ko1 $
-  $Date: 2007-11-14 07:13:04 +0900 (Wed, 14 Nov 2007) $
 
   Copyright (C) 2004-2007 Koichi Sasada
 
 **********************************************************************/
 
 
-#ifndef VM_OPTS_H_INCLUDED
-#define VM_OPTS_H_INCLUDED
+#ifndef RUBY_VM_OPTS_H
+#define RUBY_VM_OPTS_H
 
 /* Compile options.
  * You can change these options at runtime by VM::CompileOption.
@@ -32,6 +31,7 @@
 
 /* C compiler depend */
 #define OPT_DIRECT_THREADED_CODE     1
+#define OPT_TOKEN_THREADED_CODE      0
 #define OPT_CALL_THREADED_CODE       0
 
 /* VM running option */
@@ -48,5 +48,4 @@
 /* misc */
 #define SUPPORT_JOKE                 0
 
-#endif /* VM_OPTS_H_INCLUDED */
-
+#endif /* RUBY_VM_OPTS_H */
